@@ -13,7 +13,7 @@
     Artifex Software, Inc.,  101 Lucas Valley Road #110,
     San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 
-    $Id: config_win32.h 8037 2007-06-10 01:51:33Z alexcher $
+    $Id: config_win32.h 8735 2008-05-15 16:17:48Z mvrhel $
 */
 
 /* configuration header file for compiling under Microsoft Windows */
@@ -34,7 +34,12 @@
   typedef unsigned int              uint32_t;
   /* no uint64_t */
 
-#  define vsnprintf _vsnprintf
+#if defined(_MSC_VER)
+	#if _MSC_VER < 1500  /* Visual Studio 2008 has definition for vsnprintf */
+		#define vsnprintf _vsnprintf
+	#endif
+#endif
+
 #  define snprintf _snprintf
 
 #endif /* _MSC_VER */
