@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: bfont.h 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: bfont.h 8881 2008-07-27 09:38:42Z leonardo $ */
 /* Interpreter internal routines and data needed for building fonts */
 /* Requires gxfont.h */
 
@@ -42,7 +42,8 @@ typedef enum {
     bf_Encoding_optional = 1,	/* build_gs_font */
     bf_UniqueID_ignored = 4,	/* build_gs_simple_font */
     bf_CharStrings_optional = 8,	/* build_gs_primitive_font */
-    bf_notdef_required = 16	/* build_gs_primitive_font */
+    bf_notdef_required = 16,	/* build_gs_primitive_font */
+    bf_has_font_file = 32   /* build_gs_TrueType_font ( only for gs_type42_font_init) */
 } build_font_options_t;
 
 /* In zbfont.c */
