@@ -1,58 +1,46 @@
-/* Copyright (C) 1999, 2000, Ghostgum Software Pty Ltd.  All rights reserved.
+/* Copyright (C) 2001-2008 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-// $Id: dwsetup.cpp,v 1.5.6.2 2002/04/20 22:11:39 ghostgum Exp $
+// $Id: dwsetup.cpp 8877 2008-07-25 22:51:14Z ghostgum $
 //
 //
-// This is the setup program for Win32 GNU Ghostscript
+// This is the setup program for Win32 GPL Ghostscript
 //
 // The starting point is a self extracting zip archive
 // with the following contents:
 //   setupgs.exe
 //   uninstgs.exe
 //   filelist.txt      (contains list of program files)
-//   fontlist.txt      (contains list of font files)
 //   gs#.##\*          (files listed in filelist.txt)
-//   fonts\*           (fonts listed in fontlist.txt)
 // This is the same as the zip file created by Aladdin Enterprises,
-// with the addition of setupgs.exe, uninstgs.exe, filelist.txt and 
-// fontlist.txt.
+// with the addition of setupgs.exe, uninstgs.exe, and filelist.txt
 //
-// The first line of the files filelist.txt and fontlist.txt
+// The first line of the file filelist.txt
 // contains the uninstall name to be used.  
 // The second line contains name of the main directory where 
 // uninstall log files are to be placed.  
 // Subsequent lines contain files to be copied (but not directories).
 // For example, filelist.txt might contain:
-//   GNU Ghostscript 6.50
-//   gs6.50
-//   gs6.50\bin\gsdll32.dll
-//   gs6.50\lib\gs_init.ps
-// The file fontlist.txt might contain:
-//   GNU Ghostscript Fonts
-//   fonts
-//   fonts\n019003l.pfb
-//   fonts\n019023l.pfb
+//   GPL Ghostscript 8.55
+//   gs8.55
+//   gs8.55\bin\gsdll32.dll
+//   gs8.55\lib\gs_init.ps
 //
 // The default install directory is c:\gs.
 // The default Start Menu Folder is Ghostscript.
 // These are set in the resources.
 // The setup program will create the following uninstall log files
 //   c:\gs\gs#.##\uninstal.txt
-//   c:\gs\fonts\uninstal.txt
 // The uninstall program (accessed through control panel) will not 
 // remove directories nor will it remove itself.
 //
@@ -60,7 +48,7 @@
 // location, no files will be copied, but the existence of each file 
 // will be checked.  This allows the archive to be unzipped, then
 // configured in its current location.  Running the uninstall will not 
-// remove uninstgs.exe, setupgs.exe, filelist.txt or fontlist.txt.
+// remove uninstgs.exe, setupgs.exe, or filelist.txt.
 
 
 #define STRICT
@@ -80,6 +68,21 @@
 #include "dwsetup.h"
 #include "dwinst.h"
 
+extern "C" {
+typedef HRESULT (WINAPI *PFN_SHGetFolderPath)(
+  HWND hwndOwner,
+  int nFolder,
+  HANDLE hToken,
+  DWORD dwFlags,
+  LPSTR pszPath);
+
+typedef BOOL (WINAPI *PFN_SHGetSpecialFolderPath)(
+  HWND hwndOwner,
+  LPTSTR lpszPath,
+  int nFolder,
+  BOOL fCreate);
+}
+
 //#define DEBUG
 
 #define UNINSTALLPROG "uninstgs.exe"
@@ -101,7 +104,7 @@ BOOL g_bNoCopy;
 // unzip self extractor.
 CHAR g_szSourceDir[MAXSTR];
 
-// Target directory for program and fonts.
+// Target directory for program.
 // Default loaded from resources
 CHAR g_szTargetDir[MAXSTR];
 
@@ -112,7 +115,7 @@ CHAR g_szTargetGroup[MAXSTR];
 // Setup application name, loaded from resources
 CHAR g_szAppName[MAXSTR];
 
-BOOL g_bInstallFonts = TRUE;
+BOOL g_bCJKFonts = FALSE;
 BOOL g_bAllUsers = FALSE;
 
 
@@ -128,17 +131,24 @@ BOOL g_bQuit = FALSE;	// TRUE = Get out of message loop.
 BOOL g_bError = FALSE;	// TRUE = Install was not successful
 BOOL is_winnt = FALSE;	// Disable "All Users" if not NT.
 
+#ifdef _WIN64
+#define DLGRETURN INT_PTR
+#else
+#define DLGRETURN BOOL
+#endif
 
 // Prototypes
-BOOL CALLBACK MainDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
+DLGRETURN CALLBACK MainDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
 void gs_addmess_count(const char *str, int count);
 void gs_addmess(const char *str);
 void gs_addmess_update(void);
 BOOL init();
 BOOL install_all();
 BOOL install_prog();
-BOOL install_fonts();
 BOOL make_filelist(int argc, char *argv[]);
+int get_font_path(char *path, unsigned int pathlen);
+BOOL write_cidfmap(const char *gspath, const char *cidpath);
+BOOL GetProgramFiles(LPTSTR path);
 
 
 //////////////////////////////////////////////////////////////////////
@@ -187,7 +197,7 @@ char twbuf[TWLENGTH];
 int twend;
 
 // Modeless Dialog Box
-BOOL CALLBACK 
+DLGRETURN CALLBACK 
 TextWinDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
     switch(message) {
@@ -335,7 +345,7 @@ gs_addmess_update(void)
 char szFolderName[MAXSTR];
 char szDirName[MAXSTR];
 
-BOOL CALLBACK 
+DLGRETURN CALLBACK 
 DirDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
 	WORD notify_message;
@@ -486,8 +496,12 @@ init()
 	}
 	
 	// Interactive setup
+	if (!GetProgramFiles(g_szTargetDir))
+	    strcpy(g_szTargetDir, "C:\\Program Files");
+	strcat(g_szTargetDir, "\\");
 	LoadString(g_hInstance, IDS_TARGET_DIR, 
-		g_szTargetDir, sizeof(g_szTargetDir));
+	    g_szTargetDir+strlen(g_szTargetDir), 
+	    sizeof(g_szTargetDir)-strlen(g_szTargetDir));
 	
 	// main dialog box
 	g_hMain = CreateDialogParam(g_hInstance, MAKEINTRESOURCE(IDD_MAIN), (HWND)NULL, MainDlgProc, (LPARAM)NULL);
@@ -509,7 +523,6 @@ init()
 	SetDlgItemText(g_hMain, IDC_TARGET_DIR, g_szTargetDir);
 	SetDlgItemText(g_hMain, IDC_TARGET_GROUP, g_szTargetGroup);
 	SetDlgItemText(g_hMain, IDC_PRODUCT_NAME, cinst.GetUninstallName());
-	SendDlgItemMessage(g_hMain, IDC_INSTALL_FONTS, BM_SETCHECK, BST_CHECKED, 0);
 	ShowWindow(g_hMain, SW_SHOWNORMAL);
 	
 	return (g_hMain != (HWND)NULL); /* success */
@@ -517,7 +530,7 @@ init()
 
 
 // Main Modeless Dialog Box
-BOOL CALLBACK 
+DLGRETURN CALLBACK 
 MainDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
 	switch(message) {
@@ -598,8 +611,8 @@ MainDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 				g_szTargetDir, sizeof(g_szTargetDir));
 			GetDlgItemText(hwnd, IDC_TARGET_GROUP, 
 				g_szTargetGroup, sizeof(g_szTargetGroup));
-			g_bInstallFonts = (SendDlgItemMessage(g_hMain, 
-				IDC_INSTALL_FONTS, BM_GETCHECK, 0, 0) 
+			g_bCJKFonts = (SendDlgItemMessage(g_hMain, 
+				IDC_CJK_FONTS, BM_GETCHECK, 0, 0) 
 				== BST_CHECKED);
 			g_bAllUsers = (SendDlgItemMessage(hwnd, 
 				IDC_ALLUSERS, BM_GETCHECK, 0, 0
@@ -657,11 +670,6 @@ install_all()
 		g_bError = TRUE;
 		return FALSE;
 	}
-	if (g_bInstallFonts && !install_fonts()) {
-		cinst.CleanUp();
-		g_bError = TRUE;
-		return FALSE;
-	}
 	
 	gs_addmess("Install successful\n");
 	
@@ -686,14 +694,16 @@ install_all()
 BOOL
 install_prog()
 {
-	char *regkey1 = "GNU Ghostscript";
+	char *regkey1 = "GPL Ghostscript";
 	char regkey2[16];
 	char szDLL[MAXSTR];
-	char szLIB[MAXSTR];
+	char szLIB[MAXSTR+MAXSTR];
 	char szProgram[MAXSTR];
 	char szArguments[MAXSTR];
 	char szDescription[MAXSTR];
 	char szDotVersion[MAXSTR];
+	char szPlatformSuffix[MAXSTR];
+	const char *pSuffix = "";
 	
 	if (g_bQuit)
 		return FALSE;
@@ -750,7 +760,15 @@ install_prog()
 	strcat(szLIB, cinst.GetMainDir());
 	strcat(szLIB, "\\lib;");
 	strcat(szLIB, g_szTargetDir);
-	strcat(szLIB, "\\fonts");
+	strcat(szLIB, "\\fonts;");
+	strcat(szLIB, g_szTargetDir);
+	strcat(szLIB, "\\");
+	strcat(szLIB, cinst.GetMainDir());
+	strcat(szLIB, "\\Resource");
+	if (g_bCJKFonts) {
+	    strcat(szLIB, ";");
+	    get_font_path(szLIB+strlen(szLIB), sizeof(szLIB)-strlen(szLIB)-1);
+	}
 	if (!cinst.UpdateRegistryValue(regkey1, regkey2, "GS_LIB", szLIB)) {
 		gs_addmess("Failed to add registry value\n");
 		return FALSE;
@@ -764,6 +782,22 @@ install_prog()
 	
 	// Add Start Menu items
 	gs_addmess("Adding Start Menu items\n");
+
+        memset(szPlatformSuffix, 0, sizeof(szPlatformSuffix));
+	if (GetProgramFiles(szPlatformSuffix)) {
+	    /* If ProgramFiles has a suffix like " (x86)" then use
+	     * it for Start menu entries to distinguish between
+	     * 32-bit and 64-bit programs.
+	     */
+	    for (pSuffix = szPlatformSuffix; *pSuffix; pSuffix++)
+		if ((pSuffix[0] == ' ') && (pSuffix[1] == '('))
+		    break;
+	}
+	else {
+	    pSuffix = "";
+	}
+
+
 	if (!cinst.StartMenuBegin()) {
 		gs_addmess("Failed to begin Start Menu update\n");
 		return FALSE;
@@ -775,7 +809,7 @@ install_prog()
 	strcpy(szArguments, "\042-I");
 	strcat(szArguments, szLIB);
 	strcat(szArguments, "\042");
-	sprintf(szDescription, "Ghostscript %s", szDotVersion);
+	sprintf(szDescription, "Ghostscript %s%s", szDotVersion, pSuffix);
 	if (!cinst.StartMenuAdd(szDescription, szProgram, szArguments)) {
 		gs_addmess("Failed to add Start Menu item\n");
 		return FALSE;
@@ -784,7 +818,8 @@ install_prog()
 	strcat(szProgram, "\\");
 	strcat(szProgram, cinst.GetMainDir());
 	strcat(szProgram, "\\doc\\Readme.htm");
-	sprintf(szDescription, "Ghostscript Readme %s", szDotVersion);
+	sprintf(szDescription, "Ghostscript Readme %s%s", 
+		szDotVersion, pSuffix);
 	if (!cinst.StartMenuAdd(szDescription, szProgram, NULL)) {
 		gs_addmess("Failed to add Start Menu item\n");
 		return FALSE;
@@ -793,6 +828,42 @@ install_prog()
 		gs_addmess("Failed to end Start Menu update\n");
 		return FALSE;
 	}
+
+        /* Create lib/cidfmap */
+	if (g_bCJKFonts) {
+		char szCIDFmap[MAXSTR];
+		char szCIDFmap_bak[MAXSTR];
+		char szGSPATH[MAXSTR];
+
+		/* backup old cidfmap */
+		strcpy(szCIDFmap, g_szTargetDir);
+		strcat(szCIDFmap, "\\");
+		strcat(szCIDFmap, cinst.GetMainDir());
+		strcat(szCIDFmap, "\\lib\\cidfmap");
+		strcpy(szCIDFmap_bak, szCIDFmap);
+		strcat(szCIDFmap_bak, ".bak");
+		gs_addmess("Backing up\n  ");
+		gs_addmess(szCIDFmap);
+		gs_addmess("\nto\n  ");
+		gs_addmess(szCIDFmap_bak);
+		gs_addmess("\n");
+		rename(szCIDFmap, szCIDFmap_bak);
+
+		/* mark backup for uninstall */
+		cinst.AppendFileNew(szCIDFmap_bak);
+
+		/* write new cidfmap */
+		gs_addmess("Writing cidfmap\n   ");
+		gs_addmess(szCIDFmap);
+		gs_addmess("\n");
+		strcpy(szGSPATH, g_szTargetDir);
+		strcat(szGSPATH, "\\");
+		strcat(szGSPATH, cinst.GetMainDir());
+		if (!write_cidfmap(szGSPATH, szCIDFmap)) {
+			gs_addmess("Failed to write cidfmap\n");
+			return FALSE;
+		}
+	}
 	
 	// consolidate logs into one uninstall file
 	if (cinst.MakeLog()) {
@@ -818,55 +889,85 @@ install_prog()
 }
 
 
-BOOL
-install_fonts()
+
+//////////////////////////////////////////////////////////////////////
+// Create lib/cidfmap based on installed fonts
+//////////////////////////////////////////////////////////////////////
+
+/* Get the path to enumerate for fonts */
+int
+get_font_path(char *path, unsigned int pathlen)
 {
-	cinst.SetMessageFunction(gs_addmess);
-	cinst.SetTargetDir(g_szTargetDir);
-	cinst.SetTargetGroup(g_szTargetGroup);
-	cinst.SetAllUsers(g_bAllUsers);
-	if (!cinst.Init(g_szSourceDir, "fontlist.txt"))
-		return FALSE;
-	
-	// copy files
-	if (!cinst.InstallFiles(g_bNoCopy, &g_bQuit)) {
-		gs_addmess("Font install failed\n");
-		return FALSE;
-	}
-	
-	if (g_bQuit)
-		return FALSE;
-	
-	if (g_bNoCopy) {
-		// Don't write uninstall log or entry
-		// since we didn't copy any files.
-		cinst.CleanUp();
-	}
-	else {
-		// consolidate logs into one uninstall file
-		if (cinst.MakeLog()) {
-			// add uninstall entry for "Add/Remove Programs"
-			gs_addmess("Adding uninstall program\n");
-			if (!cinst.WriteUninstall(UNINSTALLPROG, g_bNoCopy)) {
-				gs_addmess("Failed to write uninstall entry\n");
-				return FALSE;
-			}
-		}
-		else {
-			gs_addmess("Failed to write uninstall log\n");
-			// If batch install, files might be on a server
-			// in a write protected directory.
-			// Don't return an error for batch install.
-			if (g_bBatch)
-				return TRUE;
-			return FALSE;
-		}
-	}
-	
-	gs_addmess("Font install successful\n");
-	return TRUE;
+    int i;
+    int len = GetWindowsDirectory(path, pathlen);
+    if (len == 0)
+       return -1;
+    if (pathlen - strlen(path) < 8)
+       return -1;
+    strncat(path, "/fonts", pathlen - strlen(path) - 7);
+    for (i = strlen(path)-1; i >= 0; i--)
+       if (path[i] == '\\')
+           path[i] = '/';
+    return len;
 }
 
+BOOL write_cidfmap(const char *gspath, const char *cidpath)
+{
+    char fontpath[MAXSTR];
+    char buf[4*MAXSTR];
+    STARTUPINFO siStartInfo;
+    PROCESS_INFORMATION piProcInfo;
+
+    get_font_path(fontpath, sizeof(fontpath)-1);
+
+    strcpy(buf, "\042");
+    strcat(buf, gspath);
+    strcat(buf, "\\bin\\gswin32c.exe\042 -q -dBATCH \042-sFONTDIR=");
+    strcat(buf, fontpath);
+    strcat(buf, "\042 \042");
+    strcat(buf, "-sCIDFMAP=");
+    strcat(buf, cidpath);
+    strcat(buf, "\042 \042");
+    strcat(buf, gspath);
+    strcat(buf, "\\lib\\mkcidfm.ps\042");
+
+    siStartInfo.cb = sizeof(STARTUPINFO);
+    siStartInfo.lpReserved = NULL;
+    siStartInfo.lpDesktop = NULL;
+    siStartInfo.lpTitle = NULL;  /* use executable name as title */
+    siStartInfo.dwX = siStartInfo.dwY = CW_USEDEFAULT;		/* ignored */
+    siStartInfo.dwXSize = siStartInfo.dwYSize = CW_USEDEFAULT;	/* ignored */
+    siStartInfo.dwXCountChars = 80;
+    siStartInfo.dwYCountChars = 25;
+    siStartInfo.dwFillAttribute = 0;			/* ignored */
+    siStartInfo.dwFlags = STARTF_USESHOWWINDOW;
+    siStartInfo.wShowWindow = SW_HIDE;
+    siStartInfo.cbReserved2 = 0;
+    siStartInfo.lpReserved2 = NULL;
+    siStartInfo.hStdInput = NULL;
+    siStartInfo.hStdOutput = NULL;
+    siStartInfo.hStdError = NULL;
+
+    /* Create the child process. */
+    if (!CreateProcess(NULL,
+        (char *)buf,  /* command line                       */
+        NULL,          /* process security attributes        */
+        NULL,          /* primary thread security attributes */
+        FALSE,         /* handles are not inherited          */
+        0,             /* creation flags                     */
+        NULL,          /* environment                        */
+        NULL,          /* use parent's current directory     */
+        &siStartInfo,  /* STARTUPINFO pointer                */
+        &piProcInfo))  /* receives PROCESS_INFORMATION  */
+	    return FALSE;
+
+    /* We don't care if ghostscript fails, so just return */
+
+    CloseHandle(piProcInfo.hProcess);
+    CloseHandle(piProcInfo.hThread);
+
+    return TRUE;
+}
 
 
 //////////////////////////////////////////////////////////////////////
@@ -1012,7 +1113,7 @@ BOOL make_filelist(int argc, char *argv[])
 		    if ((title == NULL) || (strlen(title) == 0) ||
 			(dir == NULL) || (strlen(dir) == 0) ||
 			(list == NULL) || (strlen(list) == 0)) {
-			message_box("Usage: setupgs -title \042GNU Ghostscript #.##\042 -dir \042gs#.##\042 -list \042filelist.txt\042 spec1 spec2 specn\n");
+			message_box("Usage: setupgs -title \042GPL Ghostscript #.##\042 -dir \042gs#.##\042 -list \042filelist.txt\042 spec1 spec2 specn\n");
 			return FALSE;
 		    }
 		    if (fList == (FILE *)NULL) {
@@ -1060,3 +1161,78 @@ BOOL make_filelist(int argc, char *argv[])
 }
 
 //////////////////////////////////////////////////////////////////////
+
+#ifndef CSIDL_PROGRAM_FILES
+#define CSIDL_PROGRAM_FILES 0x0026
+#endif
+#ifndef CSIDL_FLAG_CREATE
+#define CSIDL_FLAG_CREATE 0x8000
+#endif
+#ifndef SHGFP_TYPE_CURRENT
+#define SHGFP_TYPE_CURRENT 0
+#endif
+
+BOOL 
+GetProgramFiles(LPTSTR path) 
+{
+    PFN_SHGetSpecialFolderPath PSHGetSpecialFolderPath = NULL;
+    PFN_SHGetFolderPath PSHGetFolderPath = NULL;
+    HMODULE hModuleShell32 = NULL;
+    HMODULE hModuleShfolder = NULL;
+    BOOL fOk = FALSE;
+    hModuleShfolder = LoadLibrary("shfolder.dll");
+    hModuleShell32 = LoadLibrary("shell32.dll");
+
+    if (hModuleShfolder) {
+	PSHGetFolderPath = (PFN_SHGetFolderPath)
+	    GetProcAddress(hModuleShfolder, "SHGetFolderPathA");
+	if (PSHGetFolderPath) {
+	    fOk = (PSHGetFolderPath(HWND_DESKTOP, 
+		CSIDL_PROGRAM_FILES | CSIDL_FLAG_CREATE, 
+		NULL, SHGFP_TYPE_CURRENT, path) == S_OK);
+	}
+    }
+
+    if (!fOk && hModuleShell32) {
+	PSHGetFolderPath = (PFN_SHGetFolderPath)
+	    GetProcAddress(hModuleShell32, "SHGetFolderPathA");
+	if (PSHGetFolderPath) {
+	    fOk = (PSHGetFolderPath(HWND_DESKTOP, 
+		CSIDL_PROGRAM_FILES | CSIDL_FLAG_CREATE, 
+		NULL, SHGFP_TYPE_CURRENT, path) == S_OK);
+	}
+    }
+
+    if (!fOk && hModuleShell32) {
+	PSHGetSpecialFolderPath = (PFN_SHGetSpecialFolderPath)
+	    GetProcAddress(hModuleShell32, "SHGetSpecialFolderPathA");
+	if (PSHGetSpecialFolderPath) {
+	    fOk = PSHGetSpecialFolderPath(HWND_DESKTOP, path,
+		CSIDL_PROGRAM_FILES, TRUE);
+	}
+    }
+
+    if (!fOk) {
+	/* If all else fails (probably Win95), try the registry */
+	LONG rc;
+	HKEY hkey;
+	DWORD cbData;
+	DWORD keytype;
+	rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
+	    "SOFTWARE\\Microsoft\\Windows\\CurrentVersion", 0, KEY_READ, &hkey);
+	if (rc == ERROR_SUCCESS) {
+	    cbData = MAX_PATH;
+	    keytype =  REG_SZ;
+	    if (rc == ERROR_SUCCESS)
+		rc = RegQueryValueEx(hkey, "ProgramFilesDir", 0, &keytype, 
+		    (LPBYTE)path, &cbData);
+	    RegCloseKey(hkey);
+	}
+	fOk = (rc == ERROR_SUCCESS);
+    }
+    return fOk;
+}
+
+
+//////////////////////////////////////////////////////////////////////
+
