@@ -11,14 +11,14 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: dwtrace.h 8022 2007-06-05 22:23:38Z giles $ */
+/* $Id: dwtrace.h 8604 2008-03-21 05:07:53Z leonardo $ */
 /* The interface of Graphical trace server for Windows */
 
 #ifndef dwtrace_INCLUDED
 #  define dwtrace_INCLUDED
 
 extern struct vd_trace_interface_s visual_tracer;
-void visual_tracer_init(void);
+struct vd_trace_interface_s *visual_tracer_init(void);
 void visual_tracer_close(void);
 
 #endif /* dwtrace_INCLUDED */
