@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/*$Id: gdevbbox.c 8250 2007-09-25 13:31:24Z giles $ */
+/*$Id: gdevbbox.c 8803 2008-06-24 14:16:29Z leonardo $ */
 /* Device for tracking bounding box */
 #include "math_.h"
 #include "memory_.h"
@@ -755,7 +755,21 @@ bbox_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
 	 dev_proc(tdev, fill_path));
     int code;
 
-    if (!GX_DC_IS_TRANSPARENT(pdevc, bdev) && !gx_path_is_void(ppath)) {
+    if (ppath == NULL) {
+	/* A special handling of shfill with no path. */
+	gs_fixed_rect ibox;
+	gs_fixed_point adjust;
+
+	if (pcpath == NULL)
+	    return 0;
+	gx_cpath_inner_box(pcpath, &ibox);
+	adjust = params->adjust;
+	if (params->fill_zero_width)
+	    gx_adjust_if_empty(&ibox, &adjust);
+	adjust_box(&ibox, adjust);
+	BBOX_ADD_RECT(bdev, ibox.p.x, ibox.p.y, ibox.q.x, ibox.q.y);
+	return 0;
+    } else if (!GX_DC_IS_TRANSPARENT(pdevc, bdev) && !gx_path_is_void(ppath)) {
 	gs_fixed_rect ibox;
 	gs_fixed_point adjust;
 
@@ -903,7 +917,6 @@ bbox_fill_mask(gx_device * dev,
 
 typedef struct bbox_image_enum_s {
     gx_image_enum_common;
-    gs_memory_t *memory;
     gs_matrix matrix;		/* map from image space to device space */
     const gx_clip_path *pcpath;
     gx_image_enum_common_t *target_info;
@@ -1002,7 +1015,7 @@ bbox_begin_typed_image(gx_device * dev,
 	gx_device_bbox *const bdev = (gx_device_bbox *) dev;
 	gx_device *tdev = bdev->target;
 	dev_proc_begin_typed_image((*begin_typed_image));
-	byte wanted[GS_IMAGE_MAX_PLANES];
+	byte wanted[GS_IMAGE_MAX_COMPONENTS];
 
 	if (tdev == 0) {
 	    tdev = dev;
@@ -1101,7 +1114,7 @@ bbox_image_end_image(gx_image_enum_common_t * info, bool draw_last)
     bbox_image_enum *pbe = (bbox_image_enum *) info;
     int code = gx_image_end(pbe->target_info, draw_last);
 
-    gs_free_object(pbe->memory, pbe, "bbox_end_image");
+    gx_image_free_enum(&info);
     return code;
 }
 
