@@ -1,26 +1,20 @@
-/* Copyright (C) 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/*$RCSfile: gdevdbit.c,v $ $Revision: 1.2.2.1 $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/*$Id: gdevdbit.c 8588 2008-03-05 23:35:50Z leonardo $ */
 /* Default device bitmap copying implementation */
 #include "gx.h"
 #include "gpcheck.h"
+#include "gserror.h"
 #include "gserrors.h"
 #include "gsbittab.h"
 #include "gsrect.h"
@@ -56,8 +50,12 @@ gx_default_copy_mono(gx_device * dev, const byte * data,
     bool invert;
     gx_color_index color;
     gx_device_color devc;
-
+    
+    if (!data) 
+	gs_throw(-997, __func__ );
     fit_copy(dev, data, dx, raster, id, x, y, w, h);
+    if (!data) 
+	gs_throw(-997, __func__ );
     if (one != gx_no_color_index) {
 	invert = false;
 	color = one;
@@ -72,7 +70,11 @@ gx_default_copy_mono(gx_device * dev, const byte * data,
 	invert = true;
 	color = zero;
     }
-    color_set_pure(&devc, color);
+    if (!data) 
+	gs_throw(-997, __func__ );
+    set_nonclient_dev_color(&devc, color);
+    if (!data) 
+	gs_throw(-997, __func__ );
     return gx_dc_default_fill_masked
 	(&devc, data, dx, raster, id, x, y, w, h, dev, rop3_T, invert);
 }
@@ -110,6 +112,14 @@ gx_default_copy_color(gx_device * dev, const byte * data,
 	    if (depth >= 8) {
 		color = *ptr++;
 		switch (depth) {
+		    case 64:
+			color = (color << 8) + *ptr++;
+		    case 56:
+			color = (color << 8) + *ptr++;
+		    case 48:
+			color = (color << 8) + *ptr++;
+		    case 40:
+			color = (color << 8) + *ptr++;
 		    case 32:
 			color = (color << 8) + *ptr++;
 		    case 24:
@@ -171,12 +181,13 @@ gx_default_copy_alpha(gx_device * dev, const byte * data, int data_x,
 	const byte *row;
 	gs_memory_t *mem = dev->memory;
 	int bpp = dev->color_info.depth;
+	int ncomps = dev->color_info.num_components;
 	uint in_size = gx_device_raster(dev, false);
 	byte *lin;
 	uint out_size;
 	byte *lout;
 	int code = 0;
-	gx_color_value color_rgb[3];
+	gx_color_value color_cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
 	int ry;
 
 	fit_copy(dev, data, data_x, raster, id, x, y, width, height);
@@ -188,7 +199,7 @@ gx_default_copy_alpha(gx_device * dev, const byte * data, int data_x,
 	    code = gs_note_error(gs_error_VMerror);
 	    goto out;
 	}
-	(*dev_proc(dev, map_color_rgb)) (dev, color, color_rgb);
+	(*dev_proc(dev, decode_color)) (dev, color, color_cv);
 	for (ry = y; ry < y + height; row += raster, ++ry) {
 	    byte *line;
 	    int sx, rx;
@@ -217,13 +228,25 @@ gx_default_copy_alpha(gx_device * dev, const byte * data, int data_x,
 			    const byte *src = line + (bit >> 3);
 
 			    previous =
-				(*src >> (8 - (bit + bpp))) &
+				(*src >> (8 - ((bit & 7) + bpp))) &
 				((1 << bpp) - 1);
 			} else {
 			    const byte *src = line + (rx * (bpp >> 3));
 
 			    previous = 0;
 			    switch (bpp >> 3) {
+				case 8:
+				    previous += (gx_color_index) * src++ 
+					<< sample_bound_shift(previous, 56);
+				case 7:
+				    previous += (gx_color_index) * src++
+					<< sample_bound_shift(previous, 48);
+				case 6:
+				    previous += (gx_color_index) * src++
+					<< sample_bound_shift(previous, 40);
+				case 5:
+				    previous += (gx_color_index) * src++
+					<< sample_bound_shift(previous, 32);
 				case 4:
 				    previous += (gx_color_index) * src++ << 24;
 				case 3:
@@ -237,25 +260,24 @@ gx_default_copy_alpha(gx_device * dev, const byte * data, int data_x,
 		    }
 		    if (alpha == 0) {	/* Just write the old color. */
 			composite = previous;
-		    } else {	/* Blend RGB values. */
-			gx_color_value rgb[3];
+		    } else {	/* Blend values. */
+			gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
+			int i;
 
-			(*dev_proc(dev, map_color_rgb)) (dev, previous, rgb);
-#if arch_ints_are_short
+			(*dev_proc(dev, decode_color)) (dev, previous, cv);
+#if ARCH_INTS_ARE_SHORT
 #  define b_int long
 #else
 #  define b_int int
 #endif
 #define make_shade(old, clr, alpha, amax) \
   (old) + (((b_int)(clr) - (b_int)(old)) * (alpha) / (amax))
-			rgb[0] = make_shade(rgb[0], color_rgb[0], alpha, 15);
-			rgb[1] = make_shade(rgb[1], color_rgb[1], alpha, 15);
-			rgb[2] = make_shade(rgb[2], color_rgb[2], alpha, 15);
+			for (i=0; i<ncomps; i++)
+			    cv[i] = make_shade(cv[i], color_cv[i], alpha, 15);
 #undef b_int
 #undef make_shade
 			composite =
-			    (*dev_proc(dev, map_rgb_color)) (dev, rgb[0],
-							     rgb[1], rgb[2]);
+			    (*dev_proc(dev, encode_color)) (dev, cv);
 			if (composite == gx_no_color_index) {	/* The device can't represent this color. */
 			    /* Move the alpha value towards 0 or 1. */
 			    if (alpha == 7)	/* move 1/2 towards 1 */
@@ -295,111 +317,20 @@ gx_default_fill_mask(gx_device * orig_dev,
 {
     gx_device *dev;
     gx_device_clip cdev;
-    gx_color_index colors[2];
-    gx_strip_bitmap *tile;
 
-    if (gx_dc_is_pure(pdcolor)) {
-	tile = 0;
-	colors[0] = gx_no_color_index;
-	colors[1] = gx_dc_pure_color(pdcolor);
-    } else if (gx_dc_is_binary_halftone(pdcolor)) {
-	tile = gx_dc_binary_tile(pdcolor);
-	colors[0] = gx_dc_binary_color0(pdcolor);
-	colors[1] = gx_dc_binary_color1(pdcolor);
-    } else
-	return_error(gs_error_unknownerror);	/* not implemented */
     if (pcpath != 0) {
-	gx_make_clip_path_device(&cdev, pcpath);
-	cdev.target = orig_dev;
+	gx_make_clip_device_on_stack(&cdev, pcpath, orig_dev);
 	dev = (gx_device *) & cdev;
-	(*dev_proc(dev, open_device)) (dev);
     } else
 	dev = orig_dev;
     if (depth > 1) {
 	/****** CAN'T DO ROP OR HALFTONE WITH ALPHA ******/
 	return (*dev_proc(dev, copy_alpha))
-	    (dev, data, dx, raster, id, x, y, w, h, colors[1], depth);
-    }
-    if (lop != lop_default) {
-	gx_color_index scolors[2];
-
-	scolors[0] = gx_device_white(dev);
-	scolors[1] = gx_device_black(dev);
-	if (tile == 0)
-	    colors[0] = colors[1];	/* pure color */
-	/*
-	 * We want to write only where the mask is a 1, so enable source
-	 * transparency.  We have to include S in the operation,
-	 * otherwise S_transparent will be ignored.
-	 */
-	return (*dev_proc(dev, strip_copy_rop))
-	    (dev, data, dx, raster, id, scolors, tile, colors,
-	     x, y, w, h,
-	     gx_dc_phase(pdcolor).x, gx_dc_phase(pdcolor).y,
-	     lop | (rop3_S | lop_S_transparent));
-    }
-    if (tile == 0) {
-	return (*dev_proc(dev, copy_mono))
 	    (dev, data, dx, raster, id, x, y, w, h,
-	     gx_no_color_index, colors[1]);
-    }
-    /*
-     * Use the same approach as the default copy_mono (above).  We
-     * should really clip to the intersection of the bounding boxes of
-     * the device and the clipping path, but it's too much work.
-     */
-    fit_copy(orig_dev, data, dx, raster, id, x, y, w, h);
-    {
-	dev_proc_strip_tile_rectangle((*tile_proc)) =
-	    dev_proc(dev, strip_tile_rectangle);
-	const byte *row = data + (dx >> 3);
-	int dx_bit = dx & 7;
-	int wdx = w + dx_bit;
-	int iy;
-
-	for (row = data, iy = 0; iy < h; row += raster, iy++) {
-	    int ix;
-
-	    for (ix = dx_bit; ix < wdx;) {
-		int i0;
-		uint b;
-		uint len;
-		int code;
-
-		/* Skip 0-bits. */
-		b = row[ix >> 3];
-		len = byte_bit_run_length[ix & 7][b ^ 0xff];
-		if (len) {
-		    ix += ((len - 1) & 7) + 1;
-		    continue;
-		}
-		/* Scan 1-bits. */
-		i0 = ix;
-		for (;;) {
-		    b = row[ix >> 3];
-		    len = byte_bit_run_length[ix & 7][b];
-		    if (!len)
-			break;
-		    ix += ((len - 1) & 7) + 1;
-		    if (ix >= wdx) {
-			ix = wdx;
-			break;
-		    }
-		    if (len < 8)
-			break;
-		}
-		/* Now color the run from i0 to ix. */
-		code = (*tile_proc)
-		    (dev, tile, i0 - dx_bit + x, iy + y, ix - i0, 1,
-		     colors[0], colors[1],
-		     gx_dc_phase(pdcolor).x, gx_dc_phase(pdcolor).y);
-		if (code < 0)
-		    return code;
-#undef row_bit
-	    }
-	}
-    }
-    return 0;
+	     gx_dc_pure_color(pdcolor), depth);
+    } else
+        return pdcolor->type->fill_masked(pdcolor, data, dx, raster, id,
+				          x, y, w, h, dev, lop, false);
 }
 
 /* Default implementation of strip_tile_rectangle */
@@ -416,6 +347,8 @@ gx_default_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
     int shift = tiles->shift;
     gs_id tile_id = tiles->id;
 
+    if (rwidth == 0 || rheight == 0)
+	return_error(gs_error_unregistered); /* Must not happen. */
     fit_fill_xy(dev, x, y, w, h);
 
 #ifdef DEBUG
@@ -479,7 +412,7 @@ gx_default_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 	int code;
 
 	if (color0 == gx_no_color_index && color1 == gx_no_color_index)
-	    proc_color = dev_proc(dev, copy_color);
+	    proc_color = dev_proc(dev, copy_color), proc_mono = 0;
 	else
 	    proc_color = 0, proc_mono = dev_proc(dev, copy_mono);
 
@@ -489,7 +422,7 @@ gx_default_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
      (*proc_color)(dev, row, srcx, raster, id, tx, ty, tw, th) :\
      (*proc_mono)(dev, row, srcx, raster, id, tx, ty, tw, th, color0, color1));\
   if (code < 0) return_error(code);\
-  return_if_interrupt()
+  return_if_interrupt(dev->memory)
 #ifdef DEBUG
 #define copy_tile(srcx, tx, ty, tw, th, tid)\
   if_debug6('t', "   copy id=%lu sx=%d => x=%d y=%d w=%d h=%d\n",\
