@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevdflt.c 8488 2008-01-17 13:37:19Z leonardo $ */
+/* $Id: gdevdflt.c 8585 2008-03-03 16:01:12Z leonardo $ */
 /* Default device implementation */
 #include "math_.h"
 #include "gx.h"
@@ -881,6 +881,15 @@ gx_default_composite_clist_read_update(gs_composite_t *pxcte, gx_device * cdev,
     return 0;			/* Do nothing */
 }
 
+/*
+ * Default handler for get_cropping returns no cropping.
+ */
+int
+gx_default_composite_get_cropping(const gs_composite_t *pxcte, int *ry, int *rheight)
+{
+    return 0;			/* No cropping. */
+}
+
 int
 gx_default_finish_copydevice(gx_device *dev, const gx_device *from_dev)
 {
