@@ -1,20 +1,16 @@
-/* Copyright (C) 2001-2004, Ghostgum Software Pty Ltd.  All rights reserved.
-
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
+  
    This software is provided AS-IS with no warranty, either express or
    implied.
-   
-   This software is distributed under license and may not be copied,
-   modified or distributed except as expressly authorized under the terms
-   of the license contained in the file LICENSE in this distribution.
-   
-   For more information about licensing, please refer to
-   http://www.ghostscript.com/licensing/. For information on
-   commercial licensing, go to http://www.artifex.com/licensing/ or
-   contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
- */
 
-/* $Id: gdevdsp2.h,v 1.8 2004/07/07 09:33:19 ghostgum Exp $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevdsp2.h 8022 2007-06-05 22:23:38Z giles $ */
 /* gdevdsp2.c */
 
 #ifndef gdevdsp2_INCLUDED
