@@ -1,20 +1,16 @@
-/* Copyright (C) 1989, 1992, 1995 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/*$Id: gdevepsc.c,v 1.3.2.1 2002/02/22 19:45:56 ray Exp $*/
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevepsc.c 8250 2007-09-25 13:31:24Z giles $*/
 /* Epson color dot-matrix printer driver by dave@exlog.com */
 #include "gdevprn.h"
 
@@ -67,30 +63,29 @@
 **	the ESC-r n value
 */
 static char rgb_color[2][2][2] =	{
-	BLACK, VIOLET, GREEN,
-	CYAN, RED, MAGENTA,
-	YELLOW, WHITE,
+	{{BLACK, VIOLET}, {GREEN, CYAN}}, 
+	{{RED, MAGENTA}, {YELLOW, WHITE}}
 	};
 
 /* Map an RGB color to a printer color. */
 #define cv_shift (sizeof(gx_color_value) * 8 - 1)
-private gx_color_index
-epson_map_rgb_color(gx_device *dev,
-  gx_color_value r, gx_color_value g, gx_color_value b)
+static gx_color_index
+epson_map_rgb_color(gx_device *dev, const gx_color_value cv[])
 {
-if (gx_device_has_color(dev))
-	{
+
+    gx_color_value r = cv[0];
+    gx_color_value g = cv[1];
+    gx_color_value b = cv[2];
+    
+    if (gx_device_has_color(dev))
 /* use ^7 so WHITE is 0 for internal calculations */
-	return (gx_color_index)rgb_color[r >> cv_shift][g >> cv_shift][b >> cv_shift] ^ 7;	
-	}
-else
-	{
-	return gx_default_map_rgb_color(dev, r, g, b);
-	}
+        return (gx_color_index)rgb_color[r >> cv_shift][g >> cv_shift][b >> cv_shift] ^ 7;	
+    else
+	return gx_default_map_rgb_color(dev, cv);
 }
 
 /* Map the printer color back to RGB. */
-private int
+static int
 epson_map_color_rgb(gx_device *dev, gx_color_index color,
   gx_color_value prgb[3])
 {
@@ -121,9 +116,9 @@ if (gx_device_has_color(dev))
 }
 
 /* The device descriptor */
-private dev_proc_print_page(epsc_print_page);
+static dev_proc_print_page(epsc_print_page);
 
-private gx_device_procs epson_procs =
+static gx_device_procs epson_procs =
   prn_color_procs(gdev_prn_open, gdev_prn_output_page, gdev_prn_close,
     epson_map_rgb_color, epson_map_color_rgb); 
 
@@ -137,16 +132,16 @@ const gx_device_printer far_data gs_epsonc_device =
 /* ------ Internal routines ------ */
 
 /* Forward references */
-private void epsc_output_run(P6(byte *, int, int, char, FILE *, int));
+static void epsc_output_run(byte *, int, int, char, FILE *, int);
 
 /* Send the page to the printer. */
 #define DD 0x80				/* double density flag */
-private int
+static int
 epsc_print_page(gx_device_printer *pdev, FILE *prn_stream)
-{	static char graphics_modes_9[5] =
+{	static int graphics_modes_9[5] =
 	   {	-1, 0 /*60*/, 1	/*120*/, -1, DD+3 /*240*/
 	   };
-	static char graphics_modes_24[7] =
+	static int graphics_modes_24[7] =
 	   {	-1, 32 /*60*/, 33 /*120*/, 39 /*180*/,
 		-1, -1, DD+40 /*360*/
 	   };
@@ -154,12 +149,12 @@ epsc_print_page(gx_device_printer *pdev, FILE *prn_stream)
 	int y_mult = (y_24pin ? 3 : 1);
 	int line_size = (pdev->width + 7) >> 3;	/* always mono */
 	int in_size = line_size * (8 * y_mult);
-	byte *in = (byte *)gs_malloc(in_size+1, 1, "epsc_print_page(in)");
+	byte *in = (byte *)gs_malloc(pdev->memory, in_size+1, 1, "epsc_print_page(in)");
 	int out_size = ((pdev->width + 7) & -8) * y_mult;
-	byte *out = (byte *)gs_malloc(out_size+1, 1, "epsc_print_page(out)");
-	int x_dpi = pdev->x_pixels_per_inch;
-	char start_graphics =
-		(y_24pin ? graphics_modes_24 : graphics_modes_9)[x_dpi / 60];
+	byte *out = (byte *)gs_malloc(pdev->memory, out_size+1, 1, "epsc_print_page(out)");
+	int x_dpi = (int)pdev->x_pixels_per_inch;
+	char start_graphics = (char)
+		((y_24pin ? graphics_modes_24 : graphics_modes_9)[x_dpi / 60]);
 	int first_pass = (start_graphics & DD ? 1 : 0);
 	int last_pass = first_pass * 2;
 	int dots_per_space = x_dpi / 10;	/* pica space = 1/10" */
@@ -173,8 +168,8 @@ epsc_print_page(gx_device_printer *pdev, FILE *prn_stream)
 
 	/* Check allocations */
 	if ( in == 0 || out == 0 )
-	   {	if ( in ) gs_free((char *)in, in_size+1, 1, "epsc_print_page(in)");
-		if ( out ) gs_free((char *)out, out_size+1, 1, "epsc_print_page(out)");
+	    {	if ( in ) gs_free(pdev->memory, (char *)in, in_size+1, 1, "epsc_print_page(in)");
+		if ( out ) gs_free(pdev->memory, (char *)out, out_size+1, 1, "epsc_print_page(out)");
 		return -1;
 	   }
 
@@ -186,13 +181,13 @@ epsc_print_page(gx_device_printer *pdev, FILE *prn_stream)
 		{
 		color_line_size = gdev_mem_bytes_per_scan_line((gx_device *)pdev);
 		color_in_size = color_line_size * (8 * y_mult);
-		if((color_in = (byte *)gs_malloc(color_in_size+1, 1,
-			"epsc_print_page(color)")) == 0)
-			{
-			gs_free((char *)in, in_size+1, 1, "epsc_print_page(in)");
-			gs_free((char *)out, out_size+1, 1, "epsc_print_page(out)");
+		if((color_in = (byte *)gs_malloc(pdev->memory, color_in_size+1, 1,
+						 "epsc_print_page(color)")) == 0)
+		    {
+			gs_free(pdev->memory, (char *)in, in_size+1, 1, "epsc_print_page(in)");
+			gs_free(pdev->memory, (char *)out, out_size+1, 1, "epsc_print_page(out)");
 			return(-1);
-			}
+		    }
 		}
 	else
 		{
@@ -259,7 +254,7 @@ epsc_print_page(gx_device_printer *pdev, FILE *prn_stream)
 
 		if (gx_device_has_color(pdev))
 			{
-			register i,j;
+			register int i,j;
 			register byte *outbuf, *realbuf;
 			byte current_color;
 			int end_next_bits = whole_bits;
@@ -418,22 +413,22 @@ epsc_print_page(gx_device_printer *pdev, FILE *prn_stream)
 	fputs("\f\033@", prn_stream);
 
 
-	gs_free((char *)out, out_size+1, 1, "epsc_print_page(out)");
-	gs_free((char *)in, in_size+1, 1, "epsc_print_page(in)");
+	gs_free(pdev->memory, (char *)out, out_size+1, 1, "epsc_print_page(out)");
+	gs_free(pdev->memory, (char *)in, in_size+1, 1, "epsc_print_page(in)");
 	if (gx_device_has_color(pdev))
-		gs_free((char *)color_in, color_in_size+1, 1, "epsc_print_page(rin)");
+	    gs_free(pdev->memory, (char *)color_in, color_in_size+1, 1, "epsc_print_page(rin)");
 	return 0;
 }
 
 /* Output a single graphics command. */
 /* pass=0 for all columns, 1 for even columns, 2 for odd columns. */
-private void
+static void
 epsc_output_run(byte *data, int count, int y_mult,
   char start_graphics, FILE *prn_stream, int pass)
 {	int xcount = count / y_mult;
 	fputc(033, prn_stream);
 	if ( !(start_graphics & ~3) )
-	   {	fputc("KLYZ"[start_graphics], prn_stream);
+	   {	fputc("KLYZ"[(int)start_graphics], prn_stream);
 	   }
 	else
 	   {	fputc('*', prn_stream);
