@@ -1,23 +1,16 @@
-/* Copyright (C) 1992, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
-*/
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/*$RCSfile: gdevpbm.c,v $ $Revision: 1.3.2.1 $ */
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+/* $Id: gdevpbm.c 8414 2007-11-28 20:04:34Z ray $ */
 /* Portable Bit/Gray/PixMap drivers */
 #include "gdevprn.h"
 #include "gscdefs.h"
@@ -37,7 +30,7 @@
  */
 
 /*
- * There are 7 (families of) drivers here, plus one less related one:
+ * There are 8 (families of) drivers here, plus one less related one:
  *      pbm[raw] - outputs PBM (black and white).
  *      pgm[raw] - outputs PGM (gray-scale).
  *      pgnm[raw] - outputs PBM if the page contains only black and white,
@@ -48,6 +41,8 @@
  *        otherwise PPM.
  *      pkm[raw] - computes internally in CMYK, outputs PPM (RGB).
  *      pksm[raw] - computes internally in CMYK, outputs 4 PBM pages.
+ *      pamcmyk32 - outputs CMYK as PAM 8-bits per color
+ *      pam - previous name for the pamcmyk32 device retained for backwards compatibility
  *      plan9bm - outputs Plan 9 bitmap format.
  */
 
@@ -62,6 +57,7 @@
  *      ppmraw, pnmraw: 4(3x1), 8(3x2), 16(3x5), 24(3x8).  [3-24]
  *      pkm, pkmraw: 4(4x1), 8(4x2), 16(4x4), 32(4x8).  [4-32]
  *	pksm, pksmraw: ibid.
+ *      pam: 32 (CMYK)
  */
 
 /* Structure for P*M devices, which extend the generic printer device. */
@@ -107,28 +103,34 @@ typedef struct gx_device_pbm_s gx_device_pbm;
 }
 
 /* For all but PBM, we need our own color mapping and alpha procedures. */
-private dev_proc_map_rgb_color(pgm_map_rgb_color);
-private dev_proc_map_rgb_color(ppm_map_rgb_color);
-private dev_proc_map_color_rgb(pgm_map_color_rgb);
-private dev_proc_map_color_rgb(ppm_map_color_rgb);
-private dev_proc_map_cmyk_color(pkm_map_cmyk_color);
-private dev_proc_map_color_rgb(pkm_map_color_rgb);
-private dev_proc_get_params(ppm_get_params);
-private dev_proc_put_params(ppm_put_params);
-private dev_proc_copy_alpha(pnm_copy_alpha);
-private dev_proc_begin_typed_image(pnm_begin_typed_image);
+static dev_proc_map_rgb_color(pgm_map_rgb_color);
+static dev_proc_map_rgb_color(ppm_map_rgb_color);
+static dev_proc_map_color_rgb(pgm_map_color_rgb);
+static dev_proc_map_color_rgb(ppm_map_color_rgb);
+static dev_proc_map_cmyk_color(pkm_map_cmyk_color);
+static dev_proc_map_color_rgb(pkm_map_color_rgb);
+static dev_proc_get_params(ppm_get_params);
+static dev_proc_put_params(ppm_put_params);
+static dev_proc_copy_alpha(pnm_copy_alpha);
+static dev_proc_begin_typed_image(pnm_begin_typed_image);
 
 /* We need to initialize uses_color when opening the device, */
 /* and after each showpage. */
-private dev_proc_open_device(ppm_open);
-private dev_proc_output_page(ppm_output_page);
+static dev_proc_open_device(ppm_open);
+static dev_proc_output_page(ppm_output_page);
 
 /* And of course we need our own print-page routines. */
-private dev_proc_print_page(pbm_print_page);
-private dev_proc_print_page(pgm_print_page);
-private dev_proc_print_page(ppm_print_page);
-private dev_proc_print_page(pkm_print_page);
-private dev_proc_print_page(psm_print_page);
+static dev_proc_print_page(pbm_print_page);
+static dev_proc_print_page(pgm_print_page);
+static dev_proc_print_page(ppm_print_page);
+static dev_proc_print_page(pkm_print_page);
+static dev_proc_print_page(psm_print_page);
+static dev_proc_print_page(psm_print_page);
+static dev_proc_print_page(pam_print_page);
+
+static int pam_print_row(gx_device_printer * pdev, byte * data, int depth,
+	       FILE * pstream);
+static int pam_print_page(gx_device_printer * pdev, FILE * pstream);
 
 /* The device procedures */
 
@@ -140,21 +142,24 @@ private dev_proc_print_page(psm_print_page);
 	p_map_cmyk_color, NULL, NULL, NULL, gx_page_device_get_page_device\
 }
 
-private const gx_device_procs pbm_procs =
+static const gx_device_procs pbm_procs =
     pgpm_procs(gdev_prn_open, gdev_prn_get_params,
 	       gdev_prn_map_rgb_color, gdev_prn_map_color_rgb, NULL);
-private const gx_device_procs pgm_procs =
+static const gx_device_procs pgm_procs =
     pgpm_procs(ppm_open, gdev_prn_get_params,
 	       pgm_map_rgb_color, pgm_map_color_rgb, NULL);
-private const gx_device_procs ppm_procs =
+static const gx_device_procs ppm_procs =
     pgpm_procs(ppm_open, ppm_get_params,
 	       gx_default_rgb_map_rgb_color, ppm_map_color_rgb, NULL);
-private const gx_device_procs pnm_procs =
+static const gx_device_procs pnm_procs =
     pgpm_procs(ppm_open, ppm_get_params,
 	       ppm_map_rgb_color, ppm_map_color_rgb, NULL);
-private const gx_device_procs pkm_procs =
+static const gx_device_procs pkm_procs =
     pgpm_procs(ppm_open, ppm_get_params,
 	       NULL, cmyk_1bit_map_color_rgb, cmyk_1bit_map_cmyk_color);
+static const gx_device_procs pam_procs =
+    pgpm_procs(ppm_open, ppm_get_params,
+	       NULL, cmyk_8bit_map_color_rgb, cmyk_8bit_map_cmyk_color);
 
 /* The device descriptors themselves */
 const gx_device_pbm gs_pbm_device =
@@ -199,6 +204,13 @@ pbm_prn_device(pkm_procs, "pksm", '1', 0, 4, 4, 1, 1, 0,
 const gx_device_pbm gs_pksmraw_device =
 pbm_prn_device(pkm_procs, "pksmraw", '4', 1, 4, 4, 1, 1, 0,
 	       X_DPI, Y_DPI, psm_print_page);
+const gx_device_pbm gs_pamcmyk32_device =
+pbm_prn_device(pam_procs, "pamcmyk32", '7', 1, 4, 32, 255, 255, 0,
+	       X_DPI, Y_DPI, pam_print_page);
+/* Also keep the old device name so anyone using it won't be surprised */
+const gx_device_pbm gs_pam_device =
+pbm_prn_device(pam_procs, "pam", '7', 1, 4, 32, 255, 255, 0,
+	       X_DPI, Y_DPI, pam_print_page);
 
 /* Plan 9 bitmaps default to 100 dpi. */
 const gx_device_pbm gs_plan9bm_device =
@@ -208,7 +220,7 @@ pbm_prn_device(pbm_procs, "plan9bm", '9', 1, 1, 1, 1, 1, 1,
 /* ------ Initialization ------ */
 
 /* Set the copy_alpha and color mapping procedures if necessary. */
-private void
+static void
 ppm_set_dev_procs(gx_device * pdev)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -223,7 +235,10 @@ ppm_set_dev_procs(gx_device * pdev)
 	set_dev_proc(pdev, begin_typed_image, pnm_begin_typed_image);
     }
     if (bdev->color_info.num_components == 4) {
-	if (bdev->color_info.depth == 4) {
+	if (bdev->magic == 7) {
+	    set_dev_proc(pdev, map_color_rgb, cmyk_8bit_map_color_rgb);
+	    set_dev_proc(pdev, map_cmyk_color, cmyk_8bit_map_cmyk_color);
+	} else if (bdev->color_info.depth == 4) {
 	    set_dev_proc(pdev, map_color_rgb, cmyk_1bit_map_color_rgb);
 	    set_dev_proc(pdev, map_cmyk_color, cmyk_1bit_map_cmyk_color);
 	} else {
@@ -238,7 +253,7 @@ ppm_set_dev_procs(gx_device * pdev)
  * a planar device.
  */
 
-private int
+static int
 ppm_open(gx_device * pdev)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -246,13 +261,15 @@ ppm_open(gx_device * pdev)
 
     if (code < 0)
 	return code;
+    pdev->color_info.separable_and_linear = GX_CINFO_SEP_LIN;
+    set_linear_color_bits_mask_shift(pdev);
     bdev->uses_color = 0;
     ppm_set_dev_procs(pdev);
     return code;
 }
 
 /* Print a page, and reset uses_color if this is a showpage. */
-private int
+static int
 ppm_output_page(gx_device * pdev, int num_copies, int flush)
 {
     int code = gdev_prn_output_page(pdev, num_copies, flush);
@@ -269,16 +286,23 @@ ppm_output_page(gx_device * pdev, int num_copies, int flush)
 
 /* Map an RGB color to a PGM gray value. */
 /* Keep track of whether the image is black-and-white or gray. */
-private gx_color_index
-pgm_map_rgb_color(gx_device * pdev, gx_color_value r, gx_color_value g,
-		  gx_color_value b)
+static gx_color_index
+pgm_map_rgb_color(gx_device * pdev, const gx_color_value cv[])
 {				/* We round the value rather than truncating it. */
-    gx_color_value gray =
-    ((r * (ulong) lum_red_weight) +
+    gx_color_value gray;
+    /* TO_DO_DEVICEN  - Kludge to emulate pre DeviceN math errors */
+#if 1
+    gx_color_value r, g, b;
+
+    r = cv[0]; g = cv[0]; b = cv[0];
+    gray = ((r * (ulong) lum_red_weight) +
      (g * (ulong) lum_green_weight) +
      (b * (ulong) lum_blue_weight) +
      (lum_all_weights / 2)) / lum_all_weights
     * pdev->color_info.max_gray / gx_max_color_value;
+#else	    /* Should be ... */
+    gray = cv[0] * pdev->color_info.max_gray / gx_max_color_value;
+#endif
 
     if (!(gray == 0 || gray == pdev->color_info.max_gray)) {
 	gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -289,7 +313,7 @@ pgm_map_rgb_color(gx_device * pdev, gx_color_value r, gx_color_value g,
 }
 
 /* Map a PGM gray value back to an RGB color. */
-private int
+static int
 pgm_map_color_rgb(gx_device * dev, gx_color_index color,
 		  gx_color_value prgb[3])
 {
@@ -302,18 +326,42 @@ pgm_map_color_rgb(gx_device * dev, gx_color_index color,
     return 0;
 }
 
+/*
+ * Pre gs8.00 version of RGB mapping for 24-bit true (RGB) color devices
+ * It is kept here for backwards comparibility since the gs8.00 version
+ * has changed in functionality.  The new one requires that the device be
+ * 'separable'.  This routine is logically separable but does not require
+ * the various color_info fields associated with separability (comp_shift,
+ * comp_bits, and comp_mask) be setup.
+ */
+
+static gx_color_index
+gx_old_default_rgb_map_rgb_color(gx_device * dev,
+		       gx_color_value r, gx_color_value g, gx_color_value b)
+{
+    if (dev->color_info.depth == 24)
+	return gx_color_value_to_byte(b) +
+	    ((uint) gx_color_value_to_byte(g) << 8) +
+	    ((ulong) gx_color_value_to_byte(r) << 16);
+    else {
+	int bpc = dev->color_info.depth / 3;
+	int drop = sizeof(gx_color_value) * 8 - bpc;
+
+	return ((((r >> drop) << bpc) + (g >> drop)) << bpc) + (b >> drop);
+    }
+}
+
 /* Map an RGB color to a PPM color tuple. */
 /* Keep track of whether the image is black-and-white, gray, or colored. */
-private gx_color_index
-ppm_map_rgb_color(gx_device * pdev, gx_color_value r, gx_color_value g,
-		  gx_color_value b)
+static gx_color_index
+ppm_map_rgb_color(gx_device * pdev, const gx_color_value cv[])
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
-    gx_color_index color = gx_default_rgb_map_rgb_color(pdev, r, g, b);
-    int bpc = pdev->color_info.depth / 3;
+    gx_color_index color = 
+	    gx_old_default_rgb_map_rgb_color(pdev, cv[0], cv[1], cv[2]);
+    uint bpc = pdev->color_info.depth / 3;
     gx_color_index mask =
 	((gx_color_index)1 << (pdev->color_info.depth - bpc)) - 1;
-
     if (!(((color >> bpc) ^ color) & mask)) { /* gray shade */
 	if (color != 0 && (~color & mask))
 	    bdev->uses_color |= 1;
@@ -323,7 +371,7 @@ ppm_map_rgb_color(gx_device * pdev, gx_color_value r, gx_color_value g,
 }
 
 /* Map a PPM color tuple back to an RGB color. */
-private int
+static int
 ppm_map_color_rgb(gx_device * dev, gx_color_index color,
 		  gx_color_value prgb[3])
 {
@@ -341,16 +389,15 @@ ppm_map_color_rgb(gx_device * dev, gx_color_index color,
 }
 
 /* Map a CMYK color to a pixel value. */
-private gx_color_index
-pkm_map_cmyk_color(gx_device * pdev, gx_color_value c, gx_color_value m,
-		   gx_color_value y, gx_color_value k)
+static gx_color_index
+pkm_map_cmyk_color(gx_device * pdev, const gx_color_value cv[])
 {
     uint bpc = pdev->color_info.depth >> 2;
     uint max_value = pdev->color_info.max_color;
-    uint cc = c * max_value / gx_max_color_value;
-    uint mc = m * max_value / gx_max_color_value;
-    uint yc = y * max_value / gx_max_color_value;
-    uint kc = k * max_value / gx_max_color_value;
+    uint cc = cv[0] * max_value / gx_max_color_value;
+    uint mc = cv[1] * max_value / gx_max_color_value;
+    uint yc = cv[2] * max_value / gx_max_color_value;
+    uint kc = cv[3] * max_value / gx_max_color_value;
     gx_color_index color =
 	(((((cc << bpc) + mc) << bpc) + yc) << bpc) + kc;
 
@@ -358,7 +405,7 @@ pkm_map_cmyk_color(gx_device * pdev, gx_color_value c, gx_color_value m,
 }
 
 /* Map a CMYK pixel value to RGB. */
-private int
+static int
 pkm_map_color_rgb(gx_device * dev, gx_color_index color, gx_color_value rgb[3])
 {
     int bpc = dev->color_info.depth >> 2;
@@ -383,15 +430,19 @@ pkm_map_color_rgb(gx_device * dev, gx_color_index color, gx_color_value rgb[3])
 
 /* Augment get/put_params to add UsePlanarBuffer */
 
-private int
+static int
 ppm_get_params(gx_device * pdev, gs_param_list * plist)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
+    int code;
 
-    return gdev_prn_get_params_planar(pdev, plist, &bdev->UsePlanarBuffer);
+    code = gdev_prn_get_params_planar(pdev, plist, &bdev->UsePlanarBuffer);
+    if (code < 0) return code;
+    code = param_write_null(plist, "OutputIntent");
+    return code;
 }
 
-private int
+static int
 ppm_put_params(gx_device * pdev, gs_param_list * plist)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -401,8 +452,37 @@ ppm_put_params(gx_device * pdev, gs_param_list * plist)
     int ecode = 0;
     int code;
     long v;
+    gs_param_string_array intent;
     const char *vname;
 
+    if ((code = param_read_string_array(plist, "OutputIntent", &intent)) == 0) {
+	/* This device does not use the OutputIntent parameter.
+	   We include this code just as a sample how to handle it.
+	   The PDF interpreter extracts OutputIntent from a PDF file and sends it to here,
+	   if a device includes it in the .getdeviceparams response.
+	   This device does include due to ppm_get_params implementation.
+	   ppm_put_params must handle it (and ingore it) against 'rangecheck'.
+	 */
+	static const bool debug_print_OutputIntent = false;
+
+	if (debug_print_OutputIntent) {
+	    int i, j;
+
+	    dlprintf1("%d strings:\n", intent.size);
+	    for (i = 0; i < intent.size; i++) {
+		const gs_param_string *s = &intent.data[i];
+		dlprintf2("  %d: size %d:", i, s->size);
+		if (i < 4) {
+		    for (j = 0; j < s->size; j++)
+			dlprintf1("%c", s->data[j]);
+		} else {
+		    for (j = 0; j < s->size; j++)
+			dlprintf1(" %02x", s->data[j]);
+		}
+		dlprintf("\n");
+	    }
+	}
+    }
     save_info = pdev->color_info;
     if ((code = param_read_long(plist, (vname = "GrayValues"), &v)) != 1 ||
 	(code = param_read_long(plist, (vname = "RedValues"), &v)) != 1 ||
@@ -451,7 +531,7 @@ ppm_put_params(gx_device * pdev, gs_param_list * plist)
 
 /* Copy an alpha map, noting whether we may generate some non-black/white */
 /* colors through blending. */
-private int
+static int
 pnm_copy_alpha(gx_device * pdev, const byte * data, int data_x,
 	   int raster, gx_bitmap_id id, int x, int y, int width, int height,
 	       gx_color_index color, int depth)
@@ -470,7 +550,7 @@ pnm_copy_alpha(gx_device * pdev, const byte * data, int data_x,
 
 /* Begin processing an image, noting whether we may generate some */
 /* non-black/white colors in the process. */
-private int 
+static int 
 pnm_begin_typed_image(gx_device *dev,
                       const gs_imager_state *pis, const gs_matrix *pmat,
                       const gs_image_common_t *pim, const gs_int_rect *prect,
@@ -511,9 +591,9 @@ pnm_begin_typed_image(gx_device *dev,
 /* ------ Internal routines ------ */
 
 /* Print a page using a given row printing routine. */
-private int
+static int
 pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
-	     int (*row_proc) (P4(gx_device_printer *, byte *, int, FILE *)))
+	     int (*row_proc) (gx_device_printer *, byte *, int, FILE *))
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
     uint raster = gdev_prn_raster(pdev);
@@ -527,6 +607,23 @@ pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
     if (magic == '9')
 	fprintf(pstream, "%11d %11d %11d %11d %11d ",
 		0, 0, 0, pdev->width, pdev->height);
+    else if (magic == '7') {
+	int ncomps = pdev->color_info.num_components;
+	fprintf(pstream, "P%c\n", magic);
+	fprintf(pstream, "WIDTH %d\n", pdev->width);
+	fprintf(pstream, "HEIGHT %d\n", pdev->height);
+	fprintf(pstream, "DEPTH %d\n", ncomps);
+	fprintf(pstream, "MAXVAL %d\n", pdev->color_info.max_gray);
+	fprintf(pstream, "TUPLTYPE %s\n",
+	    (ncomps == 4) ? "CMYK" :
+	    ((ncomps == 3) ? "RGB" : "GRAYSCALE"));
+	if (bdev->comment[0])
+	    fprintf(pstream, "# %s\n", bdev->comment);
+	else
+	    fprintf(pstream, "# Image generated by %s (device=%s)\n",
+		    gs_product, pdev->dname);
+	fprintf(pstream, "ENDHDR\n");
+    }
     else {
 	fprintf(pstream, "P%c\n", magic);
 	if (bdev->comment[0])
@@ -539,6 +636,7 @@ pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
     switch (magic) {
 	case '1':		/* pbm */
 	case '4':		/* pbmraw */
+	case '7':		/* pam */
 	case '9':		/* plan9bm */
 	    break;
 	default:
@@ -561,7 +659,7 @@ pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
 /* ------ Individual page printing routines ------ */
 
 /* Print a monobit page. */
-private int
+static int
 pbm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	      FILE * pstream)
 {
@@ -583,7 +681,7 @@ pbm_print_row(gx_device_printer * pdev, byte * data, int depth,
     }
     return 0;
 }
-private int
+static int
 pbm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -592,7 +690,7 @@ pbm_print_page(gx_device_printer * pdev, FILE * pstream)
 }
 
 /* Print a gray-mapped page. */
-private int
+static int
 pgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	      FILE * pstream)
 {				/* Note that bpp <= 8 for raw format, bpp <= 16 for plain. */
@@ -602,7 +700,7 @@ pgm_print_row(gx_device_printer * pdev, byte * data, int depth,
      * If we're writing planes for a CMYK device, we have 0 = white,
      * mask = black, which is the opposite of the pgm convention.
      */
-    uint invert = (pdev->color_info.num_components == 4 ? mask : 0);
+    uint invert = (pdev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE);
     byte *bp;
     uint x;
     int shift;
@@ -635,7 +733,7 @@ pgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	}
     return 0;
 }
-private int
+static int
 pxm_pbm_print_row(gx_device_printer * pdev, byte * data, int depth,
 		  FILE * pstream)
 {				/* Compress a PGM or PPM row to a PBM row. */
@@ -679,20 +777,20 @@ pxm_pbm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	*dest = out;
     return pbm_print_row(pdev, data, 1, pstream);
 }
-private int
+static int
 pgm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
 
     return (bdev->uses_color == 0 && bdev->optimize ?
-	    pbm_print_page_loop(pdev, bdev->magic - 1, pstream,
+	    pbm_print_page_loop(pdev, (char)((int)bdev->magic - 1), pstream,
 				pxm_pbm_print_row) :
 	    pbm_print_page_loop(pdev, bdev->magic, pstream,
 				pgm_print_row));
 }
 
 /* Print a color-mapped page. */
-private int
+static int
 ppgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	       FILE * pstream, bool color)
 {				/* If color=false, write only one value per pixel; */
@@ -758,19 +856,19 @@ ppgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	}
     return 0;
 }
-private int
+static int
 ppm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	      FILE * pstream)
 {
     return ppgm_print_row(pdev, data, depth, pstream, true);
 }
-private int
+static int
 ppm_pgm_print_row(gx_device_printer * pdev, byte * data, int depth,
 		  FILE * pstream)
 {
     return ppgm_print_row(pdev, data, depth, pstream, false);
 }
-private int
+static int
 ppm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -779,16 +877,34 @@ ppm_print_page(gx_device_printer * pdev, FILE * pstream)
 	    pbm_print_page_loop(pdev, bdev->magic, pstream,
 				ppm_print_row) :
 	    bdev->uses_color == 1 ?
-	    pbm_print_page_loop(pdev, bdev->magic - 1, pstream,
+	    pbm_print_page_loop(pdev, (char)((int)bdev->magic - 1), pstream,
 				ppm_pgm_print_row) :
-	    pbm_print_page_loop(pdev, bdev->magic - 2, pstream,
+	    pbm_print_page_loop(pdev, (char)((int)bdev->magic - 2), pstream,
 				pxm_pbm_print_row));
 }
 
+static int
+pam_print_row(gx_device_printer * pdev, byte * data, int depth,
+	       FILE * pstream)
+{
+    if (depth == 32)
+	fwrite(data, 1, pdev->width * (depth / 8), pstream);
+    return 0;
+}
+
+static int
+pam_print_page(gx_device_printer * pdev, FILE * pstream)
+{
+    gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
+
+    return pbm_print_page_loop(pdev, bdev->magic, pstream,
+				pam_print_row);
+}
+
 /* Print a faux CMYK page. */
 /* Print a row where each pixel occupies 4 bits (depth == 4). */
 /* In this case, we also know pdev->color_info.max_color == 1. */
-private int
+static int
 pkm_print_row_4(gx_device_printer * pdev, byte * data, int depth,
 		FILE * pstream)
 {
@@ -847,7 +963,7 @@ pkm_print_row_4(gx_device_printer * pdev, byte * data, int depth,
     return 0;
 }
 /* Print a row where each pixel occupies 1 or more bytes (depth >= 8). */
-private int
+static int
 pkm_print_row(gx_device_printer * pdev, byte * data, int depth,
 	      FILE * pstream)
 {
@@ -895,7 +1011,7 @@ pkm_print_row(gx_device_printer * pdev, byte * data, int depth,
     }
     return 0;
 }
-private int
+static int
 pkm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
@@ -907,7 +1023,7 @@ pkm_print_page(gx_device_printer * pdev, FILE * pstream)
 }
 
 /* Print individual separations on a single file. */
-private int
+static int
 psm_print_page(gx_device_printer * pdev, FILE * pstream)
 {
     gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
