@@ -1,20 +1,17 @@
-    /* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfd.c,v 1.59 2005/04/04 20:28:27 igor Exp $ */
+/* $Id: gdevpdfd.c 8571 2008-03-01 10:15:02Z ken $ */
 /* Path drawing procedures for pdfwrite driver */
 #include "math_.h"
 #include "memory_.h"
@@ -30,12 +27,15 @@
 #include "gsstate.h"
 #include "gserrors.h"
 #include "gsptype2.h"
+#include "gsshade.h"
 #include "gzpath.h"
 #include "gzcpath.h"
 #include "gdevpdfx.h"
 #include "gdevpdfg.h"
 #include "gdevpdfo.h"
 #include "gsutil.h"
+#include "gdevpdtf.h"
+#include "gdevpdts.h"
 
 /* ---------------- Drawing ---------------- */
 
@@ -45,12 +45,14 @@ gdev_pdf_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 			gx_color_index color)
 {
     gx_device_pdf *pdev = (gx_device_pdf *) dev;
+    int bottom = (pdev->ResourcesBeforeUsage ? 1 : 0);
     int code;
 
     /* Make a special check for the initial fill with white, */
     /* which shouldn't cause the page to be opened. */
-    if (color == pdev->white && !is_in_page(pdev))
-	return 0;
+    if (color == pdev->white && !is_in_page(pdev) && pdev->sbstack_depth <= bottom)
+	if (x == 0 && y == 0 && w == pdev->width && h == pdev->height)
+	    return 0;
     code = pdf_open_page(pdev, PDF_IN_STREAM);
     if (code < 0)
 	return code;
@@ -61,6 +63,8 @@ gdev_pdf_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
     pdf_set_pure_color(pdev, color, &pdev->saved_fill_color,
 		       &pdev->fill_used_process_color,
 		       &psdf_set_fill_color_commands);
+    if (!pdev->HaveStrokeColor)
+	pdev->saved_stroke_color = pdev->saved_fill_color;
     pprintd4(pdev->strm, "%d %d %d %d re f\n", x, y, w, h);
     return 0;
 }
@@ -69,21 +73,21 @@ gdev_pdf_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 
 /* ------ Vector device implementation ------ */
 
-private int
+static int
 pdf_setlinewidth(gx_device_vector * vdev, floatp width)
 {
     /* Acrobat Reader doesn't accept negative line widths. */
     return psdf_setlinewidth(vdev, fabs(width));
 }
 
-private int
+static int
 pdf_can_handle_hl_color(gx_device_vector * vdev, const gs_imager_state * pis, 
 		 const gx_drawing_color * pdc)
 {
     return pis != NULL;
 }
 
-private int
+static int
 pdf_setfillcolor(gx_device_vector * vdev, const gs_imager_state * pis, 
 		 const gx_drawing_color * pdc)
 {
@@ -104,7 +108,7 @@ pdf_setfillcolor(gx_device_vector * vdev, const gs_imager_state * pis,
 				 &psdf_set_fill_color_commands);
 }
 
-private int
+static int
 pdf_setstrokecolor(gx_device_vector * vdev, const gs_imager_state * pis, 
                    const gx_drawing_color * pdc)
 {
@@ -125,14 +129,15 @@ pdf_setstrokecolor(gx_device_vector * vdev, const gs_imager_state * pis,
 				 &psdf_set_stroke_color_commands);
 }
 
-private int
+static int
 pdf_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1, fixed y1,
 	   gx_path_type_t type)
 {
     gx_device_pdf *pdev = (gx_device_pdf *)vdev;
     fixed xmax = int2fixed(vdev->width), ymax = int2fixed(vdev->height);
-    fixed xmin = (pdev->sbstack_depth ? -xmax : 0);
-    fixed ymin = (pdev->sbstack_depth ? -ymax : 0);
+    int bottom = (pdev->ResourcesBeforeUsage ? 1 : 0);
+    fixed xmin = (pdev->sbstack_depth > bottom ? -xmax : 0);
+    fixed ymin = (pdev->sbstack_depth > bottom ? -ymax : 0);
 
     /*
      * If we're doing a stroke operation, expand the checking box by the
@@ -168,7 +173,7 @@ pdf_dorect(gx_device_vector * vdev, fixed x0, fixed y0, fixed x1, fixed y1,
     return psdf_dorect(vdev, x0, y0, x1, y1, type);
 }
 
-private int
+static int
 pdf_endpath(gx_device_vector * vdev, gx_path_type_t type)
 {
     return 0;			/* always handled by caller */
@@ -221,7 +226,7 @@ pdf_remember_clip_path(gx_device_pdf * pdev, const gx_clip_path * pcpath)
 }
 
 /* Check if same clipping path. */
-private int
+static int
 pdf_is_same_clip_path(gx_device_pdf * pdev, const gx_clip_path * pcpath)
 {
     /* Used for skipping redundant clip paths. SF bug #624168. */
@@ -280,14 +285,14 @@ pdf_must_put_clip_path(gx_device_pdf * pdev, const gx_clip_path * pcpath)
 		return false;
 	if (pdf_is_same_clip_path(pdev, pcpath) > 0) {
 	    pdev->clip_path_id = pcpath->id;
-	    return 0;
+	    return false;
 	}
     }
     return true;
 }
 
 /* Put a single element of a clipping path list. */
-private int
+static int
 pdf_put_clip_path_list_elem(gx_device_pdf * pdev, gx_cpath_path_list *e, 
 	gs_path_enum *cenum, gdev_vector_dopath_state_t *state,
 	gs_fixed_point vs[3])
@@ -356,37 +361,57 @@ pdf_put_clip_path(gx_device_pdf * pdev, const gx_clip_path * pcpath)
 	    return code;
     }
     if (new_id != pdev->no_clip_path_id) {
-	gdev_vector_dopath_state_t state;
-	gs_fixed_point vs[3];
-	int pe_op;
+	const gs_fixed_rect *rect = cpath_is_rectangle(pcpath);
 
 	/* Use q to allow the new clipping path to unwind.  */
 	code = pdf_save_viewer_state(pdev, s);
 	if (code < 0)
 	    return code;
-	gdev_vector_dopath_init(&state, (gx_device_vector *)pdev,
-				gx_path_type_fill, NULL);
-	if (pcpath->path_list == NULL) {
-	    gs_cpath_enum cenum;
-
-	/*
-	 * We have to break 'const' here because the clip path
-	 * enumeration logic uses some internal mark bits.
-	 * This is very unfortunate, but until we can come up with
-	 * a better algorithm, it's necessary.
-	 */
-	gx_cpath_enum_init(&cenum, (gx_clip_path *) pcpath);
-	while ((pe_op = gx_cpath_enum_next(&cenum, vs)) > 0)
-	    gdev_vector_dopath_segment(&state, pe_op, vs);
-	pprints1(s, "%s n\n", (pcpath->rule <= 0 ? "W" : "W*"));
-	if (pe_op < 0)
-	    return pe_op;
+	if (rect != NULL) {
+	    /* Use unrounded coordinates. */
+	    pprintg4(s, "%g %g %g %g re", 
+		fixed2float(rect->p.x), fixed2float(rect->p.y),
+		fixed2float(rect->q.x - rect->p.x), 
+		fixed2float(rect->q.y - rect->p.y));
+	    pprints1(s, " %s n\n", (pcpath->rule <= 0 ? "W" : "W*"));
 	} else {
-	    gs_path_enum cenum;
+	    gdev_vector_dopath_state_t state;
+	    gs_fixed_point vs[3];
+	    int pe_op;
 
-	    code = pdf_put_clip_path_list_elem(pdev, pcpath->path_list, &cenum, &state, vs);
-	    if (code < 0)
-		return code;
+	    gdev_vector_dopath_init(&state, (gx_device_vector *)pdev,
+				    gx_path_type_fill, NULL);
+	    if (pcpath->path_list == NULL) {
+		/*
+		 * We think this should be never executed.
+		 * This obsolete branch writes a clip path intersection
+		 * as a set of rectangles computed by 
+		 * gx_cpath_intersect_path_slow.
+		 * Those rectangles use coordinates rounded to pixels,
+		 * therefore the precision may be unsatisfactory -
+		 * see Bug 688407.
+		 */
+		gs_cpath_enum cenum;
+
+		/*
+		 * We have to break 'const' here because the clip path
+		 * enumeration logic uses some internal mark bits.
+		 * This is very unfortunate, but until we can come up with
+		 * a better algorithm, it's necessary.
+		 */
+		gx_cpath_enum_init(&cenum, (gx_clip_path *) pcpath);
+		while ((pe_op = gx_cpath_enum_next(&cenum, vs)) > 0)
+		    gdev_vector_dopath_segment(&state, pe_op, vs);
+		pprints1(s, "%s n\n", (pcpath->rule <= 0 ? "W" : "W*"));
+		if (pe_op < 0)
+		    return pe_op;
+	    } else {
+		gs_path_enum cenum;
+
+		code = pdf_put_clip_path_list_elem(pdev, pcpath->path_list, &cenum, &state, vs);
+		if (code < 0)
+		    return code;
+	    }
 	}
     }
     pdev->clip_path_id = new_id;
@@ -400,7 +425,7 @@ pdf_put_clip_path(gx_device_pdf * pdev, const gx_clip_path * pcpath)
  * CTM will be multiplied by *pscale, and all coordinates will be divided by
  * *pscale.
  */
-private bool
+static bool
 make_rect_scaling(const gx_device_pdf *pdev, const gs_fixed_rect *bbox,
 		  floatp prescale, double *pscale)
 {
@@ -424,8 +449,9 @@ make_rect_scaling(const gx_device_pdf *pdev, const gs_fixed_rect *bbox,
 /*
  * Prepare a fill with a color anc a clipping path.
  * Return 1 if there is nothing to paint.
+ * Changes *box to the clipping box.
  */
-private int
+static int
 prepare_fill_with_clip(gx_device_pdf *pdev, const gs_imager_state * pis,
 	      gs_fixed_rect *box, bool have_path,
 	      const gx_drawing_color * pdcolor, const gx_clip_path * pcpath)
@@ -433,22 +459,30 @@ prepare_fill_with_clip(gx_device_pdf *pdev, const gs_imager_state * pis,
     bool new_clip;
     int code;
 
-    /*
-     * Check for an empty clipping path.
-     */
-    if (pcpath) {
-	gx_cpath_outer_box(pcpath, box);
-	if (box->p.x >= box->q.x || box->p.y >= box->q.y)
-	    return 1;		/* empty clipping path */
-    }
     if (gx_dc_is_pure(pdcolor)) {
-	/*
+        int bottom = (pdev->ResourcesBeforeUsage ? 1 : 0);
+ 	/*
 	 * Make a special check for the initial fill with white,
 	 * which shouldn't cause the page to be opened.
 	 */
 	if (gx_dc_pure_color(pdcolor) == pdev->white && 
-		!is_in_page(pdev) && pdev->sbstack_depth == 0)
-	    return 1;
+		!is_in_page(pdev) && pdev->sbstack_depth <= bottom) {
+	    if (box->p.x == 0 && box->p.y == 0 && 
+		    box->q.x == int2fixed(pdev->width) && 
+		    box->q.y == int2fixed(pdev->height)) /* See gs_fillpage */
+		return 1;
+        }
+    }
+    /*
+     * Check for an empty clipping path.
+     */
+    if (pcpath) {
+        gs_fixed_rect cbox;
+
+	gx_cpath_outer_box(pcpath, &cbox);
+	if (cbox.p.x >= cbox.q.x || cbox.p.y >= cbox.q.y)
+	    return 1;		/* empty clipping path */
+        *box = cbox;
     }
     new_clip = pdf_must_put_clip_path(pdev, pcpath);
     if (have_path || pdev->context == PDF_IN_NONE || new_clip) {
@@ -469,7 +503,7 @@ prepare_fill_with_clip(gx_device_pdf *pdev, const gs_imager_state * pis,
 
 public_st_pdf_lcvd_t();
 
-private int 
+static int 
 lcvd_fill_rectangle_shifted(gx_device *dev, int x, int y, int width, int height, gx_color_index color)
 {
     pdf_lcvd_t *cvd = (pdf_lcvd_t *)dev;
@@ -477,7 +511,7 @@ lcvd_fill_rectangle_shifted(gx_device *dev, int x, int y, int width, int height,
     return cvd->std_fill_rectangle((gx_device *)&cvd->mdev, 
 	x - cvd->mdev.mapped_x, y - cvd->mdev.mapped_y, width, height, color);
 }
-private int 
+static int 
 lcvd_fill_rectangle_shifted2(gx_device *dev, int x, int y, int width, int height, gx_color_index color)
 {
     pdf_lcvd_t *cvd = (pdf_lcvd_t *)dev;
@@ -490,22 +524,21 @@ lcvd_fill_rectangle_shifted2(gx_device *dev, int x, int y, int width, int height
     return cvd->std_fill_rectangle((gx_device *)&cvd->mdev, 
 	x - cvd->mdev.mapped_x, y - cvd->mdev.mapped_y, width, height, color);
 }
-private int 
-lcvd_fill_rectangle_shifted_from_mdev(gx_device *dev, int x, int y, int width, int height, gx_color_index color)
+static void
+lcvd_get_clipping_box_shifted_from_mdev(gx_device *dev, gs_fixed_rect *pbox)
 {
+    fixed ofs;
     pdf_lcvd_t *cvd = (pdf_lcvd_t *)dev;
 
-    return cvd->std_fill_rectangle((gx_device *)&cvd->mdev, 
-	x - cvd->mdev.mapped_x, y - cvd->mdev.mapped_y, width, height, color);
+    cvd->std_get_clipping_box((gx_device *)&cvd->mdev, pbox);
+    ofs = int2fixed(cvd->mdev.mapped_x);
+    pbox->p.x += ofs;
+    pbox->q.x += ofs;
+    ofs = int2fixed(cvd->mdev.mapped_y);
+    pbox->p.y += ofs;
+    pbox->q.y += ofs;
 }
-private void 
-lcvd_get_clipping_box_from_target(gx_device *dev, gs_fixed_rect *pbox)
-{
-    gx_device_memory *mdev = (gx_device_memory *)dev;
-
-    (*dev_proc(mdev->target, get_clipping_box))(mdev->target, pbox);
-}
-private int 
+static int 
 lcvd_pattern_manage(gx_device *pdev1, gx_bitmap_id id,
 		gs_pattern1_instance_t *pinst, pattern_manage_t function)
 {
@@ -513,7 +546,7 @@ lcvd_pattern_manage(gx_device *pdev1, gx_bitmap_id id,
 	return 1; /* Request shading area. */
     return 0;
 }
-private int 
+static int 
 lcvd_close_device_with_writing(gx_device *pdev)
 {
     /* Assuming 'mdev' is being closed before 'mask' - see gx_image3_end_image. */
@@ -525,7 +558,7 @@ lcvd_close_device_with_writing(gx_device *pdev)
     return code < 0 ? code : code1;
 }
 
-private int
+static int
 write_image(gx_device_pdf *pdev, gx_device_memory *mdev, gs_matrix *m)
 {
     gs_image_t image;
@@ -533,11 +566,8 @@ write_image(gx_device_pdf *pdev, gx_device_memory *mdev, gs_matrix *m)
     const int sourcex = 0;
     int code;
 
-    
     if (m != NULL)
-	pdf_put_matrix(pdev, "W n\n", m, " cm\n");
-    else
-	stream_puts(pdev->strm, "W n\n");
+	pdf_put_matrix(pdev, NULL, m, " cm\n");
     code = pdf_copy_color_data(pdev, mdev->base, sourcex,  
 		mdev->raster, gx_no_bitmap_id, 0, 0, mdev->width, mdev->height,
 		&image, &writer, 2);
@@ -547,7 +577,7 @@ write_image(gx_device_pdf *pdev, gx_device_memory *mdev, gs_matrix *m)
 	code = pdf_do_image(pdev, writer.pres, NULL, true);
     return code;
 }
-private int
+static int
 write_mask(gx_device_pdf *pdev, gx_device_memory *mdev, gs_matrix *m)
 {
     const int sourcex = 0;
@@ -567,7 +597,7 @@ write_mask(gx_device_pdf *pdev, gx_device_memory *mdev, gs_matrix *m)
     return code;
 }
 
-private void
+static void
 max_subimage_width(int width, byte *base, int x0, long count1, int *x1, long *count)
 {
     long c = 0, c1 = count1 - 1;
@@ -599,7 +629,7 @@ ex:
     *x1 = x;
 }
 
-private void
+static void
 compute_subimage(int width, int height, int raster, byte *base, 
 	    	 int x0, int y0, long MaxClipPathSize, int *x1, int *y1)
 {
@@ -640,7 +670,7 @@ compute_subimage(int width, int height, int raster, byte *base,
     }
 }
 
-private int
+static int
 image_line_to_clip(gx_device_pdf *pdev, byte *base, int x0, int x1, int y0, int y1, bool started)
 {   /* returns the number of segments or error code. */
     int x = x0, xx;
@@ -685,7 +715,7 @@ image_line_to_clip(gx_device_pdf *pdev, byte *base, int x0, int x1, int y0, int
     return c;
 }
 
-private int
+static int
 mask_to_clip(gx_device_pdf *pdev, int width, int height, 
 	     int raster, byte *base, int x0, int y0, int x1, int y1)
 {
@@ -709,7 +739,7 @@ mask_to_clip(gx_device_pdf *pdev, int width, int height,
     return code < 0 ? code : has_segments ? 1 : 0;
 }
 
-private int
+static int
 write_subimage(gx_device_pdf *pdev, gx_device_memory *mdev, int x, int y, int x1, int y1)
 {
     gs_image_t image;
@@ -732,14 +762,14 @@ write_subimage(gx_device_pdf *pdev, gx_device_memory *mdev, int x, int y, int x1
     return pdf_do_image(pdev, writer.pres, NULL, true);
 }
 
-private int 
+static int 
 write_image_with_clip(gx_device_pdf *pdev, pdf_lcvd_t *cvd)
 {
     int x = 0, y = 0;
     int code, code1;
 
     if (cvd->write_matrix)
-	pdf_put_matrix(pdev, NULL, &cvd->m, " cm\n");
+	pdf_put_matrix(pdev, NULL, &cvd->m, " cm q\n");
     for(;;) {
 	int x1, y1;
 	
@@ -769,6 +799,8 @@ write_image_with_clip(gx_device_pdf *pdev, pdf_lcvd_t *cvd)
 	    y = y1;
 	}
     }
+    if (cvd->write_matrix)
+	stream_puts(pdev->strm, "Q\n");
     return 0;
 }
 
@@ -777,7 +809,9 @@ pdf_dump_converted_image(gx_device_pdf *pdev, pdf_lcvd_t *cvd)
 {
     int code = 0;
 
-    if (!cvd->path_is_empty) {
+    if (!cvd->path_is_empty || cvd->has_background) {
+	if (!cvd->has_background)
+	    stream_puts(pdev->strm, "W n\n");
 	code = write_image(pdev, &cvd->mdev, (cvd->write_matrix ? &cvd->m : NULL));
 	cvd->path_is_empty = true;
     } else if (!cvd->mask_is_empty && pdev->PatternImagemask) {
@@ -807,8 +841,10 @@ pdf_dump_converted_image(gx_device_pdf *pdev, pdf_lcvd_t *cvd)
 	inst.template.YStep = (float)cvd->mdev.height;
 	code = (*dev_proc(pdev, pattern_manage))((gx_device *)pdev, 
 	    id, &inst, pattern_manage__start_accum);
-	if (code >= 0)
+	if (code >= 0) {
+	    stream_puts(pdev->strm, "W n\n");
 	    code = write_image(pdev, &cvd->mdev, NULL);
+	}
 	pres = pdev->accumulating_substream_resource;
 	if (code >= 0)
 	    code = (*dev_proc(pdev, pattern_manage))((gx_device *)pdev, 
@@ -836,7 +872,7 @@ pdf_dump_converted_image(gx_device_pdf *pdev, pdf_lcvd_t *cvd)
 		0, 0, cvd->mdev.width, cvd->mdev.height, (gx_color_index)0);
     return code;
 }
-private int
+static int
 lcvd_handle_fill_path_as_shading_coverage(gx_device *dev,
     const gs_imager_state *pis, gx_path *ppath,
     const gx_fill_params *params,
@@ -846,6 +882,8 @@ lcvd_handle_fill_path_as_shading_coverage(gx_device *dev,
     gx_device_pdf *pdev = (gx_device_pdf *)cvd->mdev.target;
     int code;
 
+    if (cvd->has_background)
+	return 0;
     if (gx_path_is_null(ppath)) {
 	/* use the mask. */
 	if (!cvd->path_is_empty) {
@@ -896,7 +934,7 @@ pdf_setup_masked_image_converter(gx_device_pdf *pdev, gs_memory_t *mem, const gs
     pdf_lcvd_t *cvd = *pcvd;
 
     if (cvd == NULL) {
-	cvd = gs_alloc_struct(mem, pdf_lcvd_t, &st_pdf_lcvd_t, "pdf_image3_make_mid");
+	cvd = gs_alloc_struct(mem, pdf_lcvd_t, &st_pdf_lcvd_t, "pdf_setup_masked_image_converter");
 	if (cvd == NULL)
 	    return_error(gs_error_VMerror);
 	*pcvd = cvd;
@@ -913,6 +951,7 @@ pdf_setup_masked_image_converter(gx_device_pdf *pdev, gs_memory_t *mem, const gs
     cvd->path_is_empty = true;
     cvd->mask_is_empty = true;
     cvd->mask_is_clean = false;
+    cvd->has_background = false;
     cvd->mask = 0;
     cvd->write_matrix = true;
     code = (*dev_proc(&cvd->mdev, open_device))((gx_device *)&cvd->mdev);
@@ -943,13 +982,16 @@ pdf_setup_masked_image_converter(gx_device_pdf *pdev, gs_memory_t *mem, const gs
     }
     cvd->std_fill_rectangle = dev_proc(&cvd->mdev, fill_rectangle);
     cvd->std_close_device = dev_proc(&cvd->mdev, close_device);
+    cvd->std_get_clipping_box = dev_proc(&cvd->mdev, get_clipping_box);
     if (!write_on_close) {
 	/* Type 3 images will write to the mask directly. */
 	dev_proc(&cvd->mdev, fill_rectangle) = (need_mask ? lcvd_fill_rectangle_shifted2 
 							  : lcvd_fill_rectangle_shifted);
-    } else
-	dev_proc(&cvd->mdev, fill_rectangle) = lcvd_fill_rectangle_shifted_from_mdev;
-    dev_proc(&cvd->mdev, get_clipping_box) = lcvd_get_clipping_box_from_target;
+	dev_proc(&cvd->mdev, get_clipping_box) = lcvd_get_clipping_box_shifted_from_mdev;
+    } else {
+	dev_proc(&cvd->mdev, fill_rectangle) = lcvd_fill_rectangle_shifted;
+	dev_proc(&cvd->mdev, get_clipping_box) = lcvd_get_clipping_box_shifted_from_mdev;
+    }
     dev_proc(&cvd->mdev, pattern_manage) = lcvd_pattern_manage;
     dev_proc(&cvd->mdev, fill_path) = lcvd_handle_fill_path_as_shading_coverage;
     cvd->m = *m;
@@ -971,28 +1013,6 @@ pdf_remove_masked_image_converter(gx_device_pdf *pdev, pdf_lcvd_t *cvd, bool nee
     }
 }
 
-private int
-path_scale(gx_path *path, double scalex, double scaley)
-{
-    segment *pseg = (segment *)path->first_subpath;
-
-    for (;pseg != NULL; pseg = pseg->next) {
-	pseg->pt.x = (fixed)floor(pseg->pt.x * scalex + 0.5);
-	pseg->pt.y = (fixed)floor(pseg->pt.y * scaley + 0.5);
-	if (pseg->type == s_curve) {
-	    curve_segment *s = (curve_segment *)pseg;
-
-	    s->p1.x = (fixed)floor(s->p1.x * scalex + 0.5);
-	    s->p1.y = (fixed)floor(s->p1.y * scaley + 0.5);
-	    s->p2.x = (fixed)floor(s->p2.x * scalex + 0.5);
-	    s->p2.y = (fixed)floor(s->p2.y * scaley + 0.5);
-	}
-    }
-    path->position.x = (fixed)floor(path->position.x * scalex + 0.5);
-    path->position.y = (fixed)floor(path->position.y * scaley + 0.5);
-    return 0;
-}
-
 /* ------ Driver procedures ------ */
 
 /* Fill a path. */
@@ -1012,7 +1032,7 @@ gdev_pdf_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath
      * drawing anything.
      */
     bool have_path;
-    gs_fixed_rect box = {{0, 0}, {0, 0}};
+    gs_fixed_rect box = {{0, 0}, {0, 0}}, box1;
 
     have_path = !gx_path_is_void(ppath);
     if (!have_path && !pdev->vg_initial_set) {
@@ -1021,6 +1041,12 @@ gdev_pdf_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath
 	pdf_reset_graphics(pdev);
 	return 0;
     }
+    if (have_path) {
+	code = gx_path_bbox(ppath, &box);
+	if (code < 0)
+	    return code;
+    }
+    box1 = box;
 
     code = prepare_fill_with_clip(pdev, pis, &box, have_path, pdcolor, pcpath);
     if (code == gs_error_rangecheck) {
@@ -1049,10 +1075,10 @@ gdev_pdf_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath
 	    gs_fixed_rect bbox, bbox1;
 	    bool need_mask = gx_dc_pattern2_can_overlap(pdcolor);
 	    gs_matrix m, save_ctm = ctm_only(pis), ms, msi, mm;
-	    gs_point p;
 	    gs_int_point rect_size;
 	    /* double scalex = 1.9, scaley = 1.4; debug purpose only. */
-	    double scale, scalex = 1.0, scaley = 1.0;
+	    double scale, scalex, scaley;
+	    int log2_scale_x = 0, log2_scale_y = 0;
 	    gx_drawing_color dc = *pdcolor;
 	    gs_pattern2_instance_t pi = *(gs_pattern2_instance_t *)dc.ccolor.pattern;
 	    gs_state *pgs = gs_state_copy(pi.saved, gs_state_memory(pi.saved));
@@ -1074,17 +1100,15 @@ gdev_pdf_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath
 		return 0;
 	    sx = fixed2int(bbox.p.x);
 	    sy = fixed2int(bbox.p.y);
-	    gs_distance_transform_inverse(sx * pdev->HWResolution[0] / 72, 
-					  sy * pdev->HWResolution[0] / 72, &ctm_only(pis), &p);
 	    gs_make_identity(&m);
 	    rect_size.x = fixed2int(bbox.q.x + fixed_half) - sx;
 	    rect_size.y = fixed2int(bbox.q.y + fixed_half) - sy;
-	    if (rect_size.x == 0 || rect_size.x == 0)
+	    if (rect_size.x == 0 || rect_size.y == 0)
 		return 0;
-	    m.tx = p.x;
-	    m.ty = p.y;
-	    cvd.path_offset.x = m.tx;
-	    cvd.path_offset.y = m.ty;
+	    m.tx = (float)sx;
+	    m.ty = (float)sy;
+	    cvd.path_offset.x = sx;
+	    cvd.path_offset.y = sy;
 	    scale = (double)rect_size.x * rect_size.y * pdev->color_info.num_components /
 		    pdev->MaxShadingBitmapSize;
 	    if (scale > 1) {
@@ -1094,61 +1118,44 @@ gdev_pdf_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath
 		   we prefer to deal only with integers being powers of 2
 		   in order to avoid possible distorsions when scaling paths.
 		*/
-		scalex = ceil(sqrt(scale));
-		scalex = scaley = 1 << ilog2((int)scalex);
-		if (scalex * scaley < scale)
-		    scalex *= 2;
-		if (scalex * scaley < scale)
-		    scaley *= 2;
+		log2_scale_x = log2_scale_y = ilog2((int)ceil(sqrt(scale)));
+		if ((double)(1 << log2_scale_x) * (1 << log2_scale_y) < scale)
+		    log2_scale_y++;
+		if ((double)(1 << log2_scale_x) * (1 << log2_scale_y) < scale)
+		    log2_scale_x++;
+		scalex = (double)(1 << log2_scale_x);
+		scaley = (double)(1 << log2_scale_y);
 		rect_size.x = (int)floor(rect_size.x / scalex + 0.5);
 		rect_size.y = (int)floor(rect_size.y / scaley + 0.5);
 		gs_make_scaling(1.0 / scalex, 1.0 / scaley, &ms);
 		gs_make_scaling(scalex, scaley, &msi);
-		gs_matrix_multiply(&m, &msi, &m);
+		gs_matrix_multiply(&msi, &m, &m);
 		gs_matrix_multiply(&ctm_only(pis), &ms, &mm);
 		gs_setmatrix((gs_state *)pis, &mm);
 		gs_matrix_multiply(&ctm_only((gs_imager_state *)pgs), &ms, &mm);
 		gs_setmatrix((gs_state *)pgs, &mm);
-		m.tx /= scalex;
-		m.ty /= scaley;
-		cvd.path_offset.x = m.tx / scalex;
-		cvd.path_offset.y = m.ty / scaley;
-		sx = (int)floor(sx / scalex + 0.5);
-		sy = (int)floor(sy / scaley + 0.5);
+		sx = fixed2int(bbox.p.x / (int)scalex);
+		sy = fixed2int(bbox.p.y / (int)scaley);
+		cvd.path_offset.x = sx; /* m.tx / scalex */
+		cvd.path_offset.y = sy;
 	    }
 	    code = pdf_setup_masked_image_converter(pdev, pdev->memory, &m, &pcvd, need_mask, sx, sy, 
 			    rect_size.x, rect_size.y, false);
+	    pcvd->has_background = gx_dc_pattern2_has_background(pdcolor);
 	    stream_puts(pdev->strm, "q\n");
 	    if (code >= 0) {
-		pdf_put_matrix(pdev, NULL, &cvd.m, " cm q\n");
-		cvd.write_matrix = false;
-	    }
-	    if (code >= 0) {
-		/* See gx_default_fill_path. */
-		gx_clip_path cpath_intersection;
-		gx_path path_intersection, path1, *p = &path_intersection;
-
-		gx_path_init_local(&path_intersection, pdev->memory);
-		gx_path_init_local(&path1, pdev->memory);
-		gx_cpath_init_local_shared(&cpath_intersection, pcpath, pdev->memory);
-		if ((code = gx_cpath_intersect(&cpath_intersection, ppath, params->rule, (gs_imager_state *)pis)) >= 0)
-		    code = gx_cpath_to_path(&cpath_intersection, &path_intersection);
-		if (code >= 0 && scale > 1) {
-		    code = gx_path_copy(&path_intersection, &path1);	
-		    if (code > 0) {
-			p = &path1;
-			code = path_scale(&path1, scalex, scaley);
-		    }
-		}
+		code = gdev_vector_dopath((gx_device_vector *)pdev, ppath,
+					gx_path_type_clip, NULL);
 		if (code >= 0)
-		    code = gx_dc_pattern2_fill_path(&dc, p, NULL, (gx_device *)&cvd.mdev);
-		gx_path_free(&path_intersection, "gdev_pdf_fill_path");
-		gx_path_free(&path1, "gdev_pdf_fill_path");
-		gx_cpath_free(&cpath_intersection, "gdev_pdf_fill_path");
+		    stream_puts(pdev->strm, "W n\n");
 	    }
-	    if (code >= 0) {
+	    pdf_put_matrix(pdev, NULL, &cvd.m, " cm q\n");
+	    cvd.write_matrix = false;
+	    if (code >= 0)
+		code = gs_shading_do_fill_rectangle(pi.template.Shading,
+		     NULL, (gx_device *)&cvd.mdev, (gs_imager_state *)pgs, !pi.shfill);
+	    if (code >= 0)
 		code = pdf_dump_converted_image(pdev, &cvd);
-	    }
 	    stream_puts(pdev->strm, "Q Q\n");
 	    pdf_remove_masked_image_converter(pdev, &cvd, need_mask);
 	    gs_setmatrix((gs_state *)pis, &save_ctm);
@@ -1163,11 +1170,7 @@ gdev_pdf_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath
 	double scale;
 	gs_matrix smat;
 	gs_matrix *psmat = NULL;
-	gs_fixed_rect box1;
 
-	code = gx_path_bbox(ppath, &box1);
-	if (code < 0)
-	    return code;
 	if (pcpath) {
  	    rect_intersect(box1, box);
  	    if (box1.p.x > box1.q.x || box1.p.y > box1.q.y)
@@ -1212,8 +1215,53 @@ gdev_pdf_stroke_path(gx_device * dev, const gs_imager_state * pis,
 	return 0;		/* won't mark the page */
     if (pdf_must_put_clip_path(pdev, pcpath))
 	code = pdf_unclip(pdev);
-    else 
-	code = pdf_open_page(pdev, PDF_IN_STREAM);
+    else if ((pdev->last_charpath_op & TEXT_DO_FALSE_CHARPATH) && ppath->current_subpath && 
+	(ppath->last_charpath_segment == ppath->current_subpath->last)) {
+	bool hl_color = pdf_can_handle_hl_color((gx_device_vector *)pdev, pis, pdcolor);
+	const gs_imager_state *pis_for_hl_color = (hl_color ? pis : NULL);
+	
+	if (pdf_modify_text_render_mode(pdev->text->text_state, 1)) {
+	    /* Set the colour for the stroke */
+	    code = pdf_reset_color(pdev, pis_for_hl_color, pdcolor, &pdev->saved_stroke_color, 
+			&pdev->stroke_used_process_color, &psdf_set_stroke_color_commands);
+	    if (code == 0) {
+		s = pdev->strm;
+		/* Text is emitted scaled so that the CTM is an identity matrix, the line width 
+		 * needs to be scaled to match otherwise we will get the default, or the current
+		 * width scaled by the CTM before the text, either of which would be wrong.
+		 */
+		if (pdev->font3 != 0) {
+		    double det, scale;
+
+		    /* Since font matrix may have different scaling effect by
+		       different directions, we need to average them into
+		       a single scaling factor. A right way is to use
+		       the geometric average of the 2 eigenvalues of the 2x2 matrix,
+		       which appears equal to sqrt(abs(det(M)) where M
+		       is the font matrix with no translation.  */
+		    det = ((double)fixed2float(pdev->charproc_ctm.xx) * fixed2float(pdev->charproc_ctm.yy)) - 
+			((double)fixed2float(pdev->charproc_ctm.xy) * fixed2float(pdev->charproc_ctm.yx));
+		    scale = fabs(sqrt(det));
+		    scale *= 72 / pdev->HWResolution[0];
+		    pprintg1(s, "%g w\n", (pis->line_params.half_width * 2) * (float)scale);
+		} else {
+		    pprintg1(s, "%g w\n", (pis->line_params.half_width * 2));
+		}
+		/* Some trickery here. We have altered the colour, text render mode and linewidth,
+		 * we don't want those to persist. By switching to a stream context we will flush the 
+		 * pending text. This has the beneficial side effect of executing a grestore. So
+		 * everything works out neatly.
+		 */
+		code = pdf_open_page(pdev, PDF_IN_STREAM);
+		return(code);
+	    }
+	}
+	/* Can only get here if any of the above steps fail, in which case we proceed to
+	 * emit the charpath as a normal path, and stroke it.
+	 */
+        code = pdf_open_page(pdev, PDF_IN_STREAM);
+    } else
+        code = pdf_open_page(pdev, PDF_IN_STREAM);
     if (code < 0)
 	return code;
     code = pdf_prepare_stroke(pdev, pis);
@@ -1239,6 +1287,22 @@ gdev_pdf_stroke_path(gx_device * dev, const gs_imager_state * pis,
      */
     set_ctm = (bool)gdev_vector_stroke_scaling((gx_device_vector *)pdev,
 					       pis, &scale, &mat);
+    if (set_ctm && ((pis->ctm.xx == 0 && pis->ctm.xy == 0) ||
+	            (pis->ctm.yx == 0 && pis->ctm.yy == 0))) {
+ 	/* Acrobat Reader 5 and Adobe Reader 6 issues 
+ 	   the "Wrong operand type" error with matrices, which have 3 zero coefs.
+	   Besides that, we found that Acrobat Reader 4, Acrobat Reader 5 
+	   and Adobe Reader 6 all store the current path in user space 
+	   and apply CTM in the time of stroking - See the bug 687901. 
+	   Therefore a precise conversion of Postscript to PDF isn't possible in this case.
+	   Adobe viewers render a line with a constant width instead. 
+	   At last, with set_ctm == true we need the inverse matrix in 
+	   gdev_vector_dopath. Therefore we exclude projection matrices
+	   (see bug 688363). */
+	set_ctm = false;
+	scale = fabs(pis->ctm.xx + pis->ctm.xy + pis->ctm.yx + pis->ctm.yy) /* Using the non-zero coeff. */
+	        / sqrt(2); /* Empirically from Adobe. */
+    }
     if (set_ctm) {
 	/*
 	 * We want a scaling factor that will bring the largest reasonable
@@ -1254,6 +1318,37 @@ gdev_pdf_stroke_path(gx_device * dev, const gs_imager_state * pis,
 	prescale = (minor == 0 || minor > 1 ? 1 : 1 / minor);
     }
     gx_path_bbox(ppath, &bbox);
+    {
+    	/* Check whether a painting appears inside the clipping box. 
+	   Doing so after writing the clipping path due to /SP pdfmark
+	   uses a special hack with painting outside the clipping box
+	   for synchronizing the clipping path (see lib/gs_pdfwr.ps).
+	   That hack appeared because there is no way to pass 
+	   the imager state through gdev_pdf_put_params,
+	   which pdfmark is implemented with.
+	*/
+	gs_fixed_rect clip_box, stroke_bbox = bbox;
+	gs_point d0, d1;
+	gs_fixed_point p0, p1;
+	fixed bbox_expansion_x, bbox_expansion_y;
+
+	gs_distance_transform(pis->line_params.half_width, 0, &ctm_only(pis), &d0);
+	gs_distance_transform(0, pis->line_params.half_width, &ctm_only(pis), &d1);
+	p0.x = float2fixed(any_abs(d0.x));
+	p0.y = float2fixed(any_abs(d0.y));
+	p1.x = float2fixed(any_abs(d1.x));
+	p1.y = float2fixed(any_abs(d1.y));
+	bbox_expansion_x = max(p0.x, p1.x) + fixed_1 * 2;
+	bbox_expansion_y = max(p0.y, p1.y) + fixed_1 * 2;
+	stroke_bbox.p.x -= bbox_expansion_x;
+	stroke_bbox.p.y -= bbox_expansion_y;
+	stroke_bbox.q.x += bbox_expansion_x;
+	stroke_bbox.q.y += bbox_expansion_y;
+	gx_cpath_outer_box(pcpath, &clip_box);
+	rect_intersect(stroke_bbox, clip_box);
+	if (stroke_bbox.q.x < stroke_bbox.p.x || stroke_bbox.q.y < stroke_bbox.p.y)
+	    return 0;
+    }
     if (make_rect_scaling(pdev, &bbox, prescale, &path_scale)) {
 	scale /= path_scale;
 	if (set_ctm)
@@ -1268,7 +1363,8 @@ gdev_pdf_stroke_path(gx_device * dev, const gs_imager_state * pis,
     if (code < 0)
 	return gx_default_stroke_path(dev, pis, ppath, params, pdcolor,
 				      pcpath);
-
+    if (!pdev->HaveStrokeColor)
+	pdev->saved_fill_color = pdev->saved_stroke_color;
     if (set_ctm)
   	pdf_put_matrix(pdev, "q ", &mat, "cm\n");
     code = gdev_vector_dopath((gx_device_vector *)pdev, ppath,
@@ -1292,36 +1388,54 @@ gdev_pdf_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
     const gx_clip_path *pcpath)
 {
     int code;
-    gs_fixed_rect box1 = *rect, box = {{0, 0}, {0, 0}};
+    gs_fixed_rect box1 = *rect, box = box1;
     gx_device_pdf *pdev = (gx_device_pdf *) dev;
     double scale;
     gs_matrix smat;
     gs_matrix *psmat = NULL;
+    const bool convert_to_image = (pdev->CompatibilityLevel <= 1.2 && 
+	    gx_dc_is_pattern2_color(pdcolor));
 
     if (rect->p.x == rect->q.x)
 	return 0;
-    code = prepare_fill_with_clip(pdev, pis, &box, true, pdcolor, pcpath);
-    if (code < 0)
+    if (!convert_to_image) {
+	code = prepare_fill_with_clip(pdev, pis, &box, true, pdcolor, pcpath);
+	if (code < 0)
+	    return code;
+	if (code == 1)
+	    return 0; /* Nothing to paint. */
+	code = pdf_setfillcolor((gx_device_vector *)pdev, pis, pdcolor);
+	if (code < 0)
+	    return code;
+	if (pcpath) 
+	    rect_intersect(box1, box);
+	if (box1.p.x > box1.q.x || box1.p.y > box1.q.y)
+  	    return 0;		/* outside the clipping path */
+	if (make_rect_scaling(pdev, &box1, 1.0, &scale)) {
+	    gs_make_scaling(pdev->scale.x * scale, pdev->scale.y * scale, &smat);
+	    pdf_put_matrix(pdev, "q ", &smat, "cm\n");
+	    psmat = &smat;
+	}
+	pprintg4(pdev->strm, "%g %g %g %g re f\n",
+		fixed2float(box1.p.x) * scale, fixed2float(box1.p.y) * scale,
+		fixed2float(box1.q.x - box1.p.x) * scale, fixed2float(box1.q.y - box1.p.y) * scale);
+	if (psmat)
+	    stream_puts(pdev->strm, "Q\n");
+	return 0;
+    } else {
+	gx_fill_params params;
+	gx_path path;
+
+	params.rule = 1; /* Not important because the path is a rectange. */
+	params.adjust.x = params.adjust.y = 0;
+        params.flatness = pis->flatness;
+	params.fill_zero_width = false;
+	gx_path_init_local(&path, pis->memory);
+	gx_path_add_rectangle(&path, rect->p.x, rect->p.y, rect->q.x, rect->q.y);
+	code = gdev_pdf_fill_path(dev, pis, &path, &params, pdcolor, pcpath);
+	gx_path_free(&path, "gdev_pdf_fill_rectangle_hl_color");
 	return code;
-    if (code == 1)
-	return 0; /* Nothing to paint. */
-    code = pdf_setfillcolor((gx_device_vector *)pdev, pis, pdcolor);
-    if (code < 0)
-	return code;
-    if (pcpath) 
-	rect_intersect(box1, box);
-    if (box1.p.x > box1.q.x || box1.p.y > box1.q.y)
-  	return 0;		/* outside the clipping path */
-    if (make_rect_scaling(pdev, &box1, 1.0, &scale)) {
-	gs_make_scaling(pdev->scale.x * scale, pdev->scale.y * scale, &smat);
-        pdf_put_matrix(pdev, "q ", &smat, "cm\n");
-	psmat = &smat;
+
     }
-    pprintg4(pdev->strm, "%g %g %g %g re f\n",
-	     fixed2float(box1.p.x) * scale, fixed2float(box1.p.y) * scale,
-	     fixed2float(box1.q.x - box1.p.x) * scale, fixed2float(box1.q.y - box1.p.y) * scale);
-    if (psmat)
-	stream_puts(pdev->strm, "Q\n");
-    return 0;
 }
 
