@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfi.c 6723 2006-04-27 17:36:41Z leonardo $ */
+/* $Id: gdevpdfi.c 8803 2008-06-24 14:16:29Z leonardo $ */
 /* Image handling for PDF-writing driver */
 #include "memory_.h"
 #include "math_.h"
@@ -35,29 +35,29 @@
 
 
 /* Forward references */
-private image_enum_proc_plane_data(pdf_image_plane_data);
-private image_enum_proc_end_image(pdf_image_end_image);
-private image_enum_proc_end_image(pdf_image_end_image_object);
-private image_enum_proc_end_image(pdf_image_end_image_object2);
-private image_enum_proc_end_image(pdf_image_end_image_cvd);
-private IMAGE3_MAKE_MID_PROC(pdf_image3_make_mid);
-private IMAGE3_MAKE_MCDE_PROC(pdf_image3_make_mcde);
-private IMAGE3X_MAKE_MID_PROC(pdf_image3x_make_mid);
-private IMAGE3X_MAKE_MCDE_PROC(pdf_image3x_make_mcde);
+static image_enum_proc_plane_data(pdf_image_plane_data);
+static image_enum_proc_end_image(pdf_image_end_image);
+static image_enum_proc_end_image(pdf_image_end_image_object);
+static image_enum_proc_end_image(pdf_image_end_image_object2);
+static image_enum_proc_end_image(pdf_image_end_image_cvd);
+static IMAGE3_MAKE_MID_PROC(pdf_image3_make_mid);
+static IMAGE3_MAKE_MCDE_PROC(pdf_image3_make_mcde);
+static IMAGE3X_MAKE_MID_PROC(pdf_image3x_make_mid);
+static IMAGE3X_MAKE_MCDE_PROC(pdf_image3x_make_mcde);
 
-private const gx_image_enum_procs_t pdf_image_enum_procs = {
+static const gx_image_enum_procs_t pdf_image_enum_procs = {
     pdf_image_plane_data,
     pdf_image_end_image
 };
-private const gx_image_enum_procs_t pdf_image_object_enum_procs = {
+static const gx_image_enum_procs_t pdf_image_object_enum_procs = {
     pdf_image_plane_data,
     pdf_image_end_image_object
 };
-private const gx_image_enum_procs_t pdf_image_object_enum_procs2 = {
+static const gx_image_enum_procs_t pdf_image_object_enum_procs2 = {
     pdf_image_plane_data,
     pdf_image_end_image_object2
 };
-private const gx_image_enum_procs_t pdf_image_cvd_enum_procs = {
+static const gx_image_enum_procs_t pdf_image_cvd_enum_procs = {
     gx_image1_plane_data,
     pdf_image_end_image_cvd,
     gx_image1_flush
@@ -68,7 +68,6 @@ private const gx_image_enum_procs_t pdf_image_cvd_enum_procs = {
 /* Define the structure for keeping track of progress through an image. */
 typedef struct pdf_image_enum_s {
     gx_image_enum_common;
-    gs_memory_t *memory;
     int width;
     int bits_per_pixel;		/* bits per pixel (per plane) */
     int rows_left;
@@ -78,7 +77,7 @@ typedef struct pdf_image_enum_s {
 gs_private_st_composite(st_pdf_image_enum, pdf_image_enum, "pdf_image_enum",
   pdf_image_enum_enum_ptrs, pdf_image_enum_reloc_ptrs);
 /* GC procedures */
-private ENUM_PTRS_WITH(pdf_image_enum_enum_ptrs, pdf_image_enum *pie)
+static ENUM_PTRS_WITH(pdf_image_enum_enum_ptrs, pdf_image_enum *pie)
     if (index < pdf_image_writer_max_ptrs) {
 	gs_ptr_type_t ret =
 	    ENUM_USING(st_pdf_image_writer, &pie->writer, sizeof(pie->writer),
@@ -91,7 +90,7 @@ private ENUM_PTRS_WITH(pdf_image_enum_enum_ptrs, pdf_image_enum *pie)
     return ENUM_USING_PREFIX(st_gx_image_enum_common,
 			     pdf_image_writer_max_ptrs);
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(pdf_image_enum_reloc_ptrs, pdf_image_enum *pie)
+static RELOC_PTRS_WITH(pdf_image_enum_reloc_ptrs, pdf_image_enum *pie)
 {
     RELOC_USING(st_pdf_image_writer, &pie->writer, sizeof(pie->writer));
     RELOC_USING(st_gx_image_enum_common, vptr, size);
@@ -102,7 +101,7 @@ RELOC_PTRS_END
  * Test whether we can write an image in-line.  This is always true,
  * because we only support PDF 1.2 and later.
  */
-private bool
+static bool
 can_write_image_in_line(const gx_device_pdf *pdev, const gs_image_t *pim)
 {
     return true;
@@ -116,14 +115,14 @@ can_write_image_in_line(const gx_device_pdf *pdev, const gs_image_t *pim)
  * with a color-key mask, at least for 1-bit-deep images using an Indexed
  * color space.
  */
-private int
+static int
 color_is_black_or_white(gx_device *dev, const gx_drawing_color *pdcolor)
 {
     return (!color_is_pure(pdcolor) ? -1 :
 	    gx_dc_pure_color(pdcolor) == gx_device_black(dev) ? 0 :
 	    gx_dc_pure_color(pdcolor) == gx_device_white(dev) ? 1 : -1);
 }
-private int
+static int
 pdf_convert_image4_to_image1(gx_device_pdf *pdev,
 			     const gs_imager_state *pis,
 			     const gx_drawing_color *pbcolor,
@@ -213,7 +212,7 @@ pdf_convert_image4_to_image1(gx_device_pdf *pdev,
     return -1;			/* arbitrary <0 */
 }
 
-private int
+static int
 pdf_begin_image_data_decoded(gx_device_pdf *pdev, int num_components, const gs_range_t *pranges, int i, 
 			     gs_pixel_image_t *pi, cos_value_t *cs_value, pdf_image_enum *pie)
 {
@@ -235,7 +234,7 @@ pdf_begin_image_data_decoded(gx_device_pdf *pdev, int num_components, const gs_r
     return pdf_begin_image_data(pdev, &pie->writer, pi, cs_value, i);
 }
 
-private int
+static int
 make_device_color_space(gx_device_pdf *pdev, 
 			gs_color_space_index output_cspace_index, 
 			gs_color_space **ppcs)
@@ -243,19 +242,15 @@ make_device_color_space(gx_device_pdf *pdev,
     gs_color_space *cs;
     gs_memory_t *mem = pdev->v_memory;
 
-    cs = gs_alloc_struct(mem, gs_color_space, &st_color_space, 
-			    "psdf_setup_image_colors_filter");
-    if (cs == NULL)
-	return_error(gs_error_VMerror);
     switch (output_cspace_index) {
 	case gs_color_space_index_DeviceGray:
-	    gs_cspace_init_DeviceGray(mem, cs);
+	    cs = gs_cspace_new_DeviceGray(mem);
 	    break;
 	case gs_color_space_index_DeviceRGB:
-	    gs_cspace_init_DeviceRGB(mem, cs); 
+	    cs = gs_cspace_new_DeviceRGB(mem); 
 	    break;
 	case gs_color_space_index_DeviceCMYK:
-	    gs_cspace_init_DeviceCMYK(mem, cs); 
+	    cs = gs_cspace_new_DeviceCMYK(mem); 
 	    break;
 	default:
 	    /* Notify the user and terminate.
@@ -265,17 +260,19 @@ make_device_color_space(gx_device_pdf *pdev,
 	    eprintf("Unsupported ProcessColorModel");
 	    return_error(gs_error_undefined);
     }
+    if (cs == NULL)
+	return_error(gs_error_VMerror);
     *ppcs = cs;
     return 0;
 }
 
-private bool
+static bool
 check_image_color_space(gs_pixel_image_t * pim, gs_color_space_index index)
 {
     if (pim->ColorSpace->type->index == index)
 	return true;
     if (pim->ColorSpace->type->index == gs_color_space_index_Indexed)
-	if (pim->ColorSpace->params.indexed.base_space.type->index == index)
+	if (pim->ColorSpace->base_space->type->index == index)
 	    return true;
     return false;
 }
@@ -290,14 +287,36 @@ typedef enum {
     PDF_IMAGE_TYPE3_MASK,	/* no in-line, don't render */
     PDF_IMAGE_TYPE3_DATA	/* no in-line */
 } pdf_typed_image_context_t;
-private int
-pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
+
+/*
+ * We define this union because psdf_setup_image_filters may alter the
+ * gs_pixel_image_t part, but pdf_begin_image_data must also have access
+ * to the type-specific parameters.
+ */
+typedef union image_union_s {
+    gs_pixel_image_t pixel;	/* we may change some components */
+    gs_image1_t type1;
+    gs_image3_t type3;
+    gs_image3x_t type3x;
+    gs_image4_t type4;
+} image_union_t;
+
+static int pdf_begin_typed_image(gx_device_pdf *pdev,
+    const gs_imager_state * pis, const gs_matrix *pmat,
+    const gs_image_common_t *pic, const gs_int_rect * prect,
+    const gx_drawing_color * pdcolor, const gx_clip_path * pcpath,
+    gs_memory_t * mem, gx_image_enum_common_t ** pinfo,
+    pdf_typed_image_context_t context);
+
+static int
+pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
 		      const gs_matrix *pmat, const gs_image_common_t *pic,
 		      const gs_int_rect * prect,
 		      const gx_drawing_color * pdcolor,
 		      const gx_clip_path * pcpath, gs_memory_t * mem,
 		      gx_image_enum_common_t ** pinfo,
-		      pdf_typed_image_context_t context)
+		      pdf_typed_image_context_t context,
+                      image_union_t *image)
 {
     cos_dict_t *pnamed = 0;
     const gs_pixel_image_t *pim;
@@ -305,29 +324,16 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
     pdf_image_enum *pie;
     gs_image_format_t format;
     const gs_color_space *pcs;
-    gs_color_space cs_gray_temp;
     cos_value_t cs_value;
     int num_components;
     bool is_mask = false, in_line = false;
     gs_int_rect rect;
-    /*
-     * We define this union because psdf_setup_image_filters may alter the
-     * gs_pixel_image_t part, but pdf_begin_image_data must also have access
-     * to the type-specific parameters.
-     */
-    union iu_ {
-	gs_pixel_image_t pixel;	/* we may change some components */
-	gs_image1_t type1;
-	gs_image3_t type3;
-	gs_image3x_t type3x;
-	gs_image4_t type4;
-    } image[4];
     int width, height;
     const gs_range_t *pranges = 0;
     const pdf_color_space_names_t *names;
     bool convert_to_process_colors = false;
     gs_color_space *pcs_device = NULL;
-    const gs_color_space *pcs_orig = NULL;
+    gs_color_space *pcs_orig = NULL;
     pdf_lcvd_t *cvd = NULL;
 
     /*
@@ -383,6 +389,15 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
 		       prect->q.y == pim3->Height))
 	    goto nyi;
 	if (pdev->CompatibilityLevel < 1.3 && !pdev->PatternImagemask) {
+	    if (pdf_must_put_clip_path(pdev, pcpath))
+		code = pdf_unclip(pdev);
+	    else 
+		code = pdf_open_page(pdev, PDF_IN_STREAM);
+	    if (code < 0)
+		return code;
+	    code = pdf_put_clip_path(pdev, pcpath);
+	    if (code < 0)
+		return code;
 	    gs_make_identity(&m);
 	    pmat1 = &m;
 	    m.tx = floor(pis->ctm.tx + 0.5); /* Round the origin against the image size distorsions */
@@ -453,6 +468,7 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
 	    code = gs_gsave(pgs);
 	    if (code < 0)
 		return code;
+	    /* {csrc}: const cast warning */
 	    code = gs_setcolorspace(pgs, ((const gs_image4_t *)pic)->ColorSpace);
 	    if (code < 0)
 		return code;
@@ -471,6 +487,15 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
 	    gs_matrix m, m1, mi;
 	    gs_image4_t pi4 = *(const gs_image4_t *)pic;
 
+	    if (pdf_must_put_clip_path(pdev, pcpath))
+		code = pdf_unclip(pdev);
+	    else 
+		code = pdf_open_page(pdev, PDF_IN_STREAM);
+	    if (code < 0)
+		return code;
+	    code = pdf_put_clip_path(pdev, pcpath);
+	    if (code < 0)
+		return code;
 	    gs_make_identity(&m1);
 	    gs_matrix_invert(&pic->ImageMatrix, &mi);
 	    gs_matrix_multiply(&mi, &ctm_only(pis), &m);
@@ -531,17 +556,14 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
 	 * color space, which pdf_color_space() can't handle.  Patch it
 	 * to DeviceGray here.
 	 */
-	gs_cspace_init_DeviceGray(pdev->memory, &cs_gray_temp);
-	pcs = &cs_gray_temp;
+	/* {csrc} make sure this gets freed */
+	pcs = gs_cspace_new_DeviceGray(pdev->memory);
     } else if (is_mask)
 	code = pdf_prepare_imagemask(pdev, pis, pdcolor);
     else
 	code = pdf_prepare_image(pdev, pis);
     if (code < 0)
 	goto nyi;
-    code = pdf_put_clip_path(pdev, pcpath);
-    if (code < 0)
-	return code;
     if (prect)
 	rect = *prect;
     else {
@@ -610,10 +632,17 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
 	    goto nyi;
 	}
     }
+    code = pdf_put_clip_path(pdev, pcpath);
+    if (code < 0)
+	return code;
     pdf_image_writer_init(&pie->writer);
     pie->writer.alt_writer_count = (in_line || 
 				    (pim->Width <= 64 && pim->Height <= 64) ||
 				    pdev->transfer_not_identity ? 1 : 2);
+    if (image[0].pixel.ColorSpace != NULL &&
+	image[0].pixel.ColorSpace->type->index == gs_color_space_index_Indexed 
+	&& pdev->params.ColorImage.DownsampleType != ds_Subsample)
+	pie->writer.alt_writer_count = 1;
     image[1] = image[0];
     names = (in_line ? &pdf_color_space_names_short : &pdf_color_space_names);
     if (!is_mask) {
@@ -771,7 +800,7 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
 	   1 means no alternative streams.
 	   2 means the main image stream and a mask stream while converting 
 		   an Image Type 4.
-	   3 means the main image steram, alternative image compression stream, 
+	   3 means the main image stream, alternative image compression stream, 
 		   and the compression chooser.
 	   4 meams 3 and a mask stream while convertingh an Image Type 4.
          */
@@ -785,6 +814,28 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
 	((gx_device *)pdev, pis, pmat, pic, prect, pdcolor, pcpath, mem,
 	 pinfo);
 }
+
+static int
+pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
+		      const gs_matrix *pmat, const gs_image_common_t *pic,
+		      const gs_int_rect * prect,
+		      const gx_drawing_color * pdcolor,
+		      const gx_clip_path * pcpath, gs_memory_t * mem,
+		      gx_image_enum_common_t ** pinfo,
+		      pdf_typed_image_context_t context)
+{
+  int code;
+  image_union_t *image = (image_union_t *)gs_malloc(mem->non_gc_memory, 4,
+                       sizeof(image_union_t), "pdf_begin_typed_image(image)");
+  if (image == 0)
+      return_error(gs_error_VMerror);
+  code = pdf_begin_typed_image_impl(pdev, pis, pmat, pic, prect,
+         pdcolor, pcpath, mem, pinfo, context, image);
+  gs_free(mem->non_gc_memory, image, 4, sizeof(image_union_t),
+                                              "pdf_begin_typed_image(image)");
+  return code;
+}
+
 int
 gdev_pdf_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
 			   const gs_matrix *pmat, const gs_image_common_t *pic,
@@ -801,7 +852,7 @@ gdev_pdf_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
 /* ---------------- All images ---------------- */
 
 /* Process the next piece of an image. */
-private int
+static int
 pdf_image_plane_data_alt(gx_image_enum_common_t * info,
 		     const gx_image_plane_t * planes, int height,
 		     int *rows_used, int alt_writer_index)
@@ -876,7 +927,7 @@ pdf_image_plane_data_alt(gx_image_enum_common_t * info,
 #undef ROW_BYTES
 }
 
-private int
+static int
 pdf_image_plane_data(gx_image_enum_common_t * info,
 		     const gx_image_plane_t * planes, int height,
 		     int *rows_used)
@@ -894,7 +945,7 @@ pdf_image_plane_data(gx_image_enum_common_t * info,
     return !pie->rows_left;
 }
 
-private int
+static int
 use_image_as_pattern(gx_device_pdf *pdev, pdf_resource_t *pres1, 
 		     const gs_matrix *pmat, gs_id id)
 {   /* See also dump_image in gdevpdfd.c . */
@@ -965,7 +1016,7 @@ typedef enum {
 } pdf_image_usage_t;
 
 /* Close PDF image and do it. */
-private int
+static int
 pdf_end_and_do_image(gx_device_pdf *pdev, pdf_image_writer *piw, 
 		     const gs_matrix *mat, gs_id ps_bitmap_id, pdf_image_usage_t do_image)
 {
@@ -1009,7 +1060,7 @@ pdf_end_and_do_image(gx_device_pdf *pdev, pdf_image_writer *piw,
 }
 
 /* Clean up by releasing the buffers. */
-private int
+static int
 pdf_image_end_image_data(gx_image_enum_common_t * info, bool draw_last,
 			 pdf_image_usage_t do_image)
 {
@@ -1054,19 +1105,19 @@ pdf_image_end_image_data(gx_image_enum_common_t * info, bool draw_last,
 	    code = pdf_end_and_do_image(pdev, &pie->writer, &pie->mat, info->id, do_image);
 	pie->writer.alt_writer_count--; /* For GC. */
     }
-    gs_free_object(pie->memory, pie, "pdf_end_image");
+    gx_image_free_enum(&info);
     return code;
 }
 
 /* End a normal image, drawing it. */
-private int
+static int
 pdf_image_end_image(gx_image_enum_common_t * info, bool draw_last)
 {
     return pdf_image_end_image_data(info, draw_last, USE_AS_IMAGE);
 }
 
 /* End an image converted with pdf_lcvd_t. */
-private int
+static int
 pdf_image_end_image_cvd(gx_image_enum_common_t * info, bool draw_last)
 {   pdf_lcvd_t *cvd = (pdf_lcvd_t *)info->dev;
     int code = pdf_dump_converted_image(cvd->pdev, cvd);
@@ -1084,7 +1135,7 @@ pdf_image_end_image_cvd(gx_image_enum_common_t * info, bool draw_last)
  * For both types of masked images, we create temporary dummy (null) devices
  * that forward the begin_typed_image call to the implementation above.
  */
-private int
+static int
 pdf_make_mxd(gx_device **pmxdev, gx_device *tdev, gs_memory_t *mem)
 {
     gx_device *fdev;
@@ -1098,13 +1149,13 @@ pdf_make_mxd(gx_device **pmxdev, gx_device *tdev, gs_memory_t *mem)
 }
 
 /* End the mask of an ImageType 3 image, not drawing it. */
-private int
+static int
 pdf_image_end_image_object(gx_image_enum_common_t * info, bool draw_last)
 {
     return pdf_image_end_image_data(info, draw_last, USE_AS_MASK);
 }
 /* End the data of an ImageType 3 image, converting it into pattern. */
-private int
+static int
 pdf_image_end_image_object2(gx_image_enum_common_t * info, bool draw_last)
 {
     return pdf_image_end_image_data(info, draw_last, USE_AS_PATTERN);
@@ -1113,8 +1164,8 @@ pdf_image_end_image_object2(gx_image_enum_common_t * info, bool draw_last)
 /* ---------------- Type 3 images ---------------- */
 
 /* Implement the mask image device. */
-private dev_proc_begin_typed_image(pdf_mid_begin_typed_image);
-private int
+static dev_proc_begin_typed_image(pdf_mid_begin_typed_image);
+static int
 pdf_image3_make_mid(gx_device **pmidev, gx_device *dev, int width, int height,
 		    gs_memory_t *mem)
 {
@@ -1144,7 +1195,7 @@ pdf_image3_make_mid(gx_device **pmidev, gx_device *dev, int width, int height,
 	return 0;
     }
 }
-private int
+static int
 pdf_mid_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
 			  const gs_matrix *pmat, const gs_image_common_t *pic,
 			  const gs_int_rect * prect,
@@ -1161,7 +1212,7 @@ pdf_mid_begin_typed_image(gx_device * dev, const gs_imager_state * pis,
 }
 
 /* Implement the mask clip device. */
-private int
+static int
 pdf_image3_make_mcde(gx_device *dev, const gs_imager_state *pis,
 		     const gs_matrix *pmat, const gs_image_common_t *pic,
 		     const gs_int_rect *prect, const gx_drawing_color *pdcolor,
@@ -1184,7 +1235,7 @@ pdf_image3_make_mcde(gx_device *dev, const gs_imager_state *pis,
 	cvd->mdev.mapped_y = origin->y;
 	*pmcdev = (gx_device *)&cvd->mdev;
 	code = gx_default_begin_typed_image
-	    ((gx_device *)&cvd->mdev, pis, pmat, pic, prect, pdcolor, pcpath, mem,
+	    ((gx_device *)&cvd->mdev, pis, pmat, pic, prect, pdcolor, NULL, mem,
 	    pinfo);
     } else {
 	code = pdf_make_mxd(pmcdev, midev, mem);
@@ -1203,7 +1254,7 @@ pdf_image3_make_mcde(gx_device *dev, const gs_imager_state *pis,
 /* ---------------- Type 3x images ---------------- */
 
 /* Implement the mask image device. */
-private int
+static int
 pdf_image3x_make_mid(gx_device **pmidev, gx_device *dev, int width, int height,
 		     int depth, gs_memory_t *mem)
 {
@@ -1216,7 +1267,7 @@ pdf_image3x_make_mid(gx_device **pmidev, gx_device *dev, int width, int height,
 }
 
 /* Implement the mask clip device. */
-private int
+static int
 pdf_image3x_make_mcde(gx_device *dev, const gs_imager_state *pis,
 		      const gs_matrix *pmat, const gs_image_common_t *pic,
 		      const gs_int_rect *prect,
@@ -1287,7 +1338,7 @@ pdf_resource_t *pdf_substitute_pattern(pdf_resource_t *pres)
 }
 
 
-private int 
+static int 
 check_unsubstituted2(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1)
 {
     pdf_pattern_t *ppat = (pdf_pattern_t *)pres0;
@@ -1295,7 +1346,7 @@ check_unsubstituted2(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t
     return ppat->substitute == NULL;
 }
 
-private int 
+static int 
 check_unsubstituted1(gx_device_pdf * pdev, pdf_resource_t *pres0)
 {
     pdf_pattern_t *ppat = (pdf_pattern_t *)pres0;
@@ -1327,6 +1378,9 @@ gdev_pdf_pattern_manage(gx_device *pdev1, gx_bitmap_id id,
 	    code = pdf_store_pattern1_params(pdev, pres, pinst);
 	    if (code < 0)
 		return code;
+	    /* Scale the coordinate system, because object handlers assume so. See none_to_stream. */
+	    pprintg2(pdev->strm, "%g 0 0 %g 0 0 cm\n",
+		     72.0 / pdev->HWResolution[0], 72.0 / pdev->HWResolution[1]);
 	    return 1;
 	case pattern_manage__finish_accum:
 	    code = pdf_add_procsets(pdev->substream_Resources, pdev->procsets);
@@ -1369,6 +1423,18 @@ gdev_pdf_pattern_manage(gx_device *pdev1, gx_bitmap_id id,
 	    return 1;
 	case pattern_manage__shading_area:
 	    return 0;
+	case pattern_manage__is_cpath_accum:
+	    return 0;
+	case pattern_manage__shfill_doesnt_need_path:
+	    return 0; /* gdev_pdf_fill_path still does need a path. */
+	case pattern_manage__handles_clip_path:
+	    /* This is important when the default implementation of 
+	       of fill_path is called due to a failure in setcolor
+	       or so, for example when a shading is incorfect. 
+	       The test case is the unfixed (buggy) Genoa test 446-01.ps . 
+	       In this case pdfwrite converts the object into rectangles,
+	       and the clipping device has to be set up. */
+	    return 0;
     }
     return_error(gs_error_unregistered);
 }
