@@ -1,20 +1,17 @@
-/* Copyright (C) 1996, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfm.c,v 1.47 2004/10/07 05:18:34 ray Exp $ */
+/* $Id: gdevpdfm.c 8359 2007-11-12 08:59:35Z ken $ */
 /* pdfmark processing for PDF-writing driver */
 #include "math_.h"
 #include "memory_.h"
@@ -87,7 +84,7 @@ pdfmark_scan_int(const gs_param_string * pstr, int *pvalue)
 /* ---------------- Private utilities ---------------- */
 
 /* Find a key in a dictionary. */
-private bool
+static bool
 pdfmark_find_key(const char *key, const gs_param_string * pairs, uint count,
 		 gs_param_string * pstr)
 {
@@ -107,7 +104,7 @@ pdfmark_find_key(const char *key, const gs_param_string * pairs, uint count,
  * Get the page number for a page referenced by number or as /Next or /Prev.
  * The result may be 0 if the page number is 0 or invalid.
  */
-private int
+static int
 pdfmark_page_number(gx_device_pdf * pdev, const gs_param_string * pnstr)
 {
     int page = pdev->next_page + 1;
@@ -127,7 +124,7 @@ pdfmark_page_number(gx_device_pdf * pdev, const gs_param_string * pnstr)
 /* Construct a destination string specified by /Page and/or /View. */
 /* Return 0 if none (but still fill in a default), 1 or 2 if present */
 /* (1 if only one of /Page or /View, 2 if both), <0 if error. */
-private int
+static int
 pdfmark_make_dest(char dstr[MAX_DEST_STRING], gx_device_pdf * pdev,
 		  const char *Page_key, const char *View_key,
 		  const gs_param_string * pairs, uint count)
@@ -166,7 +163,7 @@ pdfmark_make_dest(char dstr[MAX_DEST_STRING], gx_device_pdf * pdev,
  * If a named destination is specified by a string, convert it to a name,
  * update *dstr, and return 1; otherwise return 0.
  */
-private int
+static int
 pdfmark_coerce_dest(gs_param_string *dstr, char dest[MAX_DEST_STRING])
 {
     const byte *data = dstr->data;
@@ -184,13 +181,13 @@ pdfmark_coerce_dest(gs_param_string *dstr, char dest[MAX_DEST_STRING])
 }
 
 /* Put pairs in a dictionary. */
-private int
+static int
 pdfmark_put_c_pair(cos_dict_t *pcd, const char *key,
 		   const gs_param_string * pvalue)
 {
     return cos_dict_put_c_key_string(pcd, key, pvalue->data, pvalue->size);
 }
-private int
+static int
 pdfmark_put_pair(cos_dict_t *pcd, const gs_param_string * pair)
 {
     return cos_dict_put_string(pcd, pair->data, pair->size,
@@ -198,7 +195,7 @@ pdfmark_put_pair(cos_dict_t *pcd, const gs_param_string * pair)
 }
 
 /* Scan a Rect value. */
-private int
+static int
 pdfmark_scan_rect(gs_rect * prect, const gs_param_string * str,
 		  const gs_matrix * pctm)
 {
@@ -222,7 +219,7 @@ pdfmark_scan_rect(gs_rect * prect, const gs_param_string * str,
 }
 
 /* Make a Rect value. */
-private void
+static void
 pdfmark_make_rect(char str[MAX_RECT_STRING], const gs_rect * prect)
 {
     /*
@@ -240,7 +237,7 @@ pdfmark_make_rect(char str[MAX_RECT_STRING], const gs_rect * prect)
 }
 
 /* Write a transformed Border value on a stream. */
-private int
+static int
 pdfmark_write_border(stream *s, const gs_param_string *str,
 		     const gs_matrix *pctm)
 {
@@ -291,14 +288,14 @@ pdfmark_write_border(stream *s, const gs_param_string *str,
 }
 
 /* Put an element in a stream's dictionary. */
-private int
+static int
 cos_stream_put_c_strings(cos_stream_t *pcs, const char *key, const char *value)
 {
     return cos_dict_put_c_strings(cos_stream_dict(pcs), key, value);
 }
 
 /* Setup pdfmak stream compression. */
-private int
+static int
 setup_pdfmark_stream_compression(gx_device_psdf *pdev0,
 			cos_stream_t *pco)
 {
@@ -341,6 +338,61 @@ setup_pdfmark_stream_compression(gx_device_psdf *pdev0,
     return pdf_put_filters(cos_stream_dict(pco), pdev, pco->input_strm, &fnames);
 }
 
+static int
+pdfmark_bind_named_object(gx_device_pdf *pdev, const gs_const_string *objname, 
+			  pdf_resource_t **pres)
+{
+    int code; 
+
+    if (objname != NULL && objname->size) {
+	const cos_value_t *v = cos_dict_find(pdev->local_named_objects, objname->data, objname->size);
+
+	if (v != NULL) {
+	    if (v->value_type == COS_VALUE_OBJECT) {
+		if (cos_type(v->contents.object) == &cos_generic_procs) {
+		    /* The object was referred but not defined. 
+		       Use the old object id.
+		       The old object stub to be dropped. */
+		    pdf_reserve_object_id(pdev, *pres, v->contents.object->id);
+		} else if (!v->contents.object->written) {
+		    /* We can't know whether the old object was referred or not.
+		       Write it out for a consistent result in any case. */
+		    code = cos_write_object(v->contents.object, pdev);
+
+		    if (code < 0)
+			return code;
+		    v->contents.object->written = true;
+		}
+	    } else 
+		return_error(gs_error_rangecheck); /* Must not happen. */
+	}
+    }
+    if ((*pres)->object->id == -1) {
+	if(objname != NULL && objname->size)
+	    code = pdf_substitute_resource(pdev, pres, resourceXObject, NULL, false);
+	else
+	    code = pdf_substitute_resource(pdev, pres, resourceXObject, NULL, true);
+	if (code < 0)
+	    return code;
+    } else {
+	/* Unfortunately we can't apply pdf_substitute_resource,
+	   because the object may already be referred by its id.
+	   Redundant objects may happen in this case.
+	   For better results users should define objects before usage.
+	 */
+    }
+    if (objname != NULL && objname->size) {
+	cos_value_t value;
+
+	code = cos_dict_put(pdev->local_named_objects, objname->data,
+			    objname->size, cos_object_value(&value, (cos_object_t *)(*pres)->object));
+	if (code < 0)
+	    return code;
+    }
+    return 0;
+}
+
+
 /* ---------------- Miscellaneous pdfmarks ---------------- */
 
 /*
@@ -380,7 +432,7 @@ typedef struct ao_params_s {
     const char *subtype;	/* default Subtype in top-level dictionary */
     long src_pg;		/* set to SrcPg - 1 if any */
 } ao_params_t;
-private int
+static int
 pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 		     const gs_param_string * pairs, uint count,
 		     const gs_matrix * pctm, ao_params_t * params,
@@ -388,6 +440,7 @@ pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 {
     const gs_param_string *Action = 0;
     const gs_param_string *File = 0;
+    const gs_param_string *URI = 0;
     gs_param_string Dest;
     gs_param_string Subtype;
     uint i;
@@ -420,6 +473,9 @@ pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 	    Dest = pair[1];
 	    coerce_dest = true;
 	}
+	else if (pdf_key_eq(pair, "/URI")) {
+	    URI = pair;		/* save it for placing into the Action dict */
+	}
 	else if (pdf_key_eq(pair, "/Page") || pdf_key_eq(pair, "/View")) {
 	    /* Make a destination even if this is for an outline. */
 	    if (Dest.data == 0) {
@@ -516,7 +572,7 @@ pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 	const byte *astr = Action[1].data;
 	const uint asize = Action[1].size;
 
-	if ((File != 0 || Dest.data != 0) &&
+	if ((File != 0 || Dest.data != 0 || URI != 0) &&
 	    (pdf_key_eq(Action + 1, "/Launch") ||
 	     (pdf_key_eq(Action + 1, "/GoToR") && File) ||
 	     pdf_key_eq(Action + 1, "/Article"))
@@ -546,6 +602,12 @@ pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 		pdfmark_put_c_pair(adict, "/F", File + 1);
 		File = 0;	/* so we don't write it again */
 	    }
+	    if (URI) {
+		/* Adobe Distiller puts a /URI key from pdfmark into the */
+		/* Action dict with /S /URI as Subtype */
+		pdfmark_put_pair(adict, URI);
+		cos_dict_put_c_strings(adict, "/S", "/URI");
+	    }
 	    cos_dict_put(pcd, (const byte *)"/A", 2,
 			 COS_OBJECT_VALUE(&avalue, adict));
 	} else if (asize >= 4 && !memcmp(astr, "<<", 2)) {
@@ -559,6 +621,12 @@ pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 
 	    if (adict == 0)
 		return_error(gs_error_VMerror);
+	    if (URI) {
+		/* Adobe Distiller puts a /URI key from pdfmark into the */
+		/* Action dict with /S /URI as Subtype */
+		pdfmark_put_pair(adict, URI);
+		cos_dict_put_c_strings(adict, "/S", "/URI");
+	    }
 	    while ((code = pdf_scan_token(&scan, end, &key.data)) > 0) {
 		key.size = scan - key.data;
 		if (key.data[0] != '/' ||
@@ -624,7 +692,7 @@ pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 }
 
 /* Copy an annotation dictionary. */
-private int
+static int
 pdfmark_annot(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	      const gs_matrix * pctm, const gs_param_string *objname,
 	      const char *subtype)
@@ -669,7 +737,7 @@ pdfmark_annot(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* ANN pdfmark */
-private int
+static int
 pdfmark_ANN(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	    const gs_matrix * pctm, const gs_param_string * objname)
 {
@@ -677,7 +745,7 @@ pdfmark_ANN(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* LNK pdfmark (obsolescent) */
-private int
+static int
 pdfmark_LNK(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	    const gs_matrix * pctm, const gs_param_string * objname)
 {
@@ -685,17 +753,24 @@ pdfmark_LNK(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* Write and release one node of the outline tree. */
-private int
+static int
 pdfmark_write_outline(gx_device_pdf * pdev, pdf_outline_node_t * pnode,
 		      long next_id)
 {
     stream *s;
+    int code = 0;
 
     pdf_open_separate(pdev, pnode->id);
-    pnode->action->id = pnode->id;
+    if (pnode->action != NULL)
+	pnode->action->id = pnode->id;
+    else {
+	eprintf1("pdfmark error: Outline node %ld has no action or destination.\n", pnode->id);
+	code = gs_note_error(gs_error_undefined);
+    }
     s = pdev->strm;
     stream_puts(s, "<< ");
-    cos_dict_elements_write(pnode->action, pdev);
+    if (pnode->action != NULL)
+	cos_dict_elements_write(pnode->action, pdev);
     if (pnode->count)
 	pprintd1(s, "/Count %d ", pnode->count);
     pprintld1(s, "/Parent %ld 0 R\n", pnode->parent_id);
@@ -708,13 +783,14 @@ pdfmark_write_outline(gx_device_pdf * pdev, pdf_outline_node_t * pnode,
 		  pnode->first_id, pnode->last_id);
     stream_puts(s, ">>\n");
     pdf_end_separate(pdev);
-    COS_FREE(pnode->action, "pdfmark_write_outline");
+    if (pnode->action != NULL)
+	COS_FREE(pnode->action, "pdfmark_write_outline");
     pnode->action = 0;
-    return 0;
+    return code;
 }
 
 /* Adjust the parent's count when writing an outline node. */
-private void
+static void
 pdfmark_adjust_parent_count(pdf_outline_level_t * plevel)
 {
     pdf_outline_level_t *parent = plevel - 1;
@@ -738,12 +814,10 @@ pdfmark_close_outline(gx_device_pdf * pdev)
 {
     int depth = pdev->outline_depth;
     pdf_outline_level_t *plevel = &pdev->outline_levels[depth];
-    int code;
+    int code = 0;
 
     if (plevel->last.id) {	/* check for incomplete tree */
 	code = pdfmark_write_outline(pdev, &plevel->last, 0);
-	if (code < 0)
-	    return code;
     }
     if (depth > 0) {
 	plevel[-1].last.last_id = plevel->last.id;
@@ -753,11 +827,11 @@ pdfmark_close_outline(gx_device_pdf * pdev)
 	    pdev->closed_outline_depth--;
 	pdev->outline_depth--;
     }
-    return 0;
+    return code;
 }
 
 /* OUT pdfmark */
-private int
+static int
 pdfmark_OUT(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	    const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -830,7 +904,7 @@ pdfmark_OUT(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* Write an article bead. */
-private int
+static int
 pdfmark_write_bead(gx_device_pdf * pdev, const pdf_bead_t * pbead)
 {
     stream *s;
@@ -874,7 +948,7 @@ pdfmark_write_article(gx_device_pdf * pdev, const pdf_article_t * part)
 }
 
 /* ARTICLE pdfmark */
-private int
+static int
 pdfmark_ARTICLE(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -966,7 +1040,7 @@ pdfmark_ARTICLE(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* DEST pdfmark */
-private int
+static int
 pdfmark_DEST(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	     const gs_matrix * pctm, const gs_param_string * objname)
 {
@@ -1014,7 +1088,7 @@ pdfmark_DEST(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* Check that pass-through PostScript code is a string. */
-private bool
+static bool
 ps_source_ok(const gs_param_string * psource)
 {
     if (psource->size >= 2 && psource->data[0] == '(' &&
@@ -1033,7 +1107,7 @@ ps_source_ok(const gs_param_string * psource)
 
 /* Write the contents of pass-through PostScript code. */
 /* Return the size written on the file. */
-private uint
+static uint
 pdfmark_write_ps(stream *s, const gs_param_string * psource)
 {
     /****** REMOVE ESCAPES WITH PSSDecode, SEE gdevpdfr p. 2 ******/
@@ -1045,29 +1119,29 @@ pdfmark_write_ps(stream *s, const gs_param_string * psource)
 }
 
 /* Start a XObject. */
-private int
-start_XObject(gx_device_pdf * pdev, bool compress, cos_stream_t **ppcs,
-		const gs_param_string * objname)
+static int
+start_XObject(gx_device_pdf * pdev, bool compress, cos_stream_t **ppcs)
 {   pdf_resource_t *pres;
     cos_stream_t *pcs;
-    cos_value_t value;
     int code;
 
     code = pdf_open_page(pdev, PDF_IN_STREAM);
     if (code < 0)
 	return code;
-    code = pdf_enter_substream(pdev, resourceXObject, gs_no_id, &pres, true, 
+    code = pdf_enter_substream(pdev, resourceXObject, gs_no_id, &pres, false, 
 		pdev->CompressFonts /* Have no better switch*/);
     if (code < 0)
 	return code;
+    pdev->accumulating_a_global_object = true;
     pcs = (cos_stream_t *)pres->object;
     pdev->substream_Resources = cos_dict_alloc(pdev, "start_XObject");
     if (!pdev->substream_Resources)
 	return_error(gs_error_VMerror);
-    code = cos_dict_put(pdev->local_named_objects, objname->data,
-			    objname->size, cos_object_value(&value, pres->object));
-    if (code < 0)
-	return code;
+    if (pdev->ForOPDFRead) {
+	code = cos_dict_put_c_key_bool((cos_dict_t *)pres->object, "/.Global", true);
+	if (code < 0)
+	    return code;
+    }
     pres->named = true;
     pres->where_used = 0;	/* initially not used */
     pcs->pres = pres;
@@ -1077,7 +1151,7 @@ start_XObject(gx_device_pdf * pdev, bool compress, cos_stream_t **ppcs,
 
 /* PS pdfmark */
 #define MAX_PS_INLINE 100
-private int
+static int
 pdfmark_PS(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	   const gs_matrix * pctm, const gs_param_string * objname)
 {
@@ -1105,17 +1179,23 @@ pdfmark_PS(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	cos_stream_t *pcs;
 	int code;
 	gs_id level1_id = gs_no_id;
+	pdf_resource_t *pres;
 
 	if (level1.data != 0) {
 	    pdf_resource_t *pres;
 
 	    code = pdf_enter_substream(pdev, 
-			resourceXObject /* A stub. Actually it's not a resource. */, 
+			resourceXObject, 
 			gs_no_id, &pres, true, 
 			pdev->CompressFonts /* Have no better switch*/);
 	    if (code < 0)
 		return code;
 	    pcs = (cos_stream_t *)pres->object;
+	    if (pdev->ForOPDFRead && objname != 0) {
+		code = cos_dict_put_c_key_bool((cos_dict_t *)pres->object, "/.Global", true);
+		if (code < 0)
+		    return code;
+	    }
 	    pres->named = (objname != 0);
 	    pres->where_used = 0;
 	    pcs->pres = pres;
@@ -1128,10 +1208,10 @@ pdfmark_PS(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		return code;
 	    level1_id = pres->object->id;
 	}
-	code = start_XObject(pdev, pdev->params.CompressPages /* Have no better switch*/, 
-			    &pcs, objname);
+	code = start_XObject(pdev, pdev->params.CompressPages, &pcs);
 	if (code < 0)
 	    return code;
+	pres = pdev->accumulating_substream_resource;
 	code = cos_stream_put_c_strings(pcs, "/Type", "/XObject");
 	if (code < 0)
 	    return code;
@@ -1151,10 +1231,16 @@ pdfmark_PS(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	code = pdf_exit_substream(pdev);
 	if (code < 0)
 	    return code;
-	if (!objname) {
-	    /* Write the resource now, since it won't be written later. */
-	    COS_WRITE_OBJECT(pcs, pdev);
-	    COS_RELEASE(pcs, "pdfmark_PS");
+	{   gs_const_string objname1, *pon = NULL;
+
+	    if (objname != NULL) {
+		objname1.data = objname->data;
+		objname1.size = objname->size;
+		pon = &objname1;
+	    }
+	    code = pdfmark_bind_named_object(pdev, pon, &pres);
+	    if (code < 0)
+    		return code;
 	}
 	code = pdf_open_contents(pdev, PDF_IN_STREAM);
 	if (code < 0)
@@ -1166,7 +1252,7 @@ pdfmark_PS(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* Common code for pdfmarks that do PUT into a specific object. */
-private int
+static int
 pdfmark_put_pairs(cos_dict_t *pcd, gs_param_string * pairs, uint count)
 {
     int code = 0, i;
@@ -1179,7 +1265,7 @@ pdfmark_put_pairs(cos_dict_t *pcd, gs_param_string * pairs, uint count)
 }
 
 /* PAGES pdfmark */
-private int
+static int
 pdfmark_PAGES(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	      const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1187,7 +1273,7 @@ pdfmark_PAGES(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* PAGE pdfmark */
-private int
+static int
 pdfmark_PAGE(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	     const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1197,7 +1283,7 @@ pdfmark_PAGE(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 /* Add a page label for the current page. The last label on a page 
  * overrides all previous labels for this page. Unlabeled pages will get 
  * empty page labels. label == NULL flushes the last label */
-private int 
+static int 
 pdfmark_add_pagelabel(gx_device_pdf * pdev, const gs_param_string *label) 
 {
     cos_value_t value;
@@ -1285,7 +1371,7 @@ pdfmark_end_pagelabels(gx_device_pdf * pdev)
 
 /* [ /Label string /PlateColor string pdfmark */
 /* FIXME: /PlateColor is ignored */
-private int
+static int
 pdfmark_PAGELABEL(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	     const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1300,7 +1386,7 @@ pdfmark_PAGELABEL(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* DOCINFO pdfmark */
-private int
+static int
 pdfmark_DOCINFO(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1379,7 +1465,7 @@ pdfmark_DOCINFO(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* DOCVIEW pdfmark */
-private int
+static int
 pdfmark_DOCVIEW(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1406,7 +1492,7 @@ pdfmark_DOCVIEW(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 /* ---------------- Named object pdfmarks ---------------- */
 
 /* [ /BBox [llx lly urx ury] /_objdef {obj} /BP pdfmark */
-private int
+static int
 pdfmark_BP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	   const gs_matrix * pctm, const gs_param_string * objname)
 {
@@ -1415,6 +1501,7 @@ pdfmark_BP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
     int code;
     gs_matrix ictm;
     byte bbox_str[6 + 6 * 15], matrix_str[6 + 6 * 15];
+    char chars[MAX_RECT_STRING + 1];
     int bbox_str_len, matrix_str_len;
     stream s;
 
@@ -1423,15 +1510,26 @@ pdfmark_BP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
     code = gs_matrix_invert(pctm, &ictm);
     if (code < 0)
 	return code;
-    if (sscanf((const char *)pairs[1].data, "[%lg %lg %lg %lg]",
+    if (pairs[1].size > MAX_RECT_STRING)
+	return_error(gs_error_limitcheck);
+    memcpy(chars, pairs[1].data, pairs[1].size);
+    chars[pairs[1].size] = 0;
+    if (sscanf(chars, "[%lg %lg %lg %lg]",
 	       &bbox.p.x, &bbox.p.y, &bbox.q.x, &bbox.q.y) != 4)
 	return_error(gs_error_rangecheck);
     if ((pdev->used_mask << 1) == 0)
 	return_error(gs_error_limitcheck);
-    code = start_XObject(pdev, pdev->params.CompressPages /* Have no better switch*/, 
-			    &pcs, objname);
+    code = start_XObject(pdev, pdev->params.CompressPages, &pcs);
     if (code < 0)
 	return code;
+    {	byte *s = gs_alloc_string(pdev->memory, objname->size, "pdfmark_PS");
+	
+	if (s == NULL)
+	    return_error(gs_error_VMerror);
+	memcpy(s, objname->data, objname->size);
+	pdev->objname.data = s;
+	pdev->objname.size = objname->size;
+    }
     pcs->is_graphics = true;
     gs_bbox_transform(&bbox, pctm, &bbox);
     s_init(&s, NULL);
@@ -1454,27 +1552,40 @@ pdfmark_BP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
  					  COS_OBJECT(pdev->substream_Resources))) < 0
 	)
 	return code;
+    /* Don't add to local_named_objects untill the object is created
+       to prevent pending references, which may appear
+       if /PUT pdfmark executes before pdf_substitute_resource in pdfmark_EP 
+       drops this object.
+    */
     return 0;
 }
 
 /* [ /EP pdfmark */
-private int
+static int
 pdfmark_EP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	   const gs_matrix * pctm, const gs_param_string * no_objname)
 {
     int code;
+    pdf_resource_t *pres = pdev->accumulating_substream_resource;
+    gs_const_string objname = pdev->objname;
 
     code = pdf_add_procsets(pdev->substream_Resources, pdev->procsets);
     if (code < 0)
 	return code;
     code = pdf_exit_substream(pdev);
+    if (code < 0)
+	return code;
+    code = pdfmark_bind_named_object(pdev, &objname, &pres);
+    if (code < 0)
+	return 0;
+    gs_free_const_string(pdev->memory, objname.data, objname.size, "pdfmark_EP");
     if (code < 0)
 	return code;
     return 0;
 }
 
 /* [ {obj} /SP pdfmark */
-private int
+static int
 pdfmark_SP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	   const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1499,7 +1610,7 @@ pdfmark_SP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 /* [ /_objdef {array} /type /array /OBJ pdfmark */
 /* [ /_objdef {dict} /type /dict /OBJ pdfmark */
 /* [ /_objdef {stream} /type /stream /OBJ pdfmark */
-private int
+static int
 pdfmark_OBJ(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	    const gs_matrix * pctm, const gs_param_string * objname)
 {
@@ -1540,7 +1651,7 @@ pdfmark_OBJ(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 /* [ {array} index value /PUT pdfmark */
 /* Dictionaries are converted to .PUTDICT */
 /* Streams are converted to .PUTSTREAM */
-private int
+static int
 pdfmark_PUT(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	    const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1556,6 +1667,8 @@ pdfmark_PUT(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	return code;
     if (index < 0)
 	return_error(gs_error_rangecheck);
+    if (pco->written)
+	return_error(gs_error_rangecheck);
     return cos_array_put((cos_array_t *)pco, index,
 		cos_string_value(&value, pairs[2].data, pairs[2].size));
 }
@@ -1567,7 +1680,7 @@ pdfmark_PUT(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
  * but it's reasonable and unambiguous, and Acrobat Distiller accepts it,
  * so we do too.
  */
-private int
+static int
 pdfmark_PUTDICT(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1578,11 +1691,13 @@ pdfmark_PUTDICT(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	return code;
     if (cos_type(pco) != cos_type_dict && cos_type(pco) != cos_type_stream)
 	return_error(gs_error_typecheck);
+    if (pco->written)
+	return_error(gs_error_rangecheck);
     return pdfmark_put_pairs((cos_dict_t *)pco, pairs + 1, count - 1);
 }
 
 /* [ {stream} string ... /.PUTSTREAM pdfmark */
-private int
+static int
 pdfmark_PUTSTREAM(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		  const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1599,11 +1714,13 @@ pdfmark_PUTSTREAM(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
     for (i = 1; i < count; ++i)
 	if (sputs(pco->input_strm, pairs[i].data, pairs[i].size, &l) != 0)
 	    return_error(gs_error_ioerror);
+    if (pco->written)
+	return_error(gs_error_rangecheck);
     return code;
 }
 
 /* [ {array} value /APPEND pdfmark */
-private int
+static int
 pdfmark_APPEND(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	       const gs_matrix * pctm, const gs_param_string * objname)
 {
@@ -1620,7 +1737,7 @@ pdfmark_APPEND(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* [ {array} index value ... /.PUTINTERVAL pdfmark */
-private int
+static int
 pdfmark_PUTINTERVAL(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		 const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1643,7 +1760,7 @@ pdfmark_PUTINTERVAL(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* [ {stream} /CLOSE pdfmark */
-private int
+static int
 pdfmark_CLOSE(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	      const gs_matrix * pctm, const gs_param_string * no_objname)
 {
@@ -1662,7 +1779,7 @@ pdfmark_CLOSE(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 }
 
 /* [ /NamespacePush pdfmark */
-private int
+static int
 pdfmark_NamespacePush(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 		      const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1672,7 +1789,7 @@ pdfmark_NamespacePush(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ /NamespacePop pdfmark */
-private int
+static int
 pdfmark_NamespacePop(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 		     const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1683,7 +1800,7 @@ pdfmark_NamespacePop(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ /_objdef {image} /NI pdfmark */
-private int
+static int
 pdfmark_NI(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	   const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1701,7 +1818,7 @@ pdfmark_NI(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 /* ---------------- Named content pdfmarks ---------------- */
 
 /* [ tag /MP pdfmark */
-private int
+static int
 pdfmark_MP(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	   const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1709,7 +1826,7 @@ pdfmark_MP(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ tag propdict /DP pdfmark */
-private int
+static int
 pdfmark_DP(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	   const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1717,7 +1834,7 @@ pdfmark_DP(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ tag /BMC pdfmark */
-private int
+static int
 pdfmark_BMC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	    const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1725,7 +1842,7 @@ pdfmark_BMC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ tag propdict /BDC pdfmark */
-private int
+static int
 pdfmark_BDC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	    const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1733,7 +1850,7 @@ pdfmark_BDC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ /EMC pdfmark */
-private int
+static int
 pdfmark_EMC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	    const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1743,7 +1860,7 @@ pdfmark_EMC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 /* ---------------- Document structure pdfmarks ---------------- */
 
 /* [ newsubtype1 stdsubtype1 ... /StRoleMap pdfmark */
-private int
+static int
 pdfmark_StRoleMap(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 		  const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1751,7 +1868,7 @@ pdfmark_StRoleMap(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ class1 {attrobj1} ... /StClassMap pdfmark */
-private int
+static int
 pdfmark_StClassMap(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 		   const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1763,7 +1880,7 @@ pdfmark_StClassMap(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
  *   [/ID string] [/Class name] [/At index] [/Bookmark dict] [action_pairs...]
  *   /StPNE pdfmark
  */
-private int
+static int
 pdfmark_StPNE(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	      const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1771,7 +1888,7 @@ pdfmark_StPNE(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ [/Title string] [/Open bool] [action_pairs...] /StBookmarkRoot pdfmark */
-private int
+static int
 pdfmark_StBookmarkRoot(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 		       const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1779,7 +1896,7 @@ pdfmark_StBookmarkRoot(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ [/E {elt}] /StPush pdfmark */
-private int
+static int
 pdfmark_StPush(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	       const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1787,7 +1904,7 @@ pdfmark_StPush(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ /StPop pdfmark */
-private int
+static int
 pdfmark_StPop(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	      const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1795,7 +1912,7 @@ pdfmark_StPop(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ /StPopAll pdfmark */
-private int
+static int
 pdfmark_StPopAll(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 		 const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1803,7 +1920,7 @@ pdfmark_StPopAll(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ [/T tagname] [/At index] /StBMC pdfmark */
-private int
+static int
 pdfmark_StBMC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	      const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1811,7 +1928,7 @@ pdfmark_StBMC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ [/P propdict] [/T tagname] [/At index] /StBDC pdfmark */
-private int
+static int
 pdfmark_StBDC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	      const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1819,7 +1936,7 @@ pdfmark_StBDC(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ /Obj {obj} [/At index] /StOBJ pdfmark */
-private int
+static int
 pdfmark_StOBJ(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	      const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1827,7 +1944,7 @@ pdfmark_StOBJ(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ /Obj {obj} /StAttr pdfmark */
-private int
+static int
 pdfmark_StAttr(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 	       const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1835,7 +1952,7 @@ pdfmark_StAttr(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ /StoreName name /StStore pdfmark */
-private int
+static int
 pdfmark_StStore(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 		const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1843,7 +1960,7 @@ pdfmark_StStore(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 }
 
 /* [ /StoreName name /StRetrieve pdfmark */
-private int
+static int
 pdfmark_StRetrieve(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 		   const gs_matrix *pctm, const gs_param_string *objname)
 {
@@ -1855,7 +1972,7 @@ pdfmark_StRetrieve(gx_device_pdf *pdev, gs_param_string *pairs, uint count,
 /*
  * Define the pdfmark types we know about.
  */
-private const pdfmark_name mark_names[] =
+static const pdfmark_name mark_names[] =
 {
 	/* Miscellaneous. */
     {"ANN",          pdfmark_ANN,         PDFMARK_NAMEABLE},
@@ -1920,12 +2037,19 @@ pdfmark_process(gx_device_pdf * pdev, const gs_param_string_array * pma)
     gs_matrix ctm;
     const pdfmark_name *pmn;
     int code = 0;
+    
+    {	int cnt, len = pts[-1].size;
+	char buf[200]; /* 6 doubles should fit (%g == -0.14285714285714285e-101 = 25 chars) */
 
-    if (size < 2 ||
-	sscanf((const char *)pts[-1].data, "[%g %g %g %g %g %g]",
-	       &ctm.xx, &ctm.xy, &ctm.yx, &ctm.yy, &ctm.tx, &ctm.ty) != 6
-	)
-	return_error(gs_error_rangecheck);
+	if (len > sizeof(buf) - 1)
+	    return_error(gs_error_rangecheck);
+	memcpy(buf, pts[-1].data, len);
+	buf[len] = 0;
+	cnt = sscanf(buf, "[%g %g %g %g %g %g]",
+		   &ctm.xx, &ctm.xy, &ctm.yx, &ctm.yy, &ctm.tx, &ctm.ty);
+	if (cnt != 6)
+	    return_error(gs_error_rangecheck);
+    }
     size -= 2;			/* remove CTM & pdfmark name */
     for (pmn = mark_names; pmn->mname != 0; ++pmn)
 	if (pdf_key_eq(pts, pmn->mname)) {
