@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfo.c 8250 2007-09-25 13:31:24Z giles $ */
+/* $Id: gdevpdfo.c 8611 2008-03-27 08:37:58Z ken $ */
 /* Cos object support */
 #include "memory_.h"
 #include "string_.h"
@@ -524,6 +524,8 @@ cos_array_write(const cos_object_t *pco, gx_device_pdf *pdev, gs_id object_id)
     }
     DISCARD(cos_array_reorder(pca, first));
     stream_puts(s, "]");
+    if (pdev->PDFA)
+        stream_puts(s, "\n");
     return 0;
 }
 
@@ -790,6 +792,8 @@ cos_dict_write(const cos_object_t *pco, gx_device_pdf *pdev, gs_id object_id)
     stream_puts(s, "<<");
     cos_elements_write(s, ((const cos_dict_t *)pco)->elements, pdev, false, object_id);
     stream_puts(s, ">>");
+    if (pdev->PDFA)
+        stream_puts(s, "\n");
     return 0;
 }
 
