@@ -1,20 +1,17 @@
-/* Copyright (C) 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfo.h,v 1.19 2004/06/08 11:43:04 igor Exp $ */
+/* $Id: gdevpdfo.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Internal definitions for "objects" for pdfwrite driver. */
 
 #ifndef gdevpdfo_INCLUDED
@@ -282,6 +279,9 @@ const cos_array_element_t *
 /* Look up a key in a dictionary. */
 const cos_value_t *cos_dict_find(const cos_dict_t *, const byte *, uint);
 const cos_value_t *cos_dict_find_c_key(const cos_dict_t *, const char *);
+/* Process all entries in a dictionary. */
+int cos_dict_forall(const cos_dict_t *pcd, void *client_data, 
+	int (*proc)(void *client_data, const byte *key_data, uint key_size, const cos_value_t *v));
 
 /* Set up a parameter list that writes into a Cos dictionary. */
 typedef struct cos_param_list_writer_s {
