@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfu.c 8499 2008-01-23 16:49:51Z ken $ */
+/* $Id: gdevpdfu.c 8611 2008-03-27 08:37:58Z ken $ */
 /* Output utilities for PDF-writing driver */
 #include "memory_.h"
 #include "jpeglib_.h"		/* for sdct.h */
@@ -689,6 +689,8 @@ stream_to_none(gx_device_pdf * pdev)
 	pdf_end_encrypt(pdev);
     	s = pdev->strm;
 	length = pdf_stell(pdev) - pdev->contents_pos;
+	if (pdev->PDFA)
+	    stream_puts(s, "\n");
 	stream_puts(s, "endstream\n");
 	pdf_end_obj(pdev);
 	pdf_open_obj(pdev, pdev->contents_length_id);
