@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtb.c 6669 2006-03-21 12:09:23Z leonardo $ */
+/* $Id: gdevpdtb.c 8622 2008-04-04 08:53:57Z ken $ */
 /* BaseFont implementation for pdfwrite */
 #include "memory_.h"
 #include "ctype_.h"
@@ -43,6 +43,8 @@
 /* ---------------- Private ---------------- */
 
 private_st_pdf_base_font();
+gs_private_st_basic(st_pdf_base_font, pdf_base_font_t, "pdf_base_font_t",\
+		    pdf_base_font_ptrs, pdf_base_font_data);
 
 #define SUBSET_PREFIX_SIZE 7	/* XXXXXX+ */
 
@@ -62,7 +64,7 @@ pdf_has_subset_prefix(const byte *str, uint size)
     return true;
 }
 
-private ulong
+static inline ulong
 hash(ulong v, int index, ushort w)
 {
     return v * 3141592653u + w;
@@ -79,20 +81,19 @@ pdf_add_subset_prefix(const gx_device_pdf *pdev, gs_string *pstr, byte *used, in
 				  size + SUBSET_PREFIX_SIZE,
 				  "pdf_add_subset_prefix");
     int len = (count + 7) / 8;
+    int len0 = len & ~(sizeof(ushort) - 1);
     ulong v = 0;
-    ushort t = 0;
     int i;
 
     if (data == 0)
 	return_error(gs_error_VMerror);
+    
     /* Hash the 'used' array. */
-    for (i = 0; i < len; i += sizeof(ushort))
+    for (i = 0; i < len0; i += sizeof(ushort))
 	v = hash(v, i, *(ushort *)(used + i));
-    if (i < len) {
-	i -= sizeof(ushort);
-	memmove(&t, used + i, len - i);
-	v = hash(v, i, *(ushort *)(used + i));
-    }
+    for (; i < len; i++)
+        v = hash(v, i, used[i]);
+
     memmove(data + SUBSET_PREFIX_SIZE, data, size);
     for (i = 0; i < SUBSET_PREFIX_SIZE - 1; ++i, v /= 26)
 	data[i] = 'A' + (v % 26);
@@ -103,7 +104,7 @@ pdf_add_subset_prefix(const gx_device_pdf *pdev, gs_string *pstr, byte *used, in
 }
 
 /* Finish writing FontFile* data. */
-private int
+static int
 pdf_end_fontfile(gx_device_pdf *pdev, pdf_data_writer_t *pdw)
 {
     /* We would like to call pdf_end_data,
@@ -122,7 +123,7 @@ pdf_end_fontfile(gx_device_pdf *pdev, pdf_data_writer_t *pdw)
 int
 pdf_base_font_alloc(gx_device_pdf *pdev, pdf_base_font_t **ppbfont,
 		    gs_font_base *font, const gs_matrix *orig_matrix, 
-		    bool is_standard, bool orig_name)
+		    bool is_standard)
 {
     gs_memory_t *mem = pdev->pdf_memory;
     gs_font *copied;
@@ -130,7 +131,7 @@ pdf_base_font_alloc(gx_device_pdf *pdev, pdf_base_font_t **ppbfont,
     pdf_base_font_t *pbfont =
 	gs_alloc_struct(mem, pdf_base_font_t,
 			&st_pdf_base_font, "pdf_base_font_alloc");
-    const gs_font_name *pfname = pdf_choose_font_name((gs_font *)font, orig_name);
+    const gs_font_name *pfname = &font->font_name;
     gs_const_string font_name;
     char fnbuf[3 + sizeof(long) / 3 + 1]; /* .F#######\0 */
     int code;
@@ -199,9 +200,23 @@ pdf_base_font_alloc(gx_device_pdf *pdev, pdf_base_font_t **ppbfont,
 		goto fail;
 	}
         code = gs_copy_font_complete((gs_font *)font, complete);
-	if (code < 0)
+	if (code < 0 && pbfont->do_subset == DO_SUBSET_NO) {
+	    char buf[gs_font_name_max + 1];
+	    int l = min(copied->font_name.size, sizeof(buf) - 1);
+
+	    memcpy(buf, copied->font_name.chars, l);
+	    buf[l] = 0;
+    	    eprintf1("Can't embed the complete font %s due to font error.\n", buf);
 	    goto fail;
-	if (pbfont->num_glyphs < 0) { /* Type 1 */
+	}
+	if (code < 0) {
+	    /* A font error happened, but it may be caused by a glyph, 
+	       which is not used in the document. Continue with subsetting the font.
+	       If the failed glyph will be used in the document, 
+	       another error will hgappen when the glyph is used.
+	     */
+	    complete = copied;
+	} else if (pbfont->num_glyphs < 0) { /* Type 1 */
 	    int index, count;
 	    gs_glyph glyph;
 
@@ -379,7 +394,7 @@ pdf_write_FontFile_entry(gx_device_pdf *pdev, pdf_base_font_t *pbfont)
 /*
  * Adjust font name for Acrobat Reader 3.
  */
-private int
+static int
 pdf_adjust_font_name(gx_device_pdf *pdev, long id, pdf_base_font_t *pbfont)
 {
     /*
@@ -430,7 +445,7 @@ pdf_adjust_font_name(gx_device_pdf *pdev, long id, pdf_base_font_t *pbfont)
  * Write an embedded font.
  */
 int
-pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
+pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont, font_type FontType,
 			gs_int_rect *FontBBox, gs_id rid, cos_dict_t **ppcd)
 {
     bool do_subset = pdf_do_subset_font(pdev, pbfont, rid);
@@ -477,7 +492,7 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
     fnstr.data = pbfont->font_name.data;
     fnstr.size = pbfont->font_name.size;
     /* Now write the font (or subset). */
-    switch (out_font->FontType) {
+    switch (FontType) {
 
     case ft_composite:
 	/* Nothing to embed -- the descendant fonts do it all. */
@@ -544,6 +559,7 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_base_font_t *pbfont,
 #define TRUETYPE_OPTIONS (WRITE_TRUETYPE_NAME | WRITE_TRUETYPE_HVMTX)
 	/* Acrobat Reader 3 doesn't handle cmap format 6 correctly. */
 	const int options = TRUETYPE_OPTIONS |
+	    (pdev->PDFA ? WRITE_TRUETYPE_UNICODE_CMAP : 0) |
 	    (pdev->CompatibilityLevel <= 1.2 ?
 	     WRITE_TRUETYPE_NO_TRIMMED_TABLE : 0) |
 	    /* Generate a cmap only for incrementally downloaded fonts
