@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtf.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gdevpdtf.c 8603 2008-03-20 10:20:17Z ken $ */
 /* Font and CMap resource implementation for pdfwrite text */
 #include "memory_.h"
 #include "string_.h"
@@ -40,7 +40,7 @@ private_st_pdf_standard_font();
 private_st_pdf_standard_font_element();
 private_st_pdf_outline_fonts();
 
-private
+static
 ENUM_PTRS_WITH(pdf_font_resource_enum_ptrs, pdf_font_resource_t *pdfont)
 ENUM_PREFIX(st_pdf_resource, 12);
 case 0: return ENUM_STRING(&pdfont->BaseFont);
@@ -103,7 +103,7 @@ case 11: switch (pdfont->FontType) {
      ENUM_RETURN(0);
 }
 ENUM_PTRS_END
-private
+static
 RELOC_PTRS_WITH(pdf_font_resource_reloc_ptrs, pdf_font_resource_t *pdfont)
 {
     RELOC_PREFIX(st_pdf_resource);
@@ -154,7 +154,7 @@ typedef struct pdf_standard_font_info_s {
     int size;
     gs_encoding_index_t base_encoding;
 } pdf_standard_font_info_t;
-private const pdf_standard_font_info_t standard_font_info[] = {
+static const pdf_standard_font_info_t standard_font_info[] = {
     {"Courier",                7, ENCODING_INDEX_STANDARD},
     {"Courier-Bold",          12, ENCODING_INDEX_STANDARD},
     {"Courier-Oblique",       15, ENCODING_INDEX_STANDARD},
@@ -173,7 +173,7 @@ private const pdf_standard_font_info_t standard_font_info[] = {
 };
 
 /* Return the index of a standard font name, or -1 if missing. */
-private int
+static int
 pdf_find_standard_font_name(const byte *str, uint size)
 {
     const pdf_standard_font_info_t *ppsf;
@@ -192,7 +192,7 @@ pdf_find_standard_font_name(const byte *str, uint size)
  * identical properties, and return the standard-font index; otherwise,
  * set *psame to 0 and return -1.
  */
-private int
+static int
 find_std_appearance(const gx_device_pdf *pdev, gs_font_base *bfont,
 		    int mask, pdf_char_glyph_pair_t *pairs, int num_glyphs)
 {
@@ -246,7 +246,7 @@ find_std_appearance(const gx_device_pdf *pdev, gs_font_base *bfont,
  * were found.  A font is recognized as standard if it was loaded as a
  * resource, it has a UniqueId, and it has a standard name.
  */
-private bool
+static bool
 scan_for_standard_fonts(gx_device_pdf *pdev, const gs_font_dir *dir)
 {
     bool found = false;
@@ -328,7 +328,7 @@ pdf_clean_standard_fonts(const gx_device_pdf *pdev)
 /* ------ Private ------ */
 
 
-private int 
+static int 
 pdf_resize_array(gs_memory_t *mem, void **p, int elem_size, int old_size, int new_size)
 {
     void *q = gs_alloc_byte_array(mem, new_size, elem_size, "pdf_resize_array");
@@ -345,7 +345,7 @@ pdf_resize_array(gs_memory_t *mem, void **p, int elem_size, int old_size, int ne
 /*
  * Allocate and (minimally) initialize a font resource.
  */
-private int
+static int
 font_resource_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 		    pdf_resource_type_t rtype, gs_id rid, font_type ftype,
 		    int chars_count,
@@ -411,16 +411,18 @@ pdf_assign_font_object_id(gx_device_pdf *pdev, pdf_font_resource_t *pdfont)
 	if (pdfont->FontType == 0) {
 	    pdf_font_resource_t *pdfont1 = pdfont->u.type0.DescendantFont;
 
-	    pdf_reserve_object_id(pdev, (pdf_resource_t *)pdfont1, 0);
-	    code = pdf_mark_font_descriptor_used(pdev, pdfont1->FontDescriptor);
-	    if (code < 0)
-		return code;
+	    if (pdf_font_id(pdfont1) == -1) {
+		pdf_reserve_object_id(pdev, (pdf_resource_t *)pdfont1, 0);
+		code = pdf_mark_font_descriptor_used(pdev, pdfont1->FontDescriptor);
+		if (code < 0)
+		    return code;
+	    }
 	}
     }
     return 0;
 }
 
-private int
+static int
 font_resource_simple_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 			   gs_id rid, font_type ftype, int chars_count,
 			   pdf_font_write_contents_proc_t write_contents)
@@ -434,6 +436,8 @@ font_resource_simple_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
     pfres->u.simple.FirstChar = 256;
     pfres->u.simple.LastChar = -1;
     pfres->u.simple.BaseEncoding = -1;
+    pfres->u.simple.preferred_encoding_index = -1;
+    pfres->u.simple.last_reserved_char = -1;
     *ppfres = pfres;
     return 0;
 }
@@ -484,7 +488,7 @@ font_resource_encoded_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
 /*
  * Record whether a Type 1 or Type 2 font is a Multiple Master instance.
  */
-private void
+static void
 set_is_MM_instance(pdf_font_resource_t *pdfont, const gs_font_base *pfont)
 {
     switch (pfont->FontType) {
@@ -584,7 +588,7 @@ pdf_font_resource_font(const pdf_font_resource_t *pdfont, bool complete)
  * 14, store its index (0..13) in *pindex and its similarity to the base
  * font (as determined by the font's same_font procedure) in *psame.
  */
-private bool
+static bool
 font_is_symbolic(const gs_font *font)
 {
     if (font->FontType == ft_composite)
@@ -599,7 +603,7 @@ font_is_symbolic(const gs_font *font)
 	return true;
     }
 }
-private bool
+static bool
 embed_list_includes(const gs_param_string_array *psa, const byte *chars,
 		    uint size)
 {
@@ -610,7 +614,7 @@ embed_list_includes(const gs_param_string_array *psa, const byte *chars,
 	    return true;
     return false;
 }
-private bool
+static bool
 embed_as_standard(gx_device_pdf *pdev, gs_font *font, int index,
 		  pdf_char_glyph_pair_t *pairs, int num_glyphs)
 {
@@ -625,7 +629,7 @@ embed_as_standard(gx_device_pdf *pdev, gs_font *font, int index,
     return (find_std_appearance(pdev, (gs_font_base *)font, -1,
 				pairs, num_glyphs) == index);
 }
-private bool
+static bool
 has_extension_glyphs(gs_font *pfont)
 {
     psf_glyph_enum_t genum;
@@ -648,25 +652,43 @@ has_extension_glyphs(gs_font *pfont)
     return false;
 }
 
-/*
- * Choose a name for embedded font.
- */
-const gs_font_name *pdf_choose_font_name(gs_font *font, bool key_name)
-{
-    return key_name ? (font->key_name.size != 0 ? &font->key_name : &font->font_name)
-	             : (font->font_name.size != 0 ? &font->font_name : &font->key_name);
-}
 pdf_font_embed_t
 pdf_font_embed_status(gx_device_pdf *pdev, gs_font *font, int *pindex,
 		      pdf_char_glyph_pair_t *pairs, int num_glyphs)
 {
-    const gs_font_name *fn = pdf_choose_font_name(font, false);
+    const gs_font_name *fn = &font->font_name;
     const byte *chars = fn->chars;
     uint size = fn->size;
     int index = pdf_find_standard_font_name(chars, size);
     bool embed_as_standard_called = false;
     bool do_embed_as_standard = false; /* Quiet compiler. */
+    int code;
+    gs_font_info_t info;
 
+    memset(&info, 0x00, sizeof(gs_font_info_t));
+    code = font->procs.font_info(font, NULL, FONT_INFO_EMBEDDING_RIGHTS, &info);
+    if (code == 0 && (info.members & FONT_INFO_EMBEDDING_RIGHTS)) {
+	if ((info.EmbeddingRights == 0x0002) || (info.EmbeddingRights & 0x0200)) {
+	    /* See the OpenType specification, "The 'OS/2' and Windows Metrics Table" for details
+	       of the fstype parameter. This is a bitfield, currently we forbid embedding of fonts
+	       with these bits set:
+	       bit 1	0x0002	Fonts that have only this bit set must not be modified, embedded or
+				exchanged in any manner.
+	       bit 9	0x0200	Bitmap embedding only.
+
+	       Note for Restricted License embedding (bit 1), this must be the only level of embedding 
+	       selected (see the OpenType spec).
+             */
+	    char name[gs_font_name_max + 1];
+	    int len;
+
+	    len = min(gs_font_name_max, font->font_name.size);
+	    memcpy(name, font->font_name.chars, len);
+	    name[len] = 0;
+	    eprintf1("\nWarning: %s cannot be embedded because of licensing restrictions\n", name);
+	    return FONT_EMBED_NO;
+	}
+    }
     /*
      * The behavior of Acrobat Distiller changed between 3.0 (PDF 1.2),
      * which will never embed the base 14 fonts, and 4.0 (PDF 1.3), which
@@ -676,7 +698,7 @@ pdf_font_embed_status(gx_device_pdf *pdev, gs_font *font, int *pindex,
      */
     if (pindex)
 	*pindex = index;
-    if (pdev->PDFX)
+    if (pdev->PDFX || pdev->PDFA)
 	return FONT_EMBED_YES;
     if (pdev->CompatibilityLevel < 1.3) {
 	if (index >= 0 && 
@@ -812,6 +834,7 @@ pdf_font_type0_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
     if (code >= 0) {
 	(*ppfres)->u.type0.DescendantFont = DescendantFont;
 	(*ppfres)->u.type0.CMapName = *CMapName;
+	(*ppfres)->u.type0.font_index = 0;
 	code = pdf_compute_BaseFont(pdev, *ppfres, false);
     }
     return code;    
@@ -843,7 +866,7 @@ pdf_font_std_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
     gs_matrix *orig_matrix = (is_original ? &pfont->FontMatrix : &psf->orig_matrix);
 
     if (code < 0 ||
-	(code = pdf_base_font_alloc(pdev, &pdfont->base_font, pfont, orig_matrix, true, true)) < 0
+	(code = pdf_base_font_alloc(pdev, &pdfont->base_font, pfont, orig_matrix, true)) < 0
 	)
 	return code;
     pdfont->BaseFont.data = (byte *)psfi->fname; /* break const */
@@ -937,15 +960,9 @@ pdf_font_cidfont_alloc(gx_device_pdf *pdev, pdf_font_resource_t **ppfres,
      * Write the CIDSystemInfo now, so we don't try to access it after
      * the font may no longer be available.
      */
-    {
-	long cidsi_id = pdf_begin_separate(pdev);
-
-	code = pdf_write_cid_system_info(pdev, pcidsi, cidsi_id);
-	if (code < 0)
-	    return code;
-	pdf_end_separate(pdev);
-	pdfont->u.cidfont.CIDSystemInfo_id = cidsi_id;
-    }
+    code = pdf_write_cid_systemInfo_separate(pdev, pcidsi, &pdfont->u.cidfont.CIDSystemInfo_id);
+    if (code < 0)
+	return code;
     *ppfres = pdfont;
     return pdf_compute_BaseFont(pdev, pdfont, false);
 }
@@ -996,6 +1013,59 @@ pdf_obtain_cidfont_widths_arrays(gx_device_pdf *pdev, pdf_font_resource_t *pdfon
     return 0;
 }
 
+/*
+ * Convert True Type fonts into CID fonts for PDF/A.
+ */
+int 
+pdf_convert_truetype_font(gx_device_pdf *pdev, pdf_resource_t *pres)
+{
+    if (!pdev->PDFA)
+	return 0;
+    else {
+	pdf_font_resource_t *pdfont = (pdf_font_resource_t *)pres;
+
+	if (pdfont->FontType != ft_TrueType)
+	    return 0;
+	else if (pdf_resource_id(pres) == -1)
+	    return 0; /* An unused font. */
+	else {
+	    int code = pdf_different_encoding_index(pdfont, 0);
+
+	    if (code < 0)
+		return code;
+	    if (code == 256)
+		return 0;
+	    {	/* The encoding have a difference - do convert. */
+		pdf_font_resource_t *pdfont0;
+		gs_const_string CMapName = {(const byte *)"OneByteIdentityH", 16};
+
+		code = pdf_convert_truetype_font_descriptor(pdev, pdfont);
+		if (code < 0)
+		    return code;
+		code = pdf_font_type0_alloc(pdev, &pdfont0, pres->rid + 1, pdfont, &CMapName);
+		if (code < 0)
+		    return code;
+		/* Pass the font object ID to the type 0 font resource. */
+		pdf_reserve_object_id(pdev, (pdf_resource_t *)pdfont0, pdf_resource_id(pres));
+		pdf_reserve_object_id(pdev, (pdf_resource_t *)pdfont, gs_no_id);
+		/* Set Encoding_name because we won't call attach_cmap_resource for it : */
+		code = pdf_write_OneByteIdentityH(pdev);
+		if (code < 0)
+		    return 0;
+		pdfont->u.cidfont.CIDSystemInfo_id = pdev->IdentityCIDSystemInfo_id;
+		sprintf(pdfont0->u.type0.Encoding_name, "%ld 0 R", pdf_resource_id(pdev->OneByteIdentityH));
+		/* Move ToUnicode : */
+		pdfont0->res_ToUnicode = pdfont->res_ToUnicode; pdfont->res_ToUnicode = 0;
+		pdfont0->cmap_ToUnicode = pdfont->cmap_ToUnicode; pdfont->cmap_ToUnicode = 0;
+		/* Change the font type to CID font : */
+		pdfont->FontType = ft_CID_TrueType;
+		pdfont->write_contents = pdf_write_contents_cid2;
+		return 0;
+	    }
+	}
+    }
+}
+
 /* ---------------- CMap resources ---------------- */
 
 /*
@@ -1007,3 +1077,4 @@ pdf_cmap_alloc(gx_device_pdf *pdev, const gs_cmap_t *pcmap,
 {
     return pdf_write_cmap(pdev, pcmap, ppres, font_index_only);
 }
+
