@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdti.c 8265 2007-10-02 07:31:58Z ken $ */
+/* $Id: gdevpdti.c 8611 2008-03-27 08:37:58Z ken $ */
 /* Bitmap font implementation for pdfwrite */
 #include "memory_.h"
 #include "string_.h"
@@ -347,6 +347,8 @@ pdf_end_char_proc(gx_device_pdf * pdev, pdf_stream_position_t * ppos)
     sseek(s, start_pos - 15);
     pprintd1(s, "%d", length);
     sseek(s, end_pos);
+    if (pdev->PDFA)
+	stream_puts(s, "\n");
     stream_puts(s, "endstream\n");
     pdf_end_separate(pdev);
     return 0;
@@ -614,6 +616,9 @@ pdf_enter_substream(gx_device_pdf *pdev, pdf_resource_type_t rtype,
     pdev->context = PDF_IN_STREAM;
     pdev->accumulating_substream_resource = pres;
     pdev->last_charpath_op = 0;
+    /* Do not alter type3charpath, inherit the current value. We need to know if */
+    /* we are inside a charpath operation, and only reset this when the charpath */
+    /* is complete */
     pdf_reset_graphics(pdev);
     *ppres = pres;
     return 0;
