@@ -1,20 +1,17 @@
-/* Copyright (C) 1990, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevprn.c,v 1.19 2005/03/14 18:08:36 dan Exp $ */
+/* $Id: gdevprn.c 8887 2008-07-28 07:52:42Z leonardo $ */
 /* Generic printer driver support */
 #include "ctype_.h"
 #include "gdevprn.h"
@@ -31,14 +28,14 @@
 
 /* GC information */
 #define PRINTER_IS_CLIST(pdev) ((pdev)->buffer_space != 0)
-private
+static
 ENUM_PTRS_WITH(device_printer_enum_ptrs, gx_device_printer *pdev)
     if (PRINTER_IS_CLIST(pdev))
 	ENUM_PREFIX(st_device_clist, 0);
     else
 	ENUM_PREFIX(st_device_forward, 0);
 ENUM_PTRS_END
-private
+static
 RELOC_PTRS_WITH(device_printer_reloc_ptrs, gx_device_printer *pdev)
 {
     if (PRINTER_IS_CLIST(pdev))
@@ -57,7 +54,10 @@ const gx_device_procs prn_std_procs =
 /* Forward references */
 int gdev_prn_maybe_realloc_memory(gx_device_printer *pdev, 
 				  gdev_prn_space_params *old_space,
-				  int old_width, int old_height);
+			          int old_width, int old_height,
+			          bool old_page_uses_transparency);
+
+extern dev_proc_open_device(pattern_clist_open_device);
 
 /* ------ Open/close ------ */
 
@@ -94,7 +94,7 @@ gdev_prn_close(gx_device * pdev)
     return code;
 }
 
-private int		/* returns 0 ok, else -ve error cde */
+static int		/* returns 0 ok, else -ve error cde */
 gdev_prn_setup_as_command_list(gx_device *pdev, gs_memory_t *buffer_memory,
 			       byte **the_memory,
 			       const gdev_prn_space_params *space_params,
@@ -139,6 +139,7 @@ BACKTRACE(pdev);
 open_c:
     ppdev->buf = base;
     ppdev->buffer_space = space;
+    clist_init_io_procs(pclist_dev, false);
     clist_init_params(pclist_dev, base, space, pdev,
 		      ppdev->printer_procs.buf_procs,
 		      space_params->band, ppdev->is_async_renderer,
@@ -183,7 +184,7 @@ open_c:
     return code;
 }
 
-private bool	/* ret true if device was cmd list, else false */
+static bool	/* ret true if device was cmd list, else false */
 gdev_prn_tear_down(gx_device *pdev, byte **the_memory)
 {
     gx_device_printer * const ppdev = (gx_device_printer *)pdev;
@@ -199,6 +200,13 @@ gdev_prn_tear_down(gx_device *pdev, byte **the_memory)
 	ppdev->buf = 0;
 	ppdev->buffer_space = 0;
 	is_command_list = true;
+	
+	/* If the clist is a reader clist, free any band_complexity_array
+	 * memory used by same.
+	 */
+	if (!CLIST_IS_WRITER(pclist_dev))
+	    gx_clist_reader_free_band_complexity_array(pclist_dev);
+	
     } else {
 	/* point at the device bitmap, no need to close mem dev */
 	*the_memory = pmemdev->base;
@@ -214,7 +222,7 @@ gdev_prn_tear_down(gx_device *pdev, byte **the_memory)
     return is_command_list;
 }
 
-private int
+static int
 gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 		  int new_width, int new_height, bool reallocate)
 {
@@ -226,6 +234,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
     int save_height = 0x0badf00d; /* Quiet compiler */
     bool is_command_list = false; /* Quiet compiler */
     bool save_is_command_list = false; /* Quiet compiler */
+    bool size_ok = 0;
     int ecode = 0;
     int pass;
     gs_memory_t *buffer_memory =
@@ -268,13 +277,18 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 
 	/* Init clist/mem device-specific fields */
 	memset(ppdev->skip, 0, sizeof(ppdev->skip));
-	ppdev->printer_procs.buf_procs.size_buf_device
-	    (&buf_space, pdev, NULL, pdev->height, false);
-	if (ppdev->page_uses_transparency)
-	    pdf14_trans_buffer_size = new_height
-		* (ESTIMATED_PDF14_ROW_SPACE(new_width) >> 3);
-	mem_space = buf_space.bits + buf_space.line_ptrs
-		    + pdf14_trans_buffer_size;
+	size_ok = ppdev->printer_procs.buf_procs.size_buf_device
+	    (&buf_space, pdev, NULL, pdev->height, false) >= 0;
+	mem_space = buf_space.bits + buf_space.line_ptrs;
+	if (ppdev->page_uses_transparency) {
+	    pdf14_trans_buffer_size = (ESTIMATED_PDF14_ROW_SPACE(max(1, new_width)) >> 3);
+	    if (new_height < (max_ulong - mem_space) / pdf14_trans_buffer_size) {
+		pdf14_trans_buffer_size *= new_height;
+		mem_space += pdf14_trans_buffer_size;
+	    } else {
+		size_ok = 0;
+	    }
+	}
 
 	/* Compute desired space params: never use the space_params as-is. */
 	/* Rather, give the dev-specific driver a chance to adjust them. */
@@ -298,7 +312,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 	else {
 	    is_command_list = space_params.banding_type == BandingAlways ||
 		mem_space >= space_params.MaxBitmap ||
-		mem_space != (uint)mem_space;	    /* too big to allocate */
+		!size_ok;	    /* too big to allocate */
 	}
 	if (!is_command_list) {
 	    /* Try to allocate memory for full memory buffer */
@@ -345,6 +359,11 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 
 	    if ( code >= 0 || (reallocate && pass > 1) )
 		ppdev->procs = gs_clist_device_procs;
+	    /* 
+	     * Now the device is a clist device, we enable multi-threaded rendering.
+	     * It will remain enabled, but that doesn't really cause any problems.
+	     */
+	    clist_enable_multi_thread_render(pdev);
 	} else {
 	    /* Render entirely in memory. */
 	    gx_device *bdev = (gx_device *)pmemdev;
@@ -353,7 +372,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 	    ppdev->buffer_space = 0;
 	    if ((code = gdev_create_buf_device
 		 (ppdev->printer_procs.buf_procs.create_buf_device,
-		  &bdev, pdev, NULL, NULL, false)) < 0 ||
+		  &bdev, pdev, 0, NULL, NULL, NULL)) < 0 ||
 		(code = ppdev->printer_procs.buf_procs.setup_buf_device
 		 (bdev, base, buf_space.raster,
 		  (byte **)(base + buf_space.bits), 0, pdev->height,
@@ -397,12 +416,8 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
 	COPY_PROC(get_color_comp_index);
 	COPY_PROC(encode_color);
 	COPY_PROC(decode_color);
-	COPY_PROC(begin_image);
-	COPY_PROC(text_begin);
-	COPY_PROC(fill_path);
-	COPY_PROC(stroke_path);
-	COPY_PROC(fill_rectangle_hl_color);
 	COPY_PROC(update_spot_equivalent_colors);
+	COPY_PROC(ret_devn_params);
 #undef COPY_PROC
 	/* If using a command list, already opened the device. */
 	if (is_command_list)
@@ -488,6 +503,7 @@ gdev_prn_get_params(gx_device * pdev, gs_param_list * plist)
 	(code = param_write_int(plist, "BandWidth", &ppdev->space_params.band.BandWidth)) < 0 ||
 	(code = param_write_int(plist, "BandHeight", &ppdev->space_params.band.BandHeight)) < 0 ||
 	(code = param_write_long(plist, "BandBufferSpace", &ppdev->space_params.band.BandBufferSpace)) < 0 ||
+	(code = param_write_int(plist, "NumRenderingThreads", &ppdev->num_render_threads_requested)) < 0 ||
 	(code = param_write_bool(plist, "OpenOutputFile", &ppdev->OpenOutputFile)) < 0 ||
 	(code = param_write_bool(plist, "ReopenPerPage", &ppdev->ReopenPerPage)) < 0 ||
 	(code = param_write_bool(plist, "PageUsesTransparency",
@@ -506,7 +522,7 @@ gdev_prn_get_params(gx_device * pdev, gs_param_list * plist)
 }
 
 /* Validate an OutputFile name by checking any %-formats. */
-private int
+static int
 validate_output_file(const gs_param_string * ofs)
 {
     gs_parsed_file_name_t parsed;
@@ -528,10 +544,12 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
     bool oof = ppdev->OpenOutputFile;
     bool rpp = ppdev->ReopenPerPage;
     bool page_uses_transparency = ppdev->page_uses_transparency;
+    bool old_page_uses_transparency = ppdev->page_uses_transparency;
     bool duplex;
     int duplex_set = -1;
     int width = pdev->width;
     int height = pdev->height;
+    int nthreads = ppdev->num_render_threads_requested;
     gdev_prn_space_params sp, save_sp;
     gs_param_string ofs;
     gs_param_dict mdict;
@@ -655,6 +673,16 @@ label:\
     read_media("InputAttributes");
     read_media("OutputAttributes");
 
+    switch (code = param_read_int(plist, (param_name = "NumRenderingThreads"), &nthreads)) {
+	case 0:
+	    break;
+	default:
+	    ecode = code;
+	    param_signal_error(plist, param_name, ecode);
+	case 1:
+	    ;
+    }
+
     if (ecode < 0)
 	return ecode;
     /* Prevent gx_default_put_params from closing the printer. */
@@ -672,11 +700,13 @@ label:\
 	ppdev->Duplex_set = duplex_set;
     }
     ppdev->space_params = sp;
+    ppdev->num_render_threads_requested = nthreads;
 
     /* If necessary, free and reallocate the printer memory. */
     /* Formerly, would not reallocate if device is not open: */
     /* we had to patch this out (see News for 5.50). */
-    code = gdev_prn_maybe_realloc_memory(ppdev, &save_sp, width, height);
+    code = gdev_prn_maybe_realloc_memory(ppdev, &save_sp, width, height,
+		    				old_page_uses_transparency);
     if (code < 0)
 	return code;
 
@@ -965,11 +995,11 @@ gdev_prn_colors_used(gx_device *dev, int y, int height,
  */
 int
 gdev_create_buf_device(create_buf_device_proc_t cbd_proc, gx_device **pbdev,
-		       gx_device *target,
+		       gx_device *target, int y,
 		       const gx_render_plane_t *render_plane,
-		       gs_memory_t *mem, bool for_band)
+		       gs_memory_t *mem, gx_band_complexity_t *band_complexity)
 {
-    int code = cbd_proc(pbdev, target, render_plane, mem, for_band);
+    int code = cbd_proc(pbdev, target, y, render_plane, mem, band_complexity);
 
     if (code < 0)
 	return code;
@@ -983,8 +1013,8 @@ gdev_create_buf_device(create_buf_device_proc_t cbd_proc, gx_device **pbdev,
  * possibly preceded by a plane extraction device.
  */
 int
-gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
-    const gx_render_plane_t *render_plane, gs_memory_t *mem, bool for_band)
+gx_default_create_buf_device(gx_device **pbdev, gx_device *target, int y,
+    const gx_render_plane_t *render_plane, gs_memory_t *mem, gx_band_complexity_t *band_complexity)
 {
     int plane_index = (render_plane ? render_plane->index : -1);
     int depth;
@@ -1013,9 +1043,10 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
         check_device_separable((gx_device *)mdev);
 	gx_device_fill_in_procs((gx_device *)mdev);
     } else
-	gs_make_mem_device(mdev, mdproto, mem, (for_band ? 1 : 0),
+	gs_make_mem_device(mdev, mdproto, mem, (band_complexity == NULL ? 1 : 0),
 			   (target == (gx_device *)mdev ? NULL : target));
     mdev->width = target->width;
+    mdev->band_y = y;
     /*
      * The matrix in the memory device is irrelevant,
      * because all we do with the device is call the device-level
@@ -1039,7 +1070,8 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
     } else
 	bdev = (gx_device *)mdev;
     /****** QUESTIONABLE, BUT BETTER THAN OMITTING ******/
-    bdev->color_info = target->color_info;
+    if (&bdev->color_info != &target->color_info) /* Pacify Valgrind */
+        bdev->color_info = target->color_info;
     *pbdev = bdev;
     return 0;
 }
@@ -1048,16 +1080,20 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target,
 int
 gx_default_size_buf_device(gx_device_buf_space_t *space, gx_device *target,
 			   const gx_render_plane_t *render_plane,
-			   int height, bool for_band)
+			   int height, bool not_used)
 {
     gx_device_memory mdev;
 
+    space->line_ptrs = 0;	/*				*/
+    space->bits = 0;		/* clear in case of failure	*/
+    space->raster = 0;		/*				*/
     mdev.color_info.depth =
 	(render_plane && render_plane->index >= 0 ? render_plane->depth :
 	 target->color_info.depth);
     mdev.width = target->width;
     mdev.num_planes = 0;
-    space->bits = gdev_mem_bits_size(&mdev, target->width, height);
+    if (gdev_mem_bits_size(&mdev, target->width, height, &(space->bits)) < 0)
+	return_error(gs_error_VMerror);
     space->line_ptrs = gdev_mem_line_ptrs_size(&mdev, target->width, height);
     space->raster = gdev_mem_raster(&mdev);
     return 0;
@@ -1080,6 +1116,13 @@ gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
     if ((gx_device *)mdev == bdev && mdev->num_planes)
 	raster = bitmap_raster(mdev->planes[0].depth * mdev->width);
     if (ptrs == 0) {
+	/* 
+	 * Before allocating a new line pointer array, if there is a previous
+	 * array, free it to prevent leaks.
+	 */
+	if (mdev->line_ptrs != NULL)
+	    gs_free_object(mdev->line_pointer_memory, mdev->line_ptrs,
+		       "mem_close");
 	/*
 	 * Allocate line pointers now; free them when we close the device.
 	 * Note that for multi-planar devices, we have to allocate using
@@ -1238,7 +1281,8 @@ gdev_prn_close_printer(gx_device * pdev)
 int
 gdev_prn_maybe_realloc_memory(gx_device_printer *prdev, 
 			      gdev_prn_space_params *old_sp,
-			      int old_width, int old_height)
+			      int old_width, int old_height,
+			      bool old_page_uses_transparency)
 {
     int code = 0;
     gx_device *const pdev = (gx_device *)prdev;
@@ -1254,7 +1298,8 @@ gdev_prn_maybe_realloc_memory(gx_device_printer *prdev,
      */
     if (prdev->is_open &&
 	(memcmp(&prdev->space_params, old_sp, sizeof(*old_sp)) != 0 ||
-	 prdev->width != old_width || prdev->height != old_height )
+	 prdev->width != old_width || prdev->height != old_height ||
+	 prdev->page_uses_transparency != old_page_uses_transparency)
 	) {
 	int new_width = prdev->width;
 	int new_height = prdev->height;
