@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999, 2000, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevprn.h,v 1.14 2005/03/14 18:08:36 dan Exp $ */
+/* $Id: gdevprn.h 8721 2008-05-09 02:18:14Z ray $ */
 /* Common header file for memory-buffered printers */
 
 #ifndef gdevprn_INCLUDED
@@ -217,7 +214,6 @@ struct gdev_prn_space_params_s {
     long BufferSpace;		/* space to use for buffer */
     gx_band_params_t band;	/* see gxclist.h */
     bool params_are_read_only;	/* true if put_params may not modify this struct */
-    bool page_uses_transparency; /* PDF 1.4 transparency is used on the page */
     gdev_prn_banding_type banding_type;	/* used to force banding or bitmap */
 };
 
@@ -232,7 +228,7 @@ struct gdev_prn_space_params_s {
 		/* ------ Other device parameters ------ */\
 	bool OpenOutputFile;\
 	bool ReopenPerPage;\
-	bool page_uses_transparency;\
+        bool page_uses_transparency; /* PDF 1.4 transparency is used on page */\
 	bool Duplex;\
 	  int Duplex_set;		/* -1 = not supported */\
 		/* ------ End of parameters ------ */\
@@ -250,6 +246,7 @@ struct gdev_prn_space_params_s {
 	gx_device_printer *async_renderer;	/* in async writer, pointer to async renderer */\
 	uint clist_disable_mask;	/* mask of clist options to disable */\
 		/* ---- End async rendering support --- */\
+	int num_render_threads_requested;	/* for multiple band rendering threads */\
 	gx_device_procs orig_procs	/* original (std_)procs */
 
 /* The device descriptor */
@@ -387,6 +384,7 @@ extern const gx_device_procs prn_std_procs;
 	0/*false*/, -1,	/* Duplex[_set] */\
 	0/*false*/, 0, 0, 0, /* file_is_new ... buf */\
 	0, 0, 0, 0, 0/*false*/, 0, 0, /* buffer_memory ... clist_dis'_mask */\
+	0, 		/* num_render_threads_requested */\
 	{ 0 }	/* ... orig_procs */
 #define prn_device_body_rest_(print_page)\
   prn_device_body_rest2_(print_page, gx_default_print_page_copies)
@@ -639,9 +637,9 @@ int gdev_prn_free_memory(gx_device *pdev);
  */
 typedef dev_proc_create_buf_device((*create_buf_device_proc_t));
 int gdev_create_buf_device(create_buf_device_proc_t cbd_proc,
-			   gx_device **pbdev, gx_device *target,
+			   gx_device **pbdev, gx_device *target, int y,
 			   const gx_render_plane_t *render_plane,
-			   gs_memory_t *mem, bool for_band);
+			   gs_memory_t *mem, gx_band_complexity_t *band_complexity);
 
 /* BACKWARD COMPATIBILITY */
 #define dev_print_scan_lines(dev)\
@@ -660,20 +658,20 @@ int gdev_prn_initialize(gx_device *, const char *, dev_proc_print_page((*)));
 void gdev_prn_init_color(gx_device *, int, dev_proc_map_rgb_color((*)), dev_proc_map_color_rgb((*)));
 
 #define prn_device_type(dtname, initproc, pageproc)\
-private dev_proc_print_page(pageproc);\
+static dev_proc_print_page(pageproc);\
 device_type(dtname, st_prn_device, initproc)
 
 #define prn_device_type_mono(dtname, dname, initproc, pageproc)\
-private dev_proc_print_page(pageproc);\
-private int \
+static dev_proc_print_page(pageproc);\
+static int \
 initproc(gx_device *dev)\
 {	return gdev_prn_initialize(dev, dname, pageproc);\
 }\
 device_type(dtname, st_prn_device, initproc)
 
 #define prn_device_type_color(dtname, dname, depth, initproc, pageproc, rcproc, crproc)\
-private dev_proc_print_page(pageproc);\
-private int \
+static dev_proc_print_page(pageproc);\
+static int \
 initproc(gx_device *dev)\
 {	int code = gdev_prn_initialize(dev, dname, pageproc);\
 	gdev_prn_init_color(dev, depth, rcproc, crproc);\
