@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevps.c 8250 2007-09-25 13:31:24Z giles $ */
+/* $Id: gdevps.c 8617 2008-04-01 00:28:17Z alexcher $ */
 /* PostScript-writing driver */
 #include "math_.h"
 #include "memory_.h"
@@ -1622,7 +1622,7 @@ psw_image_plane_data(gx_image_enum_common_t * info,
 	        return_error(gs_error_rangecheck);
 	    psw_put_bits(pdev->image_stream,
                 planes[pi].data,
-		planes[pi].data_x*info->plane_depths[pi] + pie->bits_per_row*j,
+		planes[pi].data_x*info->plane_depths[pi] + planes[pi].raster*j*8,
 		planes[pi].raster,
                 pie->bits_per_row,
 		1 );
