@@ -1,20 +1,17 @@
-/* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpsdi.c,v 1.41 2005/04/04 23:00:24 igor Exp $ */
+/* $Id: gdevpsdi.c 8275 2007-10-09 08:28:49Z ken $ */
 /* Image compression for PostScript and PDF writers */
 #include "stdio_.h"		/* for jpeglib.h */
 #include "jpeglib_.h"		/* for sdct.h */
@@ -24,6 +21,9 @@
 #include "gscspace.h"
 #include "gdevpsdf.h"
 #include "gdevpsds.h"
+#include "gxdevmem.h"
+#include "gxcspace.h"
+#include "gsparamx.h"
 #include "strimpl.h"
 #include "scfx.h"
 #include "sdct.h"
@@ -32,6 +32,12 @@
 #include "spngpx.h"
 #include "srlx.h"
 #include "szlibx.h"
+#ifdef USE_LDF_JB2
+#include "sjbig2_luratech.h"
+#endif
+#ifdef USE_LWF_JP2
+#include "sjpx_luratech.h"
+#endif
 
 /* Define parameter-setting procedures. */
 extern stream_state_proc_put_params(s_CF_put_params, stream_CF_state);
@@ -43,7 +49,7 @@ extern stream_state_proc_put_params(s_CF_put_params, stream_CF_state);
  * At least one of bpc_in and bpc_out is 8; the other is 1, 2, 4, or 8,
  * except if bpc_out is 8, bpc_in may be 12.
  */
-private int
+static int
 pixel_resize(psdf_binary_writer * pbw, int width, int num_components,
 	     int bpc_in, int bpc_out)
 {
@@ -81,10 +87,131 @@ pixel_resize(psdf_binary_writer * pbw, int width, int num_components,
     return 0;
 }
 
+static int
+convert_color(gx_device *pdev, const gs_color_space *pcs, const gs_imager_state * pis, 
+	      gs_client_color *cc, float c[3])
+{
+    int code;
+    gx_device_color dc;
+
+    cs_restrict_color(cc, pcs);
+    code = pcs->type->remap_color(cc, pcs, &dc, pis, pdev, gs_color_select_texture);
+    if (code < 0)
+	return code;
+    c[0] = (float)((int)(dc.colors.pure >> pdev->color_info.comp_shift[0]) & ((1 << pdev->color_info.comp_bits[0]) - 1));
+    c[1] = (float)((int)(dc.colors.pure >> pdev->color_info.comp_shift[1]) & ((1 << pdev->color_info.comp_bits[1]) - 1));
+    c[2] = (float)((int)(dc.colors.pure >> pdev->color_info.comp_shift[2]) & ((1 << pdev->color_info.comp_bits[2]) - 1));
+    return 0;
+}
+
+/* A hewristic choice of DCT compression parameters - see bug 687174. */
+static int 
+choose_DCT_params(gx_device *pdev, const gs_color_space *pcs, 
+		  const gs_imager_state * pis, 
+		  gs_c_param_list *list, gs_c_param_list **param, 
+		  stream_state *st) 
+{   
+    gx_device_memory mdev;
+    gs_client_color cc;
+    int code;
+    float c[4][3];
+    const float MIN_FLOAT = - MAX_FLOAT;
+    const float domination = (float)0.25;
+    const int one = 1, zero = 0;
+
+    if (pcs->type->num_components(pcs) != 3)
+	return 0;
+    if (*param != NULL) {
+	/* Make a copy of the parameter list since we will modify it. */
+	code = param_list_copy((gs_param_list *)list, (gs_param_list *)*param);
+	if (code < 0)
+	    return code;
+    }
+    *param = list;
+
+    /* Create a local memory device for transforming colors to DeviceRGB. */
+    gs_make_mem_device(&mdev, gdev_mem_device_for_bits(24), pdev->memory, 0, NULL);
+    gx_device_retain((gx_device *)&mdev, true);	/* prevent freeing */
+    set_linear_color_bits_mask_shift((gx_device *)&mdev);
+    mdev.color_info.separable_and_linear = GX_CINFO_SEP_LIN;
+
+    /* Check for an RGB-like color space.  
+       To recognize that we make a matrix as it were a linear operator,
+       suppress an ununiformity by subtracting the image of {0,0,0},
+       and then check for giagonal domination.  */
+    cc.paint.values[0] = cc.paint.values[1] = cc.paint.values[2] = MIN_FLOAT;
+    convert_color((gx_device *)&mdev, pcs, pis, &cc, c[3]);
+    cc.paint.values[0] = MAX_FLOAT; cc.paint.values[1] = MIN_FLOAT; cc.paint.values[2] = MIN_FLOAT;
+    convert_color((gx_device *)&mdev, pcs, pis, &cc, c[0]);
+    cc.paint.values[0] = MIN_FLOAT; cc.paint.values[1] = MAX_FLOAT; cc.paint.values[2] = MIN_FLOAT;
+    convert_color((gx_device *)&mdev, pcs, pis, &cc, c[1]);
+    cc.paint.values[0] = MIN_FLOAT; cc.paint.values[1] = MIN_FLOAT; cc.paint.values[2] = MAX_FLOAT;
+    convert_color((gx_device *)&mdev, pcs, pis, &cc, c[2]);
+    c[0][0] -= c[3][0]; c[0][1] -= c[3][1]; c[0][2] -= c[3][2];
+    c[1][0] -= c[3][0]; c[1][1] -= c[3][1]; c[1][2] -= c[3][2];
+    c[2][0] -= c[3][0]; c[2][1] -= c[3][1]; c[2][2] -= c[3][2];
+    c[0][0] = any_abs(c[0][0]); c[0][1] = any_abs(c[0][1]); c[0][2] = any_abs(c[0][2]);
+    c[1][0] = any_abs(c[1][0]); c[1][1] = any_abs(c[1][1]); c[1][2] = any_abs(c[1][2]);
+    c[2][0] = any_abs(c[2][0]); c[2][1] = any_abs(c[2][1]); c[2][2] = any_abs(c[2][2]);
+    if (c[0][0] * domination > c[0][1] && c[0][0] * domination > c[0][2] &&
+	c[1][1] * domination > c[1][0] && c[1][1] * domination > c[1][2] &&
+	c[2][2] * domination > c[2][0] && c[2][2] * domination > c[2][1]) {
+	/* Yes, it looks like an RGB color space. 
+	   Replace ColorTransform with 1. */
+	code = param_write_int((gs_param_list *)list, "ColorTransform", &one);
+	if (code < 0)
+	    return code;
+	goto done;
+    }
+
+    /* Check for a Lab-like color space.
+       Colors {v,0,0} should map to grays. */
+    cc.paint.values[0] = MAX_FLOAT; cc.paint.values[1] = cc.paint.values[2] = 0;
+    convert_color((gx_device *)&mdev, pcs, pis, &cc, c[0]);
+    cc.paint.values[0] /= 2;
+    convert_color((gx_device *)&mdev, pcs, pis, &cc, c[1]);
+    cc.paint.values[0] /= 2;
+    convert_color((gx_device *)&mdev, pcs, pis, &cc, c[2]);
+    c[0][1] -= c[0][0]; c[0][2] -= c[0][0];
+    c[1][1] -= c[1][0]; c[1][2] -= c[1][0];
+    c[2][1] -= c[2][0]; c[2][2] -= c[2][0];
+    c[0][1] = any_abs(c[0][1]); c[0][2] = any_abs(c[0][2]);
+    c[1][1] = any_abs(c[1][1]); c[1][2] = any_abs(c[1][2]);
+    c[2][1] = any_abs(c[2][1]); c[2][2] = any_abs(c[2][2]);
+    if (c[0][0] * domination > c[0][1] && c[0][0] * domination > c[0][2] &&
+	c[1][0] * domination > c[1][1] && c[1][0] * domination > c[1][2] &&
+	c[2][0] * domination > c[2][1] && c[2][0] * domination > c[2][2]) {
+	/* Yes, it looks like an Lab color space. 
+	   Replace ColorTransform with 0. */
+	code = param_write_int((gs_param_list *)list, "ColorTransform", &zero);
+	if (code < 0)
+	    return code;
+    } else {
+	/* Unknown color space type.
+	   Replace /HSamples [1 1 1 1] /VSamples [1 1 1 1] to avoid quality degradation. */
+	gs_param_string a;
+	static const byte v[4] = {1, 1, 1, 1};
+
+	a.data = v;
+	a.size = 4;
+	a.persistent = true;
+	code = param_write_string((gs_param_list *)list, "HSamples", &a);
+	if (code < 0)
+	    return code;
+	code = param_write_string((gs_param_list *)list, "VSamples", &a);
+	if (code < 0)
+	    return code;
+    }
+done:
+    gs_c_param_list_read(list);
+    return 0;
+}
+
 /* Add the appropriate image compression filter, if any. */
-private int
+static int
 setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
-			const gs_pixel_image_t * pim, bool lossless)
+			const gs_pixel_image_t * pim, const gs_imager_state * pis, 
+			bool lossless)
 {
     gx_device_psdf *pdev = pbw->dev;
     gs_memory_t *mem = pdev->v_memory;
@@ -103,6 +230,10 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
     stream_state *st;
     int code;
 
+#   ifdef USE_LWF_JP2
+    if (lossless && template == &s_jpxe_template && !Indexed)
+	lossless_template = &s_jpxe_template;
+#   endif
     if (!pdip->Encode)		/* no compression */
 	return 0;
     if (pdip->AutoFilter) {
@@ -117,7 +248,8 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
 	 */
         if (lossless) {
             orig_template = template = lossless_template;
-        } else {
+        } else if (template == NULL || template == &s_zlibE_template || 
+		   template == &s_LZWE_template) {
             orig_template = template = &s_DCTE_template;
         }
 	dict = pdip->ACSDict;
@@ -125,7 +257,7 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
 	return gs_error_rangecheck; /* Reject the alternative stream. */   
     if (pdev->version < psdf_version_ll3 && template == &s_zlibE_template)
 	orig_template = template = lossless_template;
-    if (dict) /* NB: rather than dereference NULL lets continue on without a dict */
+    if (dict != NULL) /* Some interpreters don't supply filter parameters. */
 	gs_c_param_list_read(dict);	/* ensure param list is in read mode */
     if (template == 0)	/* no compression */
 	return 0;
@@ -185,16 +317,56 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
 	    }
 	}
     } else if (template == &s_DCTE_template) {
-	code = psdf_DCT_filter((dict != 0 && orig_template == template ?
-				(gs_param_list *)dict : NULL),
+	gs_c_param_list list, *param = dict;
+
+	gs_c_param_list_write(&list, mem);
+	code = choose_DCT_params((gx_device *)pbw->dev, pcs, pis, &list, &param, st);
+	if (code < 0) {
+    	    gs_c_param_list_release(&list);
+	    return code;
+	}
+	code = psdf_DCT_filter((gs_param_list *)param,
 			       st, pim->Width, pim->Height, Colors, pbw);
+	gs_c_param_list_release(&list);
 	if (code < 0)
 	    goto fail;
 	/* psdf_DCT_filter already did the psdf_encode_binary. */
 	return 0;
-    } else if (template == &s_LZWE_template) {
-	if (template->set_defaults)
-	    (*template->set_defaults) (st);
+    } else {
+#	ifdef USE_LDF_JB2
+	    if (template == &s_jbig2encode_template) {
+		stream_jbig2encode_state *state = (stream_jbig2encode_state *)st;
+
+		state->width = pim->Width;
+		state->height = pim->Height;
+	    }
+#	endif
+#	ifdef USE_LWF_JP2
+	    if (template == &s_jpxe_template) {
+		stream_jpxe_state *state = (stream_jpxe_state *)st;
+		int ncomps = pim->ColorSpace->type->num_components(pim->ColorSpace);
+
+		/* HACK : We choose a JPX color space from the number of components :
+		   CIEBasedA goes as gs_jpx_cs_gray,
+		   CIEBasedABC and DeviceN(3) go as gs_jpx_cs_rgb,
+		   CIEBasedABCD and DeviceN(4) go as gs_jpx_cs_cmyk.
+		*/
+		switch (ncomps) {
+		    case 1 : state->colorspace = gs_jpx_cs_gray; break;
+		    case 3 : state->colorspace = gs_jpx_cs_rgb; break;
+		    case 4 : state->colorspace = gs_jpx_cs_cmyk; break;
+		    default:
+			return_error(gs_error_unregistered); /* Must not happen. */
+		}
+		state->width = pim->Width;
+		state->height = pim->Height;
+		state->bpc = pim->BitsPerComponent;
+		state->components = ncomps;
+		state->lossless = lossless;
+		/* Other encode parameters are not implemented yet. 
+		   Therefore ACSDict is being ignored. */
+	    }
+#	endif
     }
     code = psdf_encode_binary(pbw, template, st);
     if (code >= 0)
@@ -205,7 +377,7 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
 }
 
 /* Determine whether an image should be downsampled. */
-private bool
+static bool
 do_downsample(const psdf_image_params *pdip, const gs_pixel_image_t *pim,
 	      floatp resolution)
 {
@@ -218,9 +390,10 @@ do_downsample(const psdf_image_params *pdip, const gs_pixel_image_t *pim,
 /* Add downsampling, antialiasing, and compression filters. */
 /* Uses AntiAlias, Depth, DownsampleThreshold, DownsampleType, Resolution. */
 /* Assumes do_downsampling() is true. */
-private int
+static int
 setup_downsampling(psdf_binary_writer * pbw, const psdf_image_params * pdip,
-		   gs_pixel_image_t * pim, floatp resolution, bool lossless)
+		   gs_pixel_image_t * pim, const gs_imager_state * pis, 
+		   floatp resolution, bool lossless)
 {
     gx_device_psdf *pdev = pbw->dev;
     /* Note: Bicubic is currently interpreted as Average. */
@@ -260,7 +433,7 @@ setup_downsampling(psdf_binary_writer * pbw, const psdf_image_params * pdip,
 			(double)pim->Height / orig_height,
 			&pim->ImageMatrix);
 	/****** NO ANTI-ALIASING YET ******/
-	if ((code = setup_image_compression(pbw, pdip, pim, lossless)) < 0 ||
+	if ((code = setup_image_compression(pbw, pdip, pim, pis, lossless)) < 0 ||
 	    (code = pixel_resize(pbw, pim->Width, ss->Colors,
 				 8, pdip->Depth)) < 0 ||
 	    (code = psdf_encode_binary(pbw, template, st)) < 0 ||
@@ -285,12 +458,45 @@ psdf_is_converting_image_to_RGB(const gx_device_psdf * pdev,
 	    gs_color_space_index_DeviceCMYK;
 }
 
+static inline void 
+adjust_auto_filter_strategy(gx_device_psdf *pdev, 
+		psdf_image_params *params, gs_c_param_list *plist, 
+		const gs_pixel_image_t * pim, bool in_line)
+{
+#ifdef USE_LWF_JP2
+    if (!in_line && params->Depth > 1 && pdev->ParamCompatibilityLevel >= 1.5 &&
+	    pim->ColorSpace->type->index != gs_color_space_index_Indexed &&
+	    params->AutoFilter &&
+	    !strcmp(params->AutoFilterStrategy, "/JPEG2000")) {
+	params->Filter = "/JPXEncode";
+	params->filter_template = &s_jpxe_template;
+	params->Dict = plist;
+    }
+#endif
+}
+
+static inline void 
+adjust_auto_filter_strategy_mono(gx_device_psdf *pdev, 
+		psdf_image_params *params, gs_c_param_list *plist, 
+		const gs_pixel_image_t * pim, bool in_line)
+{
+#ifdef USE_LDF_JB2
+    if (!in_line && pdev->ParamCompatibilityLevel >= 1.5 &&
+	    params->AutoFilter &&
+	    pim->ColorSpace->type->index != gs_color_space_index_Indexed) {
+	params->Filter = "/JBIG2Encode";
+	params->filter_template = &s_jbig2encode_template;
+	params->Dict = plist;
+    }
+#endif
+}
+
 /* Set up compression and downsampling filters for an image. */
 /* Note that this may modify the image parameters. */
 int
 psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
 			 gs_pixel_image_t * pim, const gs_matrix * pctm,
-			 const gs_imager_state * pis, bool lossless)
+			 const gs_imager_state * pis, bool lossless, bool in_line)
 {
     /*
      * The following algorithms are per Adobe Tech Note # 5151,
@@ -319,16 +525,23 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
 	ncomp = 1;
     } else {
 	ncomp = gs_color_space_num_components(pim->ColorSpace);
-	if (ncomp == 1) {
-	    if (bpc == 1)
-		params = pdev->params.MonoImage;
-	    else
-		params = pdev->params.GrayImage;
-	    if (params.Depth == -1)
-		params.Depth = bpc;
-	} else {
+	if (pim->ColorSpace->type->index == gs_color_space_index_Indexed) {
 	    params = pdev->params.ColorImage;
+	    /* Ensure we don't use JPEG on a /Indexed colour space */
+	    params.AutoFilter = false;
+	    params.Filter = "FlateEncode";
+	} else {
+	    if (ncomp == 1) {
+		if (bpc == 1)
+		    params = pdev->params.MonoImage;
+    		else
+		    params = pdev->params.GrayImage;
+		if (params.Depth == -1)
+		    params.Depth = bpc;
+	    } else {
+		params = pdev->params.ColorImage;
 	    /* params.Depth is reset below */
+	    }
 	}
     }
 
@@ -350,7 +563,7 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
 	resolution = 1.0 / hypot(pt.x / pdev->HWResolution[0],
 				 pt.y / pdev->HWResolution[1]);
     }
-    if (ncomp == 1) {
+    if (ncomp == 1 && pim->ColorSpace && pim->ColorSpace->type->index != gs_color_space_index_Indexed) {
 	/* Monochrome, gray, or mask */
 	/* Check for downsampling. */
 	if (do_downsample(&params, pim, resolution)) {
@@ -359,14 +572,17 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
 		params.Filter = pdev->params.MonoImage.Filter;
 		params.filter_template = pdev->params.MonoImage.filter_template;
 		params.Dict = pdev->params.MonoImage.Dict;
+		adjust_auto_filter_strategy_mono(pdev, &params, pdev->params.MonoImage.Dict, pim, in_line);
 	    } else {
 		params.Filter = pdev->params.GrayImage.Filter;
 		params.filter_template = pdev->params.GrayImage.filter_template;
 		params.Dict = pdev->params.GrayImage.Dict;
+		adjust_auto_filter_strategy(pdev, &params, pdev->params.GrayImage.Dict, pim, in_line);
 	    }
-	    code = setup_downsampling(pbw, &params, pim, resolution, lossless);
+	    code = setup_downsampling(pbw, &params, pim, pis, resolution, lossless);
 	} else {
-	    code = setup_image_compression(pbw, &params, pim, lossless);
+	    adjust_auto_filter_strategy(pdev, &params, pdev->params.GrayImage.Dict, pim, in_line);
+	    code = setup_image_compression(pbw, &params, pim, pis, lossless);
 	}
 	if (code < 0)
 	    return code;
@@ -376,20 +592,20 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
 	bool cmyk_to_rgb = psdf_is_converting_image_to_RGB(pdev, pis, pim);
 
 	if (cmyk_to_rgb) {
-	    extern_st(st_color_space);
 	    gs_memory_t *mem = pdev->v_memory;
-	    gs_color_space *rgb_cs = gs_alloc_struct(mem, 
-		    gs_color_space, &st_color_space, "psdf_setup_image_filters");
 
-	    gs_cspace_init_DeviceRGB(mem, rgb_cs);  /* idempotent initialization */
-	    pim->ColorSpace = rgb_cs;
+	    /* {csrc} decref old colorspace? */
+	    rc_decrement_only(pim->ColorSpace, "psdf_setup_image_filters");
+	    pim->ColorSpace = gs_cspace_new_DeviceRGB(mem);
 	}
 	if (params.Depth == -1)
 	    params.Depth = (cmyk_to_rgb ? 8 : bpc_out);
 	if (do_downsample(&params, pim, resolution)) {
-	    code = setup_downsampling(pbw, &params, pim, resolution, lossless);
+	    adjust_auto_filter_strategy(pdev, &params, pdev->params.ColorImage.Dict, pim, in_line);
+	    code = setup_downsampling(pbw, &params, pim, pis, resolution, lossless);
 	} else {
-	    code = setup_image_compression(pbw, &params, pim, lossless);
+	    adjust_auto_filter_strategy(pdev, &params, pdev->params.ColorImage.Dict, pim, in_line);
+	    code = setup_image_compression(pbw, &params, pim, pis, lossless);
 	}
 	if (code < 0)
 	    return code;
@@ -420,7 +636,7 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
 /* Note that this may modify the image parameters. */
 int
 psdf_setup_lossless_filters(gx_device_psdf *pdev, psdf_binary_writer *pbw,
-			    gs_pixel_image_t *pim)
+			    gs_pixel_image_t *pim, bool in_line)
 {
     /*
      * Set up a device with modified parameters for computing the image
@@ -438,7 +654,7 @@ psdf_setup_lossless_filters(gx_device_psdf *pdev, psdf_binary_writer *pbw,
     ipdev.params.GrayImage.Downsample = false;
     ipdev.params.GrayImage.Filter = "FlateEncode";
     ipdev.params.GrayImage.filter_template = &s_zlibE_template;
-    return psdf_setup_image_filters(&ipdev, pbw, pim, NULL, NULL, true);
+    return psdf_setup_image_filters(&ipdev, pbw, pim, NULL, NULL, true, in_line);
 }
 
 /* Set up image compression chooser. */
@@ -491,15 +707,11 @@ psdf_setup_image_to_mask_filter(psdf_binary_writer *pbw, gx_device_psdf *pdev,
 int
 psdf_setup_image_colors_filter(psdf_binary_writer *pbw, 
 			       gx_device_psdf *pdev, gs_pixel_image_t * pim, 
-			       const gs_imager_state *pis,
-			       gs_color_space_index output_cspace_index)
+			       const gs_imager_state *pis)
 {   /* fixme: currently it's a stub convertion to mask. */
     int code;
-    extern_st(st_color_space);
-    gs_memory_t *mem = pdev->v_memory;
     stream_state *ss = s_alloc_state(pdev->memory, s__image_colors_template.stype, 
 	"psdf_setup_image_colors_filter");
-    gs_color_space *cs;
     int i;
 
     if (ss == 0)
@@ -509,10 +721,6 @@ psdf_setup_image_colors_filter(psdf_binary_writer *pbw,
     code = psdf_encode_binary(pbw, &s__image_colors_template, ss);
     if (code < 0)
 	return code;
-    cs = gs_alloc_struct(mem, gs_color_space, &st_color_space, 
-			    "psdf_setup_image_colors_filter");
-    if (cs == NULL)
-	return_error(gs_error_VMerror);
     s_image_colors_set_dimensions((stream_image_colors_state *)ss, 
 		    pim->Width, pim->Height, 
 		    gs_color_space_num_components(pim->ColorSpace), 
@@ -524,24 +732,5 @@ psdf_setup_image_colors_filter(psdf_binary_writer *pbw,
 	pim->Decode[i * 2 + 0] = 0;
 	pim->Decode[i * 2 + 1] = 1;
     }
-    switch (output_cspace_index) {
-	case gs_color_space_index_DeviceGray:
-	    gs_cspace_init_DeviceGray(mem, cs);
-	    break;
-	case gs_color_space_index_DeviceRGB:
-	    gs_cspace_init_DeviceRGB(mem, cs); 
-	    break;
-	case gs_color_space_index_DeviceCMYK:
-	    gs_cspace_init_DeviceCMYK(mem, cs); 
-	    break;
-	default:
-	    /* Notify the user and terminate.
-	       Don't emit rangecheck becuause it would fall back
-	       to a default implementation (rasterisation). 
-	     */
-	    eprintf("Unsupported ProcessColorModel");
-	    return_error(gs_error_undefined);
-    }
-    pim->ColorSpace = cs;
     return 0;
 }
