@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpsft.c 6680 2006-03-23 12:39:23Z leonardo $ */
+/* $Id: gdevpsft.c 8622 2008-04-04 08:53:57Z ken $ */
 /* Write an embedded TrueType font */
 #include "memory_.h"
 #include <stdlib.h>		/* for qsort */
@@ -61,14 +61,8 @@
 
 /* ---------------- Utilities ---------------- */
 
-#define ACCESS(base, length, vptr)\
-  BEGIN\
-    code = string_proc(pfont, (ulong)(base), length, &vptr);\
-    if (code < 0) return code;\
-  END
-
 /* Pad to a multiple of 4 bytes. */
-private void
+static void
 put_pad(stream *s, uint length)
 {
     static const byte pad_to_4[3] = {0, 0, 0};
@@ -77,19 +71,19 @@ put_pad(stream *s, uint length)
 }
 
 /* Put short and long values on a stream. */
-private void
+static void
 put_ushort(stream *s, uint v)
 {
     stream_putc(s, (byte)(v >> 8));
     stream_putc(s, (byte)v);
 }
-private void
+static void
 put_ulong(stream *s, ulong v)
 {
     put_ushort(s, (uint)(v >> 16));
     put_ushort(s, (uint)v);
 }
-private void
+static void
 put_loca(stream *s, ulong offset, int indexToLocFormat)
 {
     if (indexToLocFormat)
@@ -104,19 +98,19 @@ put_loca(stream *s, ulong offset, int indexToLocFormat)
 #define U16(p) (((uint)((p)[0]) << 8) + (p)[1])
 #define S16(p) (int)((U16(p) ^ 0x8000) - 0x8000)
 #define u32(p) get_u32_msb(p)
-private void
+static void
 put_u16(byte *p, uint v)
 {
     p[0] = (byte)(v >> 8);
     p[1] = (byte)v;
 }
-private void
+static void
 put_u32(byte *p, ulong v)
 {
     put_u16(p, (ushort)(v >> 16));
     put_u16(p + 2, (ushort)v);
 }
-private ulong
+static ulong
 put_table(byte tab[16], const char *tname, ulong checksum, ulong offset,
 	  uint length)
 {
@@ -128,7 +122,7 @@ put_table(byte tab[16], const char *tname, ulong checksum, ulong offset,
 }
 
 /* Write one range of a TrueType font. */
-private int
+static int
 write_range(stream *s, gs_font_type42 *pfont, ulong start, uint length)
 {
     ulong base = start, size = length;
@@ -155,7 +149,7 @@ write_range(stream *s, gs_font_type42 *pfont, ulong start, uint length)
  * Determine the Macintosh glyph number for a given character, if any.
  * If no glyph can be found, return -1 and store the name in *pstr.
  */
-private int
+static int
 mac_glyph_index(gs_font *font, int ch, gs_const_string *pstr, int *index)
 {
     gs_glyph glyph = font->procs.encode_char(font, (gs_char)ch,
@@ -244,6 +238,27 @@ static const byte cmap_initial_6[] = {
     0, 0,		/* first character code */
     0, 0		/* # of entries *VARIABLE* */
 };
+static const byte cmap_unicode_initial_6[] = {
+    0, 0,		/* table version # = 0 */
+    0, 2,		/* # of encoding tables = 2 */
+
+	/* First table, Macintosh */
+    0, 1,		/* platform ID = Macintosh */
+    0, 0,		/* platform encoding ID = ??? */
+    0, 0, 0, 4+8+8,	/* offset to table start */
+	/* Second table, Windows */
+    0, 3,		/* platform ID = Microsoft */
+    0, 1,		/* platform encoding ID = Unicode */
+    0, 0, 0, 4+8+8+10,	/* offset to table start */
+			/* *VARIABLE*, add 2 x # of entries */
+
+	/* Start of Macintosh format 6 table */
+    0, 6,		/* format = 6, trimmed table mapping */
+    0, 10,		/* length *VARIABLE*, add 2 x # of entries */
+    0, 0,		/* version number */
+    0, 0,		/* first character code */
+    0, 0		/* # of entries *VARIABLE* */
+};
 static const byte cmap_initial_4[] = {
     0, 0,		/* table version # = 0 */
     0, 1,		/* # of encoding tables = 2 */
@@ -277,7 +292,7 @@ static const byte cmap_sub_initial[] = {
  * inconsistently.
  */
 #define CMAP_ENTRIES_SIZE (256 * 2)
-private void
+static void
 write_cmap_0(stream *s, byte* entries /*[CMAP_ENTRIES_SIZE]*/, uint num_glyphs)
 {
     int i;
@@ -287,7 +302,7 @@ write_cmap_0(stream *s, byte* entries /*[CMAP_ENTRIES_SIZE]*/, uint num_glyphs)
     for (i = 0; i <= 0xff; ++i)
 	sputc(s, (byte)entries[2 * i + 1]);
 }
-private void
+static void
 write_cmap_6(stream *s, byte *entries /*[CMAP_ENTRIES_SIZE]*/, uint first_code,
 	     uint first_entry, uint num_entries)
 {
@@ -307,7 +322,22 @@ write_cmap_6(stream *s, byte *entries /*[CMAP_ENTRIES_SIZE]*/, uint first_code,
     stream_write(s, cmap_data, sizeof(cmap_data));
     stream_write(s, entries + first_entry * 2, num_entries * 2);
 }
-private void
+static void write_unicode_cmap_6(stream *s, byte *entries, uint first_code,
+	     uint first_entry, uint num_entries)
+{
+    byte cmap_data[sizeof(cmap_unicode_initial_6)];
+
+    memcpy(cmap_data, cmap_unicode_initial_6, sizeof(cmap_unicode_initial_6));
+    put_u16(cmap_data + 18,
+	    U16(cmap_data + 18) + num_entries * 2);  /* offset */
+    put_u16(cmap_data + 22,
+	    U16(cmap_data + 22) + num_entries * 2);  /* length */
+    put_u16(cmap_data + 26, first_entry);
+    put_u16(cmap_data + 28, num_entries);
+    stream_write(s, cmap_data, sizeof(cmap_data));
+    stream_write(s, entries + first_entry * 2, num_entries * 2);
+}
+static void
 write_cmap(stream *s, gs_font *font, uint first_code, int num_glyphs,
 	   gs_glyph max_glyph, int options, uint cmap_length)
 {
@@ -342,6 +372,19 @@ write_cmap(stream *s, gs_font *font, uint first_code, int num_glyphs,
 
     /* Write the table header and Macintosh sub-table (if any). */
 
+    if (options & WRITE_TRUETYPE_UNICODE_CMAP) {
+	write_unicode_cmap_6(s, entries, first_code, first_entry, num_entries);
+
+	/* Write the Windows sub-table. */
+	memcpy(cmap_sub, cmap_sub_initial, sizeof(cmap_sub_initial));
+	put_u16(cmap_sub + 2, U16(cmap_sub + 2) + num_entries * 2); /* length */
+	put_u16(cmap_sub + 14, end_entry - 1); /* endCount[0] */
+	put_u16(cmap_sub + 20, first_entry); /* startCount[0] */
+	stream_write(s, cmap_sub, sizeof(cmap_sub));
+	stream_write(s, entries + first_entry * 2, num_entries * 2);
+	put_pad(s, cmap_length);
+	return;
+    } 
 #if TT_FORCE_CMAP_6 > 0
     /* Always use format 6. */
     write_cmap_6(s, entries, first_code, first_entry, num_entries);
@@ -382,7 +425,7 @@ write_cmap(stream *s, gs_font *font, uint first_code, int num_glyphs,
     stream_write(s, entries + first_entry * 2, num_entries * 2);
     put_pad(s, cmap_length);
 }
-private uint
+static uint
 size_cmap(gs_font *font, uint first_code, int num_glyphs, gs_glyph max_glyph,
 	  int options)
 {
@@ -396,7 +439,7 @@ size_cmap(gs_font *font, uint first_code, int num_glyphs, gs_glyph max_glyph,
 
 /* ------ hmtx/vmtx ------ */
 
-private void
+static void
 write_mtx(stream *s, gs_font_type42 *pfont, const gs_type42_mtx_t *pmtx,
 	  int wmode)
 {
@@ -419,7 +462,7 @@ write_mtx(stream *s, gs_font_type42 *pfont, const gs_type42_mtx_t *pmtx,
 }
 
 /* Compute the metrics from the glyph_info. */
-private uint
+static uint
 size_mtx(gs_font_type42 *pfont, gs_type42_mtx_t *pmtx, uint max_glyph,
 	 int wmode)
 {
@@ -459,12 +502,12 @@ static const byte name_initial[] = {
     0, 0,			/* length *VARIABLE* */
     0, 0			/* start of string within string storage */
 };
-private uint
+static uint
 size_name(const gs_const_string *font_name)
 {
     return sizeof(name_initial) + font_name->size;
 }
-private void
+static void
 write_name(stream *s, const gs_const_string *font_name)
 {
     byte name_bytes[sizeof(name_initial)];
@@ -479,8 +522,9 @@ write_name(stream *s, const gs_const_string *font_name)
 /* ------ OS/2 ------ */
 
 /* Write a generated OS/2 table. */
-#define OS_2_LENGTH sizeof(ttf_OS_2_t)
-private void
+#define OS_2_LENGTH1 offset_of(ttf_OS_2_t, sxHeight[0]) /* OS/2 version 1. */
+#define OS_2_LENGTH2 sizeof(ttf_OS_2_t) /* OS/2 version 2. */
+static void
 update_OS_2(ttf_OS_2_t *pos2, uint first_glyph, int num_glyphs)
 {
     put_u16(pos2->usFirstCharIndex, first_glyph);
@@ -495,24 +539,38 @@ update_OS_2(ttf_OS_2_t *pos2, uint first_glyph, int num_glyphs)
     }
 #endif
 }
-private void
+static void
 write_OS_2(stream *s, gs_font *font, uint first_glyph, int num_glyphs)
 {
     ttf_OS_2_t os2;
+    gs_font_info_t info;
+    int code;
 
     /*
      * We don't bother to set most of the fields.  The really important
      * ones, which affect character mapping, are usFirst/LastCharIndex.
      * We also need to set usWeightClass and usWidthClass to avoid
-     * crashing ttfdump.
+     * crashing ttfdump. Version 1 86-byte structure has all the fields
+     * we need.
      */
     memset(&os2, 0, sizeof(os2));
     put_u16(os2.version, 1);
     put_u16(os2.usWeightClass, 400); /* Normal */
     put_u16(os2.usWidthClass, 5); /* Normal */
     update_OS_2(&os2, first_glyph, num_glyphs);
-    stream_write(s, &os2, sizeof(os2));
-    put_pad(s, sizeof(os2));
+
+    /*
+     * We should also preserve the licensed embedding rights, to prevent
+     * 'laundering' a TrueType font. These can be non-zero even when embedding is permitted.
+     */
+    memset(&info, 0x00, sizeof(gs_font_info_t));
+    code = font->procs.font_info(font, NULL, FONT_INFO_EMBEDDING_RIGHTS, &info);
+    if (code == 0 && (info.members & FONT_INFO_EMBEDDING_RIGHTS)) {
+	put_u16(os2.fsType, info.EmbeddingRights);
+    }
+
+    stream_write(s, &os2, offset_of(ttf_OS_2_t, sxHeight[0]));
+    put_pad(s, offset_of(ttf_OS_2_t, sxHeight[0]));
 }
 
 /* ------ post ------ */
@@ -523,7 +581,7 @@ typedef struct post_glyph_s {
     byte size;
     ushort glyph_index;
 } post_glyph_t;
-private int
+static int
 compare_post_glyphs(const void *pg1, const void *pg2)
 {
     gs_glyph g1 = ((const post_glyph_t *)pg1)->glyph_index,
@@ -541,7 +599,7 @@ typedef struct post_s {
  * If necessary, compute the length of the post table.  Note that we
  * only generate post entries for characters in the Encoding.
  */
-private int
+static int
 compute_post(gs_font *font, post_t *post)
 {
     int i;
@@ -586,7 +644,7 @@ compute_post(gs_font *font, post_t *post)
 }
 
 /* Write the post table */
-private int
+static int
 write_post(stream *s, gs_font *font, post_t *post)
 {
     byte post_initial[32 + 2];
@@ -642,26 +700,23 @@ write_post(stream *s, gs_font *font, post_t *post)
 /* ---------------- Main program ---------------- */
 
 /* Write the definition of a TrueType font. */
-private int
+static int
 compare_table_tags(const void *pt1, const void *pt2)
 {
     ulong t1 = u32(pt1), t2 = u32(pt2);
 
     return (t1 < t2 ? -1 : t1 > t2 ? 1 : 0);
 }
-private int
+static int
 psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 			psf_glyph_enum_t *penum, bool is_subset,
 			const gs_const_string *alt_font_name)
 {
     gs_font *const font = (gs_font *)pfont;
     gs_const_string font_name;
-    int (*string_proc)(gs_font_type42 *, ulong, uint, const byte **) =
-	pfont->data.string_proc;
-    const byte *OffsetTable;
+    byte OffsetTable[12];
     uint numTables_stored, numTables, numTables_out;
-#define MAX_NUM_TABLES 40
-    byte tables[MAX_NUM_TABLES * 16];
+    byte tables[MAX_NUM_TT_TABLES * 16];
     uint i;
     ulong offset;
     gs_glyph glyph, glyph_prev;
@@ -688,7 +743,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
     int have_hvhea[2];
     uint cmap_length = 0;
     ulong OS_2_start = 0;
-    uint OS_2_length = OS_2_LENGTH;
+    uint OS_2_length = OS_2_LENGTH1;
     int code;
 
     have_hvhea[0] = have_hvhea[1] = 0;
@@ -704,20 +759,20 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
      * table directory.
      */
 
-    ACCESS(0, 12, OffsetTable);
+    READ_SFNTS(pfont, 0, 12, OffsetTable);
     numTables_stored = U16(OffsetTable + 4);
     for (i = numTables = 0; i < numTables_stored; ++i) {
-	const byte *tab;
-	const byte *data;
+	byte tab[16];
+	byte data[54];
 	ulong start;
 	uint length;
 
-	if (numTables == MAX_NUM_TABLES)
+	if (numTables == MAX_NUM_TT_TABLES)
 	    return_error(gs_error_limitcheck);
-	ACCESS(12 + i * 16, 16, tab);
+	READ_SFNTS(pfont, 12 + i * 16, 16, tab);
 	start = u32(tab + 8);
 	length = u32(tab + 12);
-	/* Copy the table data now, since another ACCESS may invalidate it. */
+	/* Copy the table data now (a rudiment of old code). */
 	memcpy(&tables[numTables * 16], tab, 16);
 
 #define W(a,b,c,d)\
@@ -728,7 +783,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 	    if (length < 54)
 		return_error(gs_error_invalidfont);
 	    length = 54; /* bug 688409 fig2.eps has length=56. */
-	    ACCESS(start, length, data);
+	    READ_SFNTS(pfont, start, length, data);
 	    memcpy(head, data, length);
 	    continue;
 	case W('g','l','y','f'): /* synthesized */
@@ -743,7 +798,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 	    have_cmap = true;
 	    break;
 	case W('m','a','x','p'):
-	    ACCESS(start, length, data);
+	    READ_SFNTS(pfont, start, length, data);
 	    numGlyphs = U16(data + 4);
 	    break;
 	case W('n','a','m','e'):
@@ -755,7 +810,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 	    if (writing_cid)
 		continue;
 	    have_OS_2 = true;
-	    if (length > OS_2_LENGTH)
+	    if (length > OS_2_LENGTH2)
 		return_error(gs_error_invalidfont);
 	    OS_2_start = start;
 	    OS_2_length = length;
@@ -877,7 +932,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 	+ generate_mtx * (have_hvhea[0] + have_hvhea[1]) /* hmtx, vmtx */
 	+ !have_OS_2		/* OS/2 */
 	+ !have_cmap + !have_name + !have_post;
-    if (numTables_out >= MAX_NUM_TABLES)
+    if (numTables_out >= MAX_NUM_TT_TABLES)
 	return_error(gs_error_limitcheck);
     offset = 12 + numTables_out * 16;
     for (i = 0; i < numTables; ++i) {
@@ -999,10 +1054,10 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 		     * Adjust the first and last character indices in the OS/2
 		     * table to reflect the values in the generated cmap.
 		     */
-		    const byte *pos2;
+		    byte pos2[OS_2_LENGTH2];
 		    ttf_OS_2_t os2;
 
-		    ACCESS(OS_2_start, OS_2_length, pos2);
+		    READ_SFNTS(pfont, OS_2_start, OS_2_length, pos2);
 		    memcpy(&os2, pos2, min(OS_2_length, sizeof(os2)));
 		    update_OS_2(&os2, TT_BIAS, 256);
 		    stream_write(s, &os2, OS_2_length);
