@@ -1,47 +1,46 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: gdevpsu.c,v $ $Revision: 1.2.2.4 $ */
+/* $Id: gdevpsu.c 8250 2007-09-25 13:31:24Z giles $ */
 /* PostScript-writing utilities */
 #include "math_.h"
 #include "time_.h"
+#include "stat_.h"
+#include "unistd_.h"
 #include "gx.h"
 #include "gscdefs.h"
 #include "gxdevice.h"
 #include "gdevpsu.h"
 #include "spprint.h"
 #include "stream.h"
+#include "gserrors.h"
 
 /* ---------------- Low level ---------------- */
 
 /* Write a 0-terminated array of strings as lines. */
-void
+int
 psw_print_lines(FILE *f, const char *const lines[])
 {
     int i;
-
-    for (i = 0; lines[i] != 0; ++i)
-	fprintf(f, "%s\n", lines[i]);
+    for (i = 0; lines[i] != 0; ++i) {
+	if (fprintf(f, "%s\n", lines[i]) < 0)
+            return_error(gs_error_ioerror);
+    }
+    return 0;
 }
 
 /* Write the ProcSet name. */
-private void
+static void
 psw_put_procset_name(stream *s, const gx_device *dev,
 		     const gx_device_pswrite_common_t *pdpc)
 {
@@ -51,20 +50,21 @@ psw_put_procset_name(stream *s, const gx_device *dev,
 	    (int)(pdpc->LanguageLevel * 10 + 0.5) % 10,
 	    pdpc->ProcSet_version);
 }
-private void
+static void
 psw_print_procset_name(FILE *f, const gx_device *dev,
 		       const gx_device_pswrite_common_t *pdpc)
 {
     byte buf[100];		/* arbitrary */
     stream s;
 
+    s_init(&s, dev->memory);
     swrite_file(&s, f, buf, sizeof(buf));
     psw_put_procset_name(&s, dev, pdpc);
     sflush(&s);
 }
 
 /* Write a bounding box. */
-private void
+static void
 psw_print_bbox(FILE *f, const gs_rect *pbbox)
 {
     fprintf(f, "%%%%BoundingBox: %d %d %d %d\n",
@@ -76,18 +76,18 @@ psw_print_bbox(FILE *f, const gs_rect *pbbox)
 
 /* ---------------- File level ---------------- */
 
-private const char *const psw_ps_header[] = {
+static const char *const psw_ps_header[] = {
     "%!PS-Adobe-3.0",
     "%%Pages: (atend)",
     0
 };
 
-private const char *const psw_eps_header[] = {
+static const char *const psw_eps_header[] = {
     "%!PS-Adobe-3.0 EPSF-3.0",
     0
 };
 
-private const char *const psw_begin_prolog[] = {
+static const char *const psw_begin_prolog[] = {
     "%%EndComments",
     "%%BeginProlog",
     "% This copyright applies to everything between here and the %%EndProlog:",
@@ -102,7 +102,7 @@ private const char *const psw_begin_prolog[] = {
  * parameters. The following code checks the current page size and sets
  * it only if it is necessary.
  */
-private const char *const psw_ps_procset[] = {
+static const char *const psw_ps_procset[] = {
 	/* <w> <h> <sizename> setpagesize - */
    "/PageSize 2 array def"
    "/setpagesize"              /* x y /a4 -> -          */
@@ -114,7 +114,7 @@ private const char *const psw_ps_procset[] = {
          "{ PageSize dup  1",  /* x y /a4 [  ] [  ] 1   */
            "5 -1 roll put 0 "  /* x /a4 [ y] 0          */
            "4 -1 roll put "    /* /a4                   */
-           "dup where"        
+           "dup null eq {false} {dup where} ifelse"
              "{ exch get exec" /* -                     */
              "}",
              "{ pop"           /* -                     */
@@ -136,19 +136,32 @@ private const char *const psw_ps_procset[] = {
     0
 };
 
-private const char *const psw_end_prolog[] = {
-    "end readonly def",
+static const char *const psw_end_prolog[] = {
+    "end def",
     "%%EndResource",		/* ProcSet */
     "/pagesave null def",	/* establish binding */
     "%%EndProlog",
     0
 };
 
+/* Return true when the file is seekable.
+ * On Windows NT ftell() returns some non-EOF value when used on pipes.
+ */
+static bool
+is_seekable(FILE *f)
+{ 
+    struct stat buf;
+
+    if(fstat(fileno(f), &buf))
+      return 0;
+    return S_ISREG(buf.st_mode);
+}
+
 /*
  * Write the file header, up through the BeginProlog.  This must write to a
  * file, not a stream, because it may be called during finalization.
  */
-void
+int
 psw_begin_file_header(FILE *f, const gx_device *dev, const gs_rect *pbbox,
 		      gx_device_pswrite_common_t *pdpc, bool ascii)
 {
@@ -156,7 +169,7 @@ psw_begin_file_header(FILE *f, const gx_device *dev, const gs_rect *pbbox,
     if (pbbox) {
 	psw_print_bbox(f, pbbox);
 	pdpc->bbox_position = 0;
-    } else if (ftell(f) < 0) {	/* File is not seekable. */
+    } else if (!is_seekable(f)) {	/* File is not seekable. */
 	pdpc->bbox_position = -1;
 	fputs("%%BoundingBox: (atend)\n", f);
 	fputs("%%HiResBoundingBox: (atend)\n", f);
@@ -186,44 +199,59 @@ psw_begin_file_header(FILE *f, const gx_device *dev, const gs_rect *pbbox,
     psw_print_lines(f, psw_begin_prolog);
     fprintf(f, "%% %s\n", gs_copyright);
     fputs("%%BeginResource: procset ", f);
+    fflush(f);
     psw_print_procset_name(f, dev, pdpc);
-    fputs("\n/", f);
+    fprintf(f, " %5.3lf %d\n/", pdpc->ProcSet_version / 1000.0, 0);
+    fflush(f);
     psw_print_procset_name(f, dev, pdpc);
     fputs(" 80 dict dup begin\n", f);
     psw_print_lines(f, psw_ps_procset);
+    fflush(f);
+    if (ferror(f))
+        return_error(gs_error_ioerror);
+    return 0;
 }
 
 /*
  * End the file header.
  */
-void
+int
 psw_end_file_header(FILE *f)
 {
-    psw_print_lines(f, psw_end_prolog);
+    return psw_print_lines(f, psw_end_prolog);
 }
 
 /*
  * End the file.
  */
-void
+int
 psw_end_file(FILE *f, const gx_device *dev,
-             const gx_device_pswrite_common_t *pdpc, const gs_rect *pbbox,
-             int page_count)
+	     const gx_device_pswrite_common_t *pdpc, const gs_rect *pbbox,
+             int /* should be long */ page_count)
 {
-    fprintf(f, "%%%%Trailer\n%%%%Pages: %ld\n", page_count);
+    if (f == NULL)
+        return 0;	/* clients should be more careful */
+    fprintf(f, "%%%%Trailer\n%%%%Pages: %ld\n", (long)page_count);
+    if (ferror(f))
+        return_error(gs_error_ioerror);
     if (dev->PageCount > 0 && pdpc->bbox_position != 0) {
 	if (pdpc->bbox_position >= 0) {
 	    long save_pos = ftell(f);
 
 	    fseek(f, pdpc->bbox_position, SEEK_SET);
 	    psw_print_bbox(f, pbbox);
-	    fputc('%', f);
-	    fseek(f, save_pos, SEEK_SET);
+            fputc('%', f);
+            if (ferror(f))
+                return_error(gs_error_ioerror);
+            fseek(f, save_pos, SEEK_SET);
 	} else
 	    psw_print_bbox(f, pbbox);
     }
     if (!pdpc->ProduceEPS)
 	fputs("%%EOF\n", f);
+    if (ferror(f))
+        return_error(gs_error_ioerror);
+    return 0;
 }
 
 /* ---------------- Page level ---------------- */
@@ -231,10 +259,10 @@ psw_end_file(FILE *f, const gx_device *dev,
 /*
  * Write the page header.
  */
-void
+int
 psw_write_page_header(stream *s, const gx_device *dev,
-		      const gx_device_pswrite_common_t *pdpc,
-                      bool do_scale, long page_ord)
+                      const gx_device_pswrite_common_t *pdpc,
+                      bool do_scale, long page_ord, int dictsize)
 {
     long page = dev->PageCount + 1;
 
@@ -275,22 +303,29 @@ psw_write_page_header(stream *s, const gx_device *dev,
 	pprintd2(s, "%d %d ", width, height);
 	pprints1(s, "%s setpagesize\n", p->size_name);
     }
-    stream_puts(s, "/pagesave save store 100 dict begin\n");
+    pprintd1(s, "/pagesave save store %d dict begin\n", dictsize);
     if (do_scale)
 	pprintg2(s, "%g %g scale\n",
 		 72.0 / dev->HWResolution[0], 72.0 / dev->HWResolution[1]);
     stream_puts(s, "%%EndPageSetup\ngsave mark\n");
+    if (s->end_status == ERRC)
+        return_error(gs_error_ioerror);
+    return 0;
 }
 
 /*
  * Write the page trailer.  We do this directly to the file, rather than to
  * the stream, because we may have to do it during finalization.
  */
-void
+int
 psw_write_page_trailer(FILE *f, int num_copies, int flush)
 {
+    fprintf(f, "cleartomark end end pagesave restore\n");
     if (num_copies != 1)
 	fprintf(f, "userdict /#copies %d put\n", num_copies);
-    fprintf(f, "cleartomark end end pagesave restore %s\n%%%%PageTrailer\n",
-	    (flush ? "showpage" : "copypage"));
+    fprintf(f, " %s\n%%%%PageTrailer\n", (flush ? "showpage" : "copypage"));
+    fflush(f);
+    if (ferror(f))
+        return_error(gs_error_ioerror);
+    return 0;
 }
