@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpxut.h 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gdevpxut.h 8026 2007-06-06 22:03:27Z till $ */
 /* Utilities for PCL XL generation */
 /* Requires gdevpxat.h, gdevpxen.h, gdevpxop.h */
 
@@ -29,7 +29,8 @@ int px_write_page_header(stream *s, const gx_device *dev);
 /* Write the media selection command if needed, updating the media size. */
 int px_write_select_media(stream *s, const gx_device *dev,
 			  pxeMediaSize_t *pms,
-			  byte *media_source);
+			  byte *media_source,
+			  int page, bool Duplex, bool Tumble);
 
 /*
  * Write the file trailer.  Note that this takes a FILE *, not a stream *,
