@@ -1,20 +1,17 @@
-/* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevrops.c,v 1.12 2004/06/24 05:03:36 dan Exp $ */
+/* $Id: gdevrops.c 8838 2008-07-13 22:06:19Z henrys $ */
 /* RasterOp source device */
 #include "gx.h"
 #include "gserrors.h"
@@ -24,7 +21,7 @@
 
 /* GC procedures */
 private_st_device_rop_texture();
-private ENUM_PTRS_BEGIN(device_rop_texture_enum_ptrs) {
+static ENUM_PTRS_BEGIN(device_rop_texture_enum_ptrs) {
     if (index < st_device_color_max_ptrs) {
 	gs_ptr_type_t ptype =
 	    ENUM_SUPER_ELT(gx_device_rop_texture, st_device_color, texture, 0);
@@ -35,18 +32,18 @@ private ENUM_PTRS_BEGIN(device_rop_texture_enum_ptrs) {
     }
     ENUM_PREFIX(st_device_forward, st_device_color_max_ptrs);
 } ENUM_PTRS_END
-private RELOC_PTRS_BEGIN(device_rop_texture_reloc_ptrs) {
+static RELOC_PTRS_BEGIN(device_rop_texture_reloc_ptrs) {
     RELOC_PREFIX(st_device_forward);
     RELOC_SUPER(gx_device_rop_texture, st_device_color, texture);
 } RELOC_PTRS_END
 
 /* Device for providing source data for RasterOp. */
-private dev_proc_fill_rectangle(rop_texture_fill_rectangle);
-private dev_proc_copy_mono(rop_texture_copy_mono);
-private dev_proc_copy_color(rop_texture_copy_color);
+static dev_proc_fill_rectangle(rop_texture_fill_rectangle);
+static dev_proc_copy_mono(rop_texture_copy_mono);
+static dev_proc_copy_color(rop_texture_copy_color);
 
 /* The device descriptor. */
-private const gx_device_rop_texture gs_rop_texture_device = {
+static const gx_device_rop_texture gs_rop_texture_device = {
     std_device_std_body(gx_device_rop_texture, 0, "rop source",
 			0, 0, 1, 1),
     {NULL,				/* open_device */
@@ -108,7 +105,8 @@ private const gx_device_rop_texture gs_rop_texture_device = {
      gx_forward_fill_linear_color_scanline,
      gx_forward_fill_linear_color_trapezoid,
      gx_forward_fill_linear_color_triangle,
-     gx_forward_update_spot_equivalent_colors
+     gx_forward_update_spot_equivalent_colors,
+     gx_forward_ret_devn_params
     },
     0,				/* target */
     lop_default			/* log_op */
@@ -132,7 +130,7 @@ gx_make_rop_texture_device(gx_device_rop_texture * dev, gx_device * target,
 {
     gx_device_init((gx_device *) dev,
 		   (const gx_device *)&gs_rop_texture_device,
-		   NULL, true);
+		   (target ? target->memory : NULL), true);
     gx_device_set_target((gx_device_forward *)dev, target);
     /* Drawing operations are defaulted, non-drawing are forwarded. */
     check_device_separable((gx_device *) dev);
@@ -143,7 +141,7 @@ gx_make_rop_texture_device(gx_device_rop_texture * dev, gx_device * target,
 }
 
 /* Fill a rectangle */
-private int
+static int
 rop_texture_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 			   gx_color_index color)
 {
@@ -162,7 +160,7 @@ rop_texture_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 }
 
 /* Copy a monochrome rectangle */
-private int
+static int
 rop_texture_copy_mono(gx_device * dev,
 		const byte * data, int sourcex, int raster, gx_bitmap_id id,
 		      int x, int y, int w, int h,
@@ -190,7 +188,7 @@ rop_texture_copy_mono(gx_device * dev,
 }
 
 /* Copy a color rectangle */
-private int
+static int
 rop_texture_copy_color(gx_device * dev,
 		const byte * data, int sourcex, int raster, gx_bitmap_id id,
 		       int x, int y, int w, int h)
