@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevtrac.c 8250 2007-09-25 13:31:24Z giles $ */
+/* $Id: gdevtrac.c 8803 2008-06-24 14:16:29Z leonardo $ */
 /* Tracing device (including sample high-level implementation) */
 #include "gx.h"
 #include "gserrors.h"
@@ -351,7 +351,7 @@ trace_end_image(gx_image_enum_common_t * info, bool draw_last)
 {
     trace_image_enum_t *pie = (trace_image_enum_t *)info;
 
-    gs_free_object(pie->memory, pie, "trace_end_image");
+    gx_image_free_enum(&info);
     return 0;
 }
 static const gx_image_enum_procs_t trace_image_enum_procs = {
