@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gscdevn.c 8488 2008-01-17 13:37:19Z leonardo $ */
+/* $Id: gscdevn.c 8764 2008-05-21 18:27:42Z mvrhel $ */
 /* DeviceN color space and operation definition */
 
 #include "memory_.h"
@@ -34,6 +34,7 @@
 #include "gxdevcli.h"
 #include "gsovrc.h"
 #include "stream.h"
+#include "gsnamecl.h"  /* Custom color call back define */
 
 /* ---------------- Color space ---------------- */
 
@@ -398,7 +399,7 @@ gx_remap_concrete_DeviceN(const frac * pconc, const gs_color_space * pcs,
 #endif
 
 #if ENABLE_CUSTOM_COLOR_CALLBACK
-    if (pis->custom_color_callback) {
+    if (pis->memory->gs_lib_ctx->custom_color_callback) {
 	int code = gx_remap_concrete_custom_color_DeviceN(pconc, pcs, pdc,
 						       	pis, dev, select);
 	if (code >= 0)
