@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gscolor2.c 8250 2007-09-25 13:31:24Z giles $ */
+/* $Id: gscolor2.c 8868 2008-07-23 23:29:39Z mvrhel $ */
 /* Level 2 color operators for Ghostscript library */
 #include "memory_.h"
 #include "gx.h"
@@ -24,6 +24,8 @@
 #include "gzstate.h"
 #include "gxpcolor.h"
 #include "stream.h"
+#include "gxcie.h"  
+#include "gxfrac.h"
 
 /* ---------------- General colors and color spaces ---------------- */
 
@@ -166,7 +168,12 @@ const gs_color_space_type gs_color_space_type_Indexed = {
     gx_init_paint_1, gx_restrict_Indexed,
     gx_concrete_space_Indexed,
     gx_concretize_Indexed, NULL,
-    gx_default_remap_color, gx_install_Indexed,
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    gx_remap_IndexedSpace,
+#else
+    gx_default_remap_color,
+#endif
+    gx_install_Indexed,
     gx_set_overprint_Indexed,
     gx_final_Indexed, gx_no_adjust_color_count,
     gx_serialize_Indexed,
@@ -425,15 +432,11 @@ static int
 gx_concretize_Indexed(const gs_client_color * pc, const gs_color_space * pcs,
 		      frac * pconc, const gs_imager_state * pis)
 {
-    float value = pc->paint.values[0];
-    int index =
-	(is_fneg(value) ? 0 :
-	 value >= pcs->params.indexed.hival ? pcs->params.indexed.hival :
-	 (int)value);
+
+    gs_client_color cc;
     const gs_color_space *pbcs =
 	(const gs_color_space *)pcs->base_space;
-    gs_client_color cc;
-    int code = gs_cspace_indexed_lookup(pcs, index, &cc);
+   int code = gs_indexed_limit_and_lookup(pc, pcs, &cc);
 
     if (code < 0)
 	return code;
@@ -475,6 +478,173 @@ gs_cspace_indexed_lookup(const gs_color_space *pcs, int index,
     }
 }
 
+/* Look up an index in an Indexed color space, return value as byte value(s). */
+int
+gs_cspace_indexed_lookup_bytes(const gs_color_space *pcs, float index_float,
+			unsigned char *output)
+{
+    const gs_indexed_params *pip = &pcs->params.indexed;
+    const gs_color_space *pbcs = pcs->base_space;
+    int m = cs_num_components(pbcs);
+    int index;
+ 
+    index = (is_fneg(index_float) ? 0 :
+    index_float >= pcs->params.indexed.hival ? pcs->params.indexed.hival :
+    (int) index_float);
+
+    if (pip->use_proc) {
+
+        float values[GS_CLIENT_COLOR_MAX_COMPONENTS];
+        int ok;
+
+        ok = pip->lookup.map->proc.lookup_index(pcs, index, values); 
+
+        /* Get out of float and to uchar. 
+           Note the fall through in the switch statement to 
+           handle the number of channels */
+
+        switch (m) {
+            default: {		/* DeviceN */
+	        int i;
+
+	        for (i = 0; i < m; ++i)
+	            output[i] = float_color_to_byte_color(values[i]);
+            }
+	        break;
+            case 4:
+	        output[3] = float_color_to_byte_color(values[3]);
+            case 3:
+	        output[2] = float_color_to_byte_color(values[2]);
+            case 2:
+	        output[1] = float_color_to_byte_color(values[1]);
+            case 1:
+	        output[0] = float_color_to_byte_color(values[0]);
+        }
+
+        return ok; 
+
+    } else {
+
+        /* Here it uses a 1-D LUT. Again the fall through
+            in the switch statement */
+
+        const byte *pcomp = pip->lookup.table.data + m * index;
+
+        switch (m) {
+            default: {		/* DeviceN */
+	        int i;
+
+	        for (i = 0; i < m; ++i)
+	            output[i] = pcomp[i];
+            }
+	        break;
+            case 4:
+	        output[3] = pcomp[3];
+            case 3:
+	        output[2] = pcomp[2];
+            case 2:
+	        output[1] = pcomp[1];
+            case 1:
+	        output[0] = pcomp[0];
+        }
+        return 0;
+    }
+}
+
+
+/* Look up an index in an Indexed color space, return value as frac value(s). */
+int
+gs_cspace_indexed_lookup_frac(const gs_color_space *pcs, float index_float,
+			frac *output)
+{
+    const gs_indexed_params *pip = &pcs->params.indexed;
+    const gs_color_space *pbcs = pcs->base_space;
+    int m = cs_num_components(pbcs);
+    int index;
+ 
+    index = (is_fneg(index_float) ? 0 :
+    index_float >= pcs->params.indexed.hival ? pcs->params.indexed.hival :
+    (int) index_float);
+
+    if (pip->use_proc) {
+
+        float values[GS_CLIENT_COLOR_MAX_COMPONENTS];
+        int ok;
+
+        ok = pip->lookup.map->proc.lookup_index(pcs, index, values); 
+
+        /* Get out of float and to frac. 
+           Note the fall through in the switch statement to 
+           handle the number of channels */
+
+        switch (m) {
+            default: {		/* DeviceN */
+	        int i;
+
+	        for (i = 0; i < m; ++i)
+	            output[i] = float2frac(values[i]);
+            }
+	        break;
+            case 4:
+	        output[3] = float2frac(values[3]);
+            case 3:
+	        output[2] = float2frac(values[2]);
+            case 2:
+	        output[1] = float2frac(values[1]);
+            case 1:
+	        output[0] = float2frac(values[0]);
+        }
+
+        return ok; 
+
+    } else {
+
+        /* Here it uses a 1-D LUT 
+           Again the fall through */
+
+        const byte *pcomp = pip->lookup.table.data + m * index;
+
+        switch (m) {
+            default: {		/* DeviceN */
+	        int i;
+
+	        for (i = 0; i < m; ++i)
+	            output[i] = byte2frac(pcomp[i]);
+            }
+	        break;
+            case 4:
+	        output[3] = byte2frac(pcomp[3]);
+            case 3:
+	        output[2] = byte2frac(pcomp[2]);
+            case 2:
+	        output[1] = byte2frac(pcomp[1]);
+            case 1:
+	        output[0] = byte2frac(pcomp[0]);
+        }
+        return 0;
+    }
+}
+
+
+
+
+/* Look up with restriction */
+
+int
+gs_indexed_limit_and_lookup(const gs_client_color * pc,const gs_color_space *pcs,
+			 gs_client_color *pcc)
+{     
+  
+    float value = pc->paint.values[0];
+        int index =
+	    (is_fneg(value) ? 0 :
+	     value >= pcs->params.indexed.hival ? pcs->params.indexed.hival :
+	     (int)value);
+         return(gs_cspace_indexed_lookup(pcs, index, pcc));
+
+}
+
+
 /* ---------------- Serialization. -------------------------------- */
 
 static int 
