@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsfont.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gsfont.c 8419 2007-11-30 06:43:47Z lpd $ */
 /* Font operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -40,6 +40,8 @@
 #define cmax_SMALL 500		/* cmax - # of cached chars */
 #define blimit_SMALL 100	/* blimit/upper - max size of a single cached char */
 
+extern bool CPSI_mode;
+
 /* Define a default procedure vector for fonts. */
 const gs_font_procs gs_font_procs_default = {
     gs_no_define_font,		/* (actually a default) */
@@ -58,8 +60,8 @@ const gs_font_procs gs_font_procs_default = {
 };
 
 private_st_font_dir();
-private struct_proc_enum_ptrs(font_enum_ptrs);
-private struct_proc_reloc_ptrs(font_reloc_ptrs);
+static struct_proc_enum_ptrs(font_enum_ptrs);
+static struct_proc_reloc_ptrs(font_reloc_ptrs);
 
 public_st_gs_font_info();
 public_st_gs_font();
@@ -85,7 +87,7 @@ public_st_gs_font_ptr_element();
  * relocation phase of the GC.  */
 
 /* Font directory GC procedures */
-private 
+static 
 ENUM_PTRS_WITH(font_dir_enum_ptrs, gs_font_dir *dir)
 {
     /* Enumerate pointers from cached characters to f/m pairs, */
@@ -119,7 +121,7 @@ return 0;
 font_dir_do_ptrs(e1)
 #undef e1
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(font_dir_reloc_ptrs, gs_font_dir *dir);
+static RELOC_PTRS_WITH(font_dir_reloc_ptrs, gs_font_dir *dir);
     /* Relocate the pointers from cached characters to f/m pairs. */
     /* See gxfcache.h for why we do this here. */
 {
@@ -161,6 +163,7 @@ gs_font_finalize(void *vptr)
 	    (ulong) pfont, (ulong) pfont->base, (ulong) prev, (ulong) next);
     /* Notify clients that the font is being freed. */
     gs_notify_all(&pfont->notify_list, NULL);
+    gs_purge_font_from_char_caches(pfont);
     if (pfont->dir == 0)
 	ppfirst = 0;
     else if (pfont->base == pfont)
@@ -187,7 +190,7 @@ gs_font_finalize(void *vptr)
 	*ppfirst = next;
     gs_notify_release(&pfont->notify_list);
 }
-private 
+static 
 ENUM_PTRS_WITH(font_enum_ptrs, gs_font *pfont) return ENUM_USING(st_gs_notify_list, &pfont->notify_list, sizeof(gs_notify_list_t), index - 5);
 	/* We don't enumerate next or prev of base fonts. */
 	/* See above for details. */
@@ -195,7 +198,7 @@ case 0: ENUM_RETURN((pfont->base == pfont ? 0 : pfont->next));
 case 1: ENUM_RETURN((pfont->base == pfont ? 0 : pfont->prev));
 ENUM_PTR3(2, gs_font, dir, base, client_data);
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(font_reloc_ptrs, gs_font *pfont);
+static RELOC_PTRS_WITH(font_reloc_ptrs, gs_font *pfont);
 RELOC_USING(st_gs_notify_list, &pfont->notify_list, sizeof(gs_notify_list_t));
 	/* We *do* always relocate next and prev. */
 	/* Again, see above for details. */
@@ -205,7 +208,7 @@ RELOC_PTR3(gs_font, dir, base, client_data);
 RELOC_PTRS_END
 
 /* Allocate a font directory */
-private bool
+static bool
 cc_no_mark_glyph(const gs_memory_t *mem, gs_glyph glyph, void *ignore_data)
 {
     return false;
@@ -262,6 +265,7 @@ gs_font_dir_alloc2_limits(gs_memory_t * struct_mem, gs_memory_t * bits_mem,
     pdir->grid_fit_tt = 2;
     pdir->memory = struct_mem;
     pdir->tti = 0;
+    pdir->ttm = 0;
     pdir->san = 0;
     pdir->global_glyph_code = NULL;
     pdir->text_enum_id = 0;
@@ -307,6 +311,7 @@ gs_font_alloc(gs_memory_t *mem, gs_memory_type_ptr_t pstype,
     pfont->WMode = 0;
     pfont->PaintType = 0;
     pfont->StrokeWidth = 0;
+    pfont->is_cached = false;
     pfont->procs = *procs;
     memset(&pfont->orig_FontMatrix, 0, sizeof(pfont->orig_FontMatrix));
 #endif
@@ -372,7 +377,7 @@ gs_font_notify_unregister(gs_font *font, gs_notify_proc_t proc, void *proc_data)
 }
 
 /* Link an element at the head of a chain. */
-private void
+static void
 font_link_first(gs_font **pfirst, gs_font *elt)
 {
     gs_font *first = elt->next = *pfirst;
@@ -467,8 +472,8 @@ gs_makefont(gs_font_dir * pdir, const gs_font * pfont,
 	    dlprintf1("[m]XUID(%u)", (uint) (-pbfont->UID.id));
 	else
 	    dlprintf("[m]no UID");
-	dprintf7(", FontType=%d,\n[m]  new FontMatrix=[%g %g %g %g %g %g]\n",
-		 pfont->FontType,
+	dprintf8(", FontType=%d, base=0x%lx,\n[m]  new FontMatrix=[%g %g %g %g %g %g]\n",
+		 pfont->FontType, (ulong)pfont->base,
 		 pmat->xx, pmat->xy, pmat->yx, pmat->yy,
 		 pmat->tx, pmat->ty);
     }
@@ -552,7 +557,7 @@ gs_makefont(gs_font_dir * pdir, const gs_font * pfont,
 	     */
 #if 0	    /* We disabled this code portion due to Bug 688392.
 	       The problem was dangling pointers, which appear in fm_pair instances
-	       after uid_free is applied to applied to a font's UID,
+	       after uid_free is applied to a font's UID,
 	       because they share same xvalues array. We're unable to guess 
 	       for which reason uid_free was applied to the font's UID here 
 	       5+ years ago (see gsfont.c revision 1.1).
@@ -628,20 +633,26 @@ gs_cachestatus(register const gs_font_dir * pdir, register uint pstat[7])
 /* setcacheparams */
 int
 gs_setcachesize(gs_font_dir * pdir, uint size)
-{				/* This doesn't delete anything from the cache yet. */
+{   /* This doesn't delete anything from the cache yet. */
+    if (CPSI_mode) {
+        if (size < 100000)             /* for CET 27-07 */
+            size = 100000;
+        else if (size > 100000000)     /* for CET 27-02-01 */
+            size = 100000000;
+    }
     pdir->ccache.bmax = size;
     return 0;
 }
 int
 gs_setcachelower(gs_font_dir * pdir, uint size)
 {
-    pdir->ccache.lower = size;
+    pdir->ccache.lower = ((int)size < 0) ? 0 : size; /* ?: for CET 27-07 */
     return 0;
 }
 int
 gs_setcacheupper(gs_font_dir * pdir, uint size)
-{
-    pdir->ccache.upper = size;
+{ 
+    pdir->ccache.upper = ((int)size < 0) ? 0 : size; /* ?: for CET 27-06 */
     return 0;
 }
 int
@@ -722,7 +733,7 @@ gs_purge_font(gs_font * pfont)
 
     /* Purge the font from the font/matrix pair cache, */
     /* including all cached characters rendered with that font. */
-    return gs_purge_font_from_char_caches(pdir, pfont);
+    return gs_purge_font_from_char_caches(pfont);
 }
 
 /* Locate a gs_font by gs_id. */
@@ -807,6 +818,8 @@ gs_default_font_info(gs_font *font, const gs_point *pscale, int members,
 	int fixed_width = 0;
 	int index;
 	int code = 0; /* Quiet compiler. */
+	int ecode = 0;
+	bool has_glyphs = false;
 
 	for (index = 0;
 	     fixed_width >= 0 &&
@@ -818,8 +831,10 @@ gs_default_font_info(gs_font *font, const gs_point *pscale, int members,
 	    code = font->procs.glyph_info(font, glyph, pmat,
 					  (GLYPH_INFO_WIDTH0 << wmode),
 					  &glyph_info);
-	    if (code < 0)
-		return code;
+	    if (code < 0) {
+		ecode = code;
+		continue;
+	    }
 	    if (notdef == gs_no_glyph && gs_font_glyph_is_notdef(bfont, glyph)) {
 		notdef = glyph;
 		info->MissingWidth = (int)glyph_info.width[wmode].x;
@@ -831,9 +846,10 @@ gs_default_font_info(gs_font *font, const gs_point *pscale, int members,
 		fixed_width = (int)glyph_info.width[wmode].x;
 	    else if (glyph_info.width[wmode].x != fixed_width)
 		fixed_width = min_int;
+	    has_glyphs = true;
 	}
-	if (code < 0)
-	    return code;
+	if (ecode < 0 && !has_glyphs)
+	    return ecode;
 	if (fixed_width > 0) {
 	    info->Flags |= FONT_IS_FIXED_WIDTH;
 	    info->members |= FONT_INFO_AVG_WIDTH | FONT_INFO_MAX_WIDTH |
