@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsicc.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gsicc.c 8780 2008-05-27 20:12:08Z mvrhel $ */
 /* Implementation of the ICCBased color space family */
 
 #include "math_.h"
@@ -24,10 +24,10 @@
 #include "gxarith.h"
 #include "gxcie.h"
 #include "gzstate.h"
-#include "stream.h"
 #include "icc.h"		/* must precede icc.h */
 #include "gsicc.h"
 
+#define SAVEICCPROFILE 0
 
 typedef struct _icmFileGs icmFileGs;
 
@@ -47,7 +47,7 @@ struct _icmFileGs {
  *
  * No special action is taken with respect to the stream pointer; that is
  * the responsibility of the client.  */
-private void
+static void
 cie_icc_finalize(void * pvicc_info)
 {
     gs_cie_icc *    picc_info = (gs_cie_icc *)pvicc_info;
@@ -73,29 +73,21 @@ private_st_cie_icc();
  * we must enumerate and relocate pointers in these space explicity.
  */
 gs_private_st_composite( st_color_space_CIEICC,
-                         gs_paint_color_space,
+                         gs_color_space,
                          "gs_color_space_CIEICC",
                          cs_CIEICC_enum_ptrs,
                          cs_CIEICC_reloc_ptrs );
 
 /* pointer enumeration routine */
-private
-ENUM_PTRS_WITH(cs_CIEICC_enum_ptrs, gs_color_space * pcs)
-        return ENUM_USING( *pcs->params.icc.alt_space.type->stype,
-                           &pcs->params.icc.alt_space,
-                           sizeof(pcs->params.separation.alt_space),
-                           index - 1 );
-
+static
+ENUM_PTRS_BEGIN(cs_CIEICC_enum_ptrs) return 0;
         ENUM_PTR(0, gs_color_space, params.icc.picc_info);
 ENUM_PTRS_END
 
 /* pointer relocation routine */
-private
-RELOC_PTRS_WITH(cs_CIEICC_reloc_ptrs, gs_color_space * pcs)
+static
+RELOC_PTRS_BEGIN(cs_CIEICC_reloc_ptrs)
     RELOC_PTR(gs_color_space, params.icc.picc_info);
-    RELOC_USING( *pcs->params.icc.alt_space.type->stype,
-                 &pcs->params.icc.alt_space,
-                 sizeof(pcs->params.separation.alt_space) );
 RELOC_PTRS_END
 
 
@@ -118,31 +110,36 @@ RELOC_PTRS_END
  * common elements (ranges, number of components, etc.) into the color space
  * root class.
  */
-private cs_proc_num_components(gx_num_components_CIEICC);
-private cs_proc_base_space(gx_alt_space_CIEICC);
-private cs_proc_init_color(gx_init_CIEICC);
-private cs_proc_restrict_color(gx_restrict_CIEICC);
-private cs_proc_concrete_space(gx_concrete_space_CIEICC);
-private cs_proc_concretize_color(gx_concretize_CIEICC);
-private cs_proc_adjust_cspace_count(gx_adjust_cspace_CIEICC);
-private cs_proc_serialize(gx_serialize_CIEICC);
+static cs_proc_num_components(gx_num_components_CIEICC);
+static cs_proc_init_color(gx_init_CIEICC);
+static cs_proc_restrict_color(gx_restrict_CIEICC);
+static cs_proc_concrete_space(gx_concrete_space_CIEICC);
+static cs_proc_concretize_color(gx_concretize_CIEICC);
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+static cs_proc_remap_color(gx_remap_ICCBased);
+#endif
+static cs_proc_final(gx_final_CIEICC);
+static cs_proc_serialize(gx_serialize_CIEICC);
 
-private const gs_color_space_type gs_color_space_type_CIEICC = {
+static const gs_color_space_type gs_color_space_type_CIEICC = {
     gs_color_space_index_CIEICC,    /* index */
     true,                           /* can_be_base_space */
     true,                           /* can_be_alt_space */
     &st_color_space_CIEICC,         /* stype - structure descriptor */
     gx_num_components_CIEICC,       /* num_components */
-    gx_alt_space_CIEICC,            /* base_space */
     gx_init_CIEICC,                 /* init_color */
     gx_restrict_CIEICC,             /* restrict_color */
     gx_concrete_space_CIEICC,       /* concrete_space */
     gx_concretize_CIEICC,           /* concreteize_color */
     NULL,                           /* remap_concrete_color */
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    gx_remap_ICCBased,		    /* remap_color */
+#else
     gx_default_remap_color,         /* remap_color */
+#endif
     gx_install_CIE,                 /* install_cpsace */
     gx_spot_colors_set_overprint,   /* set_overprint */
-    gx_adjust_cspace_CIEICC,        /* adjust_cspace_count */
+    gx_final_CIEICC,                /* final */
     gx_no_adjust_color_count,       /* adjust_color_count */
     gx_serialize_CIEICC,		    /* serialize */
     gx_cspace_is_linear_default
@@ -152,29 +149,17 @@ private const gs_color_space_type gs_color_space_type_CIEICC = {
 /*
  * Return the number of components used by a ICCBased color space - 1, 3, or 4
  */
-private int
+static int
 gx_num_components_CIEICC(const gs_color_space * pcs)
 {
     return pcs->params.icc.picc_info->num_components;
 }
 
-/*
- * Return the alternative space for an ICCBasee color space, but only if
- * that space is being used.
- */
-private const gs_color_space *
-gx_alt_space_CIEICC(const gs_color_space * pcs)
-{
-    return (pcs->params.icc.picc_info->picc == NULL)
-                ? (const gs_color_space *)&pcs->params.icc.alt_space
-                : NULL;
-}
-
 /*
  * Set the initial client color for an ICCBased color space. The convention
  * suggested by the ICC specification is to set all components to 0.
  */
-private void
+static void
 gx_init_CIEICC(gs_client_color * pcc, const gs_color_space * pcs)
 {
     int     i, ncomps = pcs->params.icc.picc_info->num_components;
@@ -189,7 +174,7 @@ gx_init_CIEICC(gs_client_color * pcc, const gs_color_space * pcs)
 /*
  * Restrict an color to the range specified for an ICCBased color space.
  */
-private void
+static void
 gx_restrict_CIEICC(gs_client_color * pcc, const gs_color_space * pcs)
 {
     int                 i, ncomps = pcs->params.icc.picc_info->num_components;
@@ -213,12 +198,11 @@ gx_restrict_CIEICC(gs_client_color * pcc, const gs_color_space * pcs)
  * for all CIE bases. If the alternate color space is being used, then
  * this question is passed on the the appropriate method of that space.
  */
-private const gs_color_space *
+static const gs_color_space *
 gx_concrete_space_CIEICC(const gs_color_space * pcs, const gs_imager_state * pis)
 {
     if (pcs->params.icc.picc_info->picc == NULL) {
-        const gs_color_space *  pacs = (const gs_color_space *)
-                                        &pcs->params.icc.alt_space;
+        const gs_color_space *  pacs = pcs->base_space;
 
         return cs_concrete_space(pacs, pis);
     } else
@@ -228,7 +212,7 @@ gx_concrete_space_CIEICC(const gs_color_space * pcs, const gs_imager_state * pis
 /*
  * Convert an ICCBased color space to a concrete color space.
  */
-private int
+static int
 gx_concretize_CIEICC(
     const gs_client_color * pcc,
     const gs_color_space *  pcs,
@@ -243,17 +227,22 @@ gx_concretize_CIEICC(
     cie_cached_vector3      vlmn;
     gs_client_color         lcc = *pcc;
     int                     i, ncomps = picc_info->num_components;
+    int code;
 
     /* use the altenate space concretize if appropriate */
     if (picc == NULL)
-        return picc_params->alt_space.type->concretize_color(
+        return pcs->base_space->type->concretize_color(
                             pcc,
-                            (const gs_color_space *)&picc_params->alt_space,
+                            pcs->base_space,
                             pconc,
                             pis );
 
     /* set up joint cache as required */
-    CIE_CHECK_RENDERING(pcs, pconc, pis, return 0);
+    code = gx_cie_check_rendering(pcs, pconc, pis);
+    if (code < 0)
+	return code;
+    if (code == 1)
+	return 0;
 
     /* verify and update the stream pointer */
     if (picc_info->file_id != (instrp->read_id | instrp->write_id))
@@ -264,14 +253,18 @@ gx_concretize_CIEICC(
     gx_restrict_CIEICC(&lcc, pcs);
     for (i = 0; i < ncomps; i++)
         inv[i] = lcc.paint.values[i];
+		
+	/* Since the original limits were wrong for this case, We need to adjust things a bit different */
 
     /* For input Lab color space massage the values into Lab range */
 
-    if (picc_info->plu->e_inSpace == icSigLabData) {
+   /* if (picc_info->plu->e_inSpace == icSigLabData) {
+
         inv[0] *= 100;
         inv[1] = inv[1]*255 - 128;
         inv[2] = inv[2]*255 - 128; 
-    }
+
+    } */
 
     /*
      * Perform the lookup operation. A return value of 1 indicates that
@@ -279,6 +272,7 @@ gx_concretize_CIEICC(
      * legitimate. Other non-zero return values indicate an error, which
      * should not occur in practice.
      */
+
     if (picc_info->plu->lookup(picc_info->plu, outv, inv) > 1)
         return_error(gs_error_unregistered);
 
@@ -317,36 +311,46 @@ gx_concretize_CIEICC(
     return 0;
 }
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
 /*
- * Handle a reference or de-reference of the prameter structure of an
- * ICCBased color space. For the purposes of this routine, the color space
- * is considered a reference rather than an object, and is not itself
- * reference counted (an unintuitive but otherwise legitimate state of
- * affairs).
+ * This routine is only used if ENABLE_CUSTOM_COLOR_CALLBACK is true.
+ * Otherwise we use gx_default_remap_color directly for CIEBasedDEFG color
+ * spaces.
  *
- * Because color spaces store alternative/base color space inline, these
- * need to have their reference count adjusted explicitly.
+ * Render a CIEBasedDEFG color.
  */
-private void
-gx_adjust_cspace_CIEICC(const gs_color_space * pcs, int delta)
+int
+gx_remap_ICCBased(const gs_client_color * pc, const gs_color_space * pcs,
+	gx_device_color * pdc, const gs_imager_state * pis, gx_device * dev,
+		gs_color_select_t select)
 {
-    const gs_icc_params *   picc_params = &pcs->params.icc;
+    client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) (pis->memory->gs_lib_ctx->custom_color_callback);
 
-    rc_adjust_const(picc_params->picc_info, delta, "gx_adjust_cspace_CIEICC");
-    picc_params->alt_space.type->adjust_cspace_count(
-                (const gs_color_space *)&picc_params->alt_space, delta );
+    if (pcb != NULL) {
+	if (pcb->client_procs->remap_ICCBased(pcb, pc, pcs,
+			   			pdc, pis, dev, select) == 0)
+	    return 0;
+    }
+    /* Use default routine for non custom color processing. */
+    return gx_default_remap_color(pc, pcs, pdc, pis, dev, select);
 }
+#endif
 
 /*
- * Increment color space reference counts.
+ * Finalize the contents of an ICC color space. Now that color space
+ * objects have straightforward reference counting discipline, there's
+ * nothing special about it. In the previous state of affairs, the
+ * argument in favor of correct reference counting spoke of "an
+ * unintuitive but otherwise legitimate state of affairs".
  */
-void
-gx_increment_cspace_count(const gs_color_space * pcs)
+static void
+gx_final_CIEICC(const gs_color_space * pcs)
 {
-    pcs->type->adjust_cspace_count(pcs, 1);
+    rc_decrement_only(pcs->params.icc.picc_info, "gx_final_CIEICC");
 }
 
-private int
+static int
 icmFileGs_seek(icmFile *pp, long int offset)
 {
     icmFileGs *p = (icmFileGs *)pp;
@@ -354,7 +358,7 @@ icmFileGs_seek(icmFile *pp, long int offset)
     return spseek(p->strp, offset);
 }
 
-private size_t
+static size_t
 icmFileGs_read(icmFile *pp, void *buffer, size_t size, size_t count)
 {
     icmFileGs *p = (icmFileGs *)pp;
@@ -364,7 +368,7 @@ icmFileGs_read(icmFile *pp, void *buffer, size_t size, size_t count)
     return (status < 0) ? status : tot;
 }
 
-private size_t
+static size_t
 icmFileGs_write(icmFile *pp, void *buffer, size_t size, size_t count)
 {
     icmFileGs *p = (icmFileGs *)pp;
@@ -374,7 +378,7 @@ icmFileGs_write(icmFile *pp, void *buffer, size_t size, size_t count)
     return (status < 0) ? status : tot;
 }
 
-private int
+static int
 icmFileGs_flush(icmFile *pp)
 {
     icmFileGs *p = (icmFileGs *)pp;
@@ -382,7 +386,7 @@ icmFileGs_flush(icmFile *pp)
     return s_std_write_flush(p->strp);
 }
 
-private int
+static int
 icmFileGs_delete(icmFile *pp)
 {
     free(pp);
@@ -404,7 +408,7 @@ icmFileGs_delete(icmFile *pp)
  * Return value: the stream wrapped as an icmFile object, or NULL on
  * error.
  **/
-private icmFile *
+static icmFile *
 gx_wrap_icc_stream(stream *strp)
 {
     icmFileGs *p;
@@ -429,6 +433,14 @@ gx_load_icc_profile(gs_cie_icc *picc_info)
     icc *           picc;
     icmLuBase * plu = NULL;
     icmFile *pfile = NULL;
+
+#if   SAVEICCPROFILE
+
+    unsigned int num_bytes;
+    unsigned char *iccbuffer;
+    FILE *fid;
+
+#endif
 	
     /* verify that the file is legitimate */
     if (picc_info->file_id != (instrp->read_id | instrp->write_id))
@@ -458,6 +470,20 @@ gx_load_icc_profile(gs_cie_icc *picc_info)
       
 	if ((picc->read(picc, pfile, 0)) != 0)
 	    goto return_rangecheck;
+
+#if SAVEICCPROFILE
+
+        num_bytes = picc->header->size;
+        iccbuffer = (unsigned char *) malloc(num_bytes);
+        pfile->seek(pfile,0);
+        pfile->read(pfile,iccbuffer,1,num_bytes);
+        fid = fopen("DumpedICC.icm","wb");
+        fwrite(iccbuffer,sizeof(unsigned char),num_bytes,fid);
+        fclose(fid);
+        free(iccbuffer);
+
+#endif
+        
             
 	/* verify the profile type */
 	profile_class = picc->header->deviceClass;
@@ -583,12 +609,28 @@ gx_load_icc_profile(gs_cie_icc *picc_info)
  * Note that an ICCBased color space must be installed before it is known if
  * the ICC profile or the alternate color space is to be used.
  */
-private int
-gx_install_CIEICC(const gs_color_space * pcs, gs_state * pgs)
+static int
+gx_install_CIEICC(gs_color_space * pcs, gs_state * pgs)
 {
     const gs_icc_params * picc_params = (const gs_icc_params *)&pcs->params.icc;
     gs_cie_icc *    picc_info = picc_params->picc_info;
 
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    {
+        /*
+         * Check if we want to use the callback color processing for this
+         * color space.
+         */
+        client_custom_color_params_t * pcb =
+	    (client_custom_color_params_t *) pgs->memory->gs_lib_ctx->custom_color_callback;
+
+        if (pcb != NULL) {
+	    if (pcb->client_procs->install_ICCBased(pcb, pcs, pgs))
+    	        /* Exit if the client will handle the colorspace completely */
+		return 0;
+        }
+    }
+#endif
     /* update the stub information used by the joint caches */
     gx_cie_load_common_cache(&picc_info->common, pgs);
     gx_cie_common_complete(&picc_info->common);
@@ -650,7 +692,7 @@ gs_cspace_build_CIEICC(
 
 /* ---------------- Serialization. -------------------------------- */
 
-private int 
+static int 
 gx_serialize_CIEICC(const gs_color_space * pcs, stream * s)
 {
     const gs_icc_params * p = &pcs->params.icc;
