@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsimage.c 8539 2008-02-22 20:18:08Z leonardo $ */
+/* $Id: gsimage.c 8790 2008-06-09 07:33:57Z leonardo $ */
 /* Image setup procedures for Ghostscript library */
 #include "memory_.h"
 #include "math_.h"
@@ -102,9 +102,9 @@ struct gs_image_enum_s {
 				/* only needed for gs_image_next */
     int y;
     bool error;
-    byte wanted[gs_image_max_planes]; /* cache gx_image_planes_wanted */
-    byte client_wanted[gs_image_max_planes]; /* see gsimage.h */
-    image_enum_plane_t planes[gs_image_max_planes]; /* see above */
+    byte wanted[GS_IMAGE_MAX_COMPONENTS]; /* cache gx_image_planes_wanted */
+    byte client_wanted[GS_IMAGE_MAX_COMPONENTS]; /* see gsimage.h */
+    image_enum_plane_t planes[GS_IMAGE_MAX_COMPONENTS]; /* see above */
     /*
      * To reduce setup for transferring complete rows, we maintain a
      * partially initialized parameter array for gx_image_plane_data_rows.
@@ -112,7 +112,7 @@ struct gs_image_enum_s {
      * gx_image_plane_data_rows; the data_x and raster members are reset
      * when needed.
      */
-    gx_image_plane_t image_planes[gs_image_max_planes];
+    gx_image_plane_t image_planes[GS_IMAGE_MAX_COMPONENTS];
 };
 
 gs_private_st_composite(st_gs_image_enum, gs_image_enum, "gs_image_enum",
@@ -194,6 +194,7 @@ gs_image_begin_typed(const gs_image_common_t * pic, gs_state * pgs,
     gx_clip_path *pcpath;
     int code = gx_effective_clip_path(pgs, &pcpath);
     gx_device *dev2 = dev;
+    gx_device_color dc_temp, *pdevc = pgs->dev_color;
 
     if (code < 0)
 	return code;
@@ -219,9 +220,13 @@ gs_image_begin_typed(const gs_image_common_t * pic, gs_state * pgs,
 	    if (code < 0)
 		return code;
 	}
+	if (dev2 != dev) {
+	    set_nonclient_dev_color(&dc_temp, 1);
+	    pdevc = &dc_temp;
+	}
     }
     code = gx_device_begin_typed_image(dev2, (const gs_imager_state *)pgs,
-		NULL, pic, NULL, pgs->dev_color, pcpath, pgs->memory, ppie);
+		NULL, pic, NULL, pdevc, pcpath, pgs->memory, ppie);
     if (code < 0)
 	return code;
     code = is_image_visible(pic, pgs, pcpath);
@@ -392,6 +397,7 @@ int
 gs_image_enum_init(gs_image_enum * penum, gx_image_enum_common_t * pie,
 		   const gs_data_image_t * pim, gs_state *pgs)
 {
+    pgs->device->sgr.stroke_stored = false;
     return gs_image_common_init(penum, pie, pim,
 				(pgs->in_charpath ? NULL :
 				 gs_currentdevice_inline(pgs)));
@@ -451,8 +457,8 @@ gs_image_next(gs_image_enum * penum, const byte * dbytes, uint dsize,
     int px = penum->plane_index;
     int num_planes = penum->num_planes;
     int i, code;
-    uint used[gs_image_max_planes];
-    gs_const_string plane_data[gs_image_max_planes];
+    uint used[GS_IMAGE_MAX_COMPONENTS];
+    gs_const_string plane_data[GS_IMAGE_MAX_COMPONENTS];
 
     if (penum->planes[px].source.size != 0)
 	return_error(gs_error_rangecheck);
