@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsmatrix.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gsmatrix.c 8536 2008-02-20 21:02:48Z leonardo $ */
 /* Matrix operators for Ghostscript library */
 #include "math_.h"
 #include "memory_.h"
@@ -23,7 +23,7 @@
 #include "stream.h"
 
 /* The identity matrix */
-private const gs_matrix gs_identity_matrix =
+static const gs_matrix gs_identity_matrix =
 {identity_matrix_body};
 
 /* ------ Matrix creation ------ */
@@ -110,6 +110,41 @@ gs_matrix_multiply(const gs_matrix * pm1, const gs_matrix * pm2, gs_matrix * pmr
     }
     return 0;
 }
+int
+gs_matrix_multiply_double(const gs_matrix_double * pm1, const gs_matrix * pm2, gs_matrix_double * pmr)
+{
+    double xx1 = pm1->xx, yy1 = pm1->yy;
+    double tx1 = pm1->tx, ty1 = pm1->ty;
+    double xx2 = pm2->xx, yy2 = pm2->yy;
+    double xy2 = pm2->xy, yx2 = pm2->yx;
+
+    if (is_xxyy(pm1)) {
+	pmr->tx = tx1 * xx2 + pm2->tx;
+	pmr->ty = ty1 * yy2 + pm2->ty;
+	if (is_fzero(xy2))
+	    pmr->xy = 0;
+	else
+	    pmr->xy = xx1 * xy2,
+		pmr->ty += tx1 * xy2;
+	pmr->xx = xx1 * xx2;
+	if (is_fzero(yx2))
+	    pmr->yx = 0;
+	else
+	    pmr->yx = yy1 * yx2,
+		pmr->tx += ty1 * yx2;
+	pmr->yy = yy1 * yy2;
+    } else {
+	double xy1 = pm1->xy, yx1 = pm1->yx;
+
+	pmr->xx = xx1 * xx2 + xy1 * yx2;
+	pmr->xy = xx1 * xy2 + xy1 * yy2;
+	pmr->yy = yx1 * xy2 + yy1 * yy2;
+	pmr->yx = yx1 * xx2 + yy1 * yx2;
+	pmr->tx = tx1 * xx2 + ty1 * yx2 + pm2->tx;
+	pmr->ty = tx1 * xy2 + ty1 * yy2 + pm2->ty;
+    }
+    return 0;
+}
 
 /* Invert a matrix.  Return gs_error_undefinedresult if not invertible. */
 int
@@ -124,17 +159,53 @@ gs_matrix_invert(const gs_matrix * pm, gs_matrix * pmr)
 	pmr->yx = 0.0;
 	pmr->ty = -(pmr->yy = 1.0 / pm->yy) * pm->ty;
     } else {
-	double det = pm->xx * pm->yy - pm->xy * pm->yx;
-	double mxx = pm->xx, mtx = pm->tx;
+	float mxx = pm->xx, myy = pm->yy, mxy = pm->xy, myx = pm->yx;
+        float mtx = pm->tx, mty = pm->ty;
+	float det = (float)(mxx * myy) - (float)(mxy * myx);
 
+        /*
+         * We are doing the math as floats instead of doubles to reproduce
+	 * the results in page 1 of CET 10-09.ps
+         */
 	if (det == 0)
 	    return_error(gs_error_undefinedresult);
-	pmr->xx = pm->yy / det;
-	pmr->xy = -pm->xy / det;
-	pmr->yx = -pm->yx / det;
-	pmr->yy = mxx / det;	/* xx is already changed */
-	pmr->tx = -(mtx * pmr->xx + pm->ty * pmr->yx);
-	pmr->ty = -(mtx * pmr->xy + pm->ty * pmr->yy);	/* tx ditto */
+	pmr->xx = myy / det;
+	pmr->xy = -mxy / det;
+	pmr->yx = -myx / det;
+	pmr->yy = mxx / det;
+	pmr->tx = (((float)(mty * myx) - (float)(mtx * myy))) / det;
+	pmr->ty = (((float)(mtx * mxy) - (float)(mty * mxx))) / det;
+    }
+    return 0;
+}
+int
+gs_matrix_invert_to_double(const gs_matrix * pm, gs_matrix_double * pmr)
+{				/* We have to be careful about fetch/store order, */
+    /* because pm might be the same as pmr. */
+    if (is_xxyy(pm)) {
+	if (is_fzero(pm->xx) || is_fzero(pm->yy))
+	    return_error(gs_error_undefinedresult);
+	pmr->tx = -(pmr->xx = 1.0 / pm->xx) * pm->tx;
+	pmr->xy = 0.0;
+	pmr->yx = 0.0;
+	pmr->ty = -(pmr->yy = 1.0 / pm->yy) * pm->ty;
+    } else {
+	double mxx = pm->xx, myy = pm->yy, mxy = pm->xy, myx = pm->yx;
+        double mtx = pm->tx, mty = pm->ty;
+	double det = (mxx * myy) - (mxy * myx);
+
+        /*
+         * We are doing the math as floats instead of doubles to reproduce
+	 * the results in page 1 of CET 10-09.ps
+         */
+	if (det == 0)
+	    return_error(gs_error_undefinedresult);
+	pmr->xx = myy / det;
+	pmr->xy = -mxy / det;
+	pmr->yx = -myx / det;
+	pmr->yy = mxx / det;
+	pmr->tx = (((mty * myx) - (mtx * myy))) / det;
+	pmr->ty = (((mtx * mxy) - (mty * mxx))) / det;
     }
     return 0;
 }
@@ -200,12 +271,16 @@ int
 gs_point_transform(floatp x, floatp y, const gs_matrix * pmat,
 		   gs_point * ppt)
 {
-    ppt->x = x * pmat->xx + pmat->tx;
-    ppt->y = y * pmat->yy + pmat->ty;
+    /*
+     * The float casts are there to reproduce results in CET 10-01.ps
+     * page 4.
+     */
+    ppt->x = (float)(x * pmat->xx) + pmat->tx;
+    ppt->y = (float)(y * pmat->yy) + pmat->ty;
     if (!is_fzero(pmat->yx))
-	ppt->x += y * pmat->yx;
+	ppt->x += (float)(y * pmat->yx);
     if (!is_fzero(pmat->xy))
-	ppt->y += x * pmat->xy;
+	ppt->y += (float)(x * pmat->xy);
     return 0;
 }
 
@@ -303,7 +378,7 @@ gs_points_bbox(const gs_point pts[4], gs_rect * pbox)
 
 /* Transform or inverse-transform a bounding box. */
 /* Return gs_error_undefinedresult if the matrix is not invertible. */
-private int
+static int
 bbox_transform_either_only(const gs_rect * pbox_in, const gs_matrix * pmat,
 			   gs_point pts[4],
      int (*point_xform) (floatp, floatp, const gs_matrix *, gs_point *))
@@ -319,7 +394,7 @@ bbox_transform_either_only(const gs_rect * pbox_in, const gs_matrix * pmat,
     return code;
 }
 
-private int
+static int
 bbox_transform_either(const gs_rect * pbox_in, const gs_matrix * pmat,
 		      gs_rect * pbox_out,
      int (*point_xform) (floatp, floatp, const gs_matrix *, gs_point *))
