@@ -1,20 +1,17 @@
-/* Copyright (C) 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gspcolor.h,v 1.2.6.1 2002/02/22 19:45:58 ray Exp $ */
+/* $Id: gspcolor.h 8022 2007-06-05 22:23:38Z giles $ */
 /* Client interface to Pattern color */
 
 #ifndef gspcolor_INCLUDED
@@ -51,6 +48,10 @@
 #  define gs_pattern_type_DEFINED
 typedef struct gs_pattern_type_s gs_pattern_type_t;
 #endif
+#ifndef gs_matrix_DEFINED
+#  define gs_matrix_DEFINED
+typedef struct gs_matrix_s gs_matrix;
+#endif
 
 #define gs_pattern_template_common\
   const gs_pattern_type_t *type;\
@@ -80,7 +81,8 @@ typedef struct gs_pattern_instance_s gs_pattern_instance_t;
     rc_header rc;\
     /* Following are set by makepattern */\
     const gs_pattern_type_t *type;  /* from template */\
-    gs_state *saved
+    gs_state *saved;\
+    gs_id pattern_id
 struct gs_pattern_instance_s {
     gs_pattern_instance_common;
 };
@@ -96,8 +98,8 @@ extern_st(st_pattern_instance);
 /* ---------------- Procedures ---------------- */
 
 /* Set a Pattern color or a Pattern color space. */
-int gs_setpattern(P2(gs_state *, const gs_client_color *));
-int gs_setpatternspace(P1(gs_state *));
+int gs_setpattern(gs_state *, const gs_client_color *);
+int gs_setpatternspace(gs_state *);
 
 /*
  * Construct a Pattern color of any PatternType.
@@ -105,9 +107,9 @@ int gs_setpatternspace(P1(gs_state *));
  * same allocator as for the gs_state argument.  Note that gs_make_pattern
  * uses rc_alloc_struct_1 to allocate pattern instances.
  */
-int gs_make_pattern(P5(gs_client_color *, const gs_pattern_template_t *,
-		       const gs_matrix *, gs_state *, gs_memory_t *));
-const gs_pattern_template_t *gs_get_pattern(P1(const gs_client_color *));
+int gs_make_pattern(gs_client_color *, const gs_pattern_template_t *,
+		    const gs_matrix *, gs_state *, gs_memory_t *);
+const gs_pattern_template_t *gs_get_pattern(const gs_client_color *);
 
 /*
  * Adjust the reference count of a pattern. This is intended to support
@@ -116,6 +118,6 @@ const gs_pattern_template_t *gs_get_pattern(P1(const gs_client_color *));
  * applications, they need some way to release or retain the instances as
  * needed.
  */
-void gs_pattern_reference(P2(gs_client_color * pcc, int delta));
+void gs_pattern_reference(gs_client_color * pcc, int delta);
 
 #endif /* gspcolor_INCLUDED */
