@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gstrans.h 8553 2008-02-27 19:22:57Z leonardo $ */
+/* $Id: gstrans.h 8585 2008-03-03 16:01:12Z leonardo $ */
 /* Transparency definitions and interface */
 
 #ifndef gstrans_INCLUDED
@@ -88,8 +88,9 @@ struct gs_pdf14trans_params_s {
     gs_transparency_source_t shape;
     bool mask_is_image;
     gs_matrix ctm;
-    bool idle;
     bool replacing;
+    bool idle; /* For clist reader.*/
+    uint mask_id; /* For clist reader.*/
 };
 
 #ifndef gs_pdf14trans_params_DEFINED
