@@ -1,20 +1,17 @@
-/* Copyright (C) 1993, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxacpath.c,v 1.3.6.1 2002/02/22 19:45:58 ray Exp $ */
+/*$Id: gxacpath.c 8659 2008-04-23 14:06:11Z leonardo $ */
 /* Accumulator for clipping paths */
 #include "gx.h"
 #include "gserrors.h"
@@ -31,16 +28,33 @@
 #include "gzacpath.h"
 
 /* Device procedures */
-private dev_proc_open_device(accum_open);
-private dev_proc_close_device(accum_close);
-private dev_proc_fill_rectangle(accum_fill_rectangle);
+static dev_proc_open_device(accum_open_device);
+static dev_proc_close_device(accum_close);
+static dev_proc_fill_rectangle(accum_fill_rectangle);
+static dev_proc_pattern_manage(accum_pattern_manage);
+
+/* GC information */
+extern_st(st_clip_list);
+static
+ENUM_PTRS_WITH(device_cpath_accum_enum_ptrs, gx_device_cpath_accum *pdev)
+    if (index >= st_device_max_ptrs)
+	return ENUM_USING(st_clip_list, &pdev->list, sizeof(gx_clip_list), index - st_device_max_ptrs);
+    ENUM_PREFIX(st_device, 0);
+ENUM_PTRS_END
+static
+RELOC_PTRS_WITH(device_cpath_accum_reloc_ptrs, gx_device_cpath_accum *pdev)
+{   RELOC_PREFIX(st_device);
+    RELOC_USING(st_clip_list, &pdev->list, size);
+} RELOC_PTRS_END
+
+public_st_device_cpath_accum();
 
 /* The device descriptor */
 /* Many of these procedures won't be called; they are set to NULL. */
-private const gx_device_cpath_accum gs_cpath_accum_device =
+static const gx_device_cpath_accum gs_cpath_accum_device =
 {std_device_std_body(gx_device_cpath_accum, 0, "clip list accumulator",
 		     0, 0, 1, 1),
- {accum_open,
+ {accum_open_device,
   NULL,
   NULL,
   NULL,
@@ -49,7 +63,7 @@ private const gx_device_cpath_accum gs_cpath_accum_device =
   NULL,
   accum_fill_rectangle,
   NULL,
-  NULL,
+  gx_default_copy_mono,
   NULL,
   NULL,
   NULL,
@@ -66,7 +80,7 @@ private const gx_device_cpath_accum gs_cpath_accum_device =
   NULL,
   gx_default_fill_path,
   gx_default_stroke_path,
-  NULL,
+  gx_default_fill_mask,
   gx_default_fill_trapezoid,
   gx_default_fill_parallelogram,
   gx_default_fill_triangle,
@@ -83,7 +97,17 @@ private const gx_device_cpath_accum gs_cpath_accum_device =
   NULL,
   NULL,
   gx_default_text_begin,
-  gx_default_finish_copydevice
+  gx_default_finish_copydevice,
+  NULL,	/* begin_transparency_group */
+  NULL,	/* end_transparency_group */
+  NULL,	/* begin_transparency_mask */
+  NULL,	/* end_transparency_mask */
+  NULL,	/* discard_transparency_layer */
+  NULL,	/* get_color_mapping_procs */
+  NULL, /* get_color_comp_index */
+  NULL,	/* encode_color */
+  NULL,	/* decode_color */
+  accum_pattern_manage
  }
 };
 
@@ -95,6 +119,8 @@ gx_cpath_accum_begin(gx_device_cpath_accum * padev, gs_memory_t * mem)
 		   (const gx_device *) & gs_cpath_accum_device,
 		   NULL /* allocated on stack */ , true);
     padev->list_memory = mem;
+    padev->memory = mem; /* gx_general_fill_path may need a storage 
+			    for dropout prevention buffer. */
     (*dev_proc(padev, open_device)) ((gx_device *) padev);
 }
 
@@ -120,14 +146,17 @@ gx_cpath_accum_end(const gx_device_cpath_accum * padev, gx_clip_path * pcpath)
 	return code;
     gx_cpath_init_local(&apath, padev->list_memory);
     apath.rect_list->list = padev->list;
-    apath.path.bbox.p.x = int2fixed(padev->bbox.p.x);
-    apath.path.bbox.p.y = int2fixed(padev->bbox.p.y);
-    apath.path.bbox.q.x = int2fixed(padev->bbox.q.x);
-    apath.path.bbox.q.y = int2fixed(padev->bbox.q.y);
-    /* Using the setbbox flag here is slightly bogus, */
-    /* but it's as good a way as any to indicate that */
-    /* the bbox is accurate. */
-    apath.path.bbox_set = 1;
+    if (padev->list.count == 0)
+	apath.path.bbox.p.x = apath.path.bbox.p.y =
+	apath.path.bbox.q.x = apath.path.bbox.q.y = 0;
+    else {
+	apath.path.bbox.p.x = int2fixed(padev->bbox.p.x);
+	apath.path.bbox.p.y = int2fixed(padev->bbox.p.y);
+	apath.path.bbox.q.x = int2fixed(padev->bbox.q.x);
+	apath.path.bbox.q.y = int2fixed(padev->bbox.q.y);
+    }
+    /* indicate that the bbox is accurate */
+    apath.path.bbox_accurate = 1;
     /* Note that the result of the intersection might be */
     /* a single rectangle.  This will cause clip_path_is_rect.. */
     /* to return true.  This, in turn, requires that */
@@ -141,7 +170,7 @@ gx_cpath_accum_end(const gx_device_cpath_accum * padev, gx_clip_path * pcpath)
     }
     gx_cpath_set_outer_box(&apath);
     apath.path_valid = false;
-    apath.id = gs_next_ids(1);	/* path changed => change id */
+    apath.id = gs_next_ids(padev->list_memory, 1);	/* path changed => change id */
     gx_cpath_assign_free(pcpath, &apath);
     return 0;
 }
@@ -156,7 +185,8 @@ gx_cpath_accum_discard(gx_device_cpath_accum * padev)
 /* Intersect two clipping paths using an accumulator. */
 int
 gx_cpath_intersect_path_slow(gx_clip_path * pcpath, gx_path * ppath,
-			     int rule, gs_imager_state *pis)
+			     int rule, gs_imager_state *pis,
+			     const gx_fill_params * params0)
 {
     gs_logical_operation_t save_lop = gs_current_logical_op_inline(pis);
     gx_device_cpath_accum adev;
@@ -165,12 +195,16 @@ gx_cpath_intersect_path_slow(gx_clip_path * pcpath, gx_path * ppath,
     int code;
 
     gx_cpath_accum_begin(&adev, pcpath->path.memory);
-    color_set_pure(&devc, 0);	/* arbitrary, but not transparent */
+    set_nonclient_dev_color(&devc, 0);	/* arbitrary, but not transparent */
     gs_set_logical_op_inline(pis, lop_default);
-    params.rule = rule;
-    params.adjust.x = params.adjust.y = fixed_half;
-    params.flatness = gs_currentflat_inline(pis);
-    params.fill_zero_width = true;
+    if (params0 != 0)
+	params = *params0;
+    else {
+	params.rule = rule;
+	params.adjust.x = params.adjust.y = fixed_half;
+	params.flatness = gs_currentflat_inline(pis);
+	params.fill_zero_width = true;
+    }
     code = gx_fill_path_only(ppath, (gx_device *)&adev, pis,
 			     &params, &devc, pcpath);
     if (code < 0 || (code = gx_cpath_accum_end(&adev, pcpath)) < 0)
@@ -183,7 +217,7 @@ gx_cpath_intersect_path_slow(gx_clip_path * pcpath, gx_path * ppath,
 
 #ifdef DEBUG
 /* Validate a clipping path after accumulation. */
-private bool
+static bool
 clip_list_validate(const gx_clip_list * clp)
 {
     if (clp->count <= 1)
@@ -213,8 +247,8 @@ clip_list_validate(const gx_clip_list * clp)
 #endif /* DEBUG */
 
 /* Initialize the accumulation device. */
-private int
-accum_open(register gx_device * dev)
+int
+accum_open_device(register gx_device * dev)
 {
     gx_device_cpath_accum * const adev = (gx_device_cpath_accum *)dev;
 
@@ -227,7 +261,7 @@ accum_open(register gx_device * dev)
 }
 
 /* Close the accumulation device. */
-private int
+static int
 accum_close(gx_device * dev)
 {
     gx_device_cpath_accum * const adev = (gx_device_cpath_accum *)dev;
@@ -256,6 +290,21 @@ accum_close(gx_device * dev)
     return 0;
 }
 
+
+/*
+   The pattern management device method.
+   See gxdevcli.h about return codes.
+ */
+int
+accum_pattern_manage(gx_device *pdev1, gx_bitmap_id id,
+		gs_pattern1_instance_t *pinst, pattern_manage_t function)
+{   
+    if (function == pattern_manage__is_cpath_accum)
+	return 1;
+    return 0;
+}
+
+
 /* Accumulate one rectangle. */
 /* Allocate a rectangle to be added to the list. */
 static const gx_clip_rect clip_head_rect = {
@@ -264,7 +313,7 @@ static const gx_clip_rect clip_head_rect = {
 static const gx_clip_rect clip_tail_rect = {
     0, 0, max_int, max_int, max_int, max_int
 };
-private gx_clip_rect *
+static gx_clip_rect *
 accum_alloc_rect(gx_device_cpath_accum * adev)
 {
     gs_memory_t *mem = adev->list_memory;
@@ -343,7 +392,7 @@ accum_alloc_rect(gx_device_cpath_accum * adev)
  * rectangle, we take special care to merge Y-adjacent rectangles when
  * this is possible.
  */
-private int
+static int
 accum_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 		     gx_color_index color)
 {
