@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxccman.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gxccman.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Character cache management routines for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -47,21 +47,21 @@ gs_private_st_simple(st_font_cache_bytes, byte, "font cache bytes");
 /* GC procedures */
 /* We do all the work in font_dir_enum/reloc_ptrs in gsfont.c. */
 /* See gxfcache.h for details. */
-private 
+static 
 ENUM_PTRS_BEGIN(cc_ptr_enum_ptrs) return 0;
 
 ENUM_PTRS_END
-private RELOC_PTRS_BEGIN(cc_ptr_reloc_ptrs)
+static RELOC_PTRS_BEGIN(cc_ptr_reloc_ptrs)
 {
 }
 RELOC_PTRS_END
 
 /* Forward references */
-private gx_xfont * lookup_xfont_by_name(gx_device *, const gx_xfont_procs *, gs_font_name *, int, const cached_fm_pair *, const gs_matrix *);
-private cached_char *alloc_char(gs_font_dir *, ulong);
-private cached_char *alloc_char_in_chunk(gs_font_dir *, ulong);
-private void hash_remove_cached_char(gs_font_dir *, uint);
-private void shorten_cached_char(gs_font_dir *, cached_char *, uint);
+static gx_xfont * lookup_xfont_by_name(gx_device *, const gx_xfont_procs *, gs_font_name *, int, const cached_fm_pair *, const gs_matrix *);
+static int alloc_char(gs_font_dir *, ulong, cached_char **);
+static int alloc_char_in_chunk(gs_font_dir *, ulong, cached_char **);
+static void hash_remove_cached_char(gs_font_dir *, uint);
+static void shorten_cached_char(gs_font_dir *, cached_char *, uint);
 
 /* ====== Initialization ====== */
 
@@ -148,7 +148,11 @@ gx_purge_selected_cached_chars(gs_font_dir * dir,
     for (chi = 0; chi <= cmax;) {
 	cached_char *cc = dir->ccache.table[chi];
 
-	if (cc != 0 && (*proc) (dir->memory, cc, proc_data)) {
+	if (cc != 0 &&
+#ifdef GSLITE
+		!cc->dont_evict &&
+#endif
+		(*proc) (dir->memory, cc, proc_data)) {
 	    hash_remove_cached_char(dir, chi);
 	    gx_free_cached_char(dir, cc);
 	} else
@@ -158,7 +162,7 @@ gx_purge_selected_cached_chars(gs_font_dir * dir,
 
 /* ====== font-matrix pair lists ====== */
 
-private int
+static int
 fm_pair_remove_from_list(gs_font_dir * dir, cached_fm_pair *pair, uint *head)
 {
     if (dir->fmcache.mdata + pair->index != pair)
@@ -184,7 +188,7 @@ fm_pair_remove_from_list(gs_font_dir * dir, cached_fm_pair *pair, uint *head)
     return 0;
 }
 
-private int
+static int
 fm_pair_insert_into_list(gs_font_dir * dir, cached_fm_pair *pair, uint *head)
 {
     if (dir->fmcache.mdata + pair->index != pair)
@@ -209,6 +213,64 @@ fm_pair_insert_into_list(gs_font_dir * dir, cached_fm_pair *pair, uint *head)
 
 /* ====== Font-level routines ====== */
 
+static int 
+gx_attach_tt_interpreter(gs_font_dir * dir,
+	       gs_font_type42 *font, cached_fm_pair *pair,
+	       const gs_matrix * char_tm, const gs_log2_scale_point *log2_scale,
+	       bool design_grid)
+{
+    float cxx, cxy, cyx, cyy;
+    gs_matrix m;
+    int code;
+
+    gx_compute_char_matrix(char_tm, log2_scale, &cxx, &cxy, &cyx, &cyy);
+    pair->design_grid = design_grid;
+    m.xx = cxx;
+    m.xy = cxy;
+    m.yx = cyx;
+    m.yy = cyy;
+    m.tx = m.ty = 0;
+    pair->ttr = gx_ttfReader__create(dir->memory->stable_memory);
+    if (!pair->ttr)
+	return_error(gs_error_VMerror);
+    /*  We could use a single the reader instance for all fonts ... */
+    pair->ttf = ttfFont__create(dir);
+    if (!pair->ttf)
+	return_error(gs_error_VMerror);
+    gx_ttfReader__set_font(pair->ttr, (gs_font_type42 *)font);
+    code = ttfFont__Open_aux(pair->ttf, dir->tti, pair->ttr, 
+		(gs_font_type42 *)font, &m, log2_scale, design_grid);
+    gx_ttfReader__set_font(pair->ttr, NULL);
+    return code;
+}
+
+static inline bool
+does_font_need_tt_interpreter(gs_font *font)
+{
+    if (font->FontType == ft_TrueType || font->FontType == ft_CID_TrueType) {
+	gs_font_type42 *pfont = (gs_font_type42 *)font;
+
+	if (pfont->FAPI==NULL)
+	    return true;
+    }
+    return false;
+}
+
+int 
+gx_provide_fm_pair_attributes(gs_font_dir * dir,
+	       gs_font *font, cached_fm_pair *pair,
+	       const gs_matrix * char_tm, const gs_log2_scale_point *log2_scale,
+	       bool design_grid)
+{
+    if (does_font_need_tt_interpreter(font)) {
+	if (pair->ttf != NULL)
+	    return 0; /* Already attached. */
+	return gx_attach_tt_interpreter(dir, (gs_font_type42 *)font, pair,
+			char_tm, log2_scale, design_grid);
+    }
+    return 0;
+}
+
 /* Add a font/matrix pair to the cache. */
 /* (This is only exported for gxccache.c.) */
 int
@@ -255,12 +317,21 @@ gx_add_fm_pair(register gs_font_dir * dir, gs_font * font, const gs_uid * puid,
 	pair = dir->fmcache.mdata + dir->fmcache.unused;
 	dir->fmcache.unused++;
     }
+    font->is_cached = true; /* Set this early to ensure 
+	    gs_purge_font_from_char_caches works for it in case of errors. */
     dir->fmcache.msize++;
     code = fm_pair_insert_into_list(dir, pair, &dir->fmcache.used);
     if (code < 0)
 	return code;
     pair->font = font;
     pair->UID = *puid;
+    /* Copy UID into a stable memory,
+       so that 'restore' may keep this pair. */
+    code = uid_copy(&pair->UID, dir->memory->stable_memory, "gx_add_fm_pair");
+    if (code < 0) {
+	uid_set_invalid(&pair->UID);
+	return code;
+    }
     pair->FontType = font->FontType;
     /* The OSF/1 compiler doesn't like casting a pointer to */
     /* a shorter int.... */
@@ -273,29 +344,9 @@ gx_add_fm_pair(register gs_font_dir * dir, gs_font * font, const gs_uid * puid,
     pair->ttf = 0;
     pair->ttr = 0;
     pair->design_grid = false;
-    if (font->FontType == ft_TrueType || font->FontType == ft_CID_TrueType) 
-	if (((gs_font_type42 *)font)->FAPI==NULL) {
-	    float cxx, cxy, cyx, cyy;
-	    gs_matrix m;
-	    gx_compute_char_matrix(char_tm, log2_scale, &cxx, &cxy, &cyx, &cyy);
-
-	    pair->design_grid = design_grid;
-	    m.xx = cxx;
-	    m.xy = cxy;
-	    m.yx = cyx;
-	    m.yy = cyy;
-	    m.tx = m.ty = 0;
-	    pair->ttr = gx_ttfReader__create(dir->memory);
-	    if (!pair->ttr)
-		return_error(gs_error_VMerror);
-	    /*  We could use a single the reader instance for all fonts ... */
-	    pair->ttf = ttfFont__create(dir);
-	    if (!pair->ttf)
-		return_error(gs_error_VMerror);
-	    gx_ttfReader__set_font(pair->ttr, (gs_font_type42 *)font);
-	    code = ttfFont__Open_aux(pair->ttf, dir->tti, pair->ttr, 
-			(gs_font_type42 *)font, &m, log2_scale, design_grid);
-	    gx_ttfReader__set_font(pair->ttr, NULL);
+    if (does_font_need_tt_interpreter(font)) {
+	    code = gx_attach_tt_interpreter(dir, (gs_font_type42 *)font, pair,
+				char_tm, log2_scale, design_grid);
 	    if (code < 0)
 		return code;
 	}
@@ -392,17 +443,37 @@ gx_lookup_xfont(const gs_state * pgs, cached_fm_pair * pair, int encoding_index)
 /* Purge from the caches all references to a given font/matrix pair, */
 /* or just characters that depend on its xfont. */
 #define cpair ((cached_fm_pair *)vpair)
-private bool
+static bool
 purge_fm_pair_char(const gs_memory_t *mem, cached_char * cc, void *vpair)
 {
     return cc_pair(cc) == cpair;
 }
-private bool
+static bool
 purge_fm_pair_char_xfont(const gs_memory_t *mem, cached_char * cc, void *vpair)
 {
     return cc_pair(cc) == cpair && cpair->xfont == 0 && !cc_has_bits(cc);
 }
 #undef cpair
+
+static inline void
+gs_clean_fm_pair_attributes(gs_font_dir * dir, cached_fm_pair * pair)
+{
+    if (pair->ttr)
+	gx_ttfReader__destroy(pair->ttr);
+    pair->ttr = 0;
+    if (pair->ttf)
+	ttfFont__destroy(pair->ttf, dir);
+    pair->ttf = 0;
+}
+
+void
+gs_clean_fm_pair(gs_font_dir * dir, cached_fm_pair * pair)
+{
+    if_debug1('k', "[k]cleaning pair 0x%lx\n", (ulong) pair);
+    pair->font = NULL;
+    gs_clean_fm_pair_attributes(dir, pair);
+}
+
 int
 gs_purge_fm_pair(gs_font_dir * dir, cached_fm_pair * pair, int xfont_only)
 {
@@ -418,12 +489,7 @@ gs_purge_fm_pair(gs_font_dir * dir, cached_fm_pair * pair, int xfont_only)
 				   (xfont_only ? purge_fm_pair_char_xfont :
 				    purge_fm_pair_char),
 				   pair);
-    if (pair->ttr)
-	gx_ttfReader__destroy(pair->ttr);
-    pair->ttr = 0;
-    if (pair->ttf)
-	ttfFont__destroy(pair->ttf, dir);
-    pair->ttf = 0;
+    gs_clean_fm_pair_attributes(dir, pair);
     if (!xfont_only) {
 	int code;
 
@@ -433,6 +499,11 @@ gs_purge_fm_pair(gs_font_dir * dir, cached_fm_pair * pair, int xfont_only)
 		     pair->num_chars);
 	}
 #endif
+	{   /* Free xvalues here because gx_add_fm_pair copied 
+	       them into the stable memory dir->memory. */
+	    gs_free_object(dir->memory->stable_memory, pair->UID.xvalues, "gs_purge_fm_pair");
+	    pair->UID.xvalues = 0;
+	}
 	fm_pair_set_free(pair);
 	code = fm_pair_remove_from_list(dir, pair, &dir->fmcache.used);
 	if (code < 0)
@@ -448,7 +519,7 @@ gs_purge_fm_pair(gs_font_dir * dir, cached_fm_pair * pair, int xfont_only)
 /* Look up an xfont by name. */
 /* The caller must already have done get_xfont_device to get the proper */
 /* device to pass as the first argument to lookup_font. */
-private gx_xfont *
+static gx_xfont *
 lookup_xfont_by_name(gx_device * fdev, const gx_xfont_procs * procs,
       gs_font_name * pfstr, int encoding_index, const cached_fm_pair * pair,
 		     const gs_matrix * pmat)
@@ -482,27 +553,34 @@ lookup_xfont_by_name(gx_device * fdev, const gx_xfont_procs * procs,
  * not NULL, dev should be an alpha-buffer device with dev2 (an alpha
  * device) as target.
  */
-cached_char *
+int 
 gx_alloc_char_bits(gs_font_dir * dir, gx_device_memory * dev,
 		   gx_device_memory * dev2, ushort iwidth, ushort iheight,
-		   const gs_log2_scale_point * pscale, int depth)
+		   const gs_log2_scale_point * pscale, int depth, cached_char **pcc)
 {
     int log2_xscale = pscale->x;
     int log2_yscale = pscale->y;
     int log2_depth = ilog2(depth);
     uint nwidth_bits = (iwidth >> log2_xscale) << log2_depth;
-    ulong isize, icdsize;
+    ulong isize, icdsize, isize2;
     uint iraster;
     cached_char *cc;
     gx_device_memory mdev;
     gx_device_memory *pdev = dev;
-    gx_device_memory *pdev2;
-
+    gx_device_memory *pdev2;  
+    float HWResolution0 = 72, HWResolution1 = 72;  /* default for dev == NULL */
+    int code;
+    
+    *pcc = 0;
     if (dev == NULL) {
 	mdev.memory = 0;
 	mdev.target = 0;
 	pdev = &mdev;
+    } else {
+	HWResolution0 = dev->HWResolution[0];
+	HWResolution1 = dev->HWResolution[1];
     }
+
     pdev2 = (dev2 == 0 ? pdev : dev2);
 
     /* Compute the scaled-down bitmap size, and test against */
@@ -535,7 +613,9 @@ gx_alloc_char_bits(gs_font_dir * dir, gx_device_memory * dev,
 	pdev->retained = retained;
 	pdev->width = iwidth;
 	pdev->height = iheight;
-	isize = gdev_mem_bitmap_size(pdev);
+	gdev_mem_bitmap_size(pdev, &isize);	/* Assume less than max_ulong */
+	pdev->HWResolution[0] = HWResolution0;
+	pdev->HWResolution[1] = HWResolution1;
     } else {
 	/* Use an alpha-buffer device to compress as we go. */
 	/* Preserve the reference counts, if any. */
@@ -552,11 +632,17 @@ gx_alloc_char_bits(gs_font_dir * dir, gx_device_memory * dev,
 	dev->rc = rc;
 	dev->width = iwidth;
 	dev->height = 2 << log2_yscale;
-	isize = gdev_mem_bitmap_size(dev) +
-	    gdev_mem_bitmap_size(dev2);
+	gdev_mem_bitmap_size(dev, &isize);	/* Assume less than max_ulong */
+	gdev_mem_bitmap_size(dev2, &isize2);	/* Assume less than max_ulong */
+	isize += isize2;	/* Assume less than max_ulong */
+	dev->HWResolution[0] = HWResolution0 * (1 >> log2_xscale);
+	dev->HWResolution[1] = HWResolution1 * (1 >> log2_yscale);
     }
     icdsize = isize + sizeof_cached_char;
-    cc = alloc_char(dir, icdsize);
+    code = alloc_char(dir, icdsize, &cc);
+    if (code < 0)
+	return code;
+    *pcc = cc;
     if (cc == 0)
 	return 0;
     if_debug4('k', "[k]adding char 0x%lx:%u(%u,%u)\n",
@@ -583,8 +669,9 @@ gx_alloc_char_bits(gs_font_dir * dir, gx_device_memory * dev,
     if (dev2) {			/* The second device is an alpha device that targets */
 	/* the real storage for the character. */
 	byte *bits = cc_bits(cc);
-	uint bsize = (uint) gdev_mem_bitmap_size(dev2);
+	ulong bsize;
 
+	gdev_mem_bitmap_size(dev2, &bsize);
 	memset(bits, 0, bsize);
 	dev2->base = bits;
 	(*dev_proc(dev2, open_device)) ((gx_device *) dev2);
@@ -593,7 +680,7 @@ gx_alloc_char_bits(gs_font_dir * dir, gx_device_memory * dev,
     } else if (dev)
 	gx_open_cache_device(dev, cc);
 
-    return cc;
+    return 0;
 }
 
 /* Open the cache device. */
@@ -601,10 +688,13 @@ void
 gx_open_cache_device(gx_device_memory * dev, cached_char * cc)
 {
     byte *bits = cc_bits(cc);
+    ulong bsize;
+
+    gdev_mem_bitmap_size(dev, &bsize);
 
     dev->width = cc->width;
     dev->height = cc->height;
-    memset((char *)bits, 0, (uint) gdev_mem_bitmap_size(dev));
+    memset((char *)bits, 0, bsize);
     dev->base = bits;
     (*dev_proc(dev, open_device)) ((gx_device *) dev);	/* initialize */
 }
@@ -802,18 +892,27 @@ gx_add_char_bits(gs_font_dir * dir, cached_char * cc,
 }
 
 /* Purge from the caches all references to a given font. */
-int
-gs_purge_font_from_char_caches(gs_font_dir * dir, const gs_font * font)
+static int
+gs_purge_font_from_char_caches_forced(gs_font * font, bool force)
 {
-    cached_fm_pair *pair = dir->fmcache.mdata;
-    int count = dir->fmcache.mmax;
+    gs_font_dir * dir;
+    cached_fm_pair *pair;
+    int count;
 
+    if (font->dir == NULL)
+	return 0; /* The font was not properly build due to errors. */
+    if (!font->is_cached)
+	return 0;
+    dir = font->dir;
+    pair = dir->fmcache.mdata;
+    count = dir->fmcache.mmax;
+    font->is_cached = false; /* Prevent redundant execution. */
     if_debug1('k', "[k]purging font 0x%lx\n",
 	      (ulong) font);
-    while (count--) {
+    for (; count--; pair++) {
 	if (pair->font == font) {
-	    if (uid_is_valid(&pair->UID)) {	/* Keep the entry. */
-		pair->font = 0;
+	    if (!force && uid_is_valid(&pair->UID)) {	/* Keep the entry. */
+		gs_clean_fm_pair(dir, pair);
 	    } else {
 		int code = gs_purge_fm_pair(dir, pair, 0);
 
@@ -821,19 +920,62 @@ gs_purge_font_from_char_caches(gs_font_dir * dir, const gs_font * font)
 		    return code;
 	    }
 	}
-	pair++;
     }
     return 0;
 }
 
+/* Purge from the caches all references to a given font,
+   with leaving persistent chars in the cache. */
+int
+gs_purge_font_from_char_caches(gs_font * font)
+{
+    /* This function is called when a font is being released.
+       The purpose is to remove all cache attributes,
+       which may point to the font data.
+       Note : when a font has a valid XUID, 
+       it doesn't release cache entries and cached chars,
+       so that they may be used in future 
+       if a font with same XUID appears again.
+       All this improves the performance when
+       a document executes a sequence like this :
+
+       n {
+          save /fontname findfont 10 scalefont
+	  (xyz) show
+	  restore
+       } repeat
+     */
+    return gs_purge_font_from_char_caches_forced(font, false);
+}
+
+/* Purge from the caches all references to a given font,
+   without leaving persistent chars in the cache. */
+int
+gs_purge_font_from_char_caches_completely(gs_font * font)
+{
+    /* A client should call this finction
+       when it frees a font,
+       and the client doesn't need to leave 
+       persistent cache entries for this font
+       even if the font has a valid XUID.
+     */
+    return gs_purge_font_from_char_caches_forced(font, true);
+}
+
+
+
 /* ------ Internal routines ------ */
 
 /* Allocate data space for a cached character, adding a new chunk if needed. */
-private cached_char *
-alloc_char(gs_font_dir * dir, ulong icdsize)
+static int
+alloc_char(gs_font_dir * dir, ulong icdsize, cached_char **pcc)
 {				/* Try allocating at the current position first. */
-    cached_char *cc = alloc_char_in_chunk(dir, icdsize);
+    cached_char *cc;
+    int code = alloc_char_in_chunk(dir, icdsize, &cc);
 
+    *pcc = cc;
+    if (code < 0)
+	return code;
     if (cc == 0) {
 	if (dir->ccache.bspace < dir->ccache.bmax) {	/* Allocate another chunk. */
 	    gs_memory_t *mem = dir->ccache.bits_memory;
@@ -875,53 +1017,80 @@ alloc_char(gs_font_dir * dir, ulong icdsize)
 
 	    while ((dir->ccache.chunks = cck = cck->next) != cck_init) {
 		dir->ccache.cnext = 0;
-		cc = alloc_char_in_chunk(dir, icdsize);
-		if (cc != 0)
-		    return cc;
+		code = alloc_char_in_chunk(dir, icdsize, &cc);
+		if (code < 0)
+		    return code;
+		if (cc != 0) {
+		    *pcc = cc;
+		    return 0;
+		}
 	    }
 	}
 	dir->ccache.cnext = 0;
-	cc = alloc_char_in_chunk(dir, icdsize);
+	code = alloc_char_in_chunk(dir, icdsize, &cc);
+	if (code < 0)
+	    return code;
+	*pcc = cc;
     }
-    return cc;
+    return 0;
 }
 
 /* Allocate a character in the current chunk. */
-private cached_char *
-alloc_char_in_chunk(gs_font_dir * dir, ulong icdsize)
+static int
+alloc_char_in_chunk(gs_font_dir * dir, ulong icdsize, cached_char **pcc)
 {
     char_cache_chunk *cck = dir->ccache.chunks;
     cached_char_head *cch;
 
 #define cc ((cached_char *)cch)
 
+    *pcc = 0;
     while (gx_bits_cache_alloc((gx_bits_cache *) & dir->ccache,
 			       icdsize, &cch) < 0
 	) {
 	if (cch == 0) {		/* Not enough room to allocate in this chunk. */
 	    return 0;
-	} {			/* Free the character */
+	}
+#ifdef GSLITE
+	/* We shouldn't free because it's used. */
+	if (cc->dont_evict) {
+	    return 0;
+	}
+#endif
+	else {			/* Free the character */
 	    cached_fm_pair *pair = cc_pair(cc);
 
 	    if (pair != 0) {
 		uint chi = chars_head_index(cc->code, pair);
+		uint cnt = dir->ccache.table_mask + 1;
 
-		while (dir->ccache.table[chi & dir->ccache.table_mask] != cc)
+		while (dir->ccache.table[chi & dir->ccache.table_mask] != cc) {
 		    chi++;
+		    if (cnt-- == 0)
+			return_error(gs_error_unregistered); /* Must not happen. */
+		}
 		hash_remove_cached_char(dir, chi);
 	    }
+
 	    gx_free_cached_char(dir, cc);
 	}
     }
+
+#ifdef GSLITE
+    cc->dont_evict = 0;
+#endif
+
     cc->chunk = cck;
     cc->loc = (byte *) cc - cck->data;
-    return cc;
+    *pcc = cc;
+    return 0;
+
 #undef cc
 }
 
 /* Remove the cached_char at a given index in the hash table. */
 /* In order not to slow down lookup, we relocate following entries. */
-private void
+static void
 hash_remove_cached_char(gs_font_dir * dir, uint chi)
 {
     uint mask = dir->ccache.table_mask;
@@ -948,7 +1117,7 @@ hash_remove_cached_char(gs_font_dir * dir, uint chi)
 
 /* Shorten a cached character. */
 /* diff >= sizeof(cached_char_head). */
-private void
+static void
 shorten_cached_char(gs_font_dir * dir, cached_char * cc, uint diff)
 {
     gx_bits_cache_shorten((gx_bits_cache *) & dir->ccache, &cc->head,
@@ -956,3 +1125,18 @@ shorten_cached_char(gs_font_dir * dir, cached_char * cc, uint diff)
     if_debug2('K', "[K]shortening creates free block 0x%lx(%u)\n",
 	      (ulong) ((byte *) cc + cc->head.size), diff);
 }
+
+#ifdef GSLITE
+
+void gx_retain_cached_char(cached_char *cc)
+{
+    cc->dont_evict ++;
+}
+
+void gx_release_cached_char(cached_char *cc)
+{
+    cc->dont_evict --;
+}
+
+#endif
+
