@@ -1,20 +1,17 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxchar.c,v 1.47 2005/07/21 09:53:42 igor Exp $ */
+/* $Id: gxchar.c 8477 2008-01-07 18:43:02Z henrys $ */
 /* Default implementation of text writing */
 #include "gx.h"
 #include "memory_.h"
@@ -38,11 +35,11 @@
 
 /* Define whether or not to cache characters rotated by angles other than */
 /* multiples of 90 degrees. */
-private const bool CACHE_ROTATED_CHARS = true;
+static const bool CACHE_ROTATED_CHARS = true;
 
 /* Define the maximum size of a full temporary bitmap when rasterizing, */
 /* in bits (not bytes). */
-private const uint MAX_TEMP_BITMAP_BITS = 80000;
+static const uint MAX_TEMP_BITMAP_BITS = 80000;
 
 /* Define whether the show operation uses the character outline data, */
 /* as opposed to just needing the width (or nothing). */
@@ -53,14 +50,14 @@ private const uint MAX_TEMP_BITMAP_BITS = 80000;
 public_st_gs_show_enum();
 extern_st(st_gs_text_enum);
 extern_st(st_gs_state);		/* only for testing */
-private 
+static 
 ENUM_PTRS_BEGIN(show_enum_enum_ptrs)
      return ENUM_USING(st_gs_text_enum, vptr, size, index - 5);
 ENUM_PTR(0, gs_show_enum, pgs);
 ENUM_PTR(1, gs_show_enum, show_gstate);
 ENUM_PTR3(2, gs_show_enum, dev_cache, dev_cache2, dev_null);
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(show_enum_reloc_ptrs, gs_show_enum *eptr)
+static RELOC_PTRS_WITH(show_enum_reloc_ptrs, gs_show_enum *eptr)
 {
     RELOC_USING(st_gs_text_enum, vptr, size);		/* superclass */
     RELOC_VAR(eptr->pgs);
@@ -70,13 +67,13 @@ private RELOC_PTRS_WITH(show_enum_reloc_ptrs, gs_show_enum *eptr)
 RELOC_PTRS_END
 
 /* Forward declarations */
-private int continue_kshow(gs_show_enum *);
-private int continue_show(gs_show_enum *);
-private int continue_show_update(gs_show_enum *);
-private void show_set_scale(const gs_show_enum *, gs_log2_scale_point *log2_scale);
-private int show_cache_setup(gs_show_enum *);
-private int show_state_setup(gs_show_enum *);
-private int show_origin_setup(gs_state *, fixed, fixed, gs_show_enum * penum);
+static int continue_kshow(gs_show_enum *);
+static int continue_show(gs_show_enum *);
+static int continue_show_update(gs_show_enum *);
+static void show_set_scale(const gs_show_enum *, gs_log2_scale_point *log2_scale);
+static int show_cache_setup(gs_show_enum *);
+static int show_state_setup(gs_show_enum *);
+static int show_origin_setup(gs_state *, fixed, fixed, gs_show_enum * penum);
 
 /* Accessors for current_char and current_glyph. */
 #define CURRENT_CHAR(penum) ((penum)->returned.current_char)
@@ -112,15 +109,15 @@ gs_show_enum_alloc(gs_memory_t * mem, gs_state * pgs, client_name_t cname)
 
 /* ------ Driver procedure ------ */
 
-private text_enum_proc_resync(gx_show_text_resync);
-private text_enum_proc_process(gx_show_text_process);
-private text_enum_proc_is_width_only(gx_show_text_is_width_only);
-private text_enum_proc_current_width(gx_show_text_current_width);
-private text_enum_proc_set_cache(gx_show_text_set_cache);
-private text_enum_proc_retry(gx_show_text_retry);
-private text_enum_proc_release(gx_show_text_release); /* not default */
+static text_enum_proc_resync(gx_show_text_resync);
+static text_enum_proc_process(gx_show_text_process);
+static text_enum_proc_is_width_only(gx_show_text_is_width_only);
+static text_enum_proc_current_width(gx_show_text_current_width);
+static text_enum_proc_set_cache(gx_show_text_set_cache);
+static text_enum_proc_retry(gx_show_text_retry);
+static text_enum_proc_release(gx_show_text_release); /* not default */
 
-private const gs_text_enum_procs_t default_text_procs = {
+static const gs_text_enum_procs_t default_text_procs = {
     gx_show_text_resync, gx_show_text_process,
     gx_show_text_is_width_only, gx_show_text_current_width,
     gx_show_text_set_cache, gx_show_text_retry,
@@ -186,7 +183,9 @@ gx_default_text_begin(gx_device * dev, gs_imager_state * pis,
     penum->show_gstate =
 	(propagate_charpath && (pgs->in_charpath != 0) ?
 	 pgs->show_gstate : pgs);
-    if (!(~operation & (TEXT_DO_NONE | TEXT_RETURN_WIDTH))) {
+    if((operation & 
+         (TEXT_DO_NONE | TEXT_RETURN_WIDTH | TEXT_RENDER_MODE_3)) == 
+         (TEXT_DO_NONE | TEXT_RETURN_WIDTH)) {
 	/* This is stringwidth. */
 	gx_device_null *dev_null =
 	    gs_alloc_struct(mem, gx_device_null, &st_device_null,
@@ -216,6 +215,40 @@ gx_default_text_begin(gx_device * dev, gs_imager_state * pis,
     return 0;
 }
 
+/* Compute the number of characters in a text. */
+int
+gs_text_count_chars(gs_state * pgs, gs_text_params_t *text, gs_memory_t * mem)
+{
+    font_proc_next_char_glyph((*next_proc)) = pgs->font->procs.next_char_glyph;
+
+    if (next_proc == gs_default_next_char_glyph)
+	return text->size;
+    else {
+	/* Do it the hard way. */
+	gs_text_enum_t tenum;	/* use a separate enumerator */
+	gs_char tchr;
+	gs_glyph tglyph;
+	int size = 0;
+	int code;
+
+	size = 0;
+
+        code = gs_text_enum_init(&tenum, &default_text_procs,
+			     NULL, NULL, text, pgs->root_font, 
+			     NULL, NULL, NULL, mem);
+	if (code < 0)
+	    return code;
+	while ((code = (*next_proc)(&tenum, &tchr, &tglyph)) != 2) {
+	    if (code < 0)
+		break;
+	    ++size;
+	}
+	if (code < 0)
+	    return code;
+	return size;
+    }
+}
+
 /* An auxiliary functions for pdfwrite to process type 3 fonts. */
 int
 gx_hld_stringwidth_begin(gs_imager_state * pis, gx_path **path)
@@ -249,18 +282,25 @@ gx_default_text_restore_state(gs_text_enum_t *pte)
 }
 /* ------ Width/cache setting ------ */
 
-private int
+static int
     set_cache_device(gs_show_enum *penum, gs_state *pgs,
 		     floatp llx, floatp lly, floatp urx, floatp ury);
 
 /* This is the default implementation of text enumerator set_cache. */
-private int
+static int
 gx_show_text_set_cache(gs_text_enum_t *pte, const double *pw,
 			  gs_text_cache_control_t control)
 {
     gs_show_enum *const penum = (gs_show_enum *)pte;
     gs_state *pgs = penum->pgs;
+    gs_font *pfont = gs_rootfont(pgs);
 
+    /* Detect zero FontMatrix now for Adobe compatibility with CET tests.
+       Note that matrixe\\ces like [1 0 0 0 0 0] are used in comparefiles
+       to compute a text width. See also gs_text_begin. */
+    if (pfont->FontMatrix.xx == 0 && pfont->FontMatrix.xy == 0 &&
+	pfont->FontMatrix.yx == 0 && pfont->FontMatrix.yy == 0)
+	return_error(gs_error_undefinedresult); /* sic! : CPSI compatibility */
     switch (control) {
     case TEXT_SET_CHAR_WIDTH:
 	return set_char_width(penum, pgs, pw[0], pw[1]);
@@ -277,7 +317,7 @@ gx_show_text_set_cache(gs_text_enum_t *pte, const double *pw,
 	int code;
 	bool retry = (penum->width_status == sws_retry);
 
-	if (gs_rootfont(pgs)->WMode) {
+	if (pfont->WMode) {
 	    float vx = pw[8], vy = pw[9];
 	    gs_fixed_point pvxy, dvxy;
 
@@ -354,7 +394,17 @@ set_char_width(gs_show_enum *penum, gs_state *pgs, floatp wx, floatp wy)
 	wx = p.x; 
 	wy = p.y;
     }
-    if ((code = gs_distance_transform2fixed(&pgs->ctm, wx, wy, &penum->wxy)) < 0)
+    code = gs_distance_transform2fixed(&pgs->ctm, wx, wy, &penum->wxy);
+    if (code < 0 && penum->cc == 0) {
+	/* Can't represent in 'fixed', use floats. */
+	code = gs_distance_transform(wx, wy, &ctm_only(pgs), &penum->wxy_float);
+	penum->wxy.x = penum->wxy.y = 0;
+	penum->use_wxy_float = true;
+    } else {
+	penum->use_wxy_float = false;
+	penum->wxy_float.x = penum->wxy_float.y = 0;
+    }
+    if (code < 0)
 	return code;
     /* Check whether we're setting the scalable width */
     /* for a cached xfont character. */
@@ -407,7 +457,7 @@ gx_compute_text_oversampling(const gs_show_enum * penum, const gs_font *pfont,
 }
 
 /* Compute glyph raster parameters */
-private int
+static int
 compute_glyph_raster_params(gs_show_enum *penum, bool in_setcachedevice, int *alpha_bits, 
 		    int *depth,
                     gs_fixed_point *subpix_origin, gs_log2_scale_point *log2_scale)
@@ -460,7 +510,7 @@ compute_glyph_raster_params(gs_show_enum *penum, bool in_setcachedevice, int *al
 /* Set up the cache device if relevant. */
 /* Return 1 if we just set up a cache device. */
 /* Used by setcachedevice and setcachedevice2. */
-private int
+static int
 set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
 		 floatp urx, floatp ury)
 {
@@ -559,8 +609,8 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
 	    cdim.y > max_cdim[log2_scale.y]
 	    )
 	    return 0;		/* much too big */
-	iwidth = ((ushort) fixed2int_var(cdim.x) + 2) << log2_scale.x;
-	iheight = ((ushort) fixed2int_var(cdim.y) + 2) << log2_scale.y;
+	iwidth = ((ushort) fixed2int_var(cdim.x) + 3) << log2_scale.x;
+	iheight = ((ushort) fixed2int_var(cdim.y) + 3) << log2_scale.y;
 	if_debug3('k', "[k]iwidth=%u iheight=%u dev_cache %s\n",
 		  (uint) iwidth, (uint) iheight,
 		  (penum->dev_cache == 0 ? "not set" : "set"));
@@ -576,11 +626,11 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
 	 * from oversampled bitmap strips to alpha values instead of
 	 * full oversampling with compression at the end.
 	 */
-	cc = gx_alloc_char_bits(dir, penum->dev_cache,
+	code = gx_alloc_char_bits(dir, penum->dev_cache,
 				(iwidth > MAX_TEMP_BITMAP_BITS / iheight &&
 				 log2_scale.x + log2_scale.y > alpha_bits ?
 				 penum->dev_cache2 : NULL),
-				iwidth, iheight, &log2_scale, depth);
+				iwidth, iheight, &log2_scale, depth, &cc);
 	if (cc == 0) {
 	    /* too big for cache or no cache */
 	    gx_path box_path;
@@ -610,8 +660,8 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
 	}
 	/* The mins handle transposed coordinate systems.... */
 	/* Truncate the offsets to avoid artifacts later. */
-	cc->offset.x = fixed_ceiling(-cll.x);
-	cc->offset.y = fixed_ceiling(-cll.y);
+	cc->offset.x = fixed_ceiling(-cll.x) + fixed_1;
+	cc->offset.y = fixed_ceiling(-cll.y) + fixed_1;
 	if_debug4('k', "[k]width=%u, height=%u, offset=[%g %g]\n",
 		  (uint) iwidth, (uint) iheight,
 		  fixed2float(cc->offset.x),
@@ -672,7 +722,7 @@ gs_incachedevice(const gs_state *pgs)
 /*
  * Set the encode_char procedure in an enumerator.
  */
-private void
+static void
 show_set_encode_char(gs_show_enum * penum)
 {
     penum->encode_char =
@@ -685,7 +735,7 @@ show_set_encode_char(gs_show_enum * penum)
  * Resync a text operation with a different set of parameters.
  * Currently this is implemented only for changing the data source.
  */
-private int
+static int
 gx_show_text_resync(gs_text_enum_t *pte, const gs_text_enum_t *pfrom)
 {
     gs_show_enum *const penum = (gs_show_enum *)pte;
@@ -702,7 +752,7 @@ gx_show_text_resync(gs_text_enum_t *pte, const gs_text_enum_t *pfrom)
 }
 
 /* Do the next step of a show (or stringwidth) operation */
-private int
+static int
 gx_show_text_process(gs_text_enum_t *pte)
 {
     gs_show_enum *const penum = (gs_show_enum *)pte;
@@ -711,11 +761,11 @@ gx_show_text_process(gs_text_enum_t *pte)
 }
 
 /* Continuation procedures */
-private int show_update(gs_show_enum * penum);
-private int show_move(gs_show_enum * penum);
-private int show_proceed(gs_show_enum * penum);
-private int show_finish(gs_show_enum * penum);
-private int
+static int show_update(gs_show_enum * penum);
+static int show_move(gs_show_enum * penum);
+static int show_proceed(gs_show_enum * penum);
+static int show_finish(gs_show_enum * penum);
+static int
 continue_show_update(gs_show_enum * penum)
 {
     int code = show_update(penum);
@@ -727,14 +777,14 @@ continue_show_update(gs_show_enum * penum)
 	return code;
     return show_proceed(penum);
 }
-private int
+static int
 continue_show(gs_show_enum * penum)
 {
     return show_proceed(penum);
 }
 /* For kshow, the CTM or font may have changed, so we have to reestablish */
 /* the cached values in the enumerator. */
-private int
+static int
 continue_kshow(gs_show_enum * penum)
 {   int code;
     gs_state *pgs = penum->pgs;
@@ -750,7 +800,7 @@ continue_kshow(gs_show_enum * penum)
 }
 
 /* Update position */
-private int
+static int
 show_update(gs_show_enum * penum)
 {
     gs_state *pgs = penum->pgs;
@@ -764,6 +814,8 @@ show_update(gs_show_enum * penum)
 	    /* Adobe interpreters assume a character width of 0, */
 	    /* even though the documentation says this is an error.... */
 	    penum->wxy.x = penum->wxy.y = 0;
+	    penum->wxy_float.x = penum->wxy_float.y = 0;
+	    penum->use_wxy_float = false;
 	    break;
 	case sws_cache:
 	    /* Finish installing the cache entry. */
@@ -772,6 +824,7 @@ show_update(gs_show_enum * penum)
 	    /* We have to check for this by comparing levels. */
 	    switch (pgs->level - penum->level) {
 		default:
+		    gx_free_cached_char(penum->orig_font->dir, penum->cc);
 		    return_error(gs_error_invalidfont);		/* WRONG */
 		case 2:
 		    code = gs_grestore(pgs);
@@ -786,8 +839,10 @@ show_update(gs_show_enum * penum)
 			    &penum->log2_scale, penum->charpath_flag != cpm_show, &pair);
 		if (code < 0)
 		    return code;
-		gx_add_cached_char(pgs->font->dir, penum->dev_cache,
+		code = gx_add_cached_char(pgs->font->dir, penum->dev_cache,
 			       cc, pair, &penum->log2_scale);
+		if (code < 0)
+		    return code;
 	    }
 	    if (!SHOW_USES_OUTLINE(penum) ||
 		penum->charpath_flag != cpm_show
@@ -818,7 +873,7 @@ show_update(gs_show_enum * penum)
 }
 
 /* Move to next character */
-private int
+static inline int
 show_fast_move(gs_state * pgs, gs_fixed_point * pwxy)
 {
     return gs_moveto_aux((gs_imager_state *)pgs, pgs->path,
@@ -854,15 +909,18 @@ int gx_current_char(const gs_text_enum_t * pte)
     return chr;
 }
 
-private int
+static int
 show_move(gs_show_enum * penum)
 {
     gs_state *pgs = penum->pgs;
 
     if (SHOW_IS(penum, TEXT_REPLACE_WIDTHS)) {
+	int code;
 	gs_point dpt;
 
-	gs_text_replaced_width(&penum->text, penum->xy_index - 1, &dpt);
+	code = gs_text_replaced_width(&penum->text, penum->xy_index - 1, &dpt);
+	if (code < 0)
+	    return code;
 	gs_distance_transform2fixed(&pgs->ctm, dpt.x, dpt.y, &penum->wxy);
     } else {
 	double dx = 0, dy = 0;
@@ -894,8 +952,14 @@ show_move(gs_show_enum * penum)
     }
     /* wxy is in device coordinates */
     {
-	int code = show_fast_move(pgs, &penum->wxy);
+	int code;
 
+	if (penum->use_wxy_float)
+	    code = gs_moveto_aux((gs_imager_state *)pgs, pgs->path,
+		    pgs->current_point.x + penum->wxy_float.x + fixed2float(penum->wxy.x), 
+		    pgs->current_point.y + penum->wxy_float.y + fixed2float(penum->wxy.y));
+	else
+	    code = show_fast_move(pgs, &penum->wxy);
 	if (code < 0)
 	    return code;
     }
@@ -907,7 +971,7 @@ show_move(gs_show_enum * penum)
     return 0;
 }
 /* Process next character */
-private int
+static int
 show_proceed(gs_show_enum * penum)
 {
     gs_state *pgs = penum->pgs;
@@ -1014,9 +1078,11 @@ show_proceed(gs_show_enum * penum)
 			    )
 			    goto no_cache;
 			if (pfont->BitmapWidths) {
-			    cc = gx_lookup_xfont_char(pgs, pair, chr,
-				     glyph, wmode);
-			    if (cc == 0)
+			    code = gx_lookup_xfont_char(pgs, pair, chr,
+				     glyph, wmode, &cc);
+			    if (code < 0)
+				return code;
+			    if (code == 0)
 				goto no_cache;
 			} else {
 			    if (!SHOW_USES_OUTLINE(penum) ||
@@ -1026,8 +1092,10 @@ show_proceed(gs_show_enum * penum)
 				goto no_cache;
 			    /* We might have an xfont, but we still */
 			    /* want the scalable widths. */
-			    cc = gx_lookup_xfont_char(pgs, pair, chr,
-				     glyph, wmode);
+			    code = gx_lookup_xfont_char(pgs, pair, chr,
+				     glyph, wmode, &cc);
+			    if (code < 0)
+				return code;
 			    /* Render up to the point of */
 			    /* setcharwidth or setcachedevice, */
 			    /* just as for stringwidth. */
@@ -1081,6 +1149,8 @@ show_proceed(gs_show_enum * penum)
 			    goto no_cache;
 			}
 		    }
+		    penum->use_wxy_float = false;
+		    penum->wxy_float.x = penum->wxy_float.y = 0;
 		    if (SHOW_IS_SLOW(penum)) {
 			/* Split up the assignment so that the */
 			/* Watcom compiler won't reserve esi/edi. */
@@ -1171,7 +1241,6 @@ show_proceed(gs_show_enum * penum)
 	/* Normally, char_tm is valid because of show_state_setup, */
 	/* but if we're in a cshow, it may not be. */
 	gs_currentcharmatrix(pgs, NULL, true);
-#if 1				/*USE_FPU <= 0 */
 	if (pgs->ctm.txy_fixed_valid && pgs->char_tm.txy_fixed_valid) {
 	    fixed tx = pgs->ctm.tx_fixed;
 	    fixed ty = pgs->ctm.ty_fixed;
@@ -1179,9 +1248,7 @@ show_proceed(gs_show_enum * penum)
 	    gs_settocharmatrix(pgs);
 	    cpt.x += pgs->ctm.tx_fixed - tx;
 	    cpt.y += pgs->ctm.ty_fixed - ty;
-	} else
-#endif
-	{
+	} else 	{
 	    double tx = pgs->ctm.tx;
 	    double ty = pgs->ctm.ty;
 	    double fpx, fpy;
@@ -1209,7 +1276,7 @@ show_proceed(gs_show_enum * penum)
     /* Try using the build procedure in the font. */
     /* < 0 means error, 0 means success, 1 means failure. */
     penum->cc = cc;		/* set this now for build procedure */
-    code = (*pfont->procs.build_char)((gs_text_enum_t *)penum, pgs, pfont,
+    code = (*pfont->procs.build_char)(penum, pgs, pfont,
 				      chr, glyph);
     if (code < 0) {
 	discard(gs_note_error(code));
@@ -1248,7 +1315,7 @@ show_proceed(gs_show_enum * penum)
  * Prepare to retry rendering of the current character.  (This is only used
  * in one place in zchar1.c; a different approach may be better.)
  */
-private int
+static int
 gx_show_text_retry(gs_text_enum_t *pte)
 {
     gs_show_enum *const penum = (gs_show_enum *)pte;
@@ -1267,25 +1334,34 @@ gx_show_text_retry(gs_text_enum_t *pte)
 }
 
 /* Finish show or stringwidth */
-private int
+static int
 show_finish(gs_show_enum * penum)
 {
     gs_state *pgs = penum->pgs;
-    int code, rcode;
 
+    if ((penum->text.operation & TEXT_DO_FALSE_CHARPATH) || 
+	(penum->text.operation & TEXT_DO_TRUE_CHARPATH)) {
+	if (pgs->path->current_subpath)
+	    pgs->path->last_charpath_segment = pgs->path->current_subpath->last;
+    }
     if (penum->auto_release)
 	penum->procs->release((gs_text_enum_t *)penum, "show_finish");
-    if (!SHOW_IS_STRINGWIDTH(penum))
-	return 0;
-    /* Save the accumulated width before returning, */
-    /* and undo the extra gsave. */
-    code = gs_currentpoint(pgs, &penum->returned.total_width);
-    rcode = gs_grestore(pgs);
-    return (code < 0 ? code : rcode);
+    
+    if((penum->text.operation & 
+         (TEXT_DO_NONE | TEXT_RETURN_WIDTH | TEXT_RENDER_MODE_3)) == 
+         (TEXT_DO_NONE | TEXT_RETURN_WIDTH)) {
+        /* Save the accumulated width before returning, */
+        /* and undo the extra gsave. */
+        int code = gs_currentpoint(pgs, &penum->returned.total_width);
+        int rcode = gs_grestore(pgs);
+
+        return (code < 0 ? code : rcode);
+    }
+    return 0;
 }
 
 /* Release the structure. */
-private void
+static void
 gx_show_text_release(gs_text_enum_t *pte, client_name_t cname)
 {
     gs_show_enum *const penum = (gs_show_enum *)pte;
@@ -1321,7 +1397,7 @@ gs_show_in_charpath(const gs_show_enum * penum)
 /* This is only meaningful just before calling gs_setcharwidth or */
 /* gs_setcachedevice[2]. */
 /* Note that we can't do this if the procedure has done any extra [g]saves. */
-private bool
+static bool
 gx_show_text_is_width_only(const gs_text_enum_t *pte)
 {
     const gs_show_enum *const penum = (const gs_show_enum *)pte;
@@ -1333,7 +1409,7 @@ gx_show_text_is_width_only(const gs_text_enum_t *pte)
 }
 
 /* Return the width of the just-enumerated character (for cshow). */
-private int
+static int
 gx_show_text_current_width(const gs_text_enum_t *pte, gs_point *pwidth)
 {
     const gs_show_enum *const penum = (const gs_show_enum *)pte;
@@ -1353,7 +1429,7 @@ gs_show_current_font(const gs_show_enum * penum)
 
 /* ------ Internal routines ------ */
 
-private inline bool
+static inline bool
 is_matrix_good_for_caching(const gs_matrix_fixed *m)
 {
     /* Skewing or non-rectangular rotation are not supported,
@@ -1373,7 +1449,7 @@ is_matrix_good_for_caching(const gs_matrix_fixed *m)
 /* We do this both when starting the show operation, */
 /* and when returning from the kshow callout. */
 /* Uses only penum->pgs, penum->fstack. */
-private int
+static int
 show_state_setup(gs_show_enum * penum)
 {
     gs_state *pgs = penum->pgs;
@@ -1421,14 +1497,12 @@ show_state_setup(gs_show_enum * penum)
 	penum->obox.p.y = fixed2int_var(cbox.p.y);
 	penum->obox.q.x = fixed2int_var_ceiling(cbox.q.x);
 	penum->obox.q.y = fixed2int_var_ceiling(cbox.q.y);
-#if 1				/*USE_FPU <= 0 */
 	if (pgs->ctm.txy_fixed_valid && pgs->char_tm.txy_fixed_valid) {
 	    penum->ftx = (int)fixed2long(pgs->char_tm.tx_fixed -
 					 pgs->ctm.tx_fixed);
 	    penum->fty = (int)fixed2long(pgs->char_tm.ty_fixed -
 					 pgs->ctm.ty_fixed);
 	} else {
-#endif
 	    double fdx = pgs->char_tm.tx - pgs->ctm.tx;
 	    double fdy = pgs->char_tm.ty - pgs->ctm.ty;
 
@@ -1447,7 +1521,7 @@ show_state_setup(gs_show_enum * penum)
 }
 
 /* Set the suggested oversampling scale for character rendering. */
-private void
+static void
 show_set_scale(const gs_show_enum * penum, gs_log2_scale_point *log2_scale)
 {
     /*
@@ -1498,7 +1572,7 @@ show_set_scale(const gs_show_enum * penum, gs_log2_scale_point *log2_scale)
 /* Set up the cache device and related information. */
 /* Note that we always allocate both cache devices, */
 /* even if we only use one of them. */
-private int
+static int
 show_cache_setup(gs_show_enum * penum)
 {
     gs_state *pgs = penum->pgs;
@@ -1525,6 +1599,8 @@ show_cache_setup(gs_show_enum * penum)
     penum->dev_cache = dev;
     gs_make_mem_mono_device(dev2, mem, gs_currentdevice_inline(pgs));
     penum->dev_cache2 = dev2;
+    dev->HWResolution[0] = pgs->device->HWResolution[0];
+    dev->HWResolution[1] = pgs->device->HWResolution[1];
     /* Retain these devices, since they are referenced from the enumerator. */
     gx_device_retain((gx_device *)dev, true);
     gx_device_retain((gx_device *)dev2, true);
@@ -1534,7 +1610,7 @@ show_cache_setup(gs_show_enum * penum)
 /* Set the character origin as the origin of the coordinate system. */
 /* Used before rendering characters, and for moving the origin */
 /* in setcachedevice2 when WMode=1. */
-private int
+static int
 show_origin_setup(gs_state * pgs, fixed cpt_x, fixed cpt_y, gs_show_enum * penum)
 {
     if (penum->charpath_flag == cpm_show) {
