@@ -1,20 +1,17 @@
-/* Copyright (C) 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclimag.c,v 1.12 2005/03/14 18:08:36 dan Exp $ */
+/*$Id: gxclimag.c 8803 2008-06-24 14:16:29Z leonardo $ */
 /* Higher-level image operations for band lists */
 #include "math_.h"
 #include "memory_.h"
@@ -37,6 +34,7 @@
 #include "gxcomp.h"
 #include "gsserial.h"
 #include "gxdhtserial.h"
+#include "gsptype1.h"
 
 extern_gx_image_type_table();
 
@@ -45,9 +43,9 @@ extern_gx_image_type_table();
 static const bool USE_HL_IMAGES = true;
 
 /* Forward references */
-private int cmd_put_set_data_x(gx_device_clist_writer * cldev,
+static int cmd_put_set_data_x(gx_device_clist_writer * cldev,
 			       gx_clist_state * pcls, int data_x);
-private bool check_rect_for_trivial_clip(
+static bool check_rect_for_trivial_clip(
     const gx_clip_path *pcpath,  /* May be NULL, clip to evaluate */
     int px, int py, int qx, int qy  /* corners of box to test */
 );
@@ -57,7 +55,7 @@ private bool check_rect_for_trivial_clip(
 int
 clist_fill_mask(gx_device * dev,
 		const byte * data, int data_x, int raster, gx_bitmap_id id,
-		int x, int y, int width, int height,
+		int rx, int ry, int rwidth, int rheight,
 		const gx_drawing_color * pdcolor, int depth,
 		gs_logical_operation_t lop, const gx_clip_path * pcpath)
 {
@@ -65,9 +63,9 @@ clist_fill_mask(gx_device * dev,
 	&((gx_device_clist *)dev)->writer;
     const byte *orig_data = data;	/* for writing tile */
     int orig_data_x = data_x;	/* ditto */
-    int orig_x = x;		/* ditto */
-    int orig_width = width;	/* ditto */
-    int orig_height = height;	/* ditto */
+    int orig_x = rx;		/* ditto */
+    int orig_width = rwidth;	/* ditto */
+    int orig_height = rheight;	/* ditto */
     int log2_depth = ilog2(depth);
     int y0;
     int data_x_bit;
@@ -77,14 +75,17 @@ clist_fill_mask(gx_device * dev,
     bool slow_rop =
 	cmd_slow_rop(dev, lop_know_S_0(lop), pdcolor) ||
 	cmd_slow_rop(dev, lop_know_S_1(lop), pdcolor);
+    cmd_rects_enum_t re;
 
     /* If depth > 1, this call will be translated to a copy_alpha call. */
     /* if the target device can't perform copy_alpha, exit now. */
     if (depth > 1 && (cdev->disable_mask & clist_disable_copy_alpha) != 0)
 	return_error(gs_error_unknownerror);
 
-    fit_copy(dev, data, data_x, raster, id, x, y, width, height);
-    y0 = y;			/* must do after fit_copy */
+    crop_copy(cdev, data, data_x, raster, id, rx, ry, rwidth, rheight);
+    if (rwidth <= 0 || rheight <= 0)
+	return 0;
+    y0 = ry;			/* must do after fit_copy */
 
     /* If non-trivial clipping & complex clipping disabled, default */
     /* Also default for uncached bitmap or non-defaul lop; */
@@ -93,48 +94,63 @@ clist_fill_mask(gx_device * dev,
     /* Lastly, the command list will translate calls with depth > 1 to */
     /* copy_alpha calls, so the device color must be pure */
     if (((cdev->disable_mask & clist_disable_complex_clip) &&
-	 !check_rect_for_trivial_clip(pcpath, x, y, x + width, y + height)) ||
+	 !check_rect_for_trivial_clip(pcpath, rx, ry, rx + rwidth, ry + rheight)) ||
 	gs_debug_c('`') || id == gx_no_bitmap_id || lop != lop_default ||
 	(depth > 1 && !color_writes_pure(pdcolor, lop))
 	)
   copy:
 	return gx_default_fill_mask(dev, data, data_x, raster, id,
-				    x, y, width, height, pdcolor, depth,
+				    rx, ry, rwidth, rheight, pdcolor, depth,
 				    lop, pcpath);
 
     if (cmd_check_clip_path(cdev, pcpath))
 	cmd_clear_known(cdev, clip_path_known);
     data_x_bit = data_x << log2_depth;
-    FOR_RECTS {
+    if (cdev->permanent_error < 0)
+      return (cdev->permanent_error);
+    RECT_ENUM_INIT(re, ry, rheight);
+    do {
 	int code;
-        ulong offset_temp;
+	ulong offset_temp;
 
-	TRY_RECT {
-	    code = cmd_update_lop(cdev, pcls, lop);
-	} HANDLE_RECT(code);
-	if (depth > 1 && !pcls->color_is_alpha) {
+	RECT_STEP_INIT(re);
+	do {
+	    code = cmd_update_lop(cdev, re.pcls, lop);
+	} while (RECT_RECOVER(code));
+	if (code < 0 && SET_BAND_CODE(code))
+	    goto error_in_rect;
+	if (depth > 1 && !re.pcls->color_is_alpha) {
 	    byte *dp;
 
-	    TRY_RECT {
+	    do {
 		code =
-		    set_cmd_put_op(dp, cdev, pcls, cmd_opv_set_copy_alpha, 1);
-	    } HANDLE_RECT(code);
-	    pcls->color_is_alpha = 1;
+		    set_cmd_put_op(dp, cdev, re.pcls, cmd_opv_set_copy_alpha, 1);
+	    } while (RECT_RECOVER(code));
+	    if (code < 0 && SET_BAND_CODE(code))
+			goto error_in_rect;
+	    re.pcls->color_is_alpha = 1;
 	}
-	TRY_RECT {
-	    code = cmd_do_write_unknown(cdev, pcls, clip_path_known);
+	do {
+	    code = cmd_do_write_unknown(cdev, re.pcls, clip_path_known);
 	    if (code >= 0)
-		code = cmd_do_enable_clip(cdev, pcls, pcpath != NULL);
-	} HANDLE_RECT(code);
-	TRY_RECT {
-	    code = cmd_put_drawing_color(cdev, pcls, pdcolor);
+			code = cmd_do_enable_clip(cdev, re.pcls, pcpath != NULL);
+	} while (RECT_RECOVER(code));
+	if (code < 0 && SET_BAND_CODE(code))
+	    goto error_in_rect;
+	do {
+	    code = cmd_put_drawing_color(cdev, re.pcls, pdcolor);
+	    if (code == gs_error_unregistered)
+		return code;
 	    if (depth > 1 && code >= 0)
-		code = cmd_set_color1(cdev, pcls, pdcolor->colors.pure);
-	} HANDLE_RECT(code);
-	pcls->colors_used.slow_rop |= slow_rop;
-
+			code = cmd_set_color1(cdev, re.pcls, pdcolor->colors.pure);
+	} while (RECT_RECOVER(code));
+	if (code < 0 && SET_BAND_CODE(code))
+	    goto error_in_rect;
+	re.pcls->colors_used.slow_rop |= slow_rop;
+	re.pcls->band_complexity.nontrivial_rops |= slow_rop;
+	re.pcls->band_complexity.uses_color |= (pdcolor->colors.pure != 0 && pdcolor->colors.pure != 0xffffff);
 	/* Put it in the cache if possible. */
-	if (!cls_has_tile_id(cdev, pcls, id, offset_temp)) {
+	if (!cls_has_tile_id(cdev, re.pcls, id, offset_temp)) {
 	    gx_strip_bitmap tile;
 
 	    tile.data = (byte *) orig_data;	/* actually const */
@@ -143,10 +159,11 @@ clist_fill_mask(gx_device * dev,
 	    tile.size.y = tile.rep_height = orig_height;
 	    tile.rep_shift = tile.shift = 0;
 	    tile.id = id;
-	    TRY_RECT {
-	        code = clist_change_bits(cdev, pcls, &tile, depth);
-	    } HANDLE_RECT_UNLESS(code,
-	        (code != gs_error_VMerror || !cdev->error_is_retryable) );
+	    do {
+	        code = clist_change_bits(cdev, re.pcls, &tile, depth);
+	    } while (RECT_RECOVER(code));
+	    if (code < 0 && !(code != gs_error_VMerror || !cdev->error_is_retryable) && SET_BAND_CODE(code))
+		goto error_in_rect;
 	    if (code < 0) {
 	        /* Something went wrong; just copy the bits. */
 	        goto copy;
@@ -160,15 +177,15 @@ clist_fill_mask(gx_device * dev,
 	    /* Output a command to copy the entire character. */
 	    /* It will be truncated properly per band. */
 	    rect.x = orig_x, rect.y = y0;
-	    rect.width = orig_width, rect.height = yend - y0;
+	    rect.width = orig_width, rect.height = re.yend - y0;
 	    rsize = 1 + cmd_sizexy(rect);
-	    TRY_RECT {
+	    do {
 	        code = (orig_data_x ?
-	      		cmd_put_set_data_x(cdev, pcls, orig_data_x) : 0);
+	      		cmd_put_set_data_x(cdev, re.pcls, orig_data_x) : 0);
 		if (code >= 0) {
 		    byte *dp;
 
-		    code = set_cmd_put_op(dp, cdev, pcls, op, rsize);
+		    code = set_cmd_put_op(dp, cdev, re.pcls, op, rsize);
 		    /*
 		     * The following conditional is unnecessary: the two
 		     * statements inside it should go outside the
@@ -182,14 +199,21 @@ clist_fill_mask(gx_device * dev,
 		        cmd_putxy(rect, dp);
 		    }
 		}
-	    } HANDLE_RECT(code);
-	    pcls->rect = rect;
+	    } while (RECT_RECOVER(code));
+	    if (code < 0 && SET_BAND_CODE(code))
+		goto error_in_rect;
+	    re.pcls->rect = rect;
 	    goto end;
 	}
 end:
-	;
-    } END_RECTS;
-	return 0;
+	re.y += re.height;
+	continue;
+error_in_rect:
+	if (!(cdev->error_is_retryable && cdev->driver_call_nesting == 0 &&
+		SET_BAND_CODE(clist_VMerror_recover_flush(cdev, re.band_code)) >= 0))
+	return re.band_code;
+    } while (re.y < re.yend);
+    return 0;
 }
 
 /* ------ Bitmap image driver procedures ------ */
@@ -198,7 +222,6 @@ end:
 typedef struct clist_image_enum_s {
     gx_image_enum_common;
     /* Arguments of begin_image */
-    gs_memory_t *memory;
     gs_pixel_image_t image;	/* only uses Width, Height, Interpolate */
     gx_drawing_color dcolor;	/* only pure right now */
     gs_int_rect rect;
@@ -237,27 +260,27 @@ gs_private_st_suffix_add3(st_clist_image_enum, clist_image_enum,
 			  clist_image_enum_reloc_ptrs, st_gx_image_enum_common,
 			  pis, pcpath, color_space.space);
 
-private image_enum_proc_plane_data(clist_image_plane_data);
-private image_enum_proc_end_image(clist_image_end_image);
-private const gx_image_enum_procs_t clist_image_enum_procs =
+static image_enum_proc_plane_data(clist_image_plane_data);
+static image_enum_proc_end_image(clist_image_end_image);
+static const gx_image_enum_procs_t clist_image_enum_procs =
 {
     clist_image_plane_data, clist_image_end_image
 };
 
 /* Forward declarations */
-private bool image_band_box(gx_device * dev, const clist_image_enum * pie,
+static bool image_band_box(gx_device * dev, const clist_image_enum * pie,
 			    int y, int h, gs_int_rect * pbox);
-private int begin_image_command(byte *buf, uint buf_size,
+static int begin_image_command(byte *buf, uint buf_size,
 				const gs_image_common_t *pic);
-private int cmd_image_plane_data(gx_device_clist_writer * cldev,
+static int cmd_image_plane_data(gx_device_clist_writer * cldev,
 				 gx_clist_state * pcls,
 				 const gx_image_plane_t * planes,
 				 const gx_image_enum_common_t * pie,
 				 uint bytes_per_plane,
 				 const uint * offsets, int dx, int h);
-private uint clist_image_unknowns(gx_device *dev,
+static uint clist_image_unknowns(gx_device *dev,
 				  const clist_image_enum *pie);
-private int write_image_end_all(gx_device *dev,
+static int write_image_end_all(gx_device *dev,
 				const clist_image_enum *pie);
 
 /*
@@ -268,7 +291,7 @@ private int write_image_end_all(gx_device *dev,
  * subrectangles, but for now, don't use the high-level approach if it would
  * cause the data to explode because of this.
  */
-private bool
+static bool
 image_matrix_ok_to_band(const gs_matrix * pmat)
 {
     double t;
@@ -370,8 +393,6 @@ clist_begin_typed_image(gx_device * dev,
 	cdev->image_enum_id != gs_no_id ||  /* Can't handle nested images */
 	/****** CAN'T HANDLE CIE COLOR YET ******/
 	base_index > gs_color_space_index_DeviceCMYK ||
-	/****** CAN'T HANDLE NON-PURE COLORS YET ******/
-	(uses_color && !gx_dc_is_pure(pdcolor)) ||
 	/****** CAN'T HANDLE IMAGES WITH ALPHA YET ******/
 	has_alpha ||
 	/****** CAN'T HANDLE IMAGES WITH IRREGULAR DEPTHS ******/
@@ -380,7 +401,9 @@ clist_begin_typed_image(gx_device * dev,
 	(code = gs_matrix_multiply(&mat, &ctm_only(pis), &mat)) < 0 ||
 	!(cdev->disable_mask & clist_disable_nonrect_hl_image ?
 	  (is_xxyy(&mat) || is_xyyx(&mat)) :
-	  image_matrix_ok_to_band(&mat))
+	  image_matrix_ok_to_band(&mat)) ||
+	/****** CAN'T HANDLE NON-PURE COLORS YET ******/
+	(uses_color && !gx_dc_is_pure(pdcolor) && !gx_dc_is_pattern1_color_clist_based(pdcolor))
 	)
 	goto use_default;
     {
@@ -522,8 +545,28 @@ use_default:
 					pdcolor, pcpath, mem, pinfo);
 }
 
+/* Error cleanup for clist_image_plane_data. */
+static inline int
+clist_image_plane_data_retry_cleanup(gx_device *dev, clist_image_enum *pie, int yh_used, int code)
+{
+    gx_device_clist_writer * const cdev =
+	&((gx_device_clist *)dev)->writer;
+
+    ++cdev->ignore_lo_mem_warnings;
+    ++cdev->driver_call_nesting; 
+    {
+	code = write_image_end_all(dev, pie);
+    } 
+    --cdev->driver_call_nesting;
+    --cdev->ignore_lo_mem_warnings;
+    /* Update sub-rect */
+    if (!pie->image.Interpolate)
+        pie->rect.p.y += yh_used;  /* interpolate & mem recovery currently incompat */
+    return code;
+}
+
 /* Process the next piece of an image. */
-private int
+static int
 clist_image_plane_data(gx_image_enum_common_t * info,
 		       const gx_image_plane_t * planes, int yh,
 		       int *rows_used)
@@ -536,8 +579,9 @@ clist_image_plane_data(gx_image_enum_common_t * info,
     int y_orig = pie->y;
     int yh_used = min(yh, pie->rect.q.y - y_orig);
     int y0, y1;
-    int y, height;		/* for BEGIN/END_RECT */
+    int ry, rheight;
     int code;
+    cmd_rects_enum_t re;
 
 #ifdef DEBUG
     if (pie->id != cdev->image_enum_id) {
@@ -574,7 +618,7 @@ clist_image_plane_data(gx_image_enum_common_t * info,
     {
 	int ry0 = (int)floor(dbox.p.y) - 2;
 	int ry1 = (int)ceil(dbox.q.y) + 2;
-	int band_height = cdev->page_band_height;
+	int band_height0 = cdev->page_band_height;
 
 	/*
 	 * Make sure we don't go into any bands beyond the Y range
@@ -591,19 +635,24 @@ clist_image_plane_data(gx_image_enum_common_t * info,
 	if (ry0 >= ry1)
 	    goto done;
 	/* Expand the range out to band boundaries. */
-	y = ry0 / band_height * band_height;
-	height = min(ROUND_UP(ry1, band_height), dev->height) - y;
+	ry = ry0 / band_height0 * band_height0;
+	rheight = min(ROUND_UP(ry1, band_height0), dev->height) - ry;
     }
 
-    FOR_RECTS {
+    if (cdev->permanent_error < 0)
+      return (cdev->permanent_error);
+    RECT_ENUM_INIT(re, ry, rheight);
+    do {
+	gs_int_rect ibox;
+	gs_int_rect entire_box;
+
+	RECT_STEP_INIT(re);
 	/*
 	 * Just transmit the subset of the data that intersects this band.
 	 * Note that y and height always define a complete band.
 	 */
-	gs_int_rect ibox;
-	gs_int_rect entire_box;
 
-	if (!image_band_box(dev, pie, y, height, &ibox))
+	if (!image_band_box(dev, pie, re.y, re.height, &ibox))
 	    continue;
 	/*
 	 * The transmitted subrectangle has to be computed at the time
@@ -611,19 +660,21 @@ clist_image_plane_data(gx_image_enum_common_t * info,
 	 * much of each scan line we write out.
 	 */
 	{
-	    int band_ymax = min(band_end, pie->ymax);
-	    int band_ymin = max(band_end - band_height, pie->ymin);
+	    int band_ymax = min(re.band_end, pie->ymax);
+	    int band_ymin = max(re.band_end - re.band_height, pie->ymin);
 
 	    if (!image_band_box(dev, pie, band_ymin,
 				band_ymax - band_ymin, &entire_box))
 		continue;
 	}
 
-	pcls->colors_used.or |= pie->colors_used.or;
-	pcls->colors_used.slow_rop |= pie->colors_used.slow_rop;
+	re.pcls->colors_used.or |= pie->colors_used.or;
+	re.pcls->band_complexity.nontrivial_rops |= 
+	    re.pcls->colors_used.slow_rop |= pie->colors_used.slow_rop;
+	re.pcls->band_complexity.uses_color |= (pie->colors_used.or != 0 || pie->colors_used.or != 0xffffff);
 
 	/* Write out begin_image & its preamble for this band */
-	if (!(pcls->known & begin_image_known)) {
+	if (!(re.pcls->known & begin_image_known)) {
 	    gs_logical_operation_t lop = pie->pis->log_op;
 	    byte *dp;
 	    byte *bp = pie->begin_image_command +
@@ -632,7 +683,7 @@ clist_image_plane_data(gx_image_enum_common_t * info,
 	    byte image_op = cmd_opv_begin_image;
 
 	    /* Make sure the imager state is up to date. */
-	    TRY_RECT {
+	    do {
 	        code = (pie->color_map_is_known ? 0 :
 			cmd_put_color_mapping(cdev, pie->pis));
 		pie->color_map_is_known = true;
@@ -643,17 +694,21 @@ clist_image_plane_data(gx_image_enum_common_t * info,
 				(pie->color_space.id == gs_no_id ? 0 :
 							 color_space_known);
 
-		    code = cmd_do_write_unknown(cdev, pcls, want_known);
+		    code = cmd_do_write_unknown(cdev, re.pcls, want_known);
 		}
 		if (code >= 0)
-		    code = cmd_do_enable_clip(cdev, pcls, pie->pcpath != NULL);
+		    code = cmd_do_enable_clip(cdev, re.pcls, pie->pcpath != NULL);
 		if (code >= 0)
-		    code = cmd_update_lop(cdev, pcls, lop);
-	    } HANDLE_RECT(code);
+		    code = cmd_update_lop(cdev, re.pcls, lop);
+	    } while (RECT_RECOVER(code));
+	    if (code < 0 && SET_BAND_CODE(code))
+		goto error_in_rect;
 	    if (pie->uses_color) {
- 	        TRY_RECT {
-		    code = cmd_put_drawing_color(cdev, pcls, &pie->dcolor);
-		} HANDLE_RECT(code);
+ 	        do {
+		    code = cmd_put_drawing_color(cdev, re.pcls, &pie->dcolor);
+		} while (RECT_RECOVER(code));
+		if (code < 0 && SET_BAND_CODE(code))
+		    goto error_in_rect;
 	    }
 	    if (entire_box.p.x != 0 || entire_box.p.y != 0 ||
 		entire_box.q.x != pie->image.Width ||
@@ -665,14 +720,16 @@ clist_image_plane_data(gx_image_enum_common_t * info,
 			  pie->image.Height - entire_box.q.y, bp);
  	        }
 	    len = bp - pie->begin_image_command;
-	    TRY_RECT {
+	    do {
 		code =
-		    set_cmd_put_op(dp, cdev, pcls, image_op, 1 + len);
-	    } HANDLE_RECT(code);
+		    set_cmd_put_op(dp, cdev, re.pcls, image_op, 1 + len);
+	    } while (RECT_RECOVER(code));
+	    if (code < 0 && SET_BAND_CODE(code))
+		goto error_in_rect;
 	    memcpy(dp + 1, pie->begin_image_command, len);
  
 	    /* Mark band's begin_image as known */
-	    pcls->known |= begin_image_known;
+	    re.pcls->known |= begin_image_known;
 	}
 
 	/*
@@ -690,7 +747,7 @@ clist_image_plane_data(gx_image_enum_common_t * info,
 	    int by0 = ibox.p.y, by1 = ibox.q.y;
 	    int bpp = pie->bits_per_plane;
 	    int num_planes = pie->num_planes;
-	    uint offsets[gs_image_max_planes];
+	    uint offsets[GS_IMAGE_MAX_COMPONENTS];
 	    int i, iy, ih, xskip, xoff, nrows;
 	    uint bytes_per_plane, bytes_per_row, rows_per_cmd;
 
@@ -719,39 +776,43 @@ clist_image_plane_data(gx_image_enum_common_t * info,
 	    }
 	    for (iy = by0, ih = by1 - by0; ih > 0; iy += nrows, ih -= nrows) {
 		nrows = min(ih, rows_per_cmd);
-		TRY_RECT {
-		    code = cmd_image_plane_data(cdev, pcls, planes, info,
+		do {
+		    code = cmd_image_plane_data(cdev, re.pcls, planes, info,
 						bytes_per_plane, offsets,
 						xoff - xskip, nrows);
-		} HANDLE_RECT(code);
+		} while (RECT_RECOVER(code));
+		if (code < 0 && SET_BAND_CODE(code))
+		    goto error_in_rect;
 		for (i = 0; i < num_planes; ++i)
 		    offsets[i] += planes[i].raster * nrows;
 	    }
 	}
-    } END_RECTS_ON_ERROR(
-	BEGIN
-	    ++cdev->ignore_lo_mem_warnings;
-	    NEST_RECT {
-		code = write_image_end_all(dev, pie);
-	    } UNNEST_RECT;
-	    --cdev->ignore_lo_mem_warnings;
-	    /* Update sub-rect */
-	    if (!pie->image.Interpolate)
-	        pie->rect.p.y += yh_used;  /* interpolate & mem recovery currently incompat */
-	END,
-	(code < 0 ? (band_code = code) : code) >= 0,
-	(cmd_clear_known(cdev,
-			 clist_image_unknowns(dev, pie) | begin_image_known),
-	 pie->color_map_is_known = false,
-	 cdev->image_enum_id = pie->id, true)
-	);
+    continue;
+error_in_rect:
+	if (cdev->error_is_retryable) {
+	    code = clist_image_plane_data_retry_cleanup(dev, pie, yh_used, code);
+	    if (code < 0)
+		SET_BAND_CODE(code);
+	    else if (cdev->driver_call_nesting == 0) {
+		SET_BAND_CODE(clist_VMerror_recover_flush(cdev, re.band_code));
+		if (re.band_code >= 0) {
+		    cmd_clear_known(cdev, clist_image_unknowns(dev, pie) | begin_image_known);
+		    pie->color_map_is_known = false;
+		    cdev->image_enum_id = pie->id;
+		    re.y -= re.height; /* Retry rect. */
+		    continue;
+		}
+	    }
+	}
+	return re.band_code;
+    } while ((re.y += re.height) < re.yend);
  done:
     *rows_used = pie->y - y_orig;
     return pie->y >= pie->rect.q.y;
 }
 
 /* Clean up by releasing the buffers. */
-private int
+static int
 clist_image_end_image(gx_image_enum_common_t * info, bool draw_last)
 {
     gx_device *dev = info->dev;
@@ -767,7 +828,8 @@ clist_image_end_image(gx_image_enum_common_t * info, bool draw_last)
 	return_error(gs_error_Fatal);
     }
 #endif
-    NEST_RECT {
+    ++cdev->driver_call_nesting; 
+    {
 	do {
 	    code = write_image_end_all(dev, pie);
 	} while (code < 0 && cdev->error_is_retryable &&
@@ -782,9 +844,10 @@ clist_image_end_image(gx_image_enum_common_t * info, bool draw_last)
 	    if (retry_code >= 0 && cdev->driver_call_nesting == 0)
 		code = clist_VMerror_recover_flush(cdev, code);
 	}
-    } UNNEST_RECT;
+    } 
+    --cdev->driver_call_nesting;
     cdev->image_enum_id = gs_no_id;
-    gs_free_object(pie->memory, pie, "clist_image_end_image");
+    gx_image_free_enum(&info);
     return code;
 }
 
@@ -795,8 +858,14 @@ clist_create_compositor(gx_device * dev,
 			gs_imager_state * pis, gs_memory_t * mem)
 {
     byte * dp;
-    uint size = 0;
-    int code = pcte->type->procs.write(pcte, 0, &size);
+    uint size = 0, size_dummy;
+    gx_device_clist_writer * const cdev =
+		    &((gx_device_clist *)dev)->writer;
+    int ry, rheight, cropping_op;
+    int band_height = cdev->page_info.band_params.BandHeight;
+    int last_band = (cdev->height + band_height - 1) / band_height;
+    int first_band = 0, no_of_bands = last_band + 1;
+    int code = pcte->type->procs.write(pcte, 0, &size, cdev);
 
     /* determine the amount of space required */
     if (code < 0 && code != gs_error_rangecheck)
@@ -809,28 +878,85 @@ clist_create_compositor(gx_device * dev,
     if (code < 0)
         return code;
 
-    /* overprint applies to all bands */
-    code = set_cmd_put_all_op( dp,
-                               (gx_device_clist_writer *)dev,
-                               cmd_opv_extend,
-                               size );
+    code = pcte->type->procs.get_cropping(pcte, &ry, &rheight);
     if (code < 0)
-        return code;
+	return code;
+    cropping_op = code;
+    if (cropping_op == 1) {
+	first_band = ry / band_height;
+	last_band = (ry + rheight + band_height - 1) / band_height;
+    } else if (cropping_op > 1) {
+	first_band = cdev->cropping_min / band_height;
+	last_band = (cdev->cropping_max + band_height - 1) / band_height;
+    }
+    if (last_band - first_band > no_of_bands * 2 / 3) {
+	/* Covering many bands, so write "all bands" command for shorter clist. */
+	cropping_op = 0;
+    }
+    if (cropping_op == 0) {
+	/* overprint applies to all bands */
+	size_dummy = size;
+	code = set_cmd_put_all_op( dp,
+				   (gx_device_clist_writer *)dev,
+				   cmd_opv_extend,
+				   size );
+	if (code < 0)
+	    return code;
 
-    /* insert the command and compositor identifier */
-    dp[1] = cmd_opv_ext_create_compositor;
-    dp[2] = pcte->type->comp_id;
+	/* insert the command and compositor identifier */
+	dp[1] = cmd_opv_ext_create_compositor;
+	dp[2] = pcte->type->comp_id;
 
-    /* serialize the remainder of the compositor */
-    if ((code = pcte->type->procs.write(pcte, dp + 3, &size)) < 0)
-        ((gx_device_clist_writer *)dev)->cnext = dp;
+	/* serialize the remainder of the compositor */
+	if ((code = pcte->type->procs.write(pcte, dp + 3, &size_dummy, cdev)) < 0)
+	    ((gx_device_clist_writer *)dev)->cnext = dp;
+	return code;
+    } 
+    if (cropping_op == 1) {
+	code = clist_writer_push_cropping(cdev, ry, rheight);
+	if (code < 0)
+	    return code;
+    }
+    {
+	/* The pdf14 compositor could be applied 
+	   only to bands covered by the pcte->params.bbox. */
+	cmd_rects_enum_t re;
+
+	RECT_ENUM_INIT(re, cdev->cropping_min, cdev->cropping_max - cdev->cropping_min);
+	do {
+	    RECT_STEP_INIT(re);
+	    re.pcls->band_complexity.nontrivial_rops = true;
+	    do {
+		code = set_cmd_put_op(dp, cdev, re.pcls, cmd_opv_extend, size);
+		if (code >= 0) {
+		    size_dummy = size;
+		    dp[1] = cmd_opv_ext_create_compositor;
+		    dp[2] = pcte->type->comp_id;
+		    code = pcte->type->procs.write(pcte, dp + 3, &size_dummy, cdev);
+		}
+	    } while (RECT_RECOVER(code));
+	    if (code < 0 && SET_BAND_CODE(code))
+		goto error_in_rect;
+	    re.y += re.height;
+	    continue;
+    error_in_rect:
+	    if (!(cdev->error_is_retryable && cdev->driver_call_nesting == 0 &&
+		    SET_BAND_CODE(clist_VMerror_recover_flush(cdev, re.band_code)) >= 0))
+		return re.band_code;
+	} while (re.y < re.yend);
+    }
+    if (cropping_op == 2) {
+	code = clist_writer_pop_cropping(cdev);
+	if (code < 0)
+	    return code;
+    }
     return code;
 }
 
 /* ------ Utilities ------ */
 
 /* Add a command to set data_x. */
-private int
+static int
 cmd_put_set_data_x(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 		   int data_x)
 {
@@ -899,7 +1025,7 @@ cmd_put_halftone(gx_device_clist_writer * cldev, const gx_device_halftone * pdht
     dp += 2;
     enc_u_putw(ht_size, dp);
 
-    /* see if a spearate allocated buffer is required */
+    /* see if a separate allocated buffer is required */
     if (ht_size > cbuf_ht_seg_max_size) {
         pht_buff = gs_alloc_bytes( cldev->bandlist_memory,
                                    ht_size,
@@ -978,8 +1104,8 @@ cmd_put_color_mapping(gx_device_clist_writer * cldev,
     int code;
     const gx_device_halftone *pdht = pis->dev_ht;
 
-    /* Put out the halftone. */
-    if (pdht->id != cldev->device_halftone_id) {
+    /* Put out the halftone, if present. */
+    if (pdht && pdht->id != cldev->device_halftone_id) {
 	code = cmd_put_halftone(cldev, pdht);
 	if (code < 0)
 	    return code;
@@ -1084,7 +1210,7 @@ cmd_put_color_mapping(gx_device_clist_writer * cldev,
  */
 #define I_FLOOR(x) ((int)floor(x))
 #define I_CEIL(x) ((int)ceil(x))
-private void
+static void
 box_merge_point(gs_int_rect * pbox, floatp x, floatp y)
 {
     int t;
@@ -1098,7 +1224,7 @@ box_merge_point(gs_int_rect * pbox, floatp x, floatp y)
     if ((t = I_CEIL(y)) > pbox->q.y)
 	pbox->q.y = t;
 }
-private bool
+static bool
 image_band_box(gx_device * dev, const clist_image_enum * pie, int y, int h,
 	       gs_int_rect * pbox)
 {
@@ -1253,7 +1379,7 @@ image_band_box(gx_device * dev, const clist_image_enum * pie, int y, int h,
 }
 
 /* Determine which image-related properties are unknown */
-private uint	/* mask of unknown properties(see pcls->known) */
+static uint	/* mask of unknown properties(see pcls->known) */
 clist_image_unknowns(gx_device *dev, const clist_image_enum *pie)
 {
     gx_device_clist_writer * const cdev =
@@ -1322,7 +1448,7 @@ clist_image_unknowns(gx_device *dev, const clist_image_enum *pie)
 }
 
 /* Construct the begin_image command. */
-private int
+static int
 begin_image_command(byte *buf, uint buf_size, const gs_image_common_t *pic)
 {
     int i;
@@ -1343,7 +1469,7 @@ begin_image_command(byte *buf, uint buf_size, const gs_image_common_t *pic)
 }
 
 /* Write data for a partial image. */
-private int
+static int
 cmd_image_plane_data(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 		     const gx_image_plane_t * planes,
 		     const gx_image_enum_common_t * pie,
@@ -1381,33 +1507,46 @@ cmd_image_plane_data(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 }
 
 /* Write image_end commands into all bands */
-private int	/* ret 0 ok, else -ve error status */
+static int	/* ret 0 ok, else -ve error status */
 write_image_end_all(gx_device *dev, const clist_image_enum *pie)
 {
     gx_device_clist_writer * const cdev =
 	&((gx_device_clist *)dev)->writer;
     int code;
-    int y = pie->ymin;
-    int height = pie->ymax - y;
+    int ry = pie->ymin;
+    int rheight = pie->ymax - ry;
+    cmd_rects_enum_t re;
 
     /*
      * We need to check specially for images lying entirely outside the
      * page, since FOR_RECTS doesn't do this.
      */
-    if (height <= 0)
+    if (rheight <= 0)
 	return 0;
-    FOR_RECTS {
+    if (cdev->permanent_error < 0)
+      return (cdev->permanent_error);
+    RECT_ENUM_INIT(re, ry, rheight);
+    do {
 	byte *dp;
 
-	if (!(pcls->known & begin_image_known))
-	    continue;
-	TRY_RECT {
-	    if_debug1('L', "[L]image_end for band %d\n", band);
-	    code = set_cmd_put_op(dp, cdev, pcls, cmd_opv_image_data, 2);
-	} HANDLE_RECT(code);
-	dp[1] = 0;	    /* EOD */
-	pcls->known ^= begin_image_known;
-    } END_RECTS;
+	RECT_STEP_INIT(re);
+	if (re.pcls->known & begin_image_known) {
+	    do {
+		if_debug1('L', "[L]image_end for band %d\n", re.band);
+		code = set_cmd_put_op(dp, cdev, re.pcls, cmd_opv_image_data, 2);
+	    } while (RECT_RECOVER(code));
+	    if (code < 0 && SET_BAND_CODE(code))
+		goto error_in_rect;
+	    dp[1] = 0;	    /* EOD */
+	    re.pcls->known ^= begin_image_known;
+	}
+	re.y += re.height;
+	continue;
+error_in_rect:
+	if (!(cdev->error_is_retryable && cdev->driver_call_nesting == 0 &&
+		SET_BAND_CODE(clist_VMerror_recover_flush(cdev, re.band_code)) >= 0))
+	   return re.band_code;
+    } while (re.y < re.yend);
     return 0;
 }
 
@@ -1416,7 +1555,7 @@ write_image_end_all(gx_device *dev, const clist_image_enum *pie)
  * path, if the rectangle is unclipped, or if the clipping path is a
  * rectangle and intersects the given rectangle.
  */
-private bool
+static bool
 check_rect_for_trivial_clip(
     const gx_clip_path *pcpath,	/* May be NULL, clip to evaluate */
     int px, int py, int qx, int qy	/* corners of box to test */
