@@ -1,23 +1,17 @@
-/* Copyright (C) 1998, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: gxclip.c,v $ $Revision: 1.3.2.1 $ */
+/* $Id: gxclip.c 8790 2008-06-09 07:33:57Z leonardo $ */
 /* Implementation of (path-based) clipping */
 #include "gx.h"
 #include "gxdevice.h"
@@ -33,19 +27,19 @@
 /* Device for clipping with a region. */
 /* We forward non-drawing operations, but we must be sure to intercept */
 /* all drawing operations. */
-private dev_proc_open_device(clip_open);
-private dev_proc_fill_rectangle(clip_fill_rectangle);
-private dev_proc_copy_mono(clip_copy_mono);
-private dev_proc_copy_color(clip_copy_color);
-private dev_proc_copy_alpha(clip_copy_alpha);
-private dev_proc_fill_mask(clip_fill_mask);
-private dev_proc_strip_tile_rectangle(clip_strip_tile_rectangle);
-private dev_proc_strip_copy_rop(clip_strip_copy_rop);
-private dev_proc_get_clipping_box(clip_get_clipping_box);
-private dev_proc_get_bits_rectangle(clip_get_bits_rectangle);
+static dev_proc_open_device(clip_open);
+static dev_proc_fill_rectangle(clip_fill_rectangle);
+static dev_proc_copy_mono(clip_copy_mono);
+static dev_proc_copy_color(clip_copy_color);
+static dev_proc_copy_alpha(clip_copy_alpha);
+static dev_proc_fill_mask(clip_fill_mask);
+static dev_proc_strip_tile_rectangle(clip_strip_tile_rectangle);
+static dev_proc_strip_copy_rop(clip_strip_copy_rop);
+static dev_proc_get_clipping_box(clip_get_clipping_box);
+static dev_proc_get_bits_rectangle(clip_get_bits_rectangle);
 
 /* The device descriptor. */
-private const gx_device_clip gs_clip_device =
+static const gx_device_clip gs_clip_device =
 {std_device_std_body(gx_device_clip, 0, "clipper",
 		     0, 0, 1, 1),
  {clip_open,
@@ -91,34 +85,64 @@ private const gx_device_clip gs_clip_device =
   gx_no_create_compositor,
   gx_forward_get_hardware_params,
   gx_default_text_begin,
-  gx_default_finish_copydevice
+  gx_default_finish_copydevice,
+  NULL,			/* begin_transparency_group */
+  NULL,			/* end_transparency_group */
+  NULL,			/* begin_transparency_mask */
+  NULL,			/* end_transparency_mask */
+  NULL,			/* discard_transparency_layer */
+  gx_forward_get_color_mapping_procs,
+  gx_forward_get_color_comp_index,
+  gx_forward_encode_color,
+  gx_forward_decode_color,
+  gx_forward_pattern_manage,
+  gx_forward_fill_rectangle_hl_color,
+  gx_forward_include_color_space,
+  gx_default_fill_linear_color_scanline,
+  gx_default_fill_linear_color_trapezoid,
+  gx_default_fill_linear_color_triangle,
+  gx_forward_update_spot_equivalent_colors,
+  gx_forward_ret_devn_params
  }
 };
 
 /* Make a clipping device. */
 void
-gx_make_clip_translate_device(gx_device_clip * dev, const gx_clip_list * list,
-			      int tx, int ty, gs_memory_t *mem)
+gx_make_clip_device_on_stack(gx_device_clip * dev, const gx_clip_path *pcpath, gx_device *target)
 {
-    gx_device_init((gx_device *)dev, (const gx_device *)&gs_clip_device,
-		   mem, true);
-    dev->list = *list;
-    dev->translation.x = tx;
-    dev->translation.y = ty;
+    gx_device_init((gx_device *)dev, (const gx_device *)&gs_clip_device, NULL, true);
+    dev->list = *gx_cpath_list(pcpath);
+    dev->translation.x = 0;
+    dev->translation.y = 0;
+    dev->HWResolution[0] = target->HWResolution[0];
+    dev->HWResolution[1] = target->HWResolution[1];
+    dev->sgr = target->sgr;
+    dev->target = target;
+    (*dev_proc(dev, open_device)) ((gx_device *)dev);
 }
 void
-gx_make_clip_path_device(gx_device_clip * dev, const gx_clip_path * pcpath)
+gx_make_clip_device_in_heap(gx_device_clip * dev, const gx_clip_path *pcpath, gx_device *target,
+			      gs_memory_t *mem)
 {
-    gx_make_clip_device(dev, gx_cpath_list(pcpath));
+    gx_device_init((gx_device *)dev, (const gx_device *)&gs_clip_device, mem, true);
+    dev->list = *gx_cpath_list(pcpath);
+    dev->translation.x = 0;
+    dev->translation.y = 0;
+    dev->HWResolution[0] = target->HWResolution[0];
+    dev->HWResolution[1] = target->HWResolution[1];
+    dev->sgr = target->sgr;
+    gx_device_set_target((gx_device_forward *)dev, target);
+    gx_device_retain((gx_device *)dev, true); /* will free explicitly */
+    (*dev_proc(dev, open_device)) ((gx_device *)dev);
 }
-
 /* Define debugging statistics for the clipping loops. */
 #ifdef DEBUG
 struct stats_clip_s {
     long
          loops, out, in_y, in, in1, down, up, x, no_x;
 } stats_clip;
-private uint clip_interval = 10000;
+
+static const uint clip_interval = 10000;
 
 # define INCR(v) (++(stats_clip.v))
 # define INCR_THEN(v, e) (INCR(v), (e))
@@ -131,11 +155,11 @@ private uint clip_interval = 10000;
  * Enumerate the rectangles of the x,w,y,h argument that fall within
  * the clipping region.
  */
-private int
+static int
 clip_enumerate_rest(gx_device_clip * rdev,
 		    int x, int y, int xe, int ye,
-		    int (*process)(P5(clip_callback_data_t * pccd,
-				      int xc, int yc, int xec, int yec)),
+		    int (*process)(clip_callback_data_t * pccd,
+				   int xc, int yc, int xec, int yec),
 		    clip_callback_data_t * pccd)
 {
     gx_clip_rect *rptr = rdev->current;		/* const within algorithm */
@@ -239,10 +263,10 @@ clip_enumerate_rest(gx_device_clip * rdev,
     return 0;
 }
 
-private int
+static int
 clip_enumerate(gx_device_clip * rdev, int x, int y, int w, int h,
-	       int (*process)(P5(clip_callback_data_t * pccd,
-				 int xc, int yc, int xec, int yec)),
+	       int (*process)(clip_callback_data_t * pccd,
+			      int xc, int yc, int xec, int yec),
 	       clip_callback_data_t * pccd)
 {
     int xe, ye;
@@ -266,7 +290,7 @@ clip_enumerate(gx_device_clip * rdev, int x, int y, int w, int h,
 }
 
 /* Open a clipping device */
-private int
+static int
 clip_open(gx_device * dev)
 {
     gx_device_clip *const rdev = (gx_device_clip *) dev;
@@ -281,6 +305,7 @@ clip_open(gx_device * dev)
     rdev->height = tdev->height;
     gx_device_copy_color_procs(dev, tdev);
     rdev->clipping_box_set = false;
+    rdev->memory = tdev->memory;
     return 0;
 }
 
@@ -291,7 +316,7 @@ clip_call_fill_rectangle(clip_callback_data_t * pccd, int xc, int yc, int xec, i
     return (*dev_proc(pccd->tdev, fill_rectangle))
 	(pccd->tdev, xc, yc, xec - xc, yec - yc, pccd->color[0]);
 }
-private int
+static int
 clip_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 		    gx_color_index color)
 {
@@ -347,7 +372,7 @@ clip_call_copy_mono(clip_callback_data_t * pccd, int xc, int yc, int xec, int ye
 	 pccd->sourcex + xc - pccd->x, pccd->raster, gx_no_bitmap_id,
 	 xc, yc, xec - xc, yec - yc, pccd->color[0], pccd->color[1]);
 }
-private int
+static int
 clip_copy_mono(gx_device * dev,
 	       const byte * data, int sourcex, int raster, gx_bitmap_id id,
 	       int x, int y, int w, int h,
@@ -390,7 +415,7 @@ clip_call_copy_color(clip_callback_data_t * pccd, int xc, int yc, int xec, int y
 	 pccd->sourcex + xc - pccd->x, pccd->raster, gx_no_bitmap_id,
 	 xc, yc, xec - xc, yec - yc);
 }
-private int
+static int
 clip_copy_color(gx_device * dev,
 		const byte * data, int sourcex, int raster, gx_bitmap_id id,
 		int x, int y, int w, int h)
@@ -411,7 +436,7 @@ clip_call_copy_alpha(clip_callback_data_t * pccd, int xc, int yc, int xec, int y
 	 pccd->sourcex + xc - pccd->x, pccd->raster, gx_no_bitmap_id,
 	 xc, yc, xec - xc, yec - yc, pccd->color[0], pccd->depth);
 }
-private int
+static int
 clip_copy_alpha(gx_device * dev,
 		const byte * data, int sourcex, int raster, gx_bitmap_id id,
 		int x, int y, int w, int h,
@@ -435,7 +460,7 @@ clip_call_fill_mask(clip_callback_data_t * pccd, int xc, int yc, int xec, int ye
 	 xc, yc, xec - xc, yec - yc, pccd->pdcolor, pccd->depth,
 	 pccd->lop, NULL);
 }
-private int
+static int
 clip_fill_mask(gx_device * dev,
 	       const byte * data, int sourcex, int raster, gx_bitmap_id id,
 	       int x, int y, int w, int h,
@@ -462,7 +487,7 @@ clip_call_strip_tile_rectangle(clip_callback_data_t * pccd, int xc, int yc, int
 	(pccd->tdev, pccd->tiles, xc, yc, xec - xc, yec - yc,
 	 pccd->color[0], pccd->color[1], pccd->phase.x, pccd->phase.y);
 }
-private int
+static int
 clip_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 			  int x, int y, int w, int h,
      gx_color_index color0, gx_color_index color1, int phase_x, int phase_y)
@@ -487,7 +512,7 @@ clip_call_strip_copy_rop(clip_callback_data_t * pccd, int xc, int yc, int xec, i
 	 xc, yc, xec - xc, yec - yc, pccd->phase.x, pccd->phase.y,
 	 pccd->lop);
 }
-private int
+static int
 clip_strip_copy_rop(gx_device * dev,
 	      const byte * sdata, int sourcex, uint raster, gx_bitmap_id id,
 		    const gx_color_index * scolors,
@@ -506,7 +531,7 @@ clip_strip_copy_rop(gx_device * dev,
 }
 
 /* Get the (outer) clipping box, in client coordinates. */
-private void
+static void
 clip_get_clipping_box(gx_device * dev, gs_fixed_rect * pbox)
 {
     gx_device_clip *const rdev = (gx_device_clip *) dev;
@@ -553,7 +578,7 @@ clip_get_clipping_box(gx_device * dev, gs_fixed_rect * pbox)
 }
 
 /* Get bits back from the device. */
-private int
+static int
 clip_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
 			gs_get_bits_params_t * params, gs_int_rect ** unread)
 {
