@@ -1,26 +1,24 @@
-/* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxclpath.c,v 1.20 2004/10/07 05:18:34 ray Exp $ */
+/*$Id: gxclpath.c 8815 2008-07-03 13:03:11Z leonardo $ */
 /* Higher-level path operations for band lists */
 #include "math_.h"
 #include "memory_.h"
 #include "gx.h"
 #include "gpcheck.h"
 #include "gserrors.h"
+#include "gsptype2.h"
 #include "gxdevice.h"
 #include "gxdevmem.h"		/* must precede gxcldev.h */
 #include "gxcldev.h"
@@ -39,7 +37,7 @@ ulong stats_cmd_diffs[5];
 #endif
 
 /* Forward declarations */
-private int cmd_put_path(gx_device_clist_writer * cldev,
+static int cmd_put_path(gx_device_clist_writer * cldev,
 			 gx_clist_state * pcls, const gx_path * ppath,
 			 fixed ymin, fixed ymax, byte op,
 			 bool implicit_close, segment_notes keep_notes);
@@ -47,7 +45,7 @@ private int cmd_put_path(gx_device_clist_writer * cldev,
 /* ------ Utilities ------ */
 
 /* Compute the colors used by a colored halftone. */
-private gx_color_index
+static gx_color_index
 colored_halftone_colors_used(gx_device_clist_writer *cldev,
 			     const gx_drawing_color *pdcolor)
 {
@@ -90,7 +88,7 @@ cmd_slow_rop(gx_device *dev, gs_logical_operation_t lop,
 	    rop = rop3_know_T_1(rop);
     }
     return !(rop == rop3_0 || rop == rop3_1 ||
-	     rop == rop3_D || rop == rop3_S || rop == rop3_T);
+	     rop == rop3_S || rop == rop3_T);
 }
 
 /* Write out the color for filling, stroking, or masking. */
@@ -107,6 +105,11 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     byte *                     dp;
     byte *                     dp0;
     gs_int_point               color_phase;
+    int			       buffer_space;
+    int			       offset = 0;
+    int			       left;
+    uint		       portion_size, prefix_size;
+    int			       req_size_final;
 
     /* see if the halftone must be inserted in the command list */
     if ( pdht != NULL                          &&
@@ -116,16 +119,6 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         color_unset(psdc);
     }
 
-    /* see if phase informaiton must be inserted in the command list */
-    if ( pdcolor->type->get_phase(pdcolor, &color_phase) &&
-         (color_phase.x != pcls->tile_phase.x ||
-          color_phase.y != pcls->tile_phase.y   )        &&
-	 (code = cmd_set_tile_phase( cldev,
-                                     pcls,
-                                     color_phase.x,
-                                     color_phase.y )) < 0  )
-        return code;
-
     /*
      * Get the device color type index and the required size.
      *
@@ -139,6 +132,7 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     code = pdcolor->type->write( pdcolor,
                                  psdc,
                                  (gx_device *)cldev,
+				 0,
                                  0,
                                  &dc_size );
 
@@ -147,33 +141,58 @@ cmd_put_drawing_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
         return 0;
     else if (code < 0 && code != gs_error_rangecheck)
         return code;
-    req_size = dc_size + 2 + 1 + enc_u_sizew(dc_size);
+    left = dc_size;
 
-    /*
-     * Encoded device colors are small in comparison to the command
-     * buffer size (< 64 bytes), so we can just clear space in the
-     * command buffer for them.
-     */
-    if ((code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_extend, req_size)) < 0)
-        return code;
-    dp0 = dp;
-    dp[1] = cmd_opv_ext_put_drawing_color;
-    dp += 2;
-    *dp++ = di;
-    enc_u_putw(dc_size, dp);
-    code = pdcolor->type->write( pdcolor,
-                                 &pcls->sdc,
-                                 (gx_device *)cldev,
-                                 dp,
-                                 &dc_size );
-    if (code < 0) {
-        cldev->cnext = dp0;
+    /* see if phase informaiton must be inserted in the command list */
+    if ( pdcolor->type->get_phase(pdcolor, &color_phase) &&
+         (color_phase.x != pcls->tile_phase.x ||
+          color_phase.y != pcls->tile_phase.y   )        &&
+	 (code = cmd_set_tile_phase( cldev,
+                                     pcls,
+                                     color_phase.x,
+                                     color_phase.y )) < 0  )
         return code;
+
+    while (left) {
+	prefix_size = 2 + 1 + (offset > 0 ? enc_u_sizew(offset) : 0);
+	req_size = left + prefix_size + enc_u_sizew(left);
+	code = cmd_get_buffer_space(cldev, pcls, req_size);
+	if (code < 0)
+	    return code;
+	buffer_space = min(code, req_size);
+	portion_size = buffer_space - prefix_size - enc_u_sizew(left);
+	req_size_final = portion_size + prefix_size + enc_u_sizew(portion_size);
+	if (req_size_final > buffer_space)
+	    return_error(gs_error_unregistered); /* Must not happen. */
+	if ((code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_extend, req_size_final)) < 0)
+	    return code;
+	dp0 = dp;
+	dp[1] = cmd_opv_ext_put_drawing_color;
+	dp += 2;
+	*dp++ = di | (offset > 0 ? 0x80 : 0);
+	if (offset > 0)
+	    enc_u_putw(offset, dp);
+	enc_u_putw(portion_size, dp);
+	code = pdcolor->type->write( pdcolor,
+				     &pcls->sdc,
+				     (gx_device *)cldev,
+				     offset,
+				     dp,
+				     &portion_size);
+	if (code < 0) {
+	    if (offset == 0)
+		cldev->cnext = dp0;
+	    return code;
+	}
+	offset += portion_size;
+	left -= portion_size;
     }
 
     /* should properly calculate colors_used, but for now just punt */
     pcls->colors_used.or = ((gx_color_index)1 << cldev->color_info.depth) - 1;
 
+    pcls->band_complexity.uses_color |= (pdcolor->colors.pure != 0 && pdcolor->colors.pure != 0xffffff);
+
     /* record the color we have just serialized color */
     pdcolor->type->save_dc(pdcolor, &pcls->sdc);
 
@@ -231,7 +250,7 @@ cmd_check_clip_path(gx_device_clist_writer * cldev, const gx_clip_path * pcpath)
 #define FILL_KNOWN\
  (cj_ac_sa_known | flatness_known | op_bm_tk_known | opacity_alpha_known |\
   shape_alpha_known | fill_adjust_known | alpha_known | clip_path_known)
-private void
+static void
 cmd_check_fill_known(gx_device_clist_writer *cdev, const gs_imager_state *pis,
 		     floatp flatness, const gs_fixed_point *padjust,
 		     const gx_clip_path *pcpath, uint *punknown)
@@ -289,6 +308,42 @@ cmd_check_fill_known(gx_device_clist_writer *cdev, const gs_imager_state *pis,
 	*punknown |= clip_path_known;
 }
 
+/* Compute the written CTM length. */
+int
+cmd_write_ctm_return_length(gx_device_clist_writer * cldev, const gs_matrix *m)
+{
+    stream s;
+
+    s_init(&s, cldev->memory);
+    swrite_position_only(&s);
+    sput_matrix(&s, m);
+    return (uint)stell(&s);
+}
+
+/* Compute the written CTM length. */
+int
+cmd_write_ctm_return_length_nodevice(const gs_matrix *m)
+{
+    stream s;
+
+    s_init(&s, NULL);
+    swrite_position_only(&s);
+    sput_matrix(&s, m);
+    return (uint)stell(&s);
+}
+
+/* Write out CTM. */
+int
+cmd_write_ctm(const gs_matrix *m, byte *dp, int len)
+{
+    stream s;
+
+    s_init(&s, NULL);
+    swrite_string(&s, dp + 1, len);
+    sput_matrix(&s, m);
+    return 0;
+}
+
 /* Write out values of any unknown parameters. */
 int
 cmd_write_unknown(gx_device_clist_writer * cldev, gx_clist_state * pcls,
@@ -375,18 +430,14 @@ cmd_write_unknown(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	pcls->known |= fill_adjust_known;
     }
     if (unknown & ctm_known) {
-	stream s;
-	uint len;
+	int len = cmd_write_ctm_return_length(cldev, &ctm_only(&cldev->imager_state));
 
-	s_init(&s, cldev->memory);
-	swrite_position_only(&s);
-	sput_matrix(&s, (const gs_matrix *)&cldev->imager_state.ctm);
-	len = (uint)stell(&s);
 	code = set_cmd_put_op(dp, cldev, pcls, cmd_opv_set_ctm, len + 1);
 	if (code < 0)
 	    return code;
-	swrite_string(&s, dp + 1, len);
-	sput_matrix(&s, (const gs_matrix *)&cldev->imager_state.ctm);
+	code = cmd_write_ctm(&ctm_only(&cldev->imager_state), dp, len);
+	if (code < 0)
+	    return code;
 	pcls->known |= ctm_known;
     }
     if (unknown & dash_known) {
@@ -510,8 +561,7 @@ cmd_write_unknown(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	    const gs_color_space *pcs = cldev->color_space.space;
 	    int hival = pcs->params.indexed.hival;
 	    uint num_values = (hival + 1) *
-		gs_color_space_num_components(
-		    (const gs_color_space *)&pcs->params.indexed.base_space);
+		gs_color_space_num_components(pcs->base_space);
 	    bool use_proc = cldev->color_space.byte1 & 4;
 	    const void *map_data;
 	    uint map_size;
@@ -551,14 +601,32 @@ clist_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
     gx_device_clist_writer * const cdev =
 	&((gx_device_clist *)dev)->writer;
     uint unknown = 0;
-    int y, height, y0, y1;
+    int ry, rheight, y0, y1;
     gs_logical_operation_t lop = pis->log_op;
     byte op = (byte)
 	(params->rule == gx_rule_even_odd ?
 	 cmd_opv_eofill : cmd_opv_fill);
     gs_fixed_point adjust;
     bool slow_rop = cmd_slow_rop(dev, lop_know_S_0(lop), pdcolor);
+    cmd_rects_enum_t re;
+    int code;
 
+    if (pdcolor != NULL && gx_dc_is_pattern2_color(pdcolor)) {
+	/* Here we need to intersect *ppath, *pcpath and shading bbox.
+	   Call the default implementation, which has a special
+	   branch for processing a shading fill with the clip writer device.
+	   It will call us back with pdcolor=NULL for passing
+	   the intersected clipping path, 
+	   and then will decompose the shading into trapezoids.
+	   See comment below about pdcolor == NULL.
+	 */
+	code = gx_default_fill_path(dev, pis, ppath, params, pdcolor, pcpath);
+	cdev->cropping_min = cdev->save_cropping_min;
+	cdev->cropping_max = cdev->save_cropping_max;
+	if_debug2('v', "[v] clist_fill_path: restore cropping_min=%d croping_max=%d\n",
+				cdev->save_cropping_min, cdev->save_cropping_max);
+	return code;
+    }
     if ( (cdev->disable_mask & clist_disable_fill_path) ||
 	 gs_debug_c(',')
 	 ) {
@@ -570,44 +638,81 @@ clist_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
     {
 	gs_fixed_rect bbox;
 
-	gx_path_bbox(ppath, &bbox);
-	y = fixed2int(bbox.p.y) - 1;
-	height = fixed2int_ceiling(bbox.q.y) - y + 1;
-	fit_fill_y(dev, y, height);
-	fit_fill_h(dev, y, height);
-	if (height <= 0)
+	if (ppath != NULL)
+	    gx_path_bbox(ppath, &bbox);
+	else {
+	    /* gx_default_fill_path passes the clip path for shfill. */
+	    gx_cpath_outer_box(pcpath, &bbox);
+	}
+	ry = fixed2int(bbox.p.y) - 1;
+	rheight = fixed2int_ceiling(bbox.q.y) - ry + 1;
+	crop_fill_y(cdev, ry, rheight);
+	if (rheight <= 0)
 	    return 0;
     }
-    y0 = y;
-    y1 = y + height;
+    y0 = ry;
+    y1 = ry + rheight;
     cmd_check_fill_known(cdev, pis, params->flatness, &adjust, pcpath,
 			 &unknown);
     if (unknown)
 	cmd_clear_known(cdev, unknown);
-    FOR_RECTS_NO_ERROR {
-	int code = cmd_do_write_unknown(cdev, pcls, FILL_KNOWN);
-
-	if (code < 0)
-	    return code;
-	if ((code = cmd_do_enable_clip(cdev, pcls, pcpath != NULL)) < 0 ||
-	    (code = cmd_update_lop(cdev, pcls, lop)) < 0
-	    )
-	    return code;
-	code = cmd_put_drawing_color(cdev, pcls, pdcolor);
-	if (code < 0) {
-	    /* Something went wrong, use the default implementation. */
-	    return gx_default_fill_path(dev, pis, ppath, params, pdcolor,
-					pcpath);
-	}
-	pcls->colors_used.slow_rop |= slow_rop;
-	code = cmd_put_path(cdev, pcls, ppath,
-			    int2fixed(max(y - 1, y0)),
-			    int2fixed(min(y + height + 1, y1)),
-			    op,
-			    true, sn_none /* fill doesn't need the notes */ );
-	if (code < 0)
-	    return code;
-    } END_RECTS_NO_ERROR;
+    if (cdev->permanent_error < 0)
+	return (cdev->permanent_error);
+    if (pdcolor == NULL) {
+	/* See comment above about pattern2_color.
+	   Put the clipping path only.
+	   The graphics library will call us again with subdividing 
+	   the shading into trapezoids and rectangles. 
+	   Narrow cropping_min, croping_max for such calls. */
+	cdev->save_cropping_min = cdev->cropping_min;
+	cdev->save_cropping_max = cdev->cropping_max;
+	cdev->cropping_min = max(ry, cdev->cropping_min);
+	cdev->cropping_max = min(ry + rheight, cdev->cropping_max);
+	if_debug2('v', "[v] clist_fill_path: narrow cropping_min=%d croping_max=%d\n",
+				cdev->save_cropping_min, cdev->save_cropping_max);
+	RECT_ENUM_INIT(re, ry, rheight);
+	do {
+	    RECT_STEP_INIT(re);
+	    if (pcpath != NULL) {
+		code = cmd_do_write_unknown(cdev, re.pcls, clip_path_known);
+		if (code < 0)
+		    return code;
+	    }
+	    code = cmd_do_enable_clip(cdev, re.pcls, pcpath != NULL);
+	    if (code  < 0)
+		return code;
+	    re.y += re.height;
+	} while (re.y < re.yend);
+    } else {
+	RECT_ENUM_INIT(re, ry, rheight);
+	do {
+	    RECT_STEP_INIT(re);
+	    code = cmd_do_write_unknown(cdev, re.pcls, FILL_KNOWN);
+	    if (code < 0)
+		return code;
+	    if ((code = cmd_do_enable_clip(cdev, re.pcls, pcpath != NULL)) < 0 ||
+		(code = cmd_update_lop(cdev, re.pcls, lop)) < 0
+		)
+		return code;
+	    code = cmd_put_drawing_color(cdev, re.pcls, pdcolor);
+	    if (code == gs_error_unregistered)
+		return code;
+	    if (code < 0) {
+		/* Something went wrong, use the default implementation. */
+		return gx_default_fill_path(dev, pis, ppath, params, pdcolor,
+					    pcpath);
+	    }
+	    re.pcls->colors_used.slow_rop |= slow_rop;
+	    code = cmd_put_path(cdev, re.pcls, ppath,
+				int2fixed(max(re.y - 1, y0)),
+				int2fixed(min(re.y + re.height + 1, y1)),
+				op,
+				true, sn_none /* fill doesn't need the notes */ );
+	    if (code < 0)
+		return code;
+	    re.y += re.height;
+	} while (re.y < re.yend);
+    }
     return 0;
 }
 
@@ -623,9 +728,10 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
     gs_fixed_rect bbox;
     gs_fixed_point expansion;
     int adjust_y, expansion_code;
-    int y, height;
+    int ry, rheight;
     gs_logical_operation_t lop = pis->log_op;
     bool slow_rop = cmd_slow_rop(dev, lop_know_S_0(lop), pdcolor);
+    cmd_rects_enum_t re;
 
     if ((cdev->disable_mask & clist_disable_stroke_path) ||
 	gs_debug_c(',')
@@ -641,15 +747,15 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
     if (expansion_code < 0) {
 	/* Expansion is too large: use the entire page. */
 	adjust_y = 0;
-	y = 0;
-	height = dev->height;
+	ry = 0;
+	rheight = dev->height;
     } else {
 	adjust_y = fixed2int_ceiling(expansion.y) + 1;
-	y = fixed2int(bbox.p.y) - adjust_y;
-	height = fixed2int_ceiling(bbox.q.y) - y + adjust_y;
-	fit_fill_y(dev, y, height);
-	fit_fill_h(dev, y, height);
-	if (height <= 0)
+	ry = fixed2int(bbox.p.y) - adjust_y;
+	rheight = fixed2int_ceiling(bbox.q.y) - ry + adjust_y;
+	fit_fill_y(dev, ry, rheight);
+	fit_fill_h(dev, ry, rheight);
+	if (rheight <= 0)
 	    return 0;
     }
     /* Check the dash pattern, since we bail out if */
@@ -715,21 +821,27 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
     }
     if (unknown)
 	cmd_clear_known(cdev, unknown);
-    FOR_RECTS_NO_ERROR {
+    if (cdev->permanent_error < 0)
+      return (cdev->permanent_error);
+    RECT_ENUM_INIT(re, ry, rheight);
+    do {
 	int code;
 
-	if ((code = cmd_do_write_unknown(cdev, pcls, stroke_all_known)) < 0 ||
-	    (code = cmd_do_enable_clip(cdev, pcls, pcpath != NULL)) < 0 ||
-	    (code = cmd_update_lop(cdev, pcls, lop)) < 0
+	RECT_STEP_INIT(re);
+	if ((code = cmd_do_write_unknown(cdev, re.pcls, stroke_all_known)) < 0 ||
+	    (code = cmd_do_enable_clip(cdev, re.pcls, pcpath != NULL)) < 0 ||
+	    (code = cmd_update_lop(cdev, re.pcls, lop)) < 0
 	    )
 	    return code;
-	code = cmd_put_drawing_color(cdev, pcls, pdcolor);
+	code = cmd_put_drawing_color(cdev, re.pcls, pdcolor);
+	    if (code == gs_error_unregistered)
+		return code;
 	if (code < 0) {
 	    /* Something went wrong, use the default implementation. */
 	    return gx_default_stroke_path(dev, pis, ppath, params, pdcolor,
 					  pcpath);
 	}
-	pcls->colors_used.slow_rop |= slow_rop;
+	re.pcls->colors_used.slow_rop |= slow_rop;
 	{
 	    fixed ymin, ymax;
 
@@ -744,16 +856,17 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
 		ymin = min_fixed;
 		ymax = max_fixed;
 	    } else {
-		ymin = int2fixed(y - adjust_y);
-		ymax = int2fixed(y + height + adjust_y);
+		ymin = int2fixed(re.y - adjust_y);
+		ymax = int2fixed(re.y + re.height + adjust_y);
 	    }
-	    code = cmd_put_path(cdev, pcls, ppath, ymin, ymax,
+	    code = cmd_put_path(cdev, re.pcls, ppath, ymin, ymax,
 				cmd_opv_stroke,
 				false, (segment_notes)~0);
 	    if (code < 0)
 		return code;
 	}
-    } END_RECTS_NO_ERROR;
+	re.y += re.height;
+    } while (re.y < re.yend);
     return 0;
 }
 
@@ -764,7 +877,7 @@ clist_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
  * sometimes used for images, so its performance does matter.
  */
 
-private int
+static int
 clist_put_polyfill(gx_device *dev, fixed px, fixed py,
 		   const gs_fixed_point *points, int num_points,
 		   const gx_drawing_color *pdcolor, gs_logical_operation_t lop)
@@ -775,8 +888,9 @@ clist_put_polyfill(gx_device *dev, fixed px, fixed py,
     gx_device_clist_writer * const cdev =
 	&((gx_device_clist *)dev)->writer;
     gs_fixed_rect bbox;
-    int y, height, y0, y1;
+    int ry, rheight, y0, y1;
     bool slow_rop = cmd_slow_rop(dev, lop_know_S_0(lop), pdcolor);
+    cmd_rects_enum_t re;
 
     if (gs_debug_c(','))
 	return -1;		/* path-based banding is disabled */
@@ -786,27 +900,32 @@ clist_put_polyfill(gx_device *dev, fixed px, fixed py,
 	)
 	goto out;
     gx_path_bbox(&path, &bbox);
-    y = fixed2int(bbox.p.y) - 1;
-    height = fixed2int_ceiling(bbox.q.y) - y + 1;
-    fit_fill_y(dev, y, height);
-    fit_fill_h(dev, y, height);
-    if (height <= 0)
+    ry = fixed2int(bbox.p.y) - 1;
+    rheight = fixed2int_ceiling(bbox.q.y) - ry + 1;
+    fit_fill_y(dev, ry, rheight);
+    fit_fill_h(dev, ry, rheight);
+    if (rheight <= 0)
 	return 0;
-    y0 = y;
-    y1 = y + height;
-    FOR_RECTS_NO_ERROR {
-	if ((code = cmd_update_lop(cdev, pcls, lop)) < 0 ||
-	    (code = cmd_put_drawing_color(cdev, pcls, pdcolor)) < 0)
+    y0 = ry;
+    y1 = ry + rheight;
+    if (cdev->permanent_error < 0)
+      return (cdev->permanent_error);
+    RECT_ENUM_INIT(re, ry, rheight);
+    do {
+	RECT_STEP_INIT(re);
+	if ((code = cmd_update_lop(cdev, re.pcls, lop)) < 0 ||
+	    (code = cmd_put_drawing_color(cdev, re.pcls, pdcolor)) < 0)
 	    goto out;
-	pcls->colors_used.slow_rop |= slow_rop;
-	code = cmd_put_path(cdev, pcls, &path,
-			    int2fixed(max(y - 1, y0)),
-			    int2fixed(min(y + height + 1, y1)),
+	re.pcls->colors_used.slow_rop |= slow_rop;
+	code = cmd_put_path(cdev, re.pcls, &path,
+			    int2fixed(max(re.y - 1, y0)),
+			    int2fixed(min(re.y + re.height + 1, y1)),
 			    cmd_opv_polyfill,
 			    true, sn_none /* fill doesn't need the notes */ );
 	if (code < 0)
 	    goto out;
-    } END_RECTS_NO_ERROR;
+	re.y += re.height;
+    } while (re.y < re.yend);
 out:
     gx_path_free(&path, "clist_put_polyfill");
     return code;
@@ -871,7 +990,7 @@ typedef struct cmd_segment_writer_s {
 cmd_segment_writer;
 
 /* Put out a path segment command. */
-private int
+static int
 cmd_put_segment(cmd_segment_writer * psw, byte op,
 		const fixed * operands, segment_notes notes)
 {
@@ -1007,7 +1126,7 @@ cmd_put_segment(cmd_segment_writer * psw, byte op,
  * Write a path.  We go to a lot of trouble to omit segments that are
  * entirely outside the band.
  */
-private int
+static int
 cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 	     const gx_path * ppath, fixed ymin, fixed ymax, byte path_op,
 	     bool implicit_close, segment_notes keep_notes)
@@ -1076,6 +1195,7 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 
     /* Information about the last emitted operation: */
     int open = 0;		/* -1 if last was moveto, 1 if line/curveto, */
+    struct { fixed vs[6]; } prev;
 
     /* 0 if newpath/closepath */
 
@@ -1092,7 +1212,6 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     set_first_point();
     for (;;) {
 	fixed vs[6];
-	struct { fixed vs[6]; } prev;
 
 #define A vs[0]
 #define B vs[1]
@@ -1234,6 +1353,14 @@ cmd_put_path(gx_device_clist_writer * cldev, gx_clist_state * pcls,
 		     * ends.
 		     */
 		    set_first_point();
+		    /*
+		     * If implicit_close == true, we don't need an explicit closepath,
+		     * because the filling algorithm will close subpath automatically.
+		     * Otherwise, if open < 0, we have an empty closed path.
+		     * If side != 0, it is outside the band, so we can 
+		     * safely skip it, because the band has been expanded
+		     * with line width.
+		     */
 		    if (side != 0) {
 			open = 0;
 			continue;
