@@ -1,41 +1,39 @@
-/* Copyright (C) 1996, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxdcolor.c,v 1.12 2003/08/26 12:37:13 igor Exp $ */
+/*$Id: gxdcolor.c 8655 2008-04-21 14:53:38Z leonardo $ */
 /* Pure and null device color implementation */
 #include "gx.h"
 #include "memory_.h"
 #include "gserrors.h"
 #include "gsbittab.h"
 #include "gxdcolor.h"
+#include "gxpcolor.h"
 #include "gxdevice.h"
 #include "gxdevcli.h"
 
 /* Define the standard device color types. */
 
 /* 'none' means the color is not defined. */
-private dev_color_proc_save_dc(gx_dc_no_save_dc);
-private dev_color_proc_get_dev_halftone(gx_dc_no_get_dev_halftone);
-private dev_color_proc_load(gx_dc_no_load);
-private dev_color_proc_fill_rectangle(gx_dc_no_fill_rectangle);
-private dev_color_proc_fill_masked(gx_dc_no_fill_masked);
-private dev_color_proc_equal(gx_dc_no_equal);
-private dev_color_proc_write(gx_dc_no_write);
-private dev_color_proc_read(gx_dc_no_read);
-private dev_color_proc_get_nonzero_comps(gx_dc_no_get_nonzero_comps);
+static dev_color_proc_save_dc(gx_dc_no_save_dc);
+static dev_color_proc_get_dev_halftone(gx_dc_no_get_dev_halftone);
+static dev_color_proc_load(gx_dc_no_load);
+static dev_color_proc_fill_rectangle(gx_dc_no_fill_rectangle);
+static dev_color_proc_fill_masked(gx_dc_no_fill_masked);
+static dev_color_proc_equal(gx_dc_no_equal);
+static dev_color_proc_write(gx_dc_no_write);
+static dev_color_proc_read(gx_dc_no_read);
+static dev_color_proc_get_nonzero_comps(gx_dc_no_get_nonzero_comps);
 const gx_device_color_type_t gx_dc_type_data_none = {
     &st_bytes,
     gx_dc_no_save_dc, gx_dc_no_get_dev_halftone, gx_dc_no_get_phase,
@@ -47,11 +45,11 @@ const gx_device_color_type_t *const gx_dc_type_none = &gx_dc_type_data_none;
 #define gx_dc_type_none (&gx_dc_type_data_none)
 
 /* 'null' means the color has no effect when used for drawing. */
-private dev_color_proc_load(gx_dc_null_load);
-private dev_color_proc_fill_rectangle(gx_dc_null_fill_rectangle);
-private dev_color_proc_fill_masked(gx_dc_null_fill_masked);
-private dev_color_proc_equal(gx_dc_null_equal);
-private dev_color_proc_read(gx_dc_null_read);
+static dev_color_proc_load(gx_dc_null_load);
+static dev_color_proc_fill_rectangle(gx_dc_null_fill_rectangle);
+static dev_color_proc_fill_masked(gx_dc_null_fill_masked);
+static dev_color_proc_equal(gx_dc_null_equal);
+static dev_color_proc_read(gx_dc_null_read);
 const gx_device_color_type_t gx_dc_type_data_null = {
     &st_bytes,
     gx_dc_no_save_dc, gx_dc_no_get_dev_halftone, gx_dc_no_get_phase,
@@ -62,13 +60,13 @@ const gx_device_color_type_t gx_dc_type_data_null = {
 const gx_device_color_type_t *const gx_dc_type_null = &gx_dc_type_data_null;
 #define gx_dc_type_null (&gx_dc_type_data_null)
 
-private dev_color_proc_save_dc(gx_dc_pure_save_dc);
-private dev_color_proc_load(gx_dc_pure_load);
-private dev_color_proc_fill_rectangle(gx_dc_pure_fill_rectangle);
-private dev_color_proc_fill_masked(gx_dc_pure_fill_masked);
-private dev_color_proc_equal(gx_dc_pure_equal);
-private dev_color_proc_write(gx_dc_pure_write);
-private dev_color_proc_read(gx_dc_pure_read);
+static dev_color_proc_save_dc(gx_dc_pure_save_dc);
+static dev_color_proc_load(gx_dc_pure_load);
+static dev_color_proc_fill_rectangle(gx_dc_pure_fill_rectangle);
+static dev_color_proc_fill_masked(gx_dc_pure_fill_masked);
+static dev_color_proc_equal(gx_dc_pure_equal);
+static dev_color_proc_write(gx_dc_pure_write);
+static dev_color_proc_read(gx_dc_pure_read);
 const gx_device_color_type_t gx_dc_type_data_pure = {
     &st_bytes,
     gx_dc_pure_save_dc, gx_dc_no_get_dev_halftone, gx_dc_no_get_phase,
@@ -130,8 +128,8 @@ gx_device_decache_colors(gx_device *dev)
 }
 
 /* Set a null RasterOp source. */
-private const gx_rop_source_t gx_rop_no_source_0 = {gx_rop_no_source_body(0)};
-private const gx_rop_source_t gx_rop_no_source_1 = {gx_rop_no_source_body(1)};
+static const gx_rop_source_t gx_rop_no_source_0 = {gx_rop_no_source_body(0)};
+static const gx_rop_source_t gx_rop_no_source_1 = {gx_rop_no_source_body(1)};
 void
 gx_set_rop_no_source(const gx_rop_source_t **psource,
 		     gx_rop_source_t *pno_source, gx_device *dev)
@@ -177,11 +175,11 @@ gx_device_color_equal(const gx_device_color *pdevc1,
  *
  * Returns gs_error_unknownerror for an unrecognized type.
  */
-private  const gx_device_color_type_t * dc_color_type_table[] = {
+static  const gx_device_color_type_t * dc_color_type_table[] = {
     gx_dc_type_none,            /* unset device color */
     gx_dc_type_null,            /* blank (transparent) device color */
     gx_dc_type_pure,            /* pure device color */
-    /* gx_dc_type_pattern, */   /* patterns - not used in command list */
+    gx_dc_type_pattern,         /* patterns */
     gx_dc_type_ht_binary,       /* binary halftone device colors */
     gx_dc_type_ht_colored,      /* general halftone device colors */
     gx_dc_type_wts              /* well-tempered screen device colors */
@@ -226,26 +224,26 @@ gx_dc_ht_get_phase(const gx_device_color * pdevc, gs_int_point * pphase)
 }
 
 /* ------ Undefined color ------ */
-private void
+static void
 gx_dc_no_save_dc(const gx_device_color * pdevc, gx_device_color_saved * psdc)
 {
     psdc->type = pdevc->type;
 }
 
-private const gx_device_halftone *
+static const gx_device_halftone *
 gx_dc_no_get_dev_halftone(const gx_device_color * pdevc)
 {
     return 0;
 }
 
-private int
+static int
 gx_dc_no_load(gx_device_color *pdevc, const gs_imager_state *ignore_pis,
 	      gx_device *ignore_dev, gs_color_select_t ignore_select)
 {
     return 0;
 }
 
-private int
+static int
 gx_dc_no_fill_rectangle(const gx_device_color *pdevc, int x, int y,
 			int w, int h, gx_device *dev,
 			gs_logical_operation_t lop,
@@ -261,7 +259,7 @@ gx_dc_no_fill_rectangle(const gx_device_color *pdevc, int x, int y,
     return gx_dc_pure_fill_rectangle(&filler, x, y, w, h, dev, lop, source);
 }
 
-private int
+static int
 gx_dc_no_fill_masked(const gx_device_color *pdevc, const byte *data,
 		     int data_x, int raster, gx_bitmap_id id,
 		     int x, int y, int w, int h, gx_device *dev,
@@ -272,17 +270,18 @@ gx_dc_no_fill_masked(const gx_device_color *pdevc, const byte *data,
     return_error(gs_error_Fatal);
 }
 
-private bool
+static bool
 gx_dc_no_equal(const gx_device_color *pdevc1, const gx_device_color *pdevc2)
 {
     return false;
 }
 
-private int
+static int
 gx_dc_no_write(
     const gx_device_color *         pdevc,      /* ignored */
     const gx_device_color_saved *   psdc,       /* ignored */
     const gx_device *               dev,        /* ignored */
+    uint			    offset,     /* ignored */
     byte *                          data,       /* ignored */
     uint *                          psize )
 {
@@ -290,12 +289,13 @@ gx_dc_no_write(
     return psdc != 0 && psdc->type == pdevc->type ? 1 : 0;
 }
 
-private int
+static int
 gx_dc_no_read(
     gx_device_color *       pdevc,
     const gs_imager_state * pis,                /* ignored */
     const gx_device_color * prior_devc,         /* ignored */
     const gx_device *       dev,                /* ignored */
+    uint		    offset,             /* ignored */
     const byte *            pdata,              /* ignored */
     uint                    size,               /* ignored */
     gs_memory_t *           mem )               /* ignored */
@@ -304,7 +304,33 @@ gx_dc_no_read(
     return 0;
 }
 
-private int
+int
+gx_dc_cannot_write(
+    const gx_device_color *         pdevc,      /* ignored */
+    const gx_device_color_saved *   psdc,       /* ignored */
+    const gx_device *               dev,        /* ignored */
+    uint			    offset,     /* ignored */
+    byte *                          data,       /* ignored */
+    uint *                          psize )
+{
+    return_error(gs_error_unknownerror);
+}
+
+int
+gx_dc_cannot_read(
+    gx_device_color *       pdevc,
+    const gs_imager_state * pis,                /* ignored */
+    const gx_device_color * prior_devc,         /* ignored */
+    const gx_device *       dev,                /* ignored */
+    uint		    offset,             /* ignored */
+    const byte *            pdata,              /* ignored */
+    uint                    size,               /* ignored */
+    gs_memory_t *           mem )               /* ignored */
+{
+    return_error(gs_error_unknownerror);
+}
+
+static int
 gx_dc_no_get_nonzero_comps(
     const gx_device_color * pdevc_ignored,
     const gx_device *       dev_ignored,
@@ -315,14 +341,14 @@ gx_dc_no_get_nonzero_comps(
 
 /* ------ Null color ------ */
 
-private int
+static int
 gx_dc_null_load(gx_device_color *pdevc, const gs_imager_state *ignore_pis,
 		gx_device *ignore_dev, gs_color_select_t ignore_select)
 {
     return 0;
 }
 
-private int
+static int
 gx_dc_null_fill_rectangle(const gx_device_color * pdevc, int x, int y,
 			  int w, int h, gx_device * dev,
 			  gs_logical_operation_t lop,
@@ -331,7 +357,7 @@ gx_dc_null_fill_rectangle(const gx_device_color * pdevc, int x, int y,
     return 0;
 }
 
-private int
+static int
 gx_dc_null_fill_masked(const gx_device_color * pdevc, const byte * data,
 		       int data_x, int raster, gx_bitmap_id id,
 		       int x, int y, int w, int h, gx_device * dev,
@@ -340,18 +366,19 @@ gx_dc_null_fill_masked(const gx_device_color * pdevc, const byte * data,
     return 0;
 }
 
-private bool
+static bool
 gx_dc_null_equal(const gx_device_color * pdevc1, const gx_device_color * pdevc2)
 {
     return pdevc2->type == pdevc1->type;
 }
 
-private int
+static int
 gx_dc_null_read(
     gx_device_color *       pdevc,
     const gs_imager_state * pis,                /* ignored */
     const gx_device_color * prior_devc,         /* ignored */
     const gx_device *       dev,                /* ignored */
+    uint		    offset,             /* ignored */
     const byte *            pdata,              /* ignored */
     uint                    size,               /* ignored */
     gs_memory_t *           mem )               /* ignored */
@@ -362,14 +389,14 @@ gx_dc_null_read(
 
 /* ------ Pure color ------ */
 
-private void
+static void
 gx_dc_pure_save_dc(const gx_device_color * pdevc, gx_device_color_saved * psdc)
 {
     psdc->type = pdevc->type;
     psdc->colors.pure = pdevc->colors.pure;
 }
 
-private int
+static int
 gx_dc_pure_load(gx_device_color * pdevc, const gs_imager_state * ignore_pis,
 		gx_device * ignore_dev, gs_color_select_t ignore_select)
 {
@@ -378,7 +405,7 @@ gx_dc_pure_load(gx_device_color * pdevc, const gs_imager_state * ignore_pis,
 
 /* Fill a rectangle with a pure color. */
 /* Note that we treat this as "texture" for RasterOp. */
-private int
+static int
 gx_dc_pure_fill_rectangle(const gx_device_color * pdevc, int x, int y,
 		  int w, int h, gx_device * dev, gs_logical_operation_t lop,
 			  const gx_rop_source_t * source)
@@ -402,7 +429,7 @@ gx_dc_pure_fill_rectangle(const gx_device_color * pdevc, int x, int y,
 
 /* Fill a mask with a pure color. */
 /* Note that there is no source in this case: the mask is the source. */
-private int
+static int
 gx_dc_pure_fill_masked(const gx_device_color * pdevc, const byte * data,
 	int data_x, int raster, gx_bitmap_id id, int x, int y, int w, int h,
 		   gx_device * dev, gs_logical_operation_t lop, bool invert)
@@ -420,17 +447,22 @@ gx_dc_pure_fill_masked(const gx_device_color * pdevc, const byte * data,
 	gx_color_index scolors[2];
 	gx_color_index tcolors[2];
 
-	scolors[0] = gx_device_black(dev);
-	scolors[1] = gx_device_white(dev);
+	if ( lop != lop_default ) {
+	    scolors[0] = gx_device_white(dev);
+	    scolors[1] = gx_device_black(dev);
+	} else {
+	    scolors[0] = gx_device_black(dev);
+	    scolors[1] = gx_device_white(dev);
+        }
 	tcolors[0] = tcolors[1] = pdevc->colors.pure;
 	return (*dev_proc(dev, strip_copy_rop))
 	    (dev, data, data_x, raster, id, scolors,
 	     NULL, tcolors, x, y, w, h, 0, 0,
-	     (invert ? rop3_invert_S(lop) : lop) | lop_S_transparent);
+	     (invert ? rop3_invert_S(lop) : lop) | (rop3_S | lop_S_transparent));
     }
 }
 
-private bool
+static bool
 gx_dc_pure_equal(const gx_device_color * pdevc1, const gx_device_color * pdevc2)
 {
     return pdevc2->type == pdevc1->type &&
@@ -468,11 +500,12 @@ gx_dc_pure_equal(const gx_device_color * pdevc1, const gx_device_color * pdevc2)
  *  < 0, != gs_error_rangecheck, in the event of some other error; in this
  *  case *psize is not changed.
  */
-private int
+static int
 gx_dc_pure_write(
     const gx_device_color *         pdevc,
     const gx_device_color_saved *   psdc,       /* ignored */
     const gx_device *               dev,
+    uint			    offset,     /* ignored */
     byte *                          pdata,
     uint *                          psize )
 {
@@ -514,12 +547,13 @@ gx_dc_pure_write(
  *
  *  # of bytes read if everthing OK, < 0 in the event of an error
  */
-private int
+static int
 gx_dc_pure_read(
     gx_device_color *       pdevc,
     const gs_imager_state * pis,                /* ignored */
     const gx_device_color * prior_devc,         /* ignored */
     const gx_device *       dev,
+    uint		    offset,             /* ignored */
     const byte *            pdata,
     uint                    size,
     gs_memory_t *           mem )               /* ignored */
@@ -692,7 +726,7 @@ gx_dc_write_color(
     /* check for adequate space */
     if (*psize < num_bytes) {
         *psize = num_bytes;
-        return gs_error_rangecheck;
+        return_error(gs_error_rangecheck);
     }
     *psize = num_bytes;
 
@@ -701,7 +735,7 @@ gx_dc_write_color(
         *psize = 1;
         *pdata = 0xff;
     } else {
-        if (depth < 8 * sizeof(gx_color_index))
+        if (depth < 8 * arch_sizeof_color_index)
             color &= ((gx_color_index)1 << depth) - 1;
         while (--num_bytes >= 0) {
             pdata[num_bytes] = color & 0xff;
@@ -742,7 +776,7 @@ gx_dc_read_color(
 
     /* check that enough data has been provided */
     if (size < 1 || (pdata[0] != 0xff && size < num_bytes))
-        return gs_error_rangecheck;
+        return_error(gs_error_rangecheck);
 
     /* check of gx_no_color_index */
     if (pdata[0] == 0xff) {
@@ -750,8 +784,8 @@ gx_dc_read_color(
         return 1;
     }
 
-    /* num_bytes > sizeof(gx_color_index), discard first byte */
-    for (i = (num_bytes >= sizeof(gx_color_index) ? 1 : 0); i < num_bytes; i++)
+    /* num_bytes > arch_sizeof_color_index, discard first byte */
+    for (i = (num_bytes >= arch_sizeof_color_index ? 1 : 0); i < num_bytes; i++)
         color = (color << 8) | pdata[i];
     *pcolor = color;
     return num_bytes;
