@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxdevice.h 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: gxdevice.h 8790 2008-06-09 07:33:57Z leonardo $ */
 /* Definitions for device implementors */
 
 #ifndef gxdevice_INCLUDED
@@ -102,7 +102,7 @@
 #define std_device_part3_()\
 	0/*PageCount*/, 0/*ShowpageCount*/, 1/*NumCopies*/, 0/*NumCopies_set*/,\
 	0/*IgnoreNumCopies*/, 0/*UseCIEColor*/, 0/*LockSafetyParams*/,\
-	INIT_NAMED_COLOR_PTR	/* 'Named color' callback pointer */\
+	0/*band_offset_x*/, 0/*band_offset_y*/, {false}/* sgr */,\
 	{ gx_default_install, gx_default_begin_page, gx_default_end_page }
 /*
  * We need a number of different variants of the std_device_ macro simply
@@ -274,6 +274,7 @@ dev_proc_fill_linear_color_scanline(gx_default_fill_linear_color_scanline);
 dev_proc_fill_linear_color_trapezoid(gx_default_fill_linear_color_trapezoid);
 dev_proc_fill_linear_color_triangle(gx_default_fill_linear_color_triangle);
 dev_proc_update_spot_equivalent_colors(gx_default_update_spot_equivalent_colors);
+dev_proc_ret_devn_params(gx_default_ret_devn_params);
 /* BACKWARD COMPATIBILITY */
 #define gx_non_imaging_create_compositor gx_null_create_compositor
 
@@ -353,6 +354,7 @@ dev_proc_fill_linear_color_scanline(gx_forward_fill_linear_color_scanline);
 dev_proc_fill_linear_color_trapezoid(gx_forward_fill_linear_color_trapezoid);
 dev_proc_fill_linear_color_triangle(gx_forward_fill_linear_color_triangle);
 dev_proc_update_spot_equivalent_colors(gx_forward_update_spot_equivalent_colors);
+dev_proc_ret_devn_params(gx_forward_ret_devn_params);
 
 /* ---------------- Implementation utilities ---------------- */
 
@@ -577,4 +579,6 @@ int gdev_write_output_media(int index, gs_param_dict * pdict,
 
 int gdev_end_output_media(gs_param_list * mlist, gs_param_dict * pdict);
 
+void gx_device_request_leadingedge(gx_device *dev, int le_req);
+
 #endif /* gxdevice_INCLUDED */
