@@ -1,23 +1,19 @@
-/* Copyright (C) 1989-2003 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxfdrop.c,v 1.16 2004/12/08 21:35:13 stefan Exp $ */
+/* $Id: gxfdrop.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Dropout prevention for a character rasterization. */
 
-#include <assert.h>
 #include "gx.h"
 #include "gserrors.h"
 #include "gsstruct.h"
@@ -54,10 +50,9 @@ void init_section(section *sect, int i0, int i1)
     }
 }
 
-private margin * alloc_margin(line_list * ll)
+static margin * alloc_margin(line_list * ll)
 {   margin *m;
 
-    assert(ll->fo->pseudo_rasterization);
     if (ll->free_margin_list != 0) {
 	m = ll->free_margin_list;
 	ll->free_margin_list = ll->free_margin_list->next;
@@ -72,7 +67,7 @@ private margin * alloc_margin(line_list * ll)
     return m;
 }
 
-private void release_margin_list(line_list * ll, margin_set *ms)
+static void release_margin_list(line_list * ll, margin_set *ms)
 {   margin * m1 = ms->margin_list;
 
     if (m1 == 0)
@@ -97,7 +92,7 @@ void free_all_margins(line_list * ll)
     }
 }
 
-private int store_margin(line_list * ll, margin_set * set, int ii0, int ii1)
+static int store_margin(line_list * ll, margin_set * set, int ii0, int ii1)
 {
     /*
      * We need to add margin to the ordered margin list.
@@ -106,13 +101,17 @@ private int store_margin(line_list * ll, margin_set * set, int ii0, int ii1)
     int i0 = ii0, i1 = ii1;
     margin *m0 = set->margin_touched, *m1;
 
-    assert(ii0 >= 0 && ii1 <= ll->bbox_width);
+    if (!ll->fo->pseudo_rasterization)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    if (ii0 < 0 || ii1 > ll->bbox_width)
+	return_error(gs_error_unregistered); /* Must not happen. */
     set->margin_touched = 0; /* safety */
     /* Find contacting elements. */
     if (m0 != 0) {
 	margin  *m_last = m0, *mb, *me;
 
-	assert(set->margin_list != 0);
+	if (set->margin_list == 0)
+	    return_error(gs_error_unregistered); /* Must not happen. */
 	if (i1 < m0->ibeg) {
 	    do {
 		m0 = m0->prev;
@@ -211,15 +210,15 @@ private int store_margin(line_list * ll, margin_set * set, int ii0, int ii1)
     return 0;
 }
 
-private inline int to_interval(int x, int l, int u)
+static inline int to_interval(int x, int l, int u)
 {   return x < l ? l : x > u ? u : x;
 }
 
-private inline fixed Y_AT_X(active_line *alp, fixed xp)
+static inline fixed Y_AT_X(active_line *alp, fixed xp)
 {   return alp->start.y + fixed_mult_quo(xp - alp->start.x,  alp->diff.y, alp->diff.x);
 }
 
-private int margin_boundary(line_list * ll, margin_set * set, active_line * alp, 
+static int margin_boundary(line_list * ll, margin_set * set, active_line * alp, 
 			    fixed xx0, fixed xx1, fixed yy0, fixed yy1, int dir, fixed y0, fixed y1)
 {   section *sect = set->sect;
     fixed x0, x1, xmin, xmax;
@@ -248,7 +247,8 @@ private int margin_boundary(line_list * ll, margin_set * set, active_line * alp,
 	    xp0 += fixed_1;
 	    i0++;
 	}
-	assert(i0 >= 0);
+	if (i0 < 0)
+	    return_error(gs_error_unregistered); /* Must not happen. */
 	for (i = i0, xp = xp0; xp < xmax && i < ll->bbox_width; xp += fixed_1, i++) {
 	    fixed y = (alp == 0 ? yy0 : Y_AT_X(alp, xp));
 	    fixed dy = y - set->y;
@@ -292,7 +292,8 @@ private int margin_boundary(line_list * ll, margin_set * set, active_line * alp,
 	    if (*b == -1 || (*b != -2 && ( ud ? *b > h : *b < h)))
 		*b = h;
 	}
-	assert(i0 >= 0 && i <= ll->bbox_width);
+	if (i0 < 0 || i > ll->bbox_width)
+	    return_error(gs_error_unregistered); /* Must not happen. */
 #	endif
     if (i > i0)
 	return store_margin(ll, set, i0, i);
@@ -341,7 +342,7 @@ int continue_margin_common(line_list * ll, margin_set * set, active_line * flp,
     return margin_boundary(ll, set, alp, 0, 0, yy0, yy1, -1, y0, y1);
 }
 
-private inline int mark_margin_interior(line_list * ll, margin_set * set, active_line * flp, active_line * alp, fixed y, fixed y0, fixed y1)
+static inline int mark_margin_interior(line_list * ll, margin_set * set, active_line * flp, active_line * alp, fixed y, fixed y0, fixed y1)
 {
     section *sect = set->sect;
     fixed x0 = (y == y0 ? flp->x_current : y == y1 ? flp->x_next : AL_X_AT_Y(flp, y));
@@ -353,7 +354,8 @@ private inline int mark_margin_interior(line_list * ll, margin_set * set, active
     ii0 = i0 - ll->bbox_left;
     ii1 = fixed2int_var_pixround(x1) - ll->bbox_left;
     if (ii0 < ii1) {
-	assert(ii0 >= 0 && ii1 <= ll->bbox_width);
+	if (ii0 < 0 || ii1 > ll->bbox_width)
+	    return_error(gs_error_unregistered); /* Must not happen. */
 	for (i = ii0; i < ii1; i++) {
 	    sect[i].y0 = sect[i].y1 = -2;
 	    vd_circle(int2fixed(i + ll->bbox_left) + fixed_half, y, 3, RGB(255, 0, 0));
@@ -384,7 +386,7 @@ int margin_interior(line_list * ll, active_line * flp, active_line * alp, fixed
     return 0;
 }
 
-private inline int process_h_sect(line_list * ll, margin_set * set, active_line * hlp0, 
+static inline int process_h_sect(line_list * ll, margin_set * set, active_line * hlp0, 
     active_line * plp, active_line * flp, int side, fixed y0, fixed y1)
 {
     active_line *hlp = hlp0;
@@ -409,7 +411,7 @@ private inline int process_h_sect(line_list * ll, margin_set * set, active_line
     return 0;	
 }
 
-private inline int process_h_side(line_list * ll, margin_set * set, active_line * hlp, 
+static inline int process_h_side(line_list * ll, margin_set * set, active_line * hlp, 
     active_line * plp, active_line * flp, active_line * alp, int side, fixed y0, fixed y1)
 {   if (plp != 0 || flp != 0 || (plp == 0 && flp == 0 && alp == 0)) {
 	/* We don't know here, whether the opposite (-) side is painted with
@@ -429,7 +431,7 @@ private inline int process_h_side(line_list * ll, margin_set * set, active_line
     return 0;
 }
 
-private inline int process_h_list(line_list * ll, active_line * hlp, active_line * plp, 
+static inline int process_h_list(line_list * ll, active_line * hlp, active_line * plp, 
     active_line * flp, active_line * alp, int side, fixed y0, fixed y1)
 {   fixed y = hlp->start.y;
 
@@ -470,7 +472,7 @@ int process_h_lists(line_list * ll, active_line * plp, active_line * flp, active
     return 0;
 }
 
-private inline int compute_padding(section *s)
+static inline int compute_padding(section *s)
 {
     return (s->y0 < 0 || s->y1 < 0 ? -2 : /* contacts a trapezoid - don't paint */
 	    s->y1 < fixed_half ? 0 : 
@@ -478,7 +480,7 @@ private inline int compute_padding(section *s)
 	    fixed_half - s->y0 < s->y1 - fixed_half ? 1 : 0);
 }
 
-private int fill_margin(gx_device * dev, const line_list * ll, margin_set *ms, int i0, int i1)
+static int fill_margin(gx_device * dev, const line_list * ll, margin_set *ms, int i0, int i1)
 {   /* Returns the new index (positive) or return code (negative). */
     section *sect = ms->sect;
     int iy = fixed2int_var_pixround(ms->y);
@@ -486,7 +488,8 @@ private int fill_margin(gx_device * dev, const line_list * ll, margin_set *ms, i
     const fill_options * const fo = ll->fo;
     const bool FILL_DIRECT = fo->fill_direct;
 
-    assert(i0 >= 0 && i1 <= ll->bbox_width);
+    if (i0 < 0 || i1 > ll->bbox_width)
+	return_error(gs_error_unregistered); /* Must not happen. */
     ir = i0;
     for (i = i0; i < i1; i++) {
 	int y0 = sect[i].y0, y1 = sect[i].y1, hh;
