@@ -1,20 +1,17 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gximag3x.c,v 1.20 2004/09/16 08:03:56 igor Exp $ */
+/* $Id: gximag3x.c 8803 2008-06-24 14:16:29Z leonardo $ */
 /* ImageType 3x image implementation */
 /****** THE REAL WORK IS NYI ******/
 #include "math_.h"		/* for ceil, floor */
@@ -34,11 +31,11 @@
 extern_st(st_color_space);
 
 /* Forward references */
-private dev_proc_begin_typed_image(gx_begin_image3x);
-private image_enum_proc_plane_data(gx_image3x_plane_data);
-private image_enum_proc_end_image(gx_image3x_end_image);
-private image_enum_proc_flush(gx_image3x_flush);
-private image_enum_proc_planes_wanted(gx_image3x_planes_wanted);
+static dev_proc_begin_typed_image(gx_begin_image3x);
+static image_enum_proc_plane_data(gx_image3x_plane_data);
+static image_enum_proc_end_image(gx_image3x_end_image);
+static image_enum_proc_flush(gx_image3x_flush);
+static image_enum_proc_planes_wanted(gx_image3x_planes_wanted);
 
 /* GC descriptor */
 private_st_gs_image3x();
@@ -49,13 +46,13 @@ const gx_image_type_t gs_image_type_3x = {
     gx_image_no_sput, gx_image_no_sget, gx_image_default_release,
     IMAGE3X_IMAGETYPE
 };
-private const gx_image_enum_procs_t image3x_enum_procs = {
+static const gx_image_enum_procs_t image3x_enum_procs = {
     gx_image3x_plane_data, gx_image3x_end_image,
     gx_image3x_flush, gx_image3x_planes_wanted
 };
 
 /* Initialize an ImageType 3x image. */
-private void
+static void
 gs_image3x_mask_init(gs_image3x_mask_t *pimm)
 {
     pimm->InterleaveType = 0;	/* not a valid type */
@@ -64,7 +61,7 @@ gs_image3x_mask_init(gs_image3x_mask_t *pimm)
     pimm->MaskDict.BitsPerComponent = 0;	/* not supplied */
 }
 void
-gs_image3x_t_init(gs_image3x_t * pim, const gs_color_space * color_space)
+gs_image3x_t_init(gs_image3x_t * pim, gs_color_space * color_space)
 {
     gs_pixel_image_t_init((gs_pixel_image_t *) pim, color_space);
     pim->type = &gs_image_type_3x;
@@ -98,7 +95,6 @@ typedef struct gx_image3x_enum_s {
     gx_device *pcdev;		/* gx_device_mask_clip in default impl. */
     int num_components;		/* (not counting masks) */
     int bpc;			/* pixel BitsPerComponent */
-    gs_memory_t *memory;
 #define NUM_MASKS 2		/* opacity, shape */
     image3x_channel_state_t mask[NUM_MASKS], pixel;
 } gx_image3x_enum_t;
@@ -119,7 +115,7 @@ typedef struct image3x_channel_values_s {
     gs_int_rect rect;
     gs_image_t image;
 } image3x_channel_values_t;
-private int check_image3x_mask(const gs_image3x_t *pim,
+static int check_image3x_mask(const gs_image3x_t *pim,
 			       const gs_image3x_mask_t *pimm,
 			       const image3x_channel_values_t *ppcv,
 			       image3x_channel_values_t *pmcv,
@@ -216,11 +212,9 @@ gx_begin_image3x_generic(gx_device * dev,
 	    minfo[i] = 0;
 	    continue;
 	}
-	pmcs =  gs_alloc_struct(mem, gs_color_space, &st_color_space,
-				"gx_begin_image3x_generic");
-	if (pmcs == 0)
-	    return_error(gs_error_VMerror);
-	gs_cspace_init_DevicePixel(mem, pmcs, penum->mask[i].depth);
+	code = gs_cspace_new_DevicePixel(mem, &pmcs, penum->mask[i].depth);
+	if (code < 0)
+	    return code;
 	mrect.p.x = mrect.p.y = 0;
 	mrect.q.x = penum->mask[i].width;
 	mrect.q.y = penum->mask[i].height;
@@ -260,12 +254,15 @@ gx_begin_image3x_generic(gx_device * dev,
 	    m_mat.tx -= origin[i].x;
 	    m_mat.ty -= origin[i].y;
 	    /*
-	     * Note that pis = NULL here, since we don't want to have to
-	     * create another imager state with default log_op, etc.
+	     * Peter put in a comment that said " Note that pis = NULL here,
+	     * since we don't want to have to create another imager state with
+	     * default log_op, etc." and passed NULL instead of pis to this
+	     * routine.  However Image type 1 need the imager state (see
+	     * bug 688348) thus his optimization was removed.
 	     * dcolor = NULL is OK because this is an opaque image with
 	     * CombineWithColor = false.
 	     */
-	    code = gx_device_begin_typed_image(mdev, NULL, &m_mat,
+	    code = gx_device_begin_typed_image(mdev, pis, &m_mat,
 			       (const gs_image_common_t *)&mask[i].image,
 					       &mask[i].rect, NULL, NULL,
 					       mem, &penum->mask[i].info);
@@ -353,7 +350,7 @@ gx_begin_image3x_generic(gx_device * dev,
     gs_free_object(mem, penum, "gx_begin_image3x");
     return code;
 }
-private bool
+static bool
 check_image3x_extent(floatp mask_coeff, floatp data_coeff)
 {
     if (mask_coeff == 0)
@@ -368,7 +365,7 @@ check_image3x_extent(floatp mask_coeff, floatp data_coeff)
  * pmcs->{InterleaveType,width,height,full_height,depth,data,y,skip}.
  * If the mask is omitted, sets pmcs->depth = 0 and returns normally.
  */
-private bool
+static bool
 check_image3x_mask(const gs_image3x_t *pim, const gs_image3x_mask_t *pimm,
 		   const image3x_channel_values_t *ppcv,
 		   image3x_channel_values_t *pmcv,
@@ -398,7 +395,7 @@ check_image3x_mask(const gs_image3x_t *pim, const gs_image3x_mask_t *pimm,
 	    break;
 	case interleave_separate_source:
 	    switch (pimm->MaskDict.BitsPerComponent) {
-	    case 1: case 2: case 4: case 8:
+		    case 1: case 2: case 4: case 8: case 12: case 16:
 		break;
 	    default:
 		return_error(gs_error_rangecheck);
@@ -454,7 +451,7 @@ check_image3x_mask(const gs_image3x_t *pim, const gs_image3x_mask_t *pimm,
  * Return > 0 if we want more data from channel 1 now, < 0 if we want more
  * from channel 2 now, 0 if we want both.
  */
-private int
+static int
 channel_next(const image3x_channel_state_t *pics1,
 	     const image3x_channel_state_t *pics2)
 {
@@ -482,8 +479,8 @@ channel_next(const image3x_channel_state_t *pics1,
 }
 
 /* Define the default implementation of ImageType 3 processing. */
-private IMAGE3X_MAKE_MID_PROC(make_midx_default); /* check prototype */
-private int
+static IMAGE3X_MAKE_MID_PROC(make_midx_default); /* check prototype */
+static int
 make_midx_default(gx_device **pmidev, gx_device *dev, int width, int height,
 		 int depth, gs_memory_t *mem)
 {
@@ -491,6 +488,9 @@ make_midx_default(gx_device **pmidev, gx_device *dev, int width, int height,
     gx_device_memory *midev;
     int code;
 
+    if (width != 0)
+	if (height > max_ulong/width)	/* protect against overflow in bitmap size */
+	    return_error(gs_error_VMerror);
     if (mdproto == 0)
 	return_error(gs_error_rangecheck);
     midev = gs_alloc_struct(mem, gx_device_memory, &st_device_memory,
@@ -514,8 +514,8 @@ make_midx_default(gx_device **pmidev, gx_device *dev, int width, int height,
     *pmidev = (gx_device *)midev;
     return 0;
 }
-private IMAGE3X_MAKE_MCDE_PROC(make_mcdex_default);  /* check prototype */
-private int
+static IMAGE3X_MAKE_MCDE_PROC(make_mcdex_default);  /* check prototype */
+static int
 make_mcdex_default(gx_device *dev, const gs_imager_state *pis,
 		   const gs_matrix *pmat, const gs_image_common_t *pic,
 		   const gs_int_rect *prect, const gx_drawing_color *pdcolor,
@@ -554,7 +554,7 @@ make_mcdex_default(gx_device *dev, const gs_imager_state *pis,
     *pmcdev = (gx_device *)bbdev;
     return 0;
 }
-private int
+static int
 gx_begin_image3x(gx_device * dev,
 		const gs_imager_state * pis, const gs_matrix * pmat,
 		const gs_image_common_t * pic, const gs_int_rect * prect,
@@ -567,7 +567,7 @@ gx_begin_image3x(gx_device * dev,
 }
 
 /* Process the next piece of an ImageType 3 image. */
-private int
+static int
 gx_image3x_plane_data(gx_image_enum_common_t * info,
 		     const gx_image_plane_t * planes, int height,
 		     int *rows_used)
@@ -743,10 +743,13 @@ gx_image3x_plane_data(gx_image_enum_common_t * info,
 	      h, (mask_plane[0].data ? "+" : ""), penum->mask[0].y,
 	      (mask_plane[1].data ? "+" : ""), penum->mask[1].y,
 	      (pixel_planes[0].data ? "+" : ""), penum->pixel.y);
-    if (penum->mask[0].y >= penum->mask[0].height &&
-	penum->mask[1].y >= penum->mask[1].height &&
-	penum->pixel.y >= penum->pixel.height)
-	return 1;
+    if (penum->mask[0].depth == 0 || penum->mask[0].y >= penum->mask[0].height) {
+        if (penum->mask[1].depth == 0 || penum->mask[1].y >= penum->mask[1].height) {
+	    if (penum->pixel.y >= penum->pixel.height) {
+	        return 1;
+            }
+        }
+    }
     /*
      * The mask may be complete (gx_image_plane_data_rows returned 1),
      * but there may still be pixel rows to go, so don't return 1 here.
@@ -755,7 +758,7 @@ gx_image3x_plane_data(gx_image_enum_common_t * info,
 }
 
 /* Flush buffered data. */
-private int
+static int
 gx_image3x_flush(gx_image_enum_common_t * info)
 {
     gx_image3x_enum_t * const penum = (gx_image3x_enum_t *) info;
@@ -769,7 +772,7 @@ gx_image3x_flush(gx_image_enum_common_t * info)
 }
 
 /* Determine which data planes are wanted. */
-private bool
+static bool
 gx_image3x_planes_wanted(const gx_image_enum_common_t * info, byte *wanted)
 {
     const gx_image3x_enum_t * const penum = (const gx_image3x_enum_t *) info;
@@ -821,7 +824,7 @@ gx_image3x_planes_wanted(const gx_image_enum_common_t * info, byte *wanted)
 }
 
 /* Clean up after processing an ImageType 3x image. */
-private int
+static int
 gx_image3x_end_image(gx_image_enum_common_t * info, bool draw_last)
 {
     gx_image3x_enum_t *penum = (gx_image3x_enum_t *) info;
@@ -851,6 +854,6 @@ gx_image3x_end_image(gx_image_enum_common_t * info, bool draw_last)
     gs_free_object(mem, pcdev, "gx_image3x_end_image(pcdev)");
     gs_free_object(mem, mdev0, "gx_image3x_end_image(mask[0].mdev)");
     gs_free_object(mem, mdev1, "gx_image3x_end_image(mask[1].mdev)");
-    gs_free_object(mem, penum, "gx_image3x_end_image");
+    gx_image_free_enum(&info);
     return (pcode < 0 ? pcode : scode < 0 ? scode : ocode);
 }
