@@ -1,20 +1,17 @@
-/* Copyright (C) 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxmclip.h,v 1.2.6.1 2002/02/22 19:45:59 ray Exp $ */
+/* $Id: gxmclip.h 8137 2007-07-24 21:18:53Z ray $ */
 /* Mask clipping device and interface */
 /* Requires gxdevice.h, gxdevmem.h */
 
@@ -35,7 +32,7 @@
  * stack frame.
  */
 
-#define tile_clip_buffer_request 300
+#define tile_clip_buffer_request 16384	/* enough for 2400 dpi up to 54" wide */
 #define tile_clip_buffer_size\
   ((tile_clip_buffer_request / arch_sizeof_long) * arch_sizeof_long)
 typedef struct gx_device_mask_clip_s {
@@ -62,10 +59,10 @@ extern_st(st_device_mask_clip);
  * We supply an explicit device space origin or phase.
  * Note that this procedure does not set cdev->tiles.
  */
-int gx_mask_clip_initialize(P7(gx_device_mask_clip * cdev,
-			       const gx_device_mask_clip * proto,
-			       const gx_bitmap * bits, gx_device * tdev,
-			       int tx, int ty, gs_memory_t *mem));
+int gx_mask_clip_initialize(gx_device_mask_clip * cdev,
+			    const gx_device_mask_clip * proto,
+			    const gx_bitmap * bits, gx_device * tdev,
+			    int tx, int ty, gs_memory_t *mem);
 
 /*
  * Prepare colors for a copy_mono operation.
