@@ -1,20 +1,17 @@
-/* Copyright (C) 2002 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxoprect.c,v 1.5 2004/01/29 18:19:41 ray Exp $ */
+/* $Id: gxoprect.c 8250 2007-09-25 13:31:24Z giles $ */
 /* generic (very slow) overprint fill rectangle implementation */
 
 #include "memory_.h"
@@ -33,7 +30,7 @@
  * divisor of 8 and thus a power of 2, which implies that 8 / depth is
  * also a power of 2.
  */
-private void
+static void
 unpack_scanline_lt8(
     gx_color_index *    destp,
     const byte *        srcp,
@@ -71,7 +68,7 @@ unpack_scanline_lt8(
  * Pack a scanline for a depth of < 8. Note that data prior to dest_offset
  * and any data beyond the width must be left undisturbed.
  */
-private void
+static void
 pack_scanline_lt8(
     const gx_color_index *  srcp,
     byte *                  destp,
@@ -115,7 +112,7 @@ pack_scanline_lt8(
  * Unpack a scanline for a depth >= 8. In this case, the depth must be
  * a multiple of 8.
  */
-private void
+static void
 unpack_scanline_ge8(
     gx_color_index *    destp,
     const byte *        srcp,
@@ -144,7 +141,7 @@ unpack_scanline_ge8(
 /*
  * Pack a scanline for depth >= 8.
  */
-private void
+static void
 pack_scanline_ge8(
     const gx_color_index *  srcp,
     byte *                  destp,
@@ -233,7 +230,7 @@ gx_overprint_generic_fill_rectangle(
     /* allocate space for a scanline of color indices */
     pcolor_buff = (gx_color_index *)
                       gs_alloc_bytes( mem,
-                                      w * sizeof(gx_color_index),
+                                      w *  arch_sizeof_color_index,
                                       "overprint generic fill rectangle" );
     if (pcolor_buff == 0)
         return gs_note_error(gs_error_VMerror);
@@ -353,12 +350,12 @@ gx_overprint_generic_fill_rectangle(
 /*
  * Replication of 2 and 4 bit patterns to fill a mem_mono_chunk.
  */
-private mono_fill_chunk fill_pat_2[4] = {
+static mono_fill_chunk fill_pat_2[4] = {
     mono_fill_make_pattern(0x00), mono_fill_make_pattern(0x55),
     mono_fill_make_pattern(0xaa), mono_fill_make_pattern(0xff)
 };
 
-private mono_fill_chunk fill_pat_4[16] = {
+static mono_fill_chunk fill_pat_4[16] = {
     mono_fill_make_pattern(0x00), mono_fill_make_pattern(0x11),
     mono_fill_make_pattern(0x22), mono_fill_make_pattern(0x33),
     mono_fill_make_pattern(0x44), mono_fill_make_pattern(0x55),
@@ -375,7 +372,7 @@ private mono_fill_chunk fill_pat_4[16] = {
  * Since sizeof(mono_fill_chunk) is a power of 2, this will be the case
  * if depth is a power of 2 and depth <= 8 * sizeof(mono_fill_chunk).
  */
-private mono_fill_chunk
+static mono_fill_chunk
 replicate_color(int depth, mono_fill_chunk color)
 {
     switch (depth) {
@@ -555,8 +552,8 @@ gx_overprint_sep_fill_rectangle_2(
     pcolor = (byte *)&color;
     pmask = (byte *)&retain_mask;
 #if arch_is_big_endian
-    pcolor += sizeof(gx_color_index) - byte_depth;
-    pmask += sizeof(gx_color_index) - byte_depth;
+    pcolor += arch_sizeof_color_index - byte_depth;
+    pmask += arch_sizeof_color_index - byte_depth;
 #endif
 
     /* allocate a buffer for the returned data */
