@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: gxpath.c 6651 2006-03-13 16:18:19Z stefan $ */
+/*$Id: gxpath.c 8265 2007-10-02 07:31:58Z ken $ */
 /* Internal path management routines for Ghostscript library */
 #include "gx.h"
 #include "gserrors.h"
@@ -25,11 +25,11 @@
 /* As usual, they return either 0 or a (negative) error code. */
 
 /* Forward references */
-private int path_alloc_copy(gx_path *);
-private int gx_path_new_subpath(gx_path *);
+static int path_alloc_copy(gx_path *);
+static int gx_path_new_subpath(gx_path *);
 
 #ifdef DEBUG
-private void gx_print_segment(const segment *);
+static void gx_print_segment(const segment *);
 
 #  define trace_segment(msg, pseg)\
      if ( gs_debug_c('P') ) dlprintf(msg), gx_print_segment(pseg);
@@ -50,26 +50,27 @@ public_st_path();
 private_st_path_segments();
 private_st_segment();
 private_st_line();
+private_st_dash();
 private_st_line_close();
 private_st_curve();
 private_st_subpath();
 
 /* ------ Initialize/free paths ------ */
 
-private rc_free_proc(rc_free_path_segments);
-private rc_free_proc(rc_free_path_segments_local);
+static rc_free_proc(rc_free_path_segments);
+static rc_free_proc(rc_free_path_segments_local);
 
 /*
  * Define the default virtual path interface implementation.
  */
-private int 
+static int 
     gz_path_add_point(gx_path *, fixed, fixed),
     gz_path_add_line_notes(gx_path *, fixed, fixed, segment_notes),
     gz_path_add_curve_notes(gx_path *, fixed, fixed, fixed, fixed, fixed, fixed, segment_notes),
     gz_path_close_subpath_notes(gx_path *, segment_notes);
-private byte gz_path_state_flags(gx_path *ppath, byte flags);
+static byte gz_path_state_flags(gx_path *ppath, byte flags);
 
-private gx_path_procs default_path_procs = {
+static gx_path_procs default_path_procs = {
     gz_path_add_point,
     gz_path_add_line_notes,
     gz_path_add_curve_notes,
@@ -80,13 +81,13 @@ private gx_path_procs default_path_procs = {
 /*
  * Define virtual path interface implementation for computing a path bbox.
  */
-private int 
+static int 
     gz_path_bbox_add_point(gx_path *, fixed, fixed),
     gz_path_bbox_add_line_notes(gx_path *, fixed, fixed, segment_notes),
     gz_path_bbox_add_curve_notes(gx_path *, fixed, fixed, fixed, fixed, fixed, fixed, segment_notes),
     gz_path_bbox_close_subpath_notes(gx_path *, segment_notes);
 
-private gx_path_procs path_bbox_procs = {
+static gx_path_procs path_bbox_procs = {
     gz_path_bbox_add_point,
     gz_path_bbox_add_line_notes,
     gz_path_bbox_add_curve_notes,
@@ -94,7 +95,7 @@ private gx_path_procs path_bbox_procs = {
     gz_path_state_flags
 };
 
-private void
+static void
 gx_path_init_contents(gx_path * ppath)
 {
     ppath->box_last = 0;
@@ -104,13 +105,14 @@ gx_path_init_contents(gx_path * ppath)
     path_update_newpath(ppath);
     ppath->bbox_set = 0;
     ppath->bbox_accurate = 0;
+    ppath->last_charpath_segment = 0;
 }
 
 /*
  * Initialize a path contained in an already-heap-allocated object,
  * optionally allocating its segments.
  */
-private int
+static int
 path_alloc_segments(gx_path_segments ** ppsegs, gs_memory_t * mem,
 		    client_name_t cname)
 {
@@ -354,7 +356,7 @@ gx_path_assign_free(gx_path * ppto, gx_path * ppfrom)
  * We don't have to worry about cleaning up pointers, because we're about
  * to free the segments object.
  */
-private void
+static void
 rc_free_path_segments_local(gs_memory_t * mem, void *vpsegs,
 			    client_name_t cname)
 {
@@ -373,7 +375,7 @@ rc_free_path_segments_local(gs_memory_t * mem, void *vpsegs,
 	pseg = prev;
     }
 }
-private void
+static void
 rc_free_path_segments(gs_memory_t * mem, void *vpsegs, client_name_t cname)
 {
     rc_free_path_segments_local(mem, vpsegs, cname);
@@ -444,7 +446,7 @@ gx_path_new(gx_path * ppath)
 
 /* Open a new subpath. */
 /* The client must invoke path_update_xxx. */
-private int
+static int
 gx_path_new_subpath(gx_path * ppath)
 {
     subpath *psub;
@@ -471,7 +473,7 @@ gx_path_new_subpath(gx_path * ppath)
     return 0;
 }
 
-private inline void
+static inline void
 gz_path_bbox_add(gx_path * ppath, fixed x, fixed y)
 {
     if (!ppath->bbox_set) {
@@ -490,7 +492,7 @@ gz_path_bbox_add(gx_path * ppath, fixed x, fixed y)
     }
 }
 
-private inline void
+static inline void
 gz_path_bbox_move(gx_path * ppath, fixed x, fixed y)
 {
     /* a trick : we store 'fixed' into 'double'. */
@@ -505,7 +507,7 @@ gx_path_add_point(gx_path * ppath, fixed x, fixed y)
 {
     return ppath->procs->add_point(ppath, x, y);
 }
-private int
+static int
 gz_path_add_point(gx_path * ppath, fixed x, fixed y)
 {
     if (ppath->bbox_set)
@@ -515,7 +517,7 @@ gz_path_add_point(gx_path * ppath, fixed x, fixed y)
     path_update_moveto(ppath);
     return 0;
 }
-private int
+static int
 gz_path_bbox_add_point(gx_path * ppath, fixed x, fixed y)
 {
     gz_path_bbox_move(ppath, x, y);
@@ -558,7 +560,7 @@ gx_path_add_line_notes(gx_path * ppath, fixed x, fixed y, segment_notes notes)
 {
     return ppath->procs->add_line(ppath, x, y, notes);
 }
-private int
+static int
 gz_path_add_line_notes(gx_path * ppath, fixed x, fixed y, segment_notes notes)
 {
     subpath *psub;
@@ -575,7 +577,7 @@ gz_path_add_line_notes(gx_path * ppath, fixed x, fixed y, segment_notes notes)
     trace_segment("[P]", (segment *) lp);
     return 0;
 }
-private int
+static int
 gz_path_bbox_add_line_notes(gx_path * ppath, fixed x, fixed y, segment_notes notes)
 {
     gz_path_bbox_add(ppath, x, y);
@@ -642,6 +644,28 @@ gx_path_add_lines_notes(gx_path *ppath, const gs_fixed_point *ppts, int count,
     return code;
 }
 
+/* Add a dash to the current path (lineto with a small length). */
+/* Only for internal use of the stroking algorithm. */
+int
+gx_path_add_dash_notes(gx_path * ppath, fixed x, fixed y, fixed dx, fixed dy, segment_notes notes)
+{
+    subpath *psub;
+    dash_segment *lp;
+
+    if (ppath->bbox_set)
+	check_in_bbox(ppath, x, y);
+    path_open();
+    path_alloc_segment(lp, dash_segment, &st_dash, s_dash, notes,
+		       "gx_dash_add_dash");
+    path_alloc_link(lp);
+    path_set_point(lp, x, y);
+    lp->tangent.x = dx;
+    lp->tangent.y = dy;
+    path_update_draw(ppath);
+    trace_segment("[P]", (segment *) lp);
+    return 0;
+}
+
 /* Add a rectangle to the current path. */
 /* This is a special case of adding a closed polygon. */
 int
@@ -670,7 +694,7 @@ gx_path_add_curve_notes(gx_path * ppath,
 {
     return ppath->procs->add_curve(ppath, x1, y1, x2, y2, x3, y3, notes);
 }
-private int
+static int
 gz_path_add_curve_notes(gx_path * ppath,
 		 fixed x1, fixed y1, fixed x2, fixed y2, fixed x3, fixed y3,
 			segment_notes notes)
@@ -698,7 +722,7 @@ gz_path_add_curve_notes(gx_path * ppath,
     trace_segment("[P]", (segment *) lp);
     return 0;
 }
-private int
+static int
 gz_path_bbox_add_curve_notes(gx_path * ppath,
 		 fixed x1, fixed y1, fixed x2, fixed y2, fixed x3, fixed y3,
 			segment_notes notes)
@@ -827,7 +851,7 @@ gx_path_close_subpath_notes(gx_path * ppath, segment_notes notes)
 {
     return ppath->procs->close_subpath(ppath, notes);
 }
-private int
+static int
 gz_path_close_subpath_notes(gx_path * ppath, segment_notes notes)
 {
     subpath *psub;
@@ -852,7 +876,7 @@ gz_path_close_subpath_notes(gx_path * ppath, segment_notes notes)
     trace_segment("[P]", (segment *) lp);
     return 0;
 }
-private int
+static int
 gz_path_bbox_close_subpath_notes(gx_path * ppath, segment_notes notes)
 {
     return 0;
@@ -912,7 +936,7 @@ gx_path_pop_close_notes(gx_path * ppath, segment_notes notes)
 /*
  * Copy the current path, because it was shared.
  */
-private int
+static int
 path_alloc_copy(gx_path * ppath)
 {
     gx_path path_new;
@@ -924,6 +948,7 @@ path_alloc_copy(gx_path * ppath)
 	gx_path_free(&path_new, "path_alloc_copy error");
 	return code;
     }
+    ppath->last_charpath_segment = 0;
     return gx_path_assign_free(ppath, &path_new);
 }
 
@@ -963,7 +988,7 @@ gx_path_print(const gx_path * ppath)
 	pseg = pseg->next;
     }
 }
-private void
+static void
 gx_print_segment(const segment * pseg)
 {
     double px = fixed2float(pseg->pt.x);
@@ -991,6 +1016,14 @@ gx_print_segment(const segment * pseg)
 	case s_line:
 	    dprintf3("%s: %1.4f %1.4f lineto\n", out, px, py);
 	    break;
+	case s_dash:{
+    		const dash_segment *const pd = (const dash_segment *)pseg;
+
+		dprintf5("%s: %1.4f %1.4f %1.4f  %1.4f dash\n", out, 
+		    fixed2float(pd->pt.x), fixed2float(pd->pt.y), 
+		    fixed2float(pd->tangent.x), fixed2float(pd->tangent.y));
+		break;
+	    }
 	case s_line_close:{
 		const line_close_segment *const plc =
 		(const line_close_segment *)pseg;
