@@ -1,20 +1,17 @@
-/* Copyright (C) 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxtype1.c,v 1.41 2004/11/19 04:39:11 ray Exp $ */
+/* $Id: gxtype1.c 8406 2007-11-23 09:23:06Z ken $ */
 /* Adobe Type 1 font interpreter support */
 #include "math_.h"
 #include "memory_.h"
@@ -53,7 +50,7 @@ public_st_gs_font_type1();
 /* Define the structure type for a Type 1 interpreter state. */
 public_st_gs_type1_state();
 /* GC procedures */
-private 
+static 
 ENUM_PTRS_WITH(gs_type1_state_enum_ptrs, gs_type1_state *pcis)
 {
     index -= 4;
@@ -67,7 +64,7 @@ ENUM_PTRS_WITH(gs_type1_state_enum_ptrs, gs_type1_state *pcis)
 ENUM_PTR3(0, gs_type1_state, pfont, pis, path);
 ENUM_PTR(3, gs_type1_state, callback_data);
 ENUM_PTRS_END
-private RELOC_PTRS_WITH(gs_type1_state_reloc_ptrs, gs_type1_state *pcis)
+static RELOC_PTRS_WITH(gs_type1_state_reloc_ptrs, gs_type1_state *pcis)
 {
     int i;
 
@@ -124,9 +121,11 @@ gs_type1_interp_init(register gs_type1_state * pcis, gs_imager_state * pis,
     pcis->init_done = -1;
     pcis->sb_set = false;
     pcis->width_set = false;
+    pcis->seac_flag = false;
     pcis->num_hints = 0;
     pcis->seac_accent = -1;
     pcis->log2_subpixels = *plog2_subpixels;
+    pcis->origin_offset.x = pcis->origin_offset.y = 0;
 
     /* Set the sampling scale. */
     set_pixel_scale(&pcis->scale.x, plog2_scale->x);
@@ -176,6 +175,7 @@ gs_type1_finish_init(gs_type1_state * pcis)
 
     /* Initialize hint-related scalars. */
     pcis->asb_diff = pcis->adxy.x = pcis->adxy.y = 0;
+    pcis->base_lsb = 0;
     pcis->flex_count = flex_max;	/* not in Flex */
     pcis->vs_offset.x = pcis->vs_offset.y = 0;
 
@@ -249,8 +249,8 @@ gs_type1_seac(gs_type1_state * pcis, const fixed * cstack, fixed asb,
 
     /* Save away all the operands. */
     pcis->seac_accent = fixed2int_var(cstack[3]);
-    pcis->save_asb = asb;
-    pcis->save_lsb = pcis->lsb;
+    pcis->asb = asb;
+    pcis->compound_lsb = pcis->lsb;
     pcis->save_adxy.x = cstack[0];
     pcis->save_adxy.y = cstack[1];
     pcis->os_count = 0;		/* clear */
@@ -287,7 +287,7 @@ gs_type1_endchar(gs_type1_state * pcis)
 	agdata.memory = pfont->memory;
 	pcis->seac_accent = -1;
 	/* Reset the coordinate system origin */
-	pcis->asb_diff = pcis->save_asb - pcis->save_lsb.x;
+	pcis->asb_diff = pcis->asb - pcis->compound_lsb.x;
 	pcis->adxy = pcis->save_adxy;
 	pcis->os_count = 0;	/* clear */
 	/* Clear the ipstack, in case the base character */
@@ -484,7 +484,7 @@ gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
  * and/or stores into info->pieces.  Updates info->members.  This is a
  * single-use procedure broken out only for readability.
  */
-private int
+static int
 gs_type1_glyph_pieces(gs_font_type1 *pfont, const gs_glyph_data_t *pgd,
 		      int members, gs_glyph_info_t *info)
 {
@@ -545,6 +545,7 @@ gs_type1_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
     }
 
     if (width_members) {
+	gx_path path;
 	/*
 	 * Interpret the CharString until we get to the [h]sbw.
 	 */
@@ -567,6 +568,8 @@ gs_type1_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 	if (code < 0)
 	    return code;
 	cis.no_grid_fitting = true;
+	gx_path_init_bbox_accumulator(&path);
+	cis.path = &path;
 	code = pdata->interpret(&cis, &gdata, &value);
 	switch (code) {
 	case 0:		/* done with no [h]sbw, error */
