@@ -1,20 +1,17 @@
-/* Copyright (C) 1993, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: igc.c,v 1.14 2004/09/28 05:34:29 giles Exp $ */
+/* $Id: igc.c 8496 2008-01-21 20:59:55Z leonardo $ */
 /* Garbage collector for Ghostscript */
 #include "memory_.h"
 #include "ghost.h"
@@ -35,10 +32,10 @@
 #include "opdef.h"		/* for marking oparray names */
 
 /* Define whether to force all garbage collections to be global. */
-private const bool I_FORCE_GLOBAL_GC = false;
+static const bool I_FORCE_GLOBAL_GC = false;
 
 /* Define whether to bypass the collector entirely. */
-private const bool I_BYPASS_GC = false;
+static const bool I_BYPASS_GC = false;
 
 /* Define an entry on the mark stack. */
 typedef struct {
@@ -68,35 +65,35 @@ struct gc_mark_stack_s {
 
 /* Forward references */
 
-private void gc_init_mark_stack(gc_mark_stack *, uint);
-private void gc_objects_clear_marks(const gs_memory_t *mem, chunk_t *);
-private void gc_unmark_names(name_table *);
-private int gc_trace(gs_gc_root_t *, gc_state_t *, gc_mark_stack *);
-private int gc_rescan_chunk(chunk_t *, gc_state_t *, gc_mark_stack *);
-private int gc_trace_chunk(const gs_memory_t *mem, chunk_t *, gc_state_t *, gc_mark_stack *);
-private bool gc_trace_finish(gc_state_t *);
-private void gc_clear_reloc(chunk_t *);
-private void gc_objects_set_reloc(chunk_t *);
-private void gc_do_reloc(chunk_t *, gs_ref_memory_t *, gc_state_t *);
-private void gc_objects_compact(chunk_t *, gc_state_t *);
-private void gc_free_empty_chunks(gs_ref_memory_t *);
+static void gc_init_mark_stack(gc_mark_stack *, uint);
+static void gc_objects_clear_marks(const gs_memory_t *mem, chunk_t *);
+static void gc_unmark_names(name_table *);
+static int gc_trace(gs_gc_root_t *, gc_state_t *, gc_mark_stack *);
+static int gc_rescan_chunk(chunk_t *, gc_state_t *, gc_mark_stack *);
+static int gc_trace_chunk(const gs_memory_t *mem, chunk_t *, gc_state_t *, gc_mark_stack *);
+static bool gc_trace_finish(gc_state_t *);
+static void gc_clear_reloc(chunk_t *);
+static void gc_objects_set_reloc(chunk_t *);
+static void gc_do_reloc(chunk_t *, gs_ref_memory_t *, gc_state_t *);
+static void gc_objects_compact(chunk_t *, gc_state_t *);
+static void gc_free_empty_chunks(gs_ref_memory_t *);
 
 /* Forward references for pointer types */
-private ptr_proc_unmark(ptr_struct_unmark);
-private ptr_proc_mark(ptr_struct_mark);
-private ptr_proc_unmark(ptr_string_unmark);
-private ptr_proc_mark(ptr_string_mark);
+static ptr_proc_unmark(ptr_struct_unmark);
+static ptr_proc_mark(ptr_struct_mark);
+static ptr_proc_unmark(ptr_string_unmark);
+static ptr_proc_mark(ptr_string_mark);
 /*ptr_proc_unmark(ptr_ref_unmark); *//* in igc.h */
 /*ptr_proc_mark(ptr_ref_mark); *//* in igc.h */
-private ptr_proc_reloc(igc_reloc_struct_ptr, void);
+static ptr_proc_reloc(igc_reloc_struct_ptr, void);
 
 ptr_proc_reloc(igc_reloc_ref_ptr, ref_packed);	/* in igcref.c */
 refs_proc_reloc(igc_reloc_refs);	/* in igcref.c */
 
 /* Define this GC's procedure vector. */
-private const gc_procs_with_refs_t igc_procs = {
+static const gc_procs_with_refs_t igc_procs = {
     igc_reloc_struct_ptr, igc_reloc_string, igc_reloc_const_string,
-    igc_reloc_ref_ptr, igc_reloc_refs
+    igc_reloc_param_string, igc_reloc_ref_ptr, igc_reloc_refs
 };
 
 /* Pointer type descriptors. */
@@ -118,7 +115,7 @@ const gs_ptr_procs_t ptr_ref_procs =
 
 /* Top level of garbage collector. */
 #ifdef DEBUG
-private void
+static void
 end_phase(const char *str)
 {
     if (gs_debug_c('6')) {
@@ -128,7 +125,7 @@ end_phase(const char *str)
     }
 }
 static const char *const depth_dots_string = "..........";
-private const char *
+static const char *
 depth_dots(const ms_entry * sp, const gc_mark_stack * pms)
 {
     int depth = sp - pms->entries - 1;
@@ -138,7 +135,7 @@ depth_dots(const ms_entry * sp, const gc_mark_stack * pms)
 	depth += pss->count - 1;
     return depth_dots_string + (depth >= 10 ? 0 : 10 - depth);
 }
-private void
+static void
 gc_validate_spaces(gs_ref_memory_t **spaces, int max_space, gc_state_t *gcst)
 {
     int i;
@@ -151,6 +148,7 @@ gc_validate_spaces(gs_ref_memory_t **spaces, int max_space, gc_state_t *gcst)
 #else  /* !DEBUG */
 #  define end_phase(str) DO_NOTHING
 #endif /* DEBUG */
+
 void
 gs_gc_reclaim(vm_spaces * pspaces, bool global)
 {
@@ -374,6 +372,18 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 
 	end_phase("finish trace");
     }
+
+    /* Filter save change lists with removing elements,
+       which point to unmarked blocks of refs. */
+    {
+	int i;
+
+	for_collected_spaces(i) {
+	    gs_ref_memory_t *mem = space_memories[i];
+
+	    alloc_save__filter_changes(mem);
+	}
+    }
     /* Clear marks and relocation in spaces that are only being traced. */
     /* We have to clear the marks first, because we want the */
     /* relocation to wind up as o_untraced, not o_unmarked. */
@@ -511,7 +521,6 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 		      (ulong) mem, total.allocated, total.used);
 	    gs_memory_status((gs_memory_t *) mem, &total);
 	    mem->gc_allocated = mem->allocated + total.allocated;
-	    mem->inherited = -(int)mem->allocated;
 	}
 	mem = space_memories[ispace];
 	mem->previous_status = total;
@@ -556,7 +565,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 /* ------ Unmarking phase ------ */
 
 /* Unmark a single struct. */
-private void
+static void
 ptr_struct_unmark(enum_ptr_t *pep, gc_state_t * ignored)
 {
     void *const vptr = (void *)pep->ptr; /* break const */
@@ -566,14 +575,14 @@ ptr_struct_unmark(enum_ptr_t *pep, gc_state_t * ignored)
 }
 
 /* Unmark a single string. */
-private void
+static void
 ptr_string_unmark(enum_ptr_t *pep, gc_state_t * gcst)
 {
     discard(gc_string_mark(pep->ptr, pep->size, false, gcst));
 }
 
 /* Unmark the objects in a chunk. */
-private void
+static void
 gc_objects_clear_marks(const gs_memory_t *mem, chunk_t * cp)
 {
     if_debug_chunk('6', "[6]unmarking chunk", cp);
@@ -596,7 +605,7 @@ gc_objects_clear_marks(const gs_memory_t *mem, chunk_t * cp)
 
 /* Mark 0- and 1-character names, and those referenced from the */
 /* op_array_nx_table, and unmark all the rest. */
-private void
+static void
 gc_unmark_names(name_table * nt)
 {
     uint i;
@@ -617,7 +626,7 @@ gc_unmark_names(name_table * nt)
 /* ------ Marking phase ------ */
 
 /* Initialize (a segment of) the mark stack. */
-private void
+static void
 gc_init_mark_stack(gc_mark_stack * pms, uint count)
 {
     pms->next = 0;
@@ -629,7 +638,7 @@ gc_init_mark_stack(gc_mark_stack * pms, uint count)
 
 /* Mark starting from all marked objects in the interval of a chunk */
 /* needing rescanning. */
-private int
+static int
 gc_rescan_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 {
     byte *sbot = cp->rescan_bot;
@@ -696,7 +705,7 @@ gc_rescan_chunk(chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 /* Mark starting from all the objects in a chunk. */
 /* We assume that pstate->min_collect > avm_system, */
 /* so we don't have to trace names. */
-private int
+static int
 gc_trace_chunk(const gs_memory_t *mem, chunk_t * cp, gc_state_t * pstate, gc_mark_stack * pmstack)
 {
     gs_gc_root_t root;
@@ -753,8 +762,8 @@ gc_trace_chunk(const gs_memory_t *mem, chunk_t * cp, gc_state_t * pstate, gc_mar
 /* Return -1 if we overflowed the mark stack, */
 /* 0 if we completed successfully without marking any new objects, */
 /* 1 if we completed and marked some new objects. */
-private int gc_extend_stack(gc_mark_stack *, gc_state_t *);
-private int
+static int gc_extend_stack(gc_mark_stack *, gc_state_t *);
+static int
 gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
 {
     int min_trace = pstate->min_collect;
@@ -1006,7 +1015,7 @@ gc_trace(gs_gc_root_t * rp, gc_state_t * pstate, gc_mark_stack * pmstack)
 }
 /* Link to, attempting to allocate if necessary, */
 /* another chunk of mark stack. */
-private int
+static int
 gc_extend_stack(gc_mark_stack * pms, gc_state_t * pstate)
 {
     if (pms->next == 0) {	/* Try to allocate another segment. */
@@ -1047,7 +1056,7 @@ gc_extend_stack(gc_mark_stack * pms, gc_state_t * pstate)
 }
 
 /* Mark a struct.  Return true if new mark. */
-private bool
+static bool
 ptr_struct_mark(enum_ptr_t *pep, gc_state_t * ignored)
 {
     obj_header_t *ptr = (obj_header_t *)pep->ptr;
@@ -1062,14 +1071,14 @@ ptr_struct_mark(enum_ptr_t *pep, gc_state_t * ignored)
 }
 
 /* Mark a string.  Return true if new mark. */
-private bool
+static bool
 ptr_string_mark(enum_ptr_t *pep, gc_state_t * gcst)
 {
     return gc_string_mark(pep->ptr, pep->size, true, gcst);
 }
 
 /* Finish tracing by marking names. */
-private bool
+static bool
 gc_trace_finish(gc_state_t * pstate)
 {
     name_table *nt = pstate->ntable;
@@ -1100,7 +1109,7 @@ gc_trace_finish(gc_state_t * pstate)
 /* ------ Relocation planning phase ------ */
 
 /* Initialize the relocation information in the chunk header. */
-private void
+static void
 gc_init_reloc(chunk_t * cp)
 {
     chunk_head_t *chead = cp->chead;
@@ -1113,7 +1122,7 @@ gc_init_reloc(chunk_t * cp)
 }
 
 /* Set marks and clear relocation for chunks that won't be compacted. */
-private void
+static void
 gc_clear_reloc(chunk_t * cp)
 {
     byte *pfree = (byte *) & cp->chead->free;
@@ -1135,7 +1144,7 @@ gc_clear_reloc(chunk_t * cp)
 
 /* Set the relocation for the objects in a chunk. */
 /* This will never be called for a chunk with any o_untraced objects. */
-private void
+static void
 gc_objects_set_reloc(chunk_t * cp)
 {
     uint reloc = 0;
@@ -1181,7 +1190,7 @@ gc_objects_set_reloc(chunk_t * cp)
 /* ------ Relocation phase ------ */
 
 /* Relocate the pointers in all the objects in a chunk. */
-private void
+static void
 gc_do_reloc(chunk_t * cp, gs_ref_memory_t * mem, gc_state_t * pstate)
 {
     chunk_head_t *chead = cp->chead;
@@ -1228,7 +1237,7 @@ print_reloc_proc(const void *obj, const char *cname, const void *robj)
 
 /* Relocate a pointer to an (aligned) object. */
 /* See gsmemory.h for why the argument is const and the result is not. */
-private void /*obj_header_t */ *
+static void /*obj_header_t */ *
 igc_reloc_struct_ptr(const void /*obj_header_t */ *obj, gc_state_t * gcst)
 {
     const obj_header_t *const optr = (const obj_header_t *)obj;
@@ -1286,7 +1295,7 @@ igc_reloc_struct_ptr(const void /*obj_header_t */ *obj, gc_state_t * gcst)
 
 /* Compact the objects in a chunk. */
 /* This will never be called for a chunk with any o_untraced objects. */
-private void
+static void
 gc_objects_compact(chunk_t * cp, gc_state_t * gcst)
 {
     chunk_head_t *chead = cp->chead;
@@ -1326,7 +1335,7 @@ gc_objects_compact(chunk_t * cp, gc_state_t * gcst)
 /* ------ Cleanup ------ */
 
 /* Free empty chunks. */
-private void
+static void
 gc_free_empty_chunks(gs_ref_memory_t * mem)
 {
     chunk_t *cp;
