@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: imain.c 8250 2007-09-25 13:31:24Z giles $ */
+/* $Id: imain.c 8720 2008-05-08 21:47:07Z ray $ */
 /* Common support for interpreter front ends */
 #include "malloc_.h"
 #include "memory_.h"
@@ -115,7 +115,7 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
 #   else
        /* plmain settings remain in effect */
 #   endif
-    gp_get_usertime(minst->base_time);
+    gp_get_realtime(minst->base_time);
 
     /* Initialize the file search paths. */
     paths = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
@@ -854,7 +854,7 @@ print_resource_usage(const gs_main_instance * minst, gs_dual_memory_t * dmem,
     ulong allocated = 0, used = 0;
     long utime[2];
 
-    gp_get_usertime(utime);
+    gp_get_realtime(utime);
     {
 	int i;
 
