@@ -1,27 +1,23 @@
-/* Copyright (C) 1996, 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2007 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: imainarg.c,v $ $Revision: 1.4.2.5 $ */
+/* $Id: imainarg.c 8337 2007-10-31 17:00:51Z giles $ */
 /* Command line parsing and dispatching */
 #include "ctype_.h"
 #include "memory_.h"
 #include "string_.h"
+#include <stdlib.h>	/* for qsort */
+
 #include "ghost.h"
 #include "gp.h"
 #include "gsargs.h"
@@ -32,7 +28,7 @@
 #include "gxdevmem.h"
 #include "gsdevice.h"
 #include "stream.h"
-#include "errors.h"
+#include "ierrors.h"
 #include "estack.h"
 #include "ialloc.h"
 #include "strimpl.h"		/* for sfilter.h */
@@ -42,6 +38,7 @@
 #include "iconf.h"
 #include "imain.h"
 #include "imainarg.h"
+#include "iapi.h"
 #include "iminst.h"
 #include "iname.h"
 #include "store.h"
@@ -49,10 +46,11 @@
 #include "interp.h"
 #include "iutil.h"
 #include "ivmspace.h"
+#include "vdtrace.h"
 
 /* Import operator procedures */
-extern int zflush(P1(i_ctx_t *));
-extern int zflushpage(P1(i_ctx_t *));
+extern int zflush(i_ctx_t *);
+extern int zflushpage(i_ctx_t *);
 
 #ifndef GS_LIB
 #  define GS_LIB "GS_LIB"
@@ -66,72 +64,79 @@ extern int zflushpage(P1(i_ctx_t *));
 #  define GS_MAX_LIB_DIRS 25
 #endif
 
-#ifndef GS_BUG_MAILBOX
-#  define GS_BUG_MAILBOX "bug-gs@ghostscript.com"
-#endif
-
 #define MAX_BUFFERED_SIZE 1024
 
 /* Note: sscanf incorrectly defines its first argument as char * */
 /* rather than const char *.  This accounts for the ugly casts below. */
 
-/* Redefine puts to use fprintf, so it will work even without stdio. */
+/* Redefine puts to use outprintf, */
+/* so it will work even without stdio. */
 #undef puts
-private void
-fpputs(const gs_main_instance *minst, const char *str)
-{
-    fprintf(minst->fstdout, "%s\n", str);
-}
-#define puts(str) fpputs(minst, str)
+#define puts(mem, str) outprintf(mem, "%s\n", str)
 
 /* Forward references */
 #define runInit 1
 #define runFlush 2
 #define runBuffer 4
-private int swproc(P3(gs_main_instance *, const char *, arg_list *));
-private int argproc(P2(gs_main_instance *, const char *));
-private int run_buffered(P2(gs_main_instance *, const char *));
-private int esc_strlen(P1(const char *));
-private void esc_strcat(P2(char *, const char *));
-private int runarg(P5(gs_main_instance *, const char *, const char *, const char *, int));
-private int run_string(P3(gs_main_instance *, const char *, int));
-private int run_finish(P4(gs_main_instance *, int, int, ref *));
+static int swproc(gs_main_instance *, const char *, arg_list *);
+static int argproc(gs_main_instance *, const char *);
+static int run_buffered(gs_main_instance *, const char *);
+static int esc_strlen(const char *);
+static void esc_strcat(char *, const char *);
+static int runarg(gs_main_instance *, const char *, const char *, const char *, int);
+static int run_string(gs_main_instance *, const char *, int);
+static int run_finish(gs_main_instance *, int, int, ref *);
+static int try_stdout_redirect(gs_main_instance * minst, 
+				const char *command, const char *filename);
 
 /* Forward references for help printout */
-private void print_help(P1(gs_main_instance *));
-private void print_revision(P1(const gs_main_instance *));
-private void print_version(P1(const gs_main_instance *));
-private void print_usage(P1(const gs_main_instance *));
-private void print_devices(P1(const gs_main_instance *));
-private void print_emulators(P1(const gs_main_instance *));
-private void print_paths(P1(gs_main_instance *));
-private void print_help_trailer(P1(const gs_main_instance *));
+static void print_help(gs_main_instance *);
+static void print_revision(const gs_main_instance *);
+static void print_version(const gs_main_instance *);
+static void print_usage(const gs_main_instance *);
+static void print_devices(const gs_main_instance *);
+static void print_emulators(const gs_main_instance *);
+static void print_paths(gs_main_instance *);
+static void print_help_trailer(const gs_main_instance *);
 
 /* ------ Main program ------ */
 
 /* Process the command line with a given instance. */
-private FILE *
+static FILE *
 gs_main_arg_fopen(const char *fname, void *vminst)
 {
     gs_main_set_lib_paths((gs_main_instance *) vminst);
-    return lib_fopen(fname);
+    return lib_fopen(&((gs_main_instance *)vminst)->lib_path, 
+		     ((gs_main_instance *)vminst)->heap, fname);
 }
-#define arg_heap_copy(str) arg_copy(str, &gs_memory_default)
+static void
+set_debug_flags(const char *arg, char *flags)
+{
+    byte value = (*arg == '-' ? (++arg, 0) : 0xff);
+
+    while (*arg)
+	flags[*arg++ & 127] = value;
+}
+
 int
 gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
 {
     const char *arg;
     arg_list args;
-    FILE *stdfiles[3];
     int code;
 
-    gs_get_real_stdio(stdfiles);
     arg_init(&args, (const char **)argv, argc,
 	     gs_main_arg_fopen, (void *)minst);
-    code = gs_main_init0(minst, stdfiles[0], stdfiles[1], stdfiles[2],
-			 GS_MAX_LIB_DIRS);
+    code = gs_main_init0(minst, 0, 0, 0, GS_MAX_LIB_DIRS);
     if (code < 0)
 	return code;
+/* This first check is not needed on VMS since GS_LIB evaluates to the same
+   value as that returned by gs_lib_default_path.  Also, since GS_LIB is
+   defined as a searchlist logical and getenv only returns the first entry
+   in the searchlist, it really doesn't make sense to search that particular
+   directory twice.
+*/
+#ifndef __VMS
     {
 	int len = 0;
 	int code = gp_getenv(GS_LIB, (char *)0, &len);
@@ -143,6 +148,7 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
 	    minst->lib_path.env = path;
 	}
     }
+#endif /* __VMS */
     minst->lib_path.final = gs_lib_default_path;
     code = gs_main_set_lib_paths(minst);
     if (code < 0)
@@ -163,11 +169,11 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
 		helping = true;
 	    } else if (!strcmp(argv[i], "--version")) {
 		print_version(minst);
-		puts("");	/* \n */
+		puts(minst->heap, "");	/* \n */
 		helping = true;
 	    }
 	if (helping)
-	    gs_exit(gs_exit_INFO);
+	    return e_Info;
     }
     /* Execute files named in the command line, */
     /* processing options along the way. */
@@ -184,18 +190,18 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
 	    (char *)gs_alloc_bytes(minst->heap, len, "GS_OPTIONS");
 
 	    gp_getenv(GS_OPTIONS, opts, &len);	/* can't fail */
-	    arg_push_memory_string(&args, opts, minst->heap);
+	    if (arg_push_memory_string(&args, opts, false, minst->heap))
+		return e_Fatal;
 	}
     }
-    while ((arg = arg_next(&args)) != 0) {
+    while ((arg = arg_next(&args, &code)) != 0) {
 	switch (*arg) {
 	    case '-':
 		code = swproc(minst, arg, &args);
 		if (code < 0)
 		    return code;
 		if (code > 0)
-		    fprintf(minst->fstdout,
-			    "Unknown switch %s - ignoring\n", arg);
+		    outprintf(minst->heap, "Unknown switch %s - ignoring\n", arg);
 		break;
 	    default:
 		code = argproc(minst, arg);
@@ -203,28 +209,36 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
 		    return code;
 	}
     }
+    if (code < 0)
+	return code;
 
-    return gs_main_init2(minst);
+
+    code = gs_main_init2(minst);
+    if (code < 0)
+	return code;
+
+    if (!minst->run_start)
+	return e_Quit;
+    return code ;
 }
 
 /*
  * Run the 'start' procedure (after processing the command line).
- * Note that in case of error, this procedure exits rather than returning.
  */
-void
+int
 gs_main_run_start(gs_main_instance * minst)
 {
-    run_string(minst, "systemdict /start get exec", runFlush);
+    return run_string(minst, "systemdict /start get exec", runFlush);
 }
 
 /* Process switches.  Return 0 if processed, 1 for unknown switch, */
 /* <0 if error. */
-private int
+static int
 swproc(gs_main_instance * minst, const char *arg, arg_list * pal)
 {
     char sw = arg[1];
     ref vtrue;
-    int code;
+    int code = 0;
 #undef initial_enter_name
 #define initial_enter_name(nstr, pvalue)\
   i_initial_enter_name(minst->i_ctx_p, nstr, pvalue)
@@ -236,10 +250,10 @@ swproc(gs_main_instance * minst, const char *arg, arg_list * pal)
 	    return 1;
 	case 0:		/* read stdin as a file char-by-char */
 	    /* This is a ******HACK****** for Ghostview. */
-	    minst->stdin_is_interactive = true;
+	    minst->heap->gs_lib_ctx->stdin_is_interactive = true;
 	    goto run_stdin;
 	case '_':	/* read stdin with normal buffering */
-	    minst->stdin_is_interactive = false;
+	    minst->heap->gs_lib_ctx->stdin_is_interactive = false;
 run_stdin:
 	    minst->run_start = false;	/* don't run 'start' */
 	    /* Set NOPAUSE so showpage won't try to read from stdin. */
@@ -249,31 +263,46 @@ run_stdin:
 	    code = gs_main_init2(minst);	/* Finish initialization */
 	    if (code < 0)
 		return code;
-	    gs_stdin_is_interactive = minst->stdin_is_interactive;
-	    run_string(minst, ".runstdin", runFlush);
+
+	    code = run_string(minst, ".runstdin", runFlush);
+	    if (code < 0)
+		return code;
 	    break;
 	case '-':		/* run with command line args */
 	case '+':
 	    pal->expand_ats = false;
 	case '@':		/* ditto with @-expansion */
 	    {
-		const char *psarg = arg_next(pal);
+		const char *psarg = arg_next(pal, &code);
 
+		if (code < 0)
+		    return e_Fatal;
 		if (psarg == 0) {
-		    fprintf(minst->fstdout,
-			    "Usage: gs ... -%c file.ps arg1 ... argn\n", sw);
+		    outprintf(minst->heap, "Usage: gs ... -%c file.ps arg1 ... argn\n", sw);
 		    arg_finit(pal);
-		    gs_exit(1);
+		    return e_Fatal;
 		}
-		psarg = arg_heap_copy(psarg);
+		psarg = arg_copy(psarg, minst->heap);
+		if (psarg == NULL)
+		    return e_Fatal;
 		code = gs_main_init2(minst);
 		if (code < 0)
 		    return code;
-		run_string(minst, "userdict/ARGUMENTS[", 0);
-		while ((arg = arg_next(pal)) != 0)
-		    runarg(minst, "", arg_heap_copy(arg), "", runInit);
+		code = run_string(minst, "userdict/ARGUMENTS[", 0);
+		if (code < 0)
+		    return code;
+		while ((arg = arg_next(pal, &code)) != 0) {
+		    char *fname = arg_copy(arg, minst->heap);
+		    if (fname == NULL)
+			return e_Fatal;
+		    code = runarg(minst, "", fname, "", runInit);
+		    if (code < 0)
+			return code;
+		}
+		if (code < 0)
+		    return e_Fatal;
 		runarg(minst, "]put", psarg, ".runfile", runInit | runFlush);
-		gs_exit(0);
+		return e_Quit;
 	    }
 	case 'A':		/* trace allocator */
 	    switch (*arg) {
@@ -284,8 +313,8 @@ run_stdin:
 		    gs_alloc_debug = 0;
 		    break;
 		default:
-		    puts("-A may only be followed by -");
-		    gs_exit(1);
+		    puts(minst->heap, "-A may only be followed by -");
+		    return e_Fatal;
 	    }
 	    break;
 	case 'B':		/* set run_string buffer size */
@@ -297,8 +326,10 @@ run_stdin:
 		if (sscanf((const char *)arg, "%u", &bsize) != 1 ||
 		    bsize <= 0 || bsize > MAX_BUFFERED_SIZE
 		    ) {
-		    fprintf(minst->fstdout, "-B must be followed by - or size between 1 and %u\n", MAX_BUFFERED_SIZE);
-		    gs_exit(1);
+		    outprintf(minst->heap, 
+			      "-B must be followed by - or size between 1 and %u\n", 
+			      MAX_BUFFERED_SIZE);
+		    return e_Fatal;
 		}
 		minst->run_buffer_size = bsize;
 	    }
@@ -311,18 +342,28 @@ run_stdin:
 		if (code < 0)
 		    return code;
 		pal->expand_ats = false;
-		while ((arg = arg_next(pal)) != 0) {
+		while ((arg = arg_next(pal, &code)) != 0) {
 		    char *sarg;
 
 		    if (arg[0] == '@' ||
-			(arg[0] == '-' && !isdigit(arg[1]))
+			(arg[0] == '-' && !isdigit((unsigned char)arg[1]))
 			)
 			break;
-		    sarg = arg_heap_copy(arg);
-		    runarg(minst, "", sarg, ".runstring", 0);
+		    sarg = arg_copy(arg, minst->heap);
+		    if (sarg == NULL)
+			return e_Fatal;
+		    code = runarg(minst, "", sarg, ".runstring", 0);
+		    if (code < 0)
+			return code;
+		}
+		if (code < 0)
+		    return e_Fatal;
+		if (arg != 0) {
+		    char *p = arg_copy(arg, minst->heap);
+		    if (p == NULL)
+			return e_Fatal;
+		    arg_push_string(pal, p, true);
 		}
-		if (arg != 0)
-		    arg_push_string(pal, arg_heap_copy(arg));
 		pal->expand_ats = ats;
 		break;
 	    }
@@ -335,24 +376,29 @@ run_stdin:
 		    gs_log_errors = 0;
 		    break;
 		default:
-		    puts("-E may only be followed by -");
-		    gs_exit(1);
+		    puts(minst->heap, "-E may only be followed by -");
+		    return e_Fatal;
 	    }
 	    break;
 	case 'f':		/* run file of arbitrary name */
-	    if (*arg != 0)
-		argproc(minst, arg);
+	    if (*arg != 0) {
+		code = argproc(minst, arg);
+		if (code < 0)
+		    return code;
+	    }
 	    break;
 	case 'F':		/* run file with buffer_size = 1 */
 	    if (!*arg) {
-		puts("-F requires a file name");
-		gs_exit(1);
+		puts(minst->heap, "-F requires a file name");
+		return e_Fatal;
 	    } {
 		uint bsize = minst->run_buffer_size;
 
 		minst->run_buffer_size = 1;
-		argproc(minst, arg);
+		code = argproc(minst, arg);
 		minst->run_buffer_size = bsize;
+		if (code < 0)
+		    return code;
 	    }
 	    break;
 	case 'g':		/* define device geometry */
@@ -360,10 +406,11 @@ run_stdin:
 		long width, height;
 		ref value;
 
-		gs_main_init1(minst);
+		if ((code = gs_main_init1(minst)) < 0)
+		    return code;
 		if (sscanf((const char *)arg, "%ldx%ld", &width, &height) != 2) {
-		    puts("-g must be followed by <width>x<height>");
-		    gs_exit(1);
+		    puts(minst->heap, "-g must be followed by <width>x<height>");
+		    return e_Fatal;
 		}
 		make_int(&value, width);
 		initial_enter_name("DEVICEWIDTH", &value);
@@ -375,22 +422,27 @@ run_stdin:
 	case 'h':		/* print help */
 	case '?':		/* ditto */
 	    print_help(minst);
-	    gs_exit(gs_exit_INFO);
+	    return e_Info;	/* show usage info on exit */
 	case 'I':		/* specify search path */
-	    gs_main_add_lib_path(minst, arg_heap_copy(arg));
+	    {
+		char *path = arg_copy(arg, minst->heap);
+		if (path == NULL)
+		    return e_Fatal;
+		gs_main_add_lib_path(minst, path);
+	    }
 	    break;
 	case 'K':		/* set malloc limit */
 	    {
 		long msize = 0;
+		gs_malloc_memory_t *rawheap = gs_malloc_wrapped_contents(minst->heap);
 
 		sscanf((const char *)arg, "%ld", &msize);
 		if (msize <= 0 || msize > max_long >> 10) {
-		    fprintf(minst->fstdout,
-			    "-K<numK> must have 1 <= numK <= %ld\n",
-			    max_long >> 10);
-		    gs_exit(1);
+		    outprintf(minst->heap, "-K<numK> must have 1 <= numK <= %ld\n",
+			      max_long >> 10);
+		    return e_Fatal;
 		}
-		gs_malloc_limit = msize << 10;
+	        rawheap->limit = msize << 10;
 	    }
 	    break;
 	case 'M':		/* set memory allocation increment */
@@ -398,10 +450,10 @@ run_stdin:
 		unsigned msize = 0;
 
 		sscanf((const char *)arg, "%u", &msize);
-#if arch_ints_are_short
+#if ARCH_INTS_ARE_SHORT
 		if (msize <= 0 || msize >= 64) {
-		    puts("-M must be between 1 and 63");
-		    gs_exit(1);
+		    puts(minst->heap, "-M must be between 1 and 63");
+		    return e_Fatal;
 		}
 #endif
 		minst->memory_chunk_size = msize << 10;
@@ -412,27 +464,53 @@ run_stdin:
 		unsigned nsize = 0;
 
 		sscanf((const char *)arg, "%d", &nsize);
-#if arch_ints_are_short
+#if ARCH_INTS_ARE_SHORT
 		if (nsize < 2 || nsize > 64) {
-		    puts("-N must be between 2 and 64");
-		    gs_exit(1);
+		    puts(minst->heap, "-N must be between 2 and 64");
+		    return e_Fatal;
 		}
 #endif
 		minst->name_table_size = (ulong) nsize << 10;
 	    }
 	    break;
+	case 'o':		/* set output file name and batch mode */
+	    {
+		const char *adef;
+		char *str;
+		ref value;
+		int len;
+
+		if (arg[0] == 0) {
+		    adef = arg_next(pal, &code);
+		    if (code < 0)
+			return code;
+		} else
+		    adef = arg;
+		if ((code = gs_main_init1(minst)) < 0)
+		    return code;
+		len = strlen(adef);
+		str = (char *)gs_alloc_bytes(minst->heap, (uint)len, "-o");
+		memcpy(str, adef, len);
+		make_const_string(&value, a_readonly | avm_foreign,
+				  len, (const byte *)str);
+		initial_enter_name("OutputFile", &value);
+		initial_enter_name("NOPAUSE", &vtrue);
+		initial_enter_name("BATCH", &vtrue);
+	    }
+	    break;
 	case 'P':		/* choose whether search '.' first */
 	    if (!strcmp(arg, ""))
 		minst->search_here_first = true;
 	    else if (!strcmp(arg, "-"))
 		minst->search_here_first = false;
 	    else {
-		puts("Only -P or -P- is allowed.");
-		gs_exit(1);
+		puts(minst->heap, "Only -P or -P- is allowed.");
+		return e_Fatal;
 	    }
 	    break;
 	case 'q':		/* quiet startup */
-	    gs_main_init1(minst);
+	    if ((code = gs_main_init1(minst)) < 0)
+		return code;
 	    initial_enter_name("QUIET", &vtrue);
 	    break;
 	case 'r':		/* define device resolution */
@@ -440,11 +518,12 @@ run_stdin:
 		float xres, yres;
 		ref value;
 
-		gs_main_init1(minst);
+		if ((code = gs_main_init1(minst)) < 0)
+		    return code;
 		switch (sscanf((const char *)arg, "%fx%f", &xres, &yres)) {
 		    default:
-			puts("-r must be followed by <res> or <xres>x<yres>");
-			gs_exit(1);
+			puts(minst->heap, "-r must be followed by <res> or <xres>x<yres>");
+			return e_Fatal;
 		    case 1:	/* -r<res> */
 			yres = xres;
 		    case 2:	/* -r<xres>x<yres> */
@@ -461,19 +540,24 @@ run_stdin:
 	case 'S':		/* define name as string */
 	case 's':
 	    {
-		char *adef = arg_heap_copy(arg);
-		char *eqp = strchr(adef, '=');
+		char *adef = arg_copy(arg, minst->heap);
+		char *eqp;
 		bool isd = (sw == 'D' || sw == 'd');
 		ref value;
 
+		if (adef == NULL)
+		    return e_Fatal;
+		eqp = strchr(adef, '=');
+
 		if (eqp == NULL)
 		    eqp = strchr(adef, '#');
 		/* Initialize the object memory, scanner, and */
 		/* name table now if needed. */
-		gs_main_init1(minst);
+		if ((code = gs_main_init1(minst)) < 0)
+		    return code;
 		if (eqp == adef) {
-		    puts("Usage: -dname, -dname=token, -sname=string");
-		    gs_exit(1);
+		    puts(minst->heap, "Usage: -dname, -dname=token, -sname=string");
+		    return e_Fatal;
 		}
 		if (eqp == NULL) {
 		    if (isd)
@@ -491,20 +575,20 @@ run_stdin:
 			stream astream;
 			scanner_state state;
 
+			s_init(&astream, NULL);
 			sread_string(&astream,
 				     (const byte *)eqp, strlen(eqp));
-			scanner_state_init(&state, false);
-			code = scan_token(minst->i_ctx_p, &astream, &value,
-					  &state);
+			scanner_init_stream(&state, &astream);
+			code = scan_token(minst->i_ctx_p, &value, &state);
 			if (code) {
-			    puts("-dname= must be followed by a valid token");
-			    gs_exit(1);
+			    puts(minst->heap, "-dname= must be followed by a valid token");
+			    return e_Fatal;
 			}
 			if (r_has_type_attrs(&value, t_name,
 					     a_executable)) {
 			    ref nsref;
 
-			    name_string_ref(&value, &nsref);
+			    name_string_ref(minst->heap, &value, &nsref);
 #define string_is(nsref, str, len)\
   (r_size(&(nsref)) == (len) &&\
    !strncmp((const char *)(nsref).value.const_bytes, str, (len)))
@@ -515,8 +599,9 @@ run_stdin:
 			    else if (string_is(nsref, "false", 5))
 				make_false(&value);
 			    else {
-				puts("-dvar=name requires name=null, true, or false");
-				gs_exit(1);
+				puts(minst->heap, 
+				     "-dvar=name requires name=null, true, or false");
+				return e_Fatal;
 			    }
 #undef name_is_string
 			}
@@ -528,12 +613,14 @@ run_stdin:
 
 			if (str == 0) {
 			    lprintf("Out of memory!\n");
-			    gs_exit(1);
+			    return e_Fatal;
 			}
 			memcpy(str, eqp, len);
 			make_const_string(&value,
 					  a_readonly | avm_foreign,
 					  len, (const byte *)str);
+			if ((code = try_stdout_redirect(minst, adef, eqp)) < 0)
+			    return code;
 		    }
 		    ialloc_set_space(idmemory, space);
 		}
@@ -541,17 +628,21 @@ run_stdin:
 		initial_enter_name(adef, &value);
 		break;
 	    }
+	case 'T':
+            set_debug_flags(arg, vd_flags);
+	    break;
 	case 'u':		/* undefine name */
 	    if (!*arg) {
-		puts("-u requires a name to undefine.");
-		gs_exit(1);
+		puts(minst->heap, "-u requires a name to undefine.");
+		return e_Fatal;
 	    }
-	    gs_main_init1(minst);
+	    if ((code = gs_main_init1(minst)) < 0)
+		return code;
 	    i_initial_remove_name(minst->i_ctx_p, arg);
 	    break;
 	case 'v':		/* print revision */
 	    print_revision(minst);
-	    gs_exit(0);
+	    return e_Info;
 /*#ifdef DEBUG */
 	    /*
 	     * Here we provide a place for inserting debugging code that can be
@@ -576,15 +667,10 @@ run_stdin:
 		run_x("== flush\n");
 		stop_x();
 	    }
-	    gs_exit(0);
+	    return e_Quit;
 /*#endif */
 	case 'Z':
-	    {
-		byte value = (*arg == '-' ? (++arg, 0) : 0xff);
-
-		while (*arg)
-		    gs_debug[*arg++ & 127] = value;
-	    }
+            set_debug_flags(arg, gs_debug);
 	    break;
     }
     return 0;
@@ -594,12 +680,12 @@ run_stdin:
 /* This is so we can enter escaped characters regardless of whether */
 /* the Level 1 convention of ignoring \s in strings-within-strings */
 /* is being observed (sigh). */
-private int
+static int
 esc_strlen(const char *str)
 {
     return strlen(str) * 2 + 2;
 }
-private void
+static void
 esc_strcat(char *dest, const char *src)
 {
     char *d = dest + strlen(dest);
@@ -618,20 +704,26 @@ esc_strcat(char *dest, const char *src)
 }
 
 /* Process file names */
-private int
+static int
 argproc(gs_main_instance * minst, const char *arg)
 {
-    gs_main_init1(minst);		/* need i_ctx_p to proceed */
-    minst->i_ctx_p->filearg = arg;	/* allow reading this file if SAFER set */
+    int code = gs_main_init1(minst);		/* need i_ctx_p to proceed */
+    char *filearg;
+
+    if (code < 0)
+        return code;
+    filearg = arg_copy(arg, minst->heap);
+    if (filearg == NULL)
+        return e_Fatal;
     if (minst->run_buffer_size) {
 	/* Run file with run_string. */
-	return run_buffered(minst, arg);
+	return run_buffered(minst, filearg);
     } else {
 	/* Run file directly in the normal way. */
-	return runarg(minst, "", arg, ".runfile", runInit | runFlush);
+	return runarg(minst, "", filearg, ".runfile", runInit | runFlush);
     }
 }
-private int
+static int
 run_buffered(gs_main_instance * minst, const char *arg)
 {
     FILE *in = gp_fopen(arg, gp_fmode_rb);
@@ -640,7 +732,7 @@ run_buffered(gs_main_instance * minst, const char *arg)
     int code;
 
     if (in == 0) {
-	fprintf(minst->fstdout, "Unable to open %s for reading", arg);
+	outprintf(minst->heap, "Unable to open %s for reading", arg);
 	return_error(e_invalidfileaccess);
     }
     code = gs_main_init2(minst);
@@ -670,15 +762,16 @@ run_buffered(gs_main_instance * minst, const char *arg)
     zflushpage(minst->i_ctx_p);
     return run_finish(minst, code, exit_code, &error_object);
 }
-private int
+static int
 runarg(gs_main_instance * minst, const char *pre, const char *arg,
        const char *post, int options)
 {
     int len = strlen(pre) + esc_strlen(arg) + strlen(post) + 1;
+    int code;
     char *line;
 
     if (options & runInit) {
-	int code = gs_main_init2(minst);	/* Finish initialization */
+	code = gs_main_init2(minst);	/* Finish initialization */
 
 	if (code < 0)
 	    return code;
@@ -691,9 +784,12 @@ runarg(gs_main_instance * minst, const char *pre, const char *arg,
     strcpy(line, pre);
     esc_strcat(line, arg);
     strcat(line, post);
-    return run_string(minst, line, options);
+    minst->i_ctx_p->starting_arg_file = true;
+    code = run_string(minst, line, options);
+    minst->i_ctx_p->starting_arg_file = false;
+    return code;
 }
-private int
+static int
 run_string(gs_main_instance * minst, const char *str, int options)
 {
     int exit_code;
@@ -707,30 +803,64 @@ run_string(gs_main_instance * minst, const char *str, int options)
     }
     return run_finish(minst, code, exit_code, &error_object);
 }
-private int
+static int
 run_finish(gs_main_instance *minst, int code, int exit_code,
 	   ref * perror_object)
 {
     switch (code) {
-	case 0:
-	    break;
 	case e_Quit:
-	    gs_exit(0);
-	    /* NOTREACHED */
+	case 0:
 	    break;
 	case e_Fatal:
 	    eprintf1("Unrecoverable error, exit code %d\n", exit_code);
-	    gs_exit(exit_code);
-	    /* NOTREACHED */
 	    break;
 	default:
 	    gs_main_dump_stack(minst, code, perror_object);
-	    gs_exit_with_code(255, code);
-	    /* NOTREACHED */
     }
     return code;
 }
 
+/* Redirect stdout to a file:
+ *  -sstdout=filename
+ *  -sstdout=-
+ *  -sstdout=%stdout
+ *  -sstdout=%stderr
+ * -sOutputFile=- is not affected.
+ * File is closed at program exit (if not stdout/err)
+ * or when -sstdout is used again.
+ */
+static int 
+try_stdout_redirect(gs_main_instance * minst, 
+    const char *command, const char *filename)
+{
+    if (strcmp(command, "stdout") == 0) {
+	minst->heap->gs_lib_ctx->stdout_to_stderr = 0;
+	minst->heap->gs_lib_ctx->stdout_is_redirected = 0;
+	/* If stdout already being redirected and it is not stdout
+	 * or stderr, close it
+	 */
+	if (minst->heap->gs_lib_ctx->fstdout2 
+	    && (minst->heap->gs_lib_ctx->fstdout2 != minst->heap->gs_lib_ctx->fstdout)
+	    && (minst->heap->gs_lib_ctx->fstdout2 != minst->heap->gs_lib_ctx->fstderr)) {
+	    fclose(minst->heap->gs_lib_ctx->fstdout2);
+	    minst->heap->gs_lib_ctx->fstdout2 = (FILE *)NULL;
+	}
+	/* If stdout is being redirected, set minst->fstdout2 */
+	if ( (filename != 0) && strlen(filename) &&
+	    strcmp(filename, "-") && strcmp(filename, "%stdout") ) {
+	    if (strcmp(filename, "%stderr") == 0) {
+		minst->heap->gs_lib_ctx->stdout_to_stderr = 1;
+	    }
+	    else if ((minst->heap->gs_lib_ctx->fstdout2 = 
+		      fopen(filename, "w")) == (FILE *)NULL)
+		return_error(e_invalidfileaccess);
+	    minst->heap->gs_lib_ctx->stdout_is_redirected = 1;
+	}
+	return 0;
+    }
+    return 1;
+}
+
 /* ---------------- Print information ---------------- */
 
 /*
@@ -738,24 +868,25 @@ run_finish(gs_main_instance *minst, int code, int exit_code,
  * the Watcom compiler has a limit of 510 characters for a single token.
  * For PC displays, we want to limit the strings to 24 lines.
  */
-private const char help_usage1[] = "\
+static const char help_usage1[] = "\
 Usage: gs [switches] [file1.ps file2.ps ...]\n\
 Most frequently used switches: (you can use # in place of =)\n\
  -dNOPAUSE           no pause after page   | -q       `quiet', fewer messages\n\
  -g<width>x<height>  page size in pixels   | -r<res>  pixels/inch resolution\n";
-private const char help_usage2[] = "\
+static const char help_usage2[] = "\
  -sDEVICE=<devname>  select device         | -dBATCH  exit after last file\n\
  -sOutputFile=<file> select output file: - for stdout, |command for pipe,\n\
                                          embed %d or %ld for page #\n";
-private const char help_trailer[] = "\
-For more information, see %s%sUse.htm.\n\
-Report bugs to %s, using the form in Bug-form.htm.\n";
-private const char help_devices[] = "Available devices:";
-private const char help_emulators[] = "Input formats:";
-private const char help_paths[] = "Search path:";
+static const char help_trailer[] = "\
+For more information, see %s.\n\
+Please report bugs to bugs.ghostscript.com.\n";
+static const char help_devices[] = "Available devices:";
+static const char help_default_device[] = "Default output device:";
+static const char help_emulators[] = "Input formats:";
+static const char help_paths[] = "Search path:";
 
 /* Print the standard help message. */
-private void
+static void
 print_help(gs_main_instance * minst)
 {
     print_revision(minst);
@@ -764,20 +895,17 @@ print_help(gs_main_instance * minst)
     print_devices(minst);
     print_paths(minst);
     if (gs_init_string_sizeof > 0) {
-        fprintf(minst->fstdout,
-		"Initialization files are compiled into the executable.\n");
+        outprintf(minst->heap, "Initialization files are compiled into the executable.\n");
     }
     print_help_trailer(minst);
 }
 
 /* Print the revision, revision date, and copyright. */
-private void
+static void
 print_revision(const gs_main_instance *minst)
 {
-    FILE *out = minst->fstdout;
-
-    printf_program_ident(out, gs_product, gs_revision);
-    fprintf(out, " (%d-%02d-%02d)\n%s\n",
+    printf_program_ident(minst->heap, gs_product, gs_revision);
+    outprintf(minst->heap, " (%d-%02d-%02d)\n%s\n",
 	    (int)(gs_revisiondate / 10000),
 	    (int)(gs_revisiondate / 100 % 100),
 	    (int)(gs_revisiondate % 100),
@@ -785,56 +913,79 @@ print_revision(const gs_main_instance *minst)
 }
 
 /* Print the version number. */
-private void
+static void
 print_version(const gs_main_instance *minst)
 {
-    FILE *out = minst->fstdout;
-
-    printf_program_ident(out, NULL, gs_revision);
+    printf_program_ident(minst->heap, NULL, gs_revision);
 }
 
 /* Print usage information. */
-private void
+static void
 print_usage(const gs_main_instance *minst)
 {
-    FILE *out = minst->fstdout;
+    outprintf(minst->heap, "%s", help_usage1);
+    outprintf(minst->heap, "%s", help_usage2);
+}
 
-    fprintf(out, "%s", help_usage1);
-    fprintf(out, "%s", help_usage2);
+/* compare function for qsort */
+static int
+cmpstr(const void *v1, const void *v2)
+{
+    return strcmp( *(char * const *)v1, *(char * const *)v2 );
 }
 
 /* Print the list of available devices. */
-private void
+static void
 print_devices(const gs_main_instance *minst)
 {
-    FILE *out = minst->fstdout;
-
-    fprintf(out, "%s", help_devices);
+    outprintf(minst->heap, "%s", help_default_device);
+    outprintf(minst->heap, " %s\n", gs_devicename(gs_getdefaultdevice()));
+    outprintf(minst->heap, "%s", help_devices);
     {
 	int i;
 	int pos = 100;
 	const gx_device *pdev;
+	const char **names;
+	size_t ndev = 0;
 
-	for (i = 0; (pdev = gs_getdevice(i)) != 0; i++) {
-	    const char *dname = gs_devicename(pdev);
-	    int len = strlen(dname);
+	for (i = 0; (pdev = gs_getdevice(i)) != 0; i++)
+	    ;
+	ndev = (size_t)i;
+	names = (const char **)gs_alloc_bytes(minst->heap, ndev * sizeof(const char*), "print_devices");
+	if (names == (const char **)NULL) { /* old-style unsorted device list */
+	    for (i = 0; (pdev = gs_getdevice(i)) != 0; i++) {
+		const char *dname = gs_devicename(pdev);
+		int len = strlen(dname);
 
-	    if (pos + 1 + len > 76)
-		fprintf(out, "\n  "), pos = 2;
-	    fprintf(out, " %s", dname);
-	    pos += 1 + len;
+		if (pos + 1 + len > 76)
+		    outprintf(minst->heap, "\n  "), pos = 2;
+		outprintf(minst->heap, " %s", dname);
+		pos += 1 + len;
+	    }
+	}
+	else {				/* new-style sorted device list */
+	    for (i = 0; (pdev = gs_getdevice(i)) != 0; i++)
+		names[i] = gs_devicename(pdev);
+	    qsort((void*)names, ndev, sizeof(const char*), cmpstr);
+	    for (i = 0; i < ndev; i++) {
+		int len = strlen(names[i]);
+
+		if (pos + 1 + len > 76)
+		    outprintf(minst->heap, "\n  "), pos = 2;
+		outprintf(minst->heap, " %s", names[i]);
+		pos += 1 + len;
+	    }
+	    gs_free(minst->heap, (char *)names, ndev * sizeof(const char*), 1, "print_devices");
 	}
     }
-    fprintf(out, "\n");
+    outprintf(minst->heap, "\n");
 }
 
 /* Print the list of language emulators. */
-private void
+static void
 print_emulators(const gs_main_instance *minst)
 {
-    FILE *out = minst->fstdout;
-
-    fprintf(out, "%s", help_emulators);
+    outprintf(minst->heap, "%s", help_emulators);
     {
 	const ref *pes;
 
@@ -846,18 +997,16 @@ print_emulators(const gs_main_instance *minst)
 	     * an array of string refs, each string is actually a
 	     * (null terminated) C string.
 	     */
-	    fprintf(out, " %s", (const char *)pes->value.const_bytes);
+	    outprintf(minst->heap, " %s", (const char *)pes->value.const_bytes);
     }
-    fprintf(out, "\n");
+    outprintf(minst->heap, "\n");
 }
 
 /* Print the search paths. */
-private void
+static void
 print_paths(gs_main_instance * minst)
 {
-    FILE *out = minst->fstdout;
-
-    fprintf(out, "%s", help_paths);
+    outprintf(minst->heap, "%s", help_paths);
     gs_main_set_lib_paths(minst);
     {
 	uint count = r_size(&minst->lib_path.list);
@@ -874,36 +1023,38 @@ print_paths(gs_main_instance * minst)
 	    const char *sepr = (i == count - 1 ? "" : fsepr);
 
 	    if (1 + pos + strlen(sepr) + len > 76)
-		fprintf(out, "\n  "), pos = 2;
-	    fprintf(out, " ");
+		outprintf(minst->heap, "\n  "), pos = 2;
+	    outprintf(minst->heap, " ");
 	    /*
 	     * This is really ugly, but it's necessary because some
 	     * platforms rely on all console output being funneled through
-	     * fprintf.  We wish we could just do:
-	     fwrite(prdir->value.bytes, 1, len, out);
+	     * outprintf.  We wish we could just do:
+	     fwrite(prdir->value.bytes, 1, len, minst->fstdout);
 	     */
 	    {
 		const char *p = (const char *)prdir->value.bytes;
 		uint j;
 
 		for (j = len; j; j--)
-		    fprintf(out, "%c", *p++);
+		    outprintf(minst->heap, "%c", *p++);
 	    }
-	    fprintf(out, sepr);
+	    outprintf(minst->heap, "%s", sepr);
 	    pos += 1 + len + strlen(sepr);
 	}
     }
-    fprintf(out, "\n");
+    outprintf(minst->heap, "\n");
 }
 
 /* Print the help trailer. */
-private void
+static void
 print_help_trailer(const gs_main_instance *minst)
 {
-    FILE *out = minst->fstdout;
+    char buffer[gp_file_name_sizeof];
+    const char *use_htm = "Use.htm", *p = buffer;
+    uint blen = sizeof(buffer);
 
-    fprintf(out, help_trailer, gs_doc_directory,
-	    gp_file_name_concat_string(gs_doc_directory,
-				       strlen(gs_doc_directory)),
-	    GS_BUG_MAILBOX);
+    if (gp_file_name_combine(gs_doc_directory, strlen(gs_doc_directory), 
+	    use_htm, strlen(use_htm), false, buffer, &blen) != gp_combine_success)
+	p = use_htm;
+    outprintf(minst->heap, help_trailer, p);
 }
