@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: scfd.c 8250 2007-09-25 13:31:24Z giles $ */
+/* $Id: scfd.c 8896 2008-07-29 04:44:05Z giles $ */
 /* CCITTFax decoding filter */
 #include "stdio_.h"		/* includes std.h */
 #include "memory_.h"
@@ -161,7 +161,7 @@ s_CFD_release(stream_state * st)
 /* makeup codes efficiently, since these are always a multiple of 64. */
 #define invert_data(rlen, black_byte, makeup_action, d)\
 	if ( rlen > qbit )\
-	{	*q++ ^= (1 << qbit) - 1;\
+	{	if (q >= ss->lbuf) *q++ ^= (1 << qbit) - 1; else q++;\
 		rlen -= qbit;\
 		switch ( rlen >> 3 )\
 		{\
