@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: spngp.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: spngp.c 8823 2008-07-07 08:56:59Z ken $ */
 /* PNG pixel prediction filters */
 #include "memory_.h"
 #include "strimpl.h"
@@ -30,12 +30,12 @@ private_st_PNGP_state();
 #define cOptimum 15
 #define cEncode -10
 #define cDecode -4
-private const byte pngp_case_needs_prev[] = {
+static const byte pngp_case_needs_prev[] = {
     0, 0, 1, 1, 1, 1
 };
 
 /* Set defaults */
-private void
+static void
 s_PNGP_set_defaults(stream_state * st)
 {
     stream_PNGP_state *const ss = (stream_PNGP_state *) st;
@@ -44,7 +44,7 @@ s_PNGP_set_defaults(stream_state * st)
 }
 
 /* Common (re)initialization. */
-private int
+static int
 s_PNGP_reinit(stream_state * st)
 {
     stream_PNGP_state *const ss = (stream_PNGP_state *) st;
@@ -56,7 +56,7 @@ s_PNGP_reinit(stream_state * st)
 }
 
 /* Common initialization. */
-private int
+static int
 s_pngp_init(stream_state * st, bool need_prev)
 {
     stream_PNGP_state *const ss = (stream_PNGP_state *) st;
@@ -70,6 +70,10 @@ s_pngp_init(stream_state * st, bool need_prev)
 #endif
     ss->row_count = (uint) ((bits_per_row + 7) >> 3);
     ss->end_mask = (1 << (-bits_per_row & 7)) - 1;
+
+    if (ss->Colors > s_PNG_max_Colors)
+	return ERRC; /* Too many colorants */
+
     ss->bpp = (bits_per_pixel + 7) >> 3;
     if (need_prev) {
 	prev_row = gs_alloc_bytes(st->memory, ss->bpp + ss->row_count,
@@ -84,7 +88,7 @@ s_pngp_init(stream_state * st, bool need_prev)
 }
 
 /* Initialize PNGPredictorEncode filter. */
-private int
+static int
 s_PNGPE_init(stream_state * st)
 {
     stream_PNGP_state *const ss = (stream_PNGP_state *) st;
@@ -93,14 +97,14 @@ s_PNGPE_init(stream_state * st)
 }
 
 /* Initialize PNGPredictorDecode filter. */
-private int
+static int
 s_PNGPD_init(stream_state * st)
 {
     return s_pngp_init(st, true);
 }
 
 /* Release a PNGPredictor filter. */
-private void
+static void
 s_PNGP_release(stream_state *st)
 {
     stream_PNGP_state *const ss = (stream_PNGP_state *) st;
@@ -113,10 +117,12 @@ s_PNGP_release(stream_state *st)
  * Process a partial buffer.  We pass in current and previous pointers
  * to both the current and preceding scan line.  Note that dprev is
  * p - bpp for encoding, q - bpp for decoding; similarly, the 'up' row
- * is the raw data for encoding, the filtered data for decoding.
+ * is the raw data for encoding, the filtered (encoded) data for decoding.
  * Note also that the case_index cannot be cOptimum.
+ *
+ * Uses ss->case_index; uses and updates ss->row_left, pr->ptr, pw->ptr.
  */
-private int
+static int
 paeth_predictor(int a, int b, int c)
 {
     /* The definitions of ac and bc are correct, not a typo. */
@@ -126,7 +132,7 @@ paeth_predictor(int a, int b, int c)
 
     return (pa <= pb && pa <= pc ? a : pb <= pc ? b : c);
 }
-private void
+static void
 s_pngp_process(stream_state * st, stream_cursor_write * pw,
 	       const byte * dprev, stream_cursor_read * pr,
 	       const byte * upprev, const byte * up, uint count)
@@ -180,7 +186,7 @@ s_pngp_process(stream_state * st, stream_cursor_write * pw,
 
 /* Calculate the number of bytes for the next processing step, */
 /* the min of (input data, output data, remaining row length). */
-private uint
+static uint
 s_pngp_count(const stream_state * st_const, const stream_cursor_read * pr,
 	     const stream_cursor_write * pw)
 {
@@ -207,13 +213,16 @@ s_pngp_count(const stream_state * st_const, const stream_cursor_read * pr,
  *        of the current input row.
  *      prev_row[P .. N + B - 1] contain bytes P - B .. N - 1
  *        of the previous input row.
+ * We must handle the edges of each row specially, by clearing ss->prev at
+ * the beginning of each row, and copying trailing bytes from ss->prev (and
+ * possibly the input) to prev_row at the end of each row.
  */
-private int
+static int
 optimum_predictor(const stream_state * st, const stream_cursor_read * pr)
 {
     return cSub;
 }
-private int
+static int
 s_PNGPE_process(stream_state * st, stream_cursor_read * pr,
 		stream_cursor_write * pw, bool last)
 {
@@ -253,10 +262,15 @@ s_PNGPE_process(stream_state * st, stream_cursor_read * pr,
 
 	    /* Process bytes whose predecessors are in prev. */
 	    s_pngp_process(st, pw, ss->prev, pr, up - bpp, up, n);
+	    if (ss->row_left == 0) {
+		if (ss->prev_row) {
+		    memcpy(up - bpp, ss->prev, bpp);
+		    memcpy(up, pr->ptr - (n - 1), n);
+		}
+		continue;
+	    }
 	    if (ss->prev_row)
 		memcpy(up - bpp, ss->prev, n);
-	    if (ss->row_left == 0)
-		continue;
 	    if (n < bpp) {
 		/*
 		 * We didn't have both enough input data and enough output
@@ -298,8 +312,11 @@ s_PNGPE_process(stream_state * st, stream_cursor_read * pr,
  *        of the current output row.
  *      prev_row[P .. N + B - 1] contain bytes P - B .. N - 1
  *        of the previous output row.
+ * We must handle the edges of each row specially, by clearing ss->prev at
+ * the beginning of each row, and copying trailing bytes from ss->prev (and
+ * possibly the output) to prev_row at the end of each row.
  */
-private int
+static int
 s_PNGPD_process(stream_state * st, stream_cursor_read * pr,
 		stream_cursor_write * pw, bool last)
 {
@@ -335,10 +352,15 @@ s_PNGPD_process(stream_state * st, stream_cursor_read * pr,
 
 	    /* Process bytes whose predecessors are in prev. */
 	    s_pngp_process(st, pw, ss->prev, pr, up - bpp, up, n);
+	    if (ss->row_left == 0) {
+		if (ss->prev_row) {
+		    memcpy(up - bpp, ss->prev, bpp);
+		    memcpy(up, pw->ptr - (n - 1), n);
+		}
+		continue;
+	    }
 	    if (ss->prev_row)
 		memcpy(up - bpp, ss->prev, n);
-	    if (ss->row_left == 0)
-		continue;
 	    if (n < bpp) {
 		/*
 		 * We didn't have both enough input data and enough output
