@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: ttfmain.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: ttfmain.c 8776 2008-05-25 07:23:38Z leonardo $ */
 /* A Free Type interface adapter. */
 /* Uses code fragments from the FreeType project. */
 
@@ -25,7 +25,7 @@
 #include "ttinterp.h"
 #include "ttcalc.h"
 
-private const bool skip_instructions = 0; /* Debug purpose only. */
+static const bool skip_instructions = 0; /* Debug purpose only. */
 
 typedef struct { 
     TT_Fixed a, b, c, d, tx, ty;
@@ -40,35 +40,35 @@ struct ttfSubGlyphUsage_s {
 
 /*------------------------------------------------------------------- */
 
-private TT_Fixed AVE(F26Dot6 a, F26Dot6 b)
+static TT_Fixed AVE(F26Dot6 a, F26Dot6 b)
 {   return (a + b) / 2;
 }
 
-private F26Dot6 shortToF26Dot6(short a)
+static F26Dot6 shortToF26Dot6(short a)
 {   return (F26Dot6)a << 6;
 }
 
-private F26Dot6 floatToF26Dot6(float a)
+static F26Dot6 floatToF26Dot6(float a)
 {   return (F26Dot6)(a * (1 << 6) + 0.5);
 }
 
-private TT_Fixed floatToF16Dot16(float a)
+static TT_Fixed floatToF16Dot16(float a)
 {   return (F26Dot6)(a * (1 << 16) + 0.5);
 }
 
-private void TransformF26Dot6PointFix(F26Dot6Point *pt, F26Dot6 dx, F26Dot6 dy, FixMatrix *m)
+static void TransformF26Dot6PointFix(F26Dot6Point *pt, F26Dot6 dx, F26Dot6 dy, FixMatrix *m)
 {   pt->x = MulDiv(dx, m->a, 65536) + MulDiv(dy, m->c, 65536) + (m->tx >> 10);
     pt->y = MulDiv(dx, m->b, 65536) + MulDiv(dy, m->d, 65536) + (m->ty >> 10);
 }
 
-private void TransformF26Dot6PointFloat(FloatPoint *pt, F26Dot6 dx, F26Dot6 dy, FloatMatrix *m)
+static void TransformF26Dot6PointFloat(FloatPoint *pt, F26Dot6 dx, F26Dot6 dy, FloatMatrix *m)
 {   pt->x = dx * m->a / 64 + dy * m->c / 64 + m->tx;
     pt->y = dx * m->b / 64 + dy * m->d / 64 + m->ty;
 }
 
 /*-------------------------------------------------------------------*/
 
-private ttfPtrElem *ttfFont__get_table_ptr(ttfFont *f, char *id)
+static ttfPtrElem *ttfFont__get_table_ptr(ttfFont *f, char *id)
 {
     if (!memcmp(id, "cvt ", 4))
 	return &f->t_cvt_;
@@ -173,7 +173,6 @@ void ttfInterpreter__release(ttfInterpreter **ptti)
     mem->free(mem, tti->usage, "ttfInterpreter__release");
     mem->free(mem, tti->exec, "ttfInterpreter__release");
     mem->free(mem, *ptti, "ttfInterpreter__release");
-    mem->free(mem, mem, "ttfInterpreter__release");
     *ptti = 0;
 }
 
@@ -191,8 +190,15 @@ void ttfFont__init(ttfFont *this, ttfMemory *mem,
 void ttfFont__finit(ttfFont *this)
 {   ttfMemory *mem = this->tti->ttf_memory;
 
-    if (this->exec)
-	Context_Destroy(this->exec);
+    if (this->exec) {
+	if (this->inst)
+	    Context_Destroy(this->exec);
+	else {
+	    /* Context_Create was not called - see ttfFont__Open.
+	       Must not call Context_Destroy for proper 'lock' count.
+	     */
+	}
+    }
     this->exec = NULL;
     if (this->inst)
 	Instance_Destroy(this->inst);
@@ -211,9 +217,10 @@ void ttfFont__finit(ttfFont *this)
 FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r, 
 				    unsigned int nTTC, float w, float h, 
 				    bool design_grid)
-{   char sVersion[4], sVersion0[4] = {0, 1, 0, 0};
+{   char sVersion[4], sVersion1[4] = {0, 1, 0, 0};
+    char sVersion2[4] = {0, 2, 0, 0};
     unsigned int nNumTables, i;
-    TT_Error code;
+    TT_Error code, code1 = 0;
     int k;
     TT_Instance I;
     ttfMemory *mem = tti->ttf_memory;
@@ -227,7 +234,7 @@ FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r,
 	unsigned int nPos = 0xbaadf00d; /* Quiet compiler. */
 
 	r->Read(r, sVersion, 4);
-	if(memcmp(sVersion, sVersion0, 4))
+       if(memcmp(sVersion, sVersion1, 4) && memcmp(sVersion, sVersion2, 4))
 	    return fUnimplemented;
 	nFonts = ttfReader__UInt(r);
 	if (nFonts == 0)
@@ -239,7 +246,7 @@ FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r,
 	r->Seek(r, nPos);
 	r->Read(r, sVersion, 4);
     }
-    if(memcmp(sVersion, sVersion0, 4) && memcmp(sVersion, "true", 4))
+    if(memcmp(sVersion, sVersion1, 4) && memcmp(sVersion, "true", 4))
 	return fUnimplemented;
     nNumTables    = ttfReader__UShort(r);
     ttfReader__UShort(r); /* nSearchRange */
@@ -321,8 +328,8 @@ FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r,
     if (code == TT_Err_Out_Of_Memory)
 	return fMemoryError;
     if (code >= TT_Err_Invalid_Opcode && code <= TT_Err_Invalid_Displacement)
-	return fBadInstruction;
-    if (code)
+	code1 = fBadInstruction;
+    else if (code)
 	return fBadFontData;
     I.z = this->inst;
     if (design_grid)
@@ -342,10 +349,12 @@ FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r,
 	return fBadInstruction;
     if (code)
 	return fBadFontData;
+    if (code1)
+	return code1;
     return code;
 }
 
-private void ttfFont__StartGlyph(ttfFont *this)
+static void ttfFont__StartGlyph(ttfFont *this)
 {   Context_Load( this->exec, this->inst );
     if ( this->inst->GS.instruct_control & 2 )
 	this->exec->GS = Default_GraphicsState;
@@ -354,14 +363,14 @@ private void ttfFont__StartGlyph(ttfFont *this)
     this->tti->usage_top = 0;
 }
 
-private void ttfFont__StopGlyph(ttfFont *this)
+static void ttfFont__StopGlyph(ttfFont *this)
 {
     Context_Save(this->exec, this->inst);
 }
 
 /*-------------------------------------------------------------------*/
 
-private void  mount_zone( PGlyph_Zone  source,
+static void  mount_zone( PGlyph_Zone  source,
                           PGlyph_Zone  target )
 {
     Int  np, nc;
@@ -381,7 +390,7 @@ private void  mount_zone( PGlyph_Zone  source,
     target->n_contours = 0;
 }
 
-private void  Init_Glyph_Component( PSubglyph_Record    element,
+static void  Init_Glyph_Component( PSubglyph_Record    element,
                                    PSubglyph_Record    original,
                                    PExecution_Context  exec )
 {
@@ -415,7 +424,7 @@ private void  Init_Glyph_Component( PSubglyph_Record    element,
     element->advanceWidth = 0;
   }
 
-private void  cur_to_org( Int  n, PGlyph_Zone  zone )
+static void  cur_to_org( Int  n, PGlyph_Zone  zone )
 {
     Int  k;
 
@@ -426,7 +435,7 @@ private void  cur_to_org( Int  n, PGlyph_Zone  zone )
 	zone->org_y[k] = zone->cur_y[k];
 }
 
-private void  org_to_cur( Int  n, PGlyph_Zone  zone )
+static void  org_to_cur( Int  n, PGlyph_Zone  zone )
 {
     Int  k;
 
@@ -451,7 +460,7 @@ void ttfOutliner__init(ttfOutliner *this, ttfFont *f, ttfReader *r, ttfExport *e
     this->exp = exp;
 }
 
-private void MoveGlyphOutline(TGlyph_Zone *pts, int nOffset, ttfGlyphOutline *out, FixMatrix *m)
+static void MoveGlyphOutline(TGlyph_Zone *pts, int nOffset, ttfGlyphOutline *out, FixMatrix *m)
 {   F26Dot6* x = pts->org_x + nOffset;
     F26Dot6* y = pts->org_y + nOffset;
     short count = out->pointCount;
@@ -468,7 +477,7 @@ private void MoveGlyphOutline(TGlyph_Zone *pts, int nOffset, ttfGlyphOutline *ou
     }
 }
 
-private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyphIndex, 
+static FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyphIndex, 
 	    FixMatrix *m_orig, ttfGlyphOutline* gOutline)
 {   ttfFont *pFont = this->pFont;
     ttfReader *r = this->r;
@@ -477,7 +486,7 @@ private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyph
     FontError error = fNoError;
     short arg1, arg2;
     short count;
-    unsigned int nMtxPos, nMtxGlyph = glyphIndex, nLongMetrics, i;
+    unsigned int i;
     unsigned short nAdvance;
     unsigned int nNextGlyphPtr = 0;
     unsigned int nPosBeg;
@@ -488,31 +497,10 @@ private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyph
     const byte *glyph = NULL;
     int glyph_size;
 
-    if(this->bVertical && pFont->t_vhea.nPos && pFont->t_vmtx.nPos) {
-	nLongMetrics = pFont->nLongMetricsVert;
-	nMtxPos = pFont->t_vmtx.nPos;
-    } else {
-	nLongMetrics = pFont->nLongMetricsHorz;
-	nMtxPos = pFont->t_hmtx.nPos;
-    }
-    if (this->bVertical && (!pFont->t_vhea.nPos || pFont->t_vmtx.nPos) && nMtxGlyph < nLongMetrics) {
-	/* A bad font fix. */
-	nMtxGlyph = nLongMetrics;
-	if(nMtxGlyph >= pFont->nNumGlyphs)
-	    nMtxGlyph = pFont->nNumGlyphs - 1;
-    }
-    if (nMtxGlyph < nLongMetrics) {
-	r->Seek(r, nMtxPos + 4 * nMtxGlyph);
-	nAdvance = ttfReader__Short(r);
-	sideBearing = ttfReader__Short(r);
-    } else {
-	r->Seek(r, nMtxPos + 4 * (nLongMetrics - 1));
-	nAdvance = ttfReader__Short(r);
-	r->Seek(r, nMtxPos + 4 * nLongMetrics + 2 * (nMtxGlyph - nLongMetrics));
-	sideBearing = ttfReader__Short(r);
-    }
-    if (r->Error(r))
+    if (r->get_metrics(r, glyphIndex, this->bVertical, &sideBearing, &nAdvance) < 0) {
+	/* fixme: the error code is missing due to interface restrictions. */
 	goto errex;
+    }
     gOutline->sideBearing = shortToF26Dot6(sideBearing);
     gOutline->advance.x = shortToF26Dot6(nAdvance);
     gOutline->advance.y = 0;
@@ -845,7 +833,7 @@ ex:;
     return error;
 }
 
-private FontError ttfOutliner__BuildGlyphOutline(ttfOutliner *this, int glyphIndex, 
+static FontError ttfOutliner__BuildGlyphOutline(ttfOutliner *this, int glyphIndex, 
 	    float orig_x, float orig_y, ttfGlyphOutline* gOutline)
 {
     FixMatrix m_orig = {1 << 16, 0, 0, 1 << 16, 0, 0};
