@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zarith.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: zarith.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Arithmetic operators */
 #include "math_.h"
 #include "ghost.h"
@@ -155,8 +155,8 @@ zmul(i_ctx_t *i_ctx_p)
 	case t_integer: {
 	    long int1 = op[-1].value.intval;
 	    long int2 = op->value.intval;
-	    long abs1 = (int1 >= 0 ? int1 : -int1);
-	    long abs2 = (int2 >= 0 ? int2 : -int2);
+	    unsigned long abs1 = (unsigned long)(int1 >= 0 ? int1 : -int1);
+	    unsigned long abs2 = (unsigned long)(int2 >= 0 ? int2 : -int2);
 	    float fprod;
 
 	    if ((abs1 > MAX_HALF_INTVAL || abs2 > MAX_HALF_INTVAL) &&
@@ -239,13 +239,11 @@ zidiv(i_ctx_t *i_ctx_p)
 
     check_type(*op, t_integer);
     check_type(op[-1], t_integer);
-    if (op->value.intval == 0)
+    if ((op->value.intval == 0) || (op[-1].value.intval == MIN_INTVAL && op->value.intval == -1)) {
+	/* Anomalous boundary case: -MININT / -1, fail. */
 	return_error(e_undefinedresult);
-    if ((op[-1].value.intval /= op->value.intval) ==
-	MIN_INTVAL && op->value.intval == -1
-	) {			/* Anomalous boundary case, fail. */
-	return_error(e_rangecheck);
     }
+    op[-1].value.intval /= op->value.intval;
     pop(1);
     return 0;
 }
@@ -377,7 +375,7 @@ ztruncate(i_ctx_t *i_ctx_p)
 /* Non-standard operators */
 
 /* <int1> <int2> .bitadd <sum> */
-private int
+static int
 zbitadd(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
