@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zcontrol.c 8250 2007-09-25 13:31:24Z giles $ */
+/* $Id: zcontrol.c 8901 2008-07-29 19:11:31Z alexcher $ */
 /* Control operators */
 #include "string_.h"
 #include "ghost.h"
@@ -816,9 +816,10 @@ do_execstack(i_ctx_t *i_ctx_p, bool include_marks, os_ptr op1)
 	    }
 	    case t_struct:
 	    case t_astruct: {
-		const char *tname =
+		const char *tname = rq->value.pstruct ?
 		    gs_struct_type_name_string(
-				gs_object_type(imemory, rq->value.pstruct));
+				gs_object_type(imemory, rq->value.pstruct))
+                    : "NULL";
 
 		make_const_string(rq, a_readonly | avm_foreign,
 				  strlen(tname), (const byte *)tname);
