@@ -1,20 +1,17 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfcid0.c,v 1.25 2004/11/19 04:39:11 ray Exp $ */
+/* $Id: zfcid0.c 8250 2007-09-25 13:31:24Z giles $ */
 /* CIDFontType 0 operators */
 #include "memory_.h"
 #include "ghost.h"
@@ -47,7 +44,7 @@ font_proc_glyph_outline(zcharstring_glyph_outline);
 /* ------ Accessing ------ */
 
 /* Parse a multi-byte integer from a string. */
-private int
+static int
 get_index(gs_glyph_data_t *pgd, int count, ulong *pval)
 {
     int i;
@@ -63,7 +60,7 @@ get_index(gs_glyph_data_t *pgd, int count, ulong *pval)
 }
 
 /* Get bytes from GlyphData or DataSource. */
-private int
+static int
 cid0_read_bytes(gs_font_cid0 *pfont, ulong base, uint count, byte *buf,
 		gs_glyph_data_t *pgd)
 {
@@ -169,7 +166,7 @@ cid0_read_bytes(gs_font_cid0 *pfont, ulong base, uint count, byte *buf,
 /* Get the CharString data for a CIDFontType 0 font. */
 /* This is the glyph_data procedure in the font itself. */
 /* Note that pgd may be NULL. */
-private int
+static int
 z9_glyph_data(gs_font_base *pbfont, gs_glyph glyph, gs_glyph_data_t *pgd,
 	      int *pfidx)
 {
@@ -248,7 +245,7 @@ z9_glyph_data(gs_font_base *pbfont, gs_glyph glyph, gs_glyph_data_t *pgd,
 }
 
 /* Get the outline of a CIDFontType 0 glyph. */
-private int
+static int
 z9_glyph_outline(gs_font *font, int WMode, gs_glyph glyph, const gs_matrix *pmat,
 		 gx_path *ppath, double sbw[4])
 {
@@ -269,7 +266,7 @@ z9_glyph_outline(gs_font *font, int WMode, gs_glyph glyph, const gs_matrix *pmat
     return ocode;
 }
 
-private int
+static int
 z9_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 		     int members, gs_glyph_info_t *info)
 {   /* fixme : same as z11_glyph_info. */
@@ -285,13 +282,13 @@ z9_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
  * outlines are always provided externally.  Replace the accessor procedures
  * with ones that will give an error if called.
  */
-private int
+static int
 z9_FDArray_glyph_data(gs_font_type1 * pfont, gs_glyph glyph,
 		      gs_glyph_data_t *pgd)
 {
     return_error(e_invalidfont);
 }
-private int
+static int
 z9_FDArray_seac_data(gs_font_type1 *pfont, int ccode, gs_glyph *pglyph,
 		     gs_const_string *gstr, gs_glyph_data_t *pgd)
 {
@@ -301,7 +298,7 @@ z9_FDArray_seac_data(gs_font_type1 *pfont, int ccode, gs_glyph *pglyph,
 /* ------ Defining ------ */
 
 /* Get one element of a FDArray. */
-private int
+static int
 fd_array_element(i_ctx_t *i_ctx_p, gs_font_type1 **ppfont, ref *prfd)
 {
     charstring_font_refs_t refs;
@@ -367,7 +364,7 @@ fd_array_element(i_ctx_t *i_ctx_p, gs_font_type1 **ppfont, ref *prfd)
 
 
 
-private int 
+static int 
 notify_remove_font_type9(void *proc_data, void *event_data)
 {  /* Likely type 9 font descendents are never released explicitly.
       So releaseing a type 9 font we must reset pointers in descendents.
@@ -386,7 +383,7 @@ notify_remove_font_type9(void *proc_data, void *event_data)
 }
 
 /* <string|name> <font_dict> .buildfont9 <string|name> <font> */
-private int
+static int
 zbuildfont9(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -468,12 +465,24 @@ zbuildfont9(i_ctx_t *i_ctx_p)
 	if (code < 0)
 	    goto fail;
     }
-    code = build_gs_simple_font(i_ctx_p, op, &pfont, ft_CID_encrypted,
+    code = build_gs_outline_font(i_ctx_p, op, &pfont, ft_CID_encrypted, 
 				&st_gs_font_cid0, &build,
-				bf_Encoding_optional |
-				bf_UniqueID_ignored);
+				bf_Encoding_optional | bf_UniqueID_ignored, 
+				build_gs_simple_font);
     if (code < 0)
 	goto fail;
+    if (code == 1) {
+	/* The font already has a FID, don't need to build it again. 
+	   Release FDArray and return normally.
+	   fixme: FDArray fonts are thrown for garbager.
+	   We're not safe to build them after 
+	   build_gs_simple_font(..., &pfont, ...),
+	   because a failure in building them would throw
+	   an underbuilt font with unclear consequences.
+	 */
+	ifree_object(FDArray, "buildfont9(FDarray)");
+	return 0;
+    }
     pfont->procs.enumerate_glyph = gs_font_cid0_enumerate_glyph;
     pfont->procs.glyph_outline = z9_glyph_outline;
     pfont->procs.glyph_info = z9_glyph_info;
@@ -485,8 +494,10 @@ zbuildfont9(i_ctx_t *i_ctx_p)
     pfcid->cidata.FDBytes = FDBytes;
     pfcid->cidata.glyph_data = z9_glyph_data;
     pfcid->cidata.proc_data = 0;	/* for GC */
-    get_font_name(imemory, &cfnstr, &CIDFontName);
-    copy_font_name(&pfcid->font_name, &cfnstr);
+    if (pfcid->font_name.size == 0) {
+	get_font_name(imemory, &cfnstr, &CIDFontName);
+	copy_font_name(&pfcid->font_name, &cfnstr);
+    }
     ref_assign(&pfont_data(pfont)->u.cid0.GlyphDirectory, &GlyphDirectory);
     ref_assign(&pfont_data(pfont)->u.cid0.GlyphData, &GlyphData);
     ref_assign(&pfont_data(pfont)->u.cid0.DataSource, &DataSource);
@@ -531,7 +542,7 @@ ztype9mapcid(i_ctx_t *i_ctx_p)
     if (code < 0) { /* failed to load glyph data, put CID 0 */
        int default_fallback_CID = 0 ;
 
-       if_debug2('J', "[J]ztype9cidmap() use CID %d instead of glyph-missing CID %d\n", default_fallback_CID, op->value.intval);
+       if_debug2('J', "[J]ztype9cidmap() use CID %d instead of glyph-missing CID %ld\n", default_fallback_CID, op->value.intval);
 
        op->value.intval = default_fallback_CID;
 
@@ -542,7 +553,7 @@ ztype9mapcid(i_ctx_t *i_ctx_p)
                                    &gdata, &fidx);
 
        if (code < 0) {
-           if_debug1('J', "[J]ztype9cidmap() could not load default glyph (CID %d)\n", op->value.intval);
+           if_debug1('J', "[J]ztype9cidmap() could not load default glyph (CID %ld)\n", op->value.intval);
            return_error(e_invalidfont);
        }
 
