@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfile.c 8250 2007-09-25 13:31:24Z giles $ */
+/* $Id: zfile.c 8878 2008-07-27 02:52:11Z alexcher $ */
 /* Non-I/O file operators */
 #include "memory_.h"
 #include "string_.h"
@@ -42,6 +42,7 @@
 #include "files.h"
 #include "main.h"		/* for gs_lib_paths */
 #include "store.h"
+#include "zfile.h"
 
 /* Import the IODevice table. */
 extern_gx_io_device_table();
@@ -59,9 +60,6 @@ static int parse_file_access_string(const ref *op, char file_access[4]);
 /* Forward references: other. */
 static int execfile_finish(i_ctx_t *);
 static int execfile_cleanup(i_ctx_t *);
-static int zopen_file(i_ctx_t *, const gs_parsed_file_name_t *pfn,
-		       const char *file_access, stream **ps,
-		       gs_memory_t *mem);
 static iodev_proc_open_file(iodev_os_open_file);
 stream_proc_report_error(filter_report_error);
 
@@ -288,7 +286,7 @@ zfile(i_ctx_t *i_ctx_p)
  * The names 'SAFETY' and 'tempfiles' are defined by gs_init.ps
 */
 static bool
-file_is_tempfile(i_ctx_t *i_ctx_p, const ref *op)
+file_is_tempfile(i_ctx_t *i_ctx_p, const uchar *fname, int len)
 {
     ref *SAFETY;
     ref *tempfiles;
@@ -297,7 +295,7 @@ file_is_tempfile(i_ctx_t *i_ctx_p, const ref *op)
     if (dict_find_string(systemdict, "SAFETY", &SAFETY) <= 0 ||
 	    dict_find_string(SAFETY, "tempfiles", &tempfiles) <= 0)
 	return false;
-    if (name_ref(imemory, op->value.bytes, r_size(op), &kname, -1) < 0 ||
+    if (name_ref(imemory, fname, len, &kname, -1) < 0 ||
 	    dict_find(tempfiles, &kname, &SAFETY) <= 0)
 	return false;
     return true;
@@ -318,7 +316,7 @@ zdeletefile(i_ctx_t *i_ctx_p)
     if (pname.iodev == iodev_default) {
 	if ((code = check_file_permissions(i_ctx_p, pname.fname, pname.len,
 		"PermitFileControl")) < 0 &&
-		 !file_is_tempfile(i_ctx_p, op)) {
+		 !file_is_tempfile(i_ctx_p, op->value.bytes, r_size(op))) {
 	    return code;
 	}
     }
@@ -447,7 +445,7 @@ zrenamefile(i_ctx_t *i_ctx_p)
 		 */
 	      ((check_file_permissions(i_ctx_p, pname1.fname, pname1.len,
 	      				"PermitFileControl") < 0 &&
-	          !file_is_tempfile(i_ctx_p, op - 1)) ||
+	          !file_is_tempfile(i_ctx_p, op[-1].value.bytes, r_size(op - 1))) ||
 	      (check_file_permissions(i_ctx_p, pname2.fname, pname2.len,
 	      				"PermitFileControl") < 0 ||
 	      check_file_permissions(i_ctx_p, pname2.fname, pname2.len,
@@ -831,7 +829,7 @@ parse_file_access_string(const ref *op, char file_access[4])
  * Open a file specified by a parsed file name (which may be only a
  * device).
  */
-static int
+int
 zopen_file(i_ctx_t *i_ctx_p, const gs_parsed_file_name_t *pfn,
 	   const char *file_access, stream **ps, gs_memory_t *mem)
 {
@@ -849,7 +847,8 @@ zopen_file(i_ctx_t *i_ctx_p, const gs_parsed_file_name_t *pfn,
 	    int code = check_file_permissions(i_ctx_p, pfn->fname, pfn->len,
 		file_access[0] == 'r' ? "PermitFileReading" : "PermitFileWriting");
 
-	    if (code < 0)
+	    if (code < 0 && !file_is_tempfile(i_ctx_p,
+                                          (const uchar *)pfn->fname, pfn->len))
 		return code;
 	}
 	return open_file(iodev, pfn->fname, pfn->len, file_access, ps, mem);
