@@ -1,23 +1,17 @@
-/* Copyright (C) 1991, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$RCSfile: zfont2.c,v $ $Revision: 1.3.2.2 $ */
+/* $Id: zfont2.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Type 2 font creation operators */
 #include "ghost.h"
 #include "oper.h"
@@ -30,9 +24,10 @@
 #include "idparam.h"
 #include "ifont1.h"
 #include "ifont2.h"
+#include "ialloc.h"
 
 /* Private utilities */
-private uint
+static uint
 subr_bias(const ref * psubrs)
 {
     uint size = r_size(psubrs);
@@ -89,13 +84,13 @@ type2_font_params(const_os_ptr op, charstring_font_refs_t *pfr,
 
 /* <string|name> <font_dict> .buildfont2 <string|name> <font> */
 /* Build a type 2 (compact Adobe encrypted) font. */
-private int
+static int
 zbuildfont2(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     charstring_font_refs_t refs;
     build_proc_refs build;
-    int code = build_proc_name_refs(&build,
+    int code = build_proc_name_refs(imemory, &build,
 				    "%Type2BuildChar", "%Type2BuildGlyph");
     gs_type1_data data1;
 
