@@ -1,20 +1,17 @@
-/* Copyright (C) 1996, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zfont42.c,v 1.21 2004/09/22 13:52:33 igor Exp $ */
+/* $Id: zfont42.c 8881 2008-07-27 09:38:42Z leonardo $ */
 /* Type 42 font creation operator */
 #include "memory_.h"
 #include "ghost.h"
@@ -28,18 +25,20 @@
 #include "idict.h"
 #include "idparam.h"
 #include "ifont42.h"
+#include "ichar1.h"
 #include "iname.h"
 #include "store.h"
 
 /* Forward references */
-private int z42_string_proc(gs_font_type42 *, ulong, uint, const byte **);
-private uint z42_get_glyph_index(gs_font_type42 *, gs_glyph);
-private int z42_gdir_get_outline(gs_font_type42 *, uint, gs_glyph_data_t *);
-private font_proc_enumerate_glyph(z42_enumerate_glyph);
-private font_proc_enumerate_glyph(z42_gdir_enumerate_glyph);
-private font_proc_encode_char(z42_encode_char);
-private font_proc_glyph_info(z42_glyph_info);
-private font_proc_glyph_outline(z42_glyph_outline);
+static int z42_string_proc(gs_font_type42 *, ulong, uint, const byte **);
+static uint z42_get_glyph_index(gs_font_type42 *, gs_glyph);
+static int z42_gdir_get_outline(gs_font_type42 *, uint, gs_glyph_data_t *);
+static font_proc_enumerate_glyph(z42_enumerate_glyph);
+static font_proc_enumerate_glyph(z42_gdir_enumerate_glyph);
+static font_proc_encode_char(z42_encode_char);
+static font_proc_glyph_info(z42_glyph_info);
+static font_proc_glyph_outline(z42_glyph_outline);
+static font_proc_font_info(z42_font_info);
 
 /* <string|name> <font_dict> .buildfont11/42 <string|name> <font> */
 /* Build a type 11 (TrueType CID-keyed) or 42 (TrueType) font. */
@@ -74,13 +73,17 @@ build_gs_TrueType_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_type42 **ppfont,
     pfont = *ppfont;
     pdata = pfont_data(pfont);
     ref_assign(&pdata->u.type42.sfnts, &sfnts);
+    pdata->u.type42.mru_sfnts_index = 0;
+    pdata->u.type42.mru_sfnts_pos = 0;
     make_null_new(&pdata->u.type42.CIDMap);
     ref_assign(&pdata->u.type42.GlyphDirectory, &GlyphDirectory);
     pfont->data.string_proc = z42_string_proc;
     pfont->data.proc_data = (char *)pdata;
-    code = gs_type42_font_init(pfont);
+    pfont->is_resource = (options & bf_has_font_file ? true : false);
+    code = gs_type42_font_init(pfont, 0);
     if (code < 0)
 	return code;
+    pfont->procs.font_info = z42_font_info;
     /*
      * If the font has a GlyphDictionary, this replaces loca and glyf for
      * accessing character outlines.  In this case, we use alternate
@@ -101,7 +104,7 @@ build_gs_TrueType_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_type42 **ppfont,
     pfont->procs.glyph_outline = z42_glyph_outline;
     return 0;
 }
-private int
+static int
 zbuildfont42(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -167,15 +170,35 @@ font_GlyphDirectory_param(os_ptr op, ref *pGlyphDirectory)
  */
 int
 string_array_access_proc(const gs_memory_t *mem, 
-			 const ref *psa, int modulus, ulong offset,
-			 uint length, const byte **pdata)
+			 const ref *psa, int modulus, ulong offset, uint length,
+			 uint *mru_index, ulong *mru_pos,
+			 const byte **pdata)
 {
-    ulong left = offset;
-    uint index = 0;
+    ulong left;
+    uint index;
+    bool backwards;
 
     if (length == 0)
         return 0;
-    for (;; ++index) {
+    if (mru_index && mru_pos && offset >= (*mru_pos >> 1)) {
+	/*    offset in or after mru string				*/
+	/* OR offset in 2nd half of the fragment before the mru string	*/
+	backwards = (*mru_pos > offset);
+	if (backwards) {
+	    index = *mru_index - 1;	/* 1st string to examine */
+	    left = *mru_pos - offset;	/* how many bytes to seek backwards */
+	} else {
+	    index = *mru_index;		/* 1st string to examine */
+	    left = offset - *mru_pos;	/* how many bytes to seek forward */
+	}
+    } else {
+	/*    no mru							*/
+	/* OR offset in 1st half of the fragment before the mru string	*/
+	backwards = false;
+	index = 0;
+	left = offset;
+    }
+    for (;;) {
 	ref rstr;
 	int code = array_get(mem, psa, index, &rstr);
 	uint size;
@@ -190,13 +213,29 @@ string_array_access_proc(const gs_memory_t *mem,
 	 * the additional byte is padding and should be ignored.
 	 */
 	size = r_size(&rstr) & -modulus;
+	if (backwards) {
+	    if (left <= size) {
+		left = size - left;
+		backwards = false;
+		/* "index" does not change */
+	    } else {
+		left -= size;
+		--index;
+		continue;
+	    }
+	}
 	if (left < size) {
 	    *pdata = rstr.value.const_bytes + left;
+	    if (mru_index)
+		*mru_index = index;
+	    if (mru_pos)
+		*mru_pos = offset - left;
 	    if (left + length > size)
 		return size - left;
 	    return 0;
 	}
 	left -= size;
+	++index;
     }
 }
 
@@ -209,7 +248,7 @@ const op_def zfont42_op_defs[] =
 };
 
 /* Reduce a glyph name to a glyph index if needed. */
-private gs_glyph
+static gs_glyph
 glyph_to_index(const gs_font *font, gs_glyph glyph)
 {
     ref gref;
@@ -228,7 +267,7 @@ glyph_to_index(const gs_font *font, gs_glyph glyph)
     }
     return GS_MIN_GLYPH_INDEX;	/* glyph 0 is notdef */
 }
-private uint
+static uint
 z42_get_glyph_index(gs_font_type42 *pfont, gs_glyph glyph)
 {
     gs_glyph gid = glyph_to_index((gs_font *)pfont, glyph);
@@ -268,7 +307,7 @@ font_gdir_get_outline(const gs_memory_t *mem,
     }
     return 0;
 }
-private int
+static int
 z42_gdir_get_outline(gs_font_type42 * pfont, uint glyph_index,
 		     gs_glyph_data_t *pgd)
 {
@@ -279,7 +318,7 @@ z42_gdir_get_outline(gs_font_type42 * pfont, uint glyph_index,
 }
 
 /* Enumerate glyphs from CharStrings or loca / glyf. */
-private int
+static int
 z42_enumerate_glyph(gs_font *font, int *pindex, gs_glyph_space_t glyph_space,
 		    gs_glyph *pglyph)
 {
@@ -293,7 +332,7 @@ z42_enumerate_glyph(gs_font *font, int *pindex, gs_glyph_space_t glyph_space,
 }
 
 /* Enumerate glyphs (keys) from GlyphDirectory instead of loca / glyf. */
-private int
+static int
 z42_gdir_enumerate_glyph(gs_font *font, int *pindex,
 			 gs_glyph_space_t glyph_space, gs_glyph *pglyph)
 {
@@ -329,7 +368,7 @@ z42_gdir_enumerate_glyph(gs_font *font, int *pindex,
  * Define font procedures that accept either a character name or a glyph
  * index as the glyph.
  */
-private gs_glyph
+static gs_glyph
 z42_encode_char(gs_font *font, gs_char chr, gs_glyph_space_t glyph_space)
 {
     gs_glyph glyph = zfont_encode_char(font, chr, glyph_space);
@@ -337,29 +376,43 @@ z42_encode_char(gs_font *font, gs_char chr, gs_glyph_space_t glyph_space)
     return (glyph_space == GLYPH_SPACE_INDEX && glyph != gs_no_glyph ?
 	    glyph_to_index(font, glyph) : glyph);
 }
-private int
+static int
 z42_glyph_outline(gs_font *font, int WMode, gs_glyph glyph, const gs_matrix *pmat,
 		  gx_path *ppath, double sbw[4])
 {
     return gs_type42_glyph_outline(font, WMode, glyph_to_index(font, glyph),
 				   pmat, ppath, sbw);
 }
-private int
+static int
 z42_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 	       int members, gs_glyph_info_t *info)
-{
-    return gs_type42_glyph_info(font, glyph_to_index(font, glyph),
-				pmat, members, info);
+{   /* fixme : same as z1_glyph_info. */
+    int wmode = font->WMode;
+
+    return z1_glyph_info_generic(font, glyph, pmat, members, info, gs_type42_glyph_info, wmode);
 }
 
 /* Procedure for accessing the sfnts array.
  * Return code : 0 - success, <0 - error, 
  *               >0 - number of accessible bytes (client must cycle).
  */
-private int
+static int
 z42_string_proc(gs_font_type42 * pfont, ulong offset, uint length,
 		const byte ** pdata)
 {
     return string_array_access_proc(pfont->memory, &pfont_data(pfont)->u.type42.sfnts, 2,
-				    offset, length, pdata);
+				    offset, length, &pfont_data(pfont)->u.type42.mru_sfnts_index,
+				    &pfont_data(pfont)->u.type42.mru_sfnts_pos, pdata);
 }
+
+static int
+z42_font_info(gs_font *font, const gs_point *pscale, int members,
+	   gs_font_info_t *info)
+{
+    int code = zfont_info(font, pscale, members, info);
+
+    if (code < 0)
+	return code;
+    return gs_truetype_font_info(font, pscale, members, info);
+}
+
