@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,10 +11,11 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zimage.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: zimage.c 8767 2008-05-21 19:46:47Z giles $ */
 /* Image operators */
 #include "math_.h"
 #include "memory_.h"
+#include "stat_.h" /* get system header early to avoid name clash on Cygwin */
 #include "ghost.h"
 #include "oper.h"
 #include "gscolor.h"
@@ -35,15 +36,16 @@
 #include "stream.h"
 #include "ifilter.h"		/* for stream exception handling */
 #include "iimage.h"
+#include "gxcspace.h"
 
 /* Forward references */
-private int zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
+static int zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
 				 gx_image_enum_common_t * pie,
 				 const ref * sources, int npop);
-private int image_proc_process(i_ctx_t *);
-private int image_file_continue(i_ctx_t *);
-private int image_string_continue(i_ctx_t *);
-private int image_cleanup(i_ctx_t *);
+static int image_proc_process(i_ctx_t *);
+static int image_file_continue(i_ctx_t *);
+static int image_string_continue(i_ctx_t *);
+static int image_cleanup(i_ctx_t *);
 
 
 
@@ -87,13 +89,26 @@ data_image_params(const gs_memory_t *mem,
 	return 1;		/* no data source */
     }
     if (pip->MultipleDataSources) {
-	long i, n = num_components + (has_alpha ? 1 : 0);
+	ref *ds = pip->DataSource;
+        long i, n = num_components + (has_alpha ? 1 : 0);
         if (!r_is_array(pds))
             return_error(e_typecheck);
 	if (r_size(pds) != n)
 	    return_error(e_rangecheck);
 	for (i = 0; i < n; ++i)
-	    array_get(mem, pds, i, &pip->DataSource[i]);
+	    array_get(mem, pds, i, &ds[i]);
+        if (r_type(&ds[0]) == t_string) {
+            /* We don't have a problem with the strings of different length
+             * but Adobe does and CET tast 12-02.ps reports this as an error.
+             */
+	    if (has_alpha)
+                n--;
+            for (i = 1; i < n; ++i) {
+                if (r_type(&ds[i]) == t_string && r_size(&ds[i]) != r_size(&ds[0])) {
+	            return_error(e_rangecheck);
+                }
+            }
+        }
     } else
 	pip->DataSource[0] = *pds;
     return 0;
@@ -103,15 +118,15 @@ data_image_params(const gs_memory_t *mem,
 int
 pixel_image_params(i_ctx_t *i_ctx_p, const ref *op, gs_pixel_image_t *pim,
 		   image_params *pip, int max_bits_per_component,
-		   bool has_alpha)
+		   bool has_alpha, gs_color_space *csp)
 {
     int num_components =
-	gs_color_space_num_components(gs_currentcolorspace(igs));
+	gs_color_space_num_components(csp);
     int code;
 
     if (num_components < 1)
 	return_error(e_rangecheck);	/* Pattern space not allowed */
-    pim->ColorSpace = gs_currentcolorspace(igs);
+    pim->ColorSpace = csp;
     code = data_image_params(imemory, op, (gs_data_image_t *) pim, pip, true,
 			     num_components, max_bits_per_component,
 			     has_alpha);
@@ -148,14 +163,28 @@ image1_setup(i_ctx_t * i_ctx_p, bool has_alpha)
     gs_image_t      image;
     image_params    ip;
     int             code;
+    gs_color_space *csp = gs_currentcolorspace(igs);
+    extern bool CPSI_mode;
 
-    gs_image_t_init(&image, gs_currentcolorspace(igs));
+    /* Adobe interpreters accept sampled images when the current color
+     * space is a pattern color space using the base color space instead
+     * of the pattern space. CET 12-07a-12
+     * If all conditions are not met the pattern color space goes through
+     * triggering a rangecheck error.
+     */
+    if (CPSI_mode && gs_color_space_num_components(csp) < 1) {
+       gs_color_space *bsp = csp->base_space;
+       if (bsp)
+         csp = bsp;
+    }
+
+    gs_image_t_init(&image, csp);
     code = pixel_image_params( i_ctx_p,
                                op,
                                (gs_pixel_image_t *)&image,
                                &ip,
 			       (level2_enabled ? 16 : 8),
-                               has_alpha );
+                               has_alpha, csp);
     if (code < 0)
 	return code;
 
@@ -168,14 +197,14 @@ image1_setup(i_ctx_t * i_ctx_p, bool has_alpha)
 }
 
 /* <dict> .image1 - */
-private int
+static int
 zimage1(i_ctx_t *i_ctx_p)
 {
     return image1_setup(i_ctx_p, false);
 }
 
 /* <dict> .imagemask1 - */
-private int
+static int
 zimagemask1(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -189,8 +218,6 @@ zimagemask1(i_ctx_t *i_ctx_p)
 			     &ip, true, 1, 1, false);
     if (code < 0)
 	return code;
-    if (ip.MultipleDataSources)
-	return_error(e_rangecheck);
     return zimage_setup(i_ctx_p, (gs_pixel_image_t *)&image, &ip.DataSource[0],
 			true, 1);
 }
@@ -227,7 +254,7 @@ zimagemask1(i_ctx_t *i_ctx_p)
   ((ep) - 4 - (i) * 2)
 #define ETOP_PLANE_INDEX(ep) ((ep) - 2)
 #define ETOP_NUM_SOURCES(ep) ((ep) - 1)
-private int
+static int
 zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
 		  gx_image_enum_common_t * pie, const ref * sources, int npop)
 {
@@ -237,6 +264,7 @@ zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
     gs_image_enum *penum;
     int px;
     const ref *pp;
+    bool string_sources = true;
 
     check_estack(inumpush + 2);	/* stuff above, + continuation + proc */
     make_int(EBOT_NUM_SOURCES(esp), num_sources);
@@ -273,6 +301,7 @@ zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
 			    break;
 			}
 		}
+		string_sources = false;
 		/* falls through */
 	    case t_string:
 		if (r_type(pp) != r_type(sources)) {
@@ -284,11 +313,14 @@ zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
 		break;
 	    default:
 		if (!r_is_proc(sources)) {
-    		    if (pie != NULL)
-		        gx_image_end(pie, false);    /* Clean up pie */
+    		    static const char ds[] = "DataSource";
+                    if (pie != NULL)
+                        gx_image_end(pie, false);    /* Clean up pie */
+                    gs_errorinfo_put_pair(i_ctx_p, ds, sizeof(ds) - 1, pp);
 		    return_error(e_typecheck);
 		}
 		check_proc(*pp);
+		string_sources = false;
 	}
 	*ep = *pp;
     }
@@ -299,8 +331,8 @@ zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
     if ((penum = gs_image_enum_alloc(imemory_local, "image_setup")) == 0)
 	return_error(e_VMerror);
     code = gs_image_enum_init(penum, pie, (const gs_data_image_t *)pim, igs);
-    if (code != 0) {		/* error, or empty image */
-	int code1 = gs_image_cleanup_and_free_enum(penum);
+    if (code != 0 || (pie->skipping && string_sources)) {		/* error, or empty image */
+	int code1 = gs_image_cleanup_and_free_enum(penum, igs);
 
 	if (code >= 0)		/* empty image */
 	    pop(npop);
@@ -328,7 +360,7 @@ zimage_data_setup(i_ctx_t *i_ctx_p, const gs_pixel_image_t * pim,
     return o_push_estack;
 }
 /* Pop all the control information off the e-stack. */
-private es_ptr
+static es_ptr
 zimage_pop_estack(es_ptr tep)
 {
     return tep - NUM_PUSH(ETOP_NUM_SOURCES(tep)->value.intval);
@@ -339,15 +371,15 @@ zimage_pop_estack(es_ptr tep)
  * to remember whether we've just called the procedure (1) or whether we're
  * returning from a RemapColor callout (0).
  */
-private int
+static int
 image_proc_continue(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
     gs_image_enum *penum = r_ptr(esp, gs_image_enum);
     int px = ETOP_PLANE_INDEX(esp)->value.intval;
     int num_sources = ETOP_NUM_SOURCES(esp)->value.intval;
-    uint size, used[gs_image_max_planes];
-    gs_const_string plane_data[gs_image_max_planes];
+    uint size, used[GS_IMAGE_MAX_COMPONENTS];
+    gs_const_string plane_data[GS_IMAGE_MAX_COMPONENTS];
     const byte *wanted;
     int i, code;
 
@@ -389,7 +421,7 @@ image_proc_continue(i_ctx_t *i_ctx_p)
     ETOP_PLANE_INDEX(esp)->value.intval = px;
     return image_proc_process(i_ctx_p);
 }
-private int
+static int
 image_proc_process(i_ctx_t *i_ctx_p)
 {
     int px = ETOP_PLANE_INDEX(esp)->value.intval;
@@ -411,7 +443,7 @@ image_proc_process(i_ctx_t *i_ctx_p)
 }
 
 /* Continue processing data from an image with file data sources. */
-private int
+static int
 image_file_continue(i_ctx_t *i_ctx_p)
 {
     gs_image_enum *penum = r_ptr(esp, gs_image_enum);
@@ -419,7 +451,7 @@ image_file_continue(i_ctx_t *i_ctx_p)
 
     for (;;) {
 	uint min_avail = max_int;
-	gs_const_string plane_data[gs_image_max_planes];
+	gs_const_string plane_data[GS_IMAGE_MAX_COMPONENTS];
 	int code;
 	int px;
 	const ref *pp;
@@ -483,7 +515,7 @@ image_file_continue(i_ctx_t *i_ctx_p)
 
 	{
 	    int pi;
-	    uint used[gs_image_max_planes];
+	    uint used[GS_IMAGE_MAX_COMPONENTS];
 
 	    code = gs_image_next_planes(penum, plane_data, used);
 	    /* Now that used has been set, update the streams. */
@@ -508,13 +540,13 @@ image_file_continue(i_ctx_t *i_ctx_p)
 
 /* Process data from an image with string data sources. */
 /* This may still encounter a RemapColor callback. */
-private int
+static int
 image_string_continue(i_ctx_t *i_ctx_p)
 {
     gs_image_enum *penum = r_ptr(esp, gs_image_enum);
     int num_sources = ETOP_NUM_SOURCES(esp)->value.intval;
-    gs_const_string sources[gs_image_max_planes];
-    uint used[gs_image_max_planes];
+    gs_const_string sources[GS_IMAGE_MAX_COMPONENTS];
+    uint used[GS_IMAGE_MAX_COMPONENTS];
 
     /* Pass no data initially, to find out how much is retained. */
     memset(sources, 0, sizeof(sources[0]) * num_sources);
@@ -546,13 +578,13 @@ image_string_continue(i_ctx_t *i_ctx_p)
 }
 
 /* Clean up after enumerating an image */
-private int
+static int
 image_cleanup(i_ctx_t *i_ctx_p)
 {
     es_ptr ep_top = esp + NUM_PUSH(EBOT_NUM_SOURCES(esp)->value.intval);
     gs_image_enum *penum = r_ptr(ep_top, gs_image_enum);
     
-    return gs_image_cleanup_and_free_enum(penum);
+    return gs_image_cleanup_and_free_enum(penum, igs);
 }
 
 /* ------ Initialization procedure ------ */
