@@ -1,20 +1,17 @@
-/* Copyright (C) 1993, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zmedia2.c,v 1.18 2005/04/06 16:45:18 raph Exp $ */
+/* $Id: zmedia2.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Media matching for setpagedevice */
 #include "math_.h"
 #include "memory_.h"
@@ -29,7 +26,7 @@
 /* <pagedict> <attrdict> <policydict> <keys> .matchmedia <key> true */
 /* <pagedict> <attrdict> <policydict> <keys> .matchmedia false */
 /* <pagedict> null <policydict> <keys> .matchmedia null true */
-private int zmatch_page_size(const gs_memory_t *mem,
+static int zmatch_page_size(const gs_memory_t *mem,
 			     const ref * pvreq, const ref * pvmed,
 			     int policy, int orient, bool roll,
 			     float *best_mismatch, gs_matrix * pmat,
@@ -38,14 +35,14 @@ typedef struct match_record_s {
     ref best_key, match_key;
     uint priority, no_match_priority;
 } match_record_t;
-private void
+static void
 reset_match(match_record_t *match)
 {
     make_null(&match->best_key);
     make_null(&match->match_key);
     match->priority = match->no_match_priority;
 }
-private int
+static int
 zmatchmedia(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -233,7 +230,7 @@ no:;
  *     <policy> <orient|null> <roll> <matrix|null> .matchpagesize
  *   <matrix|null> <med_x> <med_y> true   -or-  false
  */
-private int
+static int
 zmatchpagesize(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -278,13 +275,13 @@ zmatchpagesize(i_ctx_t *i_ctx_p)
     return 0;
 }
 /* Match the PageSize.  See below for details. */
-private int
+static int
 match_page_size(const gs_point * request,
 			     const gs_rect * medium,
 			     int policy, int orient, bool roll,
 			     float *best_mismatch, gs_matrix * pmat,
 			     gs_point * pmsize);
-private int
+static int
 zmatch_page_size(const gs_memory_t *mem, const ref * pvreq, const ref * pvmed,
 		 int policy, int orient, bool roll,
 		 float *best_mismatch, gs_matrix * pmat, gs_point * pmsize)
@@ -338,11 +335,11 @@ zmatch_page_size(const gs_memory_t *mem, const ref * pvreq, const ref * pvmed,
  * NOTE: The algorithm here doesn't work properly for variable-size media
  * when the match isn't exact.  We'll fix it if we ever need to.
  */
-private void make_adjustment_matrix(const gs_point * request,
+static void make_adjustment_matrix(const gs_point * request,
 				    const gs_rect * medium,
 				    gs_matrix * pmat,
 				    bool scale, int rotate);
-private int
+static int
 match_page_size(const gs_point * request, const gs_rect * medium, int policy,
 		int orient, bool roll, float *best_mismatch, gs_matrix * pmat,
 		gs_point * pmsize)
@@ -453,7 +450,7 @@ match_page_size(const gs_point * request, const gs_rect * medium, int policy,
  * we must adjust its size in that dimension to match the request.
  * We recognize this by an unreasonably small medium->p.{x,y}.
  */
-private void 
+static void 
 make_adjustment_matrix(const gs_point * request, const gs_rect * medium,
 		       gs_matrix * pmat, bool scale, int rotate)
 {
@@ -469,18 +466,20 @@ make_adjustment_matrix(const gs_point * request, const gs_rect * medium,
     /* If 'medium' is flexible, adjust 'mx' and 'my' towards 'rx' and 'ry',
        respectively. Note that 'mx' and 'my' have just acquired the largest
        permissible value, medium->q. */
-    if (medium->p.x < mx)	/* non-empty width range */
+    if (medium->p.x < mx) {	/* non-empty width range */
 	if (rx < medium->p.x)
 	    mx = medium->p.x;	/* use minimum of the range */
 	else if (rx < mx)
 	    mx = rx;		/* fits */
 		/* else leave mx == medium->q.x, i.e., the maximum */
-    if (medium->p.y < my)	/* non-empty height range */
+    }
+    if (medium->p.y < my) {	/* non-empty height range */
 	if (ry < medium->p.y)
 	    my = medium->p.y;	/* use minimum of the range */
 	else if (ry < my)
 	    my = ry;		/* fits */
 	    /* else leave my == medium->q.y, i.e., the maximum */
+    }
 
     /* Translate to align the centers. */ 
     gs_make_translation(mx / 2, my / 2, pmat);
