@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2006 artofcode LLC.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
   
    This software is provided AS-IS with no warranty, either express or
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zmisc.c 6651 2006-03-13 16:18:19Z stefan $ */
+/* $Id: zmisc.c 8727 2008-05-13 03:58:10Z alexcher $ */
 /* Miscellaneous operators */
 
 #include "errno_.h"
@@ -29,14 +29,16 @@
 #include "ivmspace.h"
 #include "store.h"
 
+/**********************************************************************/
+
 /* <proc> bind <proc> */
-inline private bool
+static inline bool
 r_is_ex_oper(const ref *rp)
 {
     return (r_has_attr(rp, a_executable) &&
 	    (r_btype(rp) == t_operator || r_type(rp) == t_oparray));
 }
-private int
+static int
 zbind(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -46,6 +48,9 @@ zbind(i_ctx_t *i_ctx_p)
 
     switch (r_type(op)) {
 	case t_array:
+	    if (!r_has_attr(op, a_write)) {
+		return 0;	/* per PLRM3 */
+	    }
 	case t_mixedarray:
 	case t_shortarray:
 	    defn = *op;
@@ -155,7 +160,7 @@ zbind(i_ctx_t *i_ctx_p)
 }
 
 /* - serialnumber <int> */
-private int
+static int
 zserialnumber(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -166,9 +171,9 @@ zserialnumber(i_ctx_t *i_ctx_p)
 }
 
 /* some FTS tests work better if realtime starts from 0 at boot time */
-private long    real_time_0[2];
+static long    real_time_0[2];
 
-private int
+static int
 zmisc_init_realtime(i_ctx_t * i_ctx_p)
 {
     gp_get_realtime(real_time_0);
@@ -176,7 +181,7 @@ zmisc_init_realtime(i_ctx_t * i_ctx_p)
 }
 
 /* - realtime <int> */
-private int
+static int
 zrealtime(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -191,7 +196,7 @@ zrealtime(i_ctx_t *i_ctx_p)
 }
 
 /* - usertime <int> */
-private int
+static int
 zusertime(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -207,7 +212,7 @@ zusertime(i_ctx_t *i_ctx_p)
 
 /* <string> getenv <value_string> true */
 /* <string> getenv false */
-private int
+static int
 zgetenv(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -241,7 +246,7 @@ zgetenv(i_ctx_t *i_ctx_p)
 }
 
 /* <name> <proc> .makeoperator <oper> */
-private int
+static int
 zmakeoperator(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -284,7 +289,7 @@ zmakeoperator(i_ctx_t *i_ctx_p)
 }
 
 /* - .oserrno <int> */
-private int
+static int
 zoserrno(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -295,7 +300,7 @@ zoserrno(i_ctx_t *i_ctx_p)
 }
 
 /* <int> .setoserrno - */
-private int
+static int
 zsetoserrno(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -308,7 +313,7 @@ zsetoserrno(i_ctx_t *i_ctx_p)
 
 /* <int> .oserrorstring <string> true */
 /* <int> .oserrorstring false */
-private int
+static int
 zoserrorstring(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -338,7 +343,7 @@ zoserrorstring(i_ctx_t *i_ctx_p)
 }
 
 /* <string> <bool> .setdebug - */
-private int
+static int
 zsetdebug(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -355,6 +360,40 @@ zsetdebug(i_ctx_t *i_ctx_p)
     return 0;
 }
 
+/* There are a few cases where a customer/user might want CPSI behavior 
+ * instead of the GS default behavior. cmyk_to_rgb and Type 1 char fill
+ * method are two that have come up so far. This operator allows a PS
+ * program to control the behavior without needing to recompile
+ *
+ * While this would better if it were in some context 'state', we use
+ * a global, which we don't really like, but at least it is better
+ * than a compile time #define, as in #USE_ADOBE_CMYK_RGB and allows
+ * us to easily test with/without and not require recompilation.
+ */
+extern bool CPSI_mode;		/* not worth polluting a header file */
+
+/* <bool> .setCPSImode - */
+static int
+zsetCPSImode(i_ctx_t *i_ctx_p)
+{
+    os_ptr op = osp;
+    check_type(*op, t_boolean);
+    CPSI_mode = op->value.boolval;
+    pop(1);
+    return 0;
+}
+
+/* - .getCPSImode <bool> */
+static int
+zgetCPSImode(i_ctx_t *i_ctx_p)
+{
+    os_ptr op = osp;
+
+    push(1);
+    make_bool(op, CPSI_mode);
+    return 0;
+}
+
 /* ------ gs persistent cache operators ------ */
 /* these are for testing only. they're disabled in the normal build
  * to prevent access to the cache by malicious postscript files
@@ -367,7 +406,7 @@ zsetdebug(i_ctx_t *i_ctx_p)
 #ifdef DEBUG_CACHE
 
 /* <string> <string> .pcacheinsert */
-private int
+static int
 zpcacheinsert(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -392,7 +431,7 @@ zpcacheinsert(i_ctx_t *i_ctx_p)
 }
 
 /* allocation callback for query result */
-private void *
+static void *
 pcache_alloc_callback(void *userdata, int bytes)
 {
     i_ctx_t *i_ctx_p = (i_ctx_t*)userdata;    
@@ -401,7 +440,7 @@ pcache_alloc_callback(void *userdata, int bytes)
 
 /* <string> .pcachequery <string> true */
 /* <string> .pcachequery false */
-private int
+static int
 zpcachequery(i_ctx_t *i_ctx_p)
 {
 	os_ptr op = osp;
@@ -444,6 +483,8 @@ const op_def zmisc_op_defs[] =
     {"2.setdebug", zsetdebug},
     {"1.setoserrno", zsetoserrno},
     {"0usertime", zusertime},
+    {"1.setCPSImode", zsetCPSImode},
+    {"0.getCPSImode", zgetCPSImode},
 #ifdef DEBUG_CACHE
 	/* pcache test */
     {"2.pcacheinsert", zpcacheinsert},
