@@ -1,22 +1,17 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
   
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License as published by the
-  Free Software Foundation; either version 2 of the License, or (at your
-  option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zpath.c,v 1.2.6.1.2.1 2003/01/17 00:49:06 giles Exp $ */
+/* $Id: zpath.c 8250 2007-09-25 13:31:24Z giles $ */
 /* Basic path operators */
 #include "math_.h"
 #include "ghost.h"
@@ -27,20 +22,20 @@
 #include "store.h"
 
 /* Forward references */
-private int common_to(P2(i_ctx_t *,
-			 int (*)(P3(gs_state *, floatp, floatp))));
-private int common_curve(P2(i_ctx_t *,
-  int (*)(P7(gs_state *, floatp, floatp, floatp, floatp, floatp, floatp))));
+static int common_to(i_ctx_t *,
+		      int (*)(gs_state *, floatp, floatp));
+static int common_curve(i_ctx_t *,
+  int (*)(gs_state *, floatp, floatp, floatp, floatp, floatp, floatp));
 
 /* - newpath - */
-private int
+static int
 znewpath(i_ctx_t *i_ctx_p)
 {
     return gs_newpath(igs);
 }
 
 /* - currentpoint <x> <y> */
-private int
+static int
 zcurrentpoint(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -84,9 +79,9 @@ zrlineto(i_ctx_t *i_ctx_p)
 }
 
 /* Common code for [r](move/line)to */
-private int
+static int
 common_to(i_ctx_t *i_ctx_p,
-	  int (*add_proc)(P3(gs_state *, floatp, floatp)))
+	  int (*add_proc)(gs_state *, floatp, floatp))
 {
     os_ptr op = osp;
     double opxy[2];
@@ -115,9 +110,9 @@ zrcurveto(i_ctx_t *i_ctx_p)
 }
 
 /* Common code for [r]curveto */
-private int
+static int
 common_curve(i_ctx_t *i_ctx_p,
-	     int (*add_proc)(P7(gs_state *, floatp, floatp, floatp, floatp, floatp, floatp)))
+	     int (*add_proc)(gs_state *, floatp, floatp, floatp, floatp, floatp, floatp))
 {
     os_ptr op = osp;
     double opxy[6];
@@ -139,21 +134,21 @@ zclosepath(i_ctx_t *i_ctx_p)
 }
 
 /* - initclip - */
-private int
+static int
 zinitclip(i_ctx_t *i_ctx_p)
 {
     return gs_initclip(igs);
 }
 
 /* - clip - */
-private int
+static int
 zclip(i_ctx_t *i_ctx_p)
 {
     return gs_clip(igs);
 }
 
 /* - eoclip - */
-private int
+static int
 zeoclip(i_ctx_t *i_ctx_p)
 {
     return gs_eoclip(igs);
