@@ -1,21 +1,17 @@
+/* Copyright (C) 2001-2006 Artifex Software, Inc.
+   All Rights Reserved.
+  
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-/* Copyright (C) 1996, 2000 Aladdin Enterprises.  All rights reserved.
-  
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zusparam.c,v 1.3.6.2 2002/02/22 19:46:01 ray Exp $ */
+/* $Id: zusparam.c 8631 2008-04-09 15:28:45Z ken $ */
 /* User and system parameter operators */
 #include "memory_.h"
 #include "string_.h"
@@ -38,6 +34,7 @@
 #include "iutil2.h"
 #include "ivmem2.h"
 #include "store.h"
+#include "gsnamecl.h"
 
 /* The (global) font directory */
 extern gs_font_dir *ifont_dir;	/* in zfont.c */
@@ -54,26 +51,28 @@ typedef struct param_def_s {
 typedef struct long_param_def_s {
     param_def_common;
     long min_value, max_value;
-    long (*current)(P1(i_ctx_t *));
-    int (*set)(P2(i_ctx_t *, long));
+    long (*current)(i_ctx_t *);
+    int (*set)(i_ctx_t *, long);
 } long_param_def_t;
 
 #if arch_sizeof_long > arch_sizeof_int
 #  define MAX_UINT_PARAM max_uint
+#  define MIN_INT_PARAM min_int
 #else
 #  define MAX_UINT_PARAM max_long
+#  define MIN_INT_PARAM min_long
 #endif
 
 typedef struct bool_param_def_s {
     param_def_common;
-    bool (*current)(P1(i_ctx_t *));
-    int (*set)(P2(i_ctx_t *, bool));
+    bool (*current)(i_ctx_t *);
+    int (*set)(i_ctx_t *, bool);
 } bool_param_def_t;
 
 typedef struct string_param_def_s {
     param_def_common;
-    void (*current)(P2(i_ctx_t *, gs_param_string *));
-    int (*set)(P2(i_ctx_t *, gs_param_string *));
+    void (*current)(i_ctx_t *, gs_param_string *);
+    int (*set)(i_ctx_t *, gs_param_string *);
 } string_param_def_t;
 
 /* Define a parameter set (user or system). */
@@ -87,14 +86,14 @@ typedef struct param_set_s {
 } param_set;
 
 /* Forward references */
-private int setparams(P3(i_ctx_t *, gs_param_list *, const param_set *));
-private int currentparams(P2(i_ctx_t *, const param_set *));
-private int currentparam1(P2(i_ctx_t *, const param_set *));
+static int setparams(i_ctx_t *, gs_param_list *, const param_set *);
+static int currentparams(i_ctx_t *, const param_set *);
+static int currentparam1(i_ctx_t *, const param_set *);
 
 /* ------ Passwords ------ */
 
 /* <string|int> .checkpassword <0|1|2> */
-private int
+static int
 zcheckpassword(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -102,7 +101,7 @@ zcheckpassword(i_ctx_t *i_ctx_p)
     array_param_list list;
     gs_param_list *const plist = (gs_param_list *)&list;
     int result = 0;
-    int code = name_ref((const byte *)"Password", 8, &params[0], 0);
+    int code = name_ref(imemory, (const byte *)"Password", 8, &params[0], 0);
     password pass;
 
     if (code < 0)
@@ -125,24 +124,24 @@ zcheckpassword(i_ctx_t *i_ctx_p)
 /* ------ System parameters ------ */
 
 /* Integer values */
-private long
+static long
 current_BuildTime(i_ctx_t *i_ctx_p)
 {
     return gs_buildtime;
 }
-private long
+static long
 current_MaxFontCache(i_ctx_t *i_ctx_p)
 {
     return gs_currentcachesize(ifont_dir);
 }
-private int
+static int
 set_MaxFontCache(i_ctx_t *i_ctx_p, long val)
 {
     return gs_setcachesize(ifont_dir,
 			   (uint)(val < 0 ? 0 : val > max_uint ? max_uint :
 				   val));
 }
-private long
+static long
 current_CurFontCache(i_ctx_t *i_ctx_p)
 {
     uint cstat[7];
@@ -150,7 +149,7 @@ current_CurFontCache(i_ctx_t *i_ctx_p)
     gs_cachestatus(ifont_dir, cstat);
     return cstat[0];
 }
-private long
+static long
 current_MaxGlobalVM(i_ctx_t *i_ctx_p)
 {
     gs_memory_gc_status_t stat;
@@ -158,7 +157,7 @@ current_MaxGlobalVM(i_ctx_t *i_ctx_p)
     gs_memory_gc_status(iimemory_global, &stat);
     return stat.max_vm;
 }
-private int
+static int
 set_MaxGlobalVM(i_ctx_t *i_ctx_p, long val)
 {
     gs_memory_gc_status_t stat;
@@ -168,12 +167,12 @@ set_MaxGlobalVM(i_ctx_t *i_ctx_p, long val)
     gs_memory_set_gc_status(iimemory_global, &stat);
     return 0;
 }
-private long
+static long
 current_Revision(i_ctx_t *i_ctx_p)
 {
     return gs_revision;
 }
-private const long_param_def_t system_long_params[] =
+static const long_param_def_t system_long_params[] =
 {
     {"BuildTime", min_long, max_long, current_BuildTime, NULL},
 {"MaxFontCache", 0, MAX_UINT_PARAM, current_MaxFontCache, set_MaxFontCache},
@@ -184,21 +183,21 @@ private const long_param_def_t system_long_params[] =
 };
 
 /* Boolean values */
-private bool
+static bool
 current_ByteOrder(i_ctx_t *i_ctx_p)
 {
     return !arch_is_big_endian;
 }
-private const bool_param_def_t system_bool_params[] =
+static const bool_param_def_t system_bool_params[] =
 {
     {"ByteOrder", current_ByteOrder, NULL}
 };
 
 /* String values */
-private void
+static void
 current_RealFormat(i_ctx_t *i_ctx_p, gs_param_string * pval)
 {
-#if arch_floats_are_IEEE
+#if ARCH_FLOATS_ARE_IEEE
     static const char *const rfs = "IEEE";
 #else
     static const char *const rfs = "not IEEE";
@@ -208,13 +207,13 @@ current_RealFormat(i_ctx_t *i_ctx_p, gs_param_string * pval)
     pval->size = strlen(rfs);
     pval->persistent = true;
 }
-private const string_param_def_t system_string_params[] =
+static const string_param_def_t system_string_params[] =
 {
     {"RealFormat", current_RealFormat, NULL}
 };
 
 /* The system parameter set */
-private const param_set system_param_set =
+static const param_set system_param_set =
 {
     system_long_params, countof(system_long_params),
     system_bool_params, countof(system_bool_params),
@@ -222,7 +221,7 @@ private const param_set system_param_set =
 };
 
 /* <dict> .setsystemparams - */
-private int
+static int
 zsetsystemparams(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -270,6 +269,13 @@ zsetsystemparams(i_ctx_t *i_ctx_p)
 	    if (code < 0)
 		goto out;
     }
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    /* The custom color callback pointer */
+    code = custom_color_callback_put_params(i_ctx_p->pgs, plist);
+    if (code < 0)
+	goto out;
+#endif
+
     code = setparams(i_ctx_p, plist, &system_param_set);
   out:
     iparam_list_release(&list);
@@ -280,14 +286,14 @@ zsetsystemparams(i_ctx_t *i_ctx_p)
 }
 
 /* - .currentsystemparams <name1> <value1> ... */
-private int
+static int
 zcurrentsystemparams(i_ctx_t *i_ctx_p)
 {
     return currentparams(i_ctx_p, &system_param_set);
 }
 
 /* <name> .getsystemparam <value> */
-private int
+static int
 zgetsystemparam(i_ctx_t *i_ctx_p)
 {
     return currentparam1(i_ctx_p, &system_param_set);
@@ -296,67 +302,67 @@ zgetsystemparam(i_ctx_t *i_ctx_p)
 /* ------ User parameters ------ */
 
 /* Integer values */
-private long
+static long
 current_JobTimeout(i_ctx_t *i_ctx_p)
 {
     return 0;
 }
-private int
+static int
 set_JobTimeout(i_ctx_t *i_ctx_p, long val)
 {
     return 0;
 }
-private long
+static long
 current_MaxFontItem(i_ctx_t *i_ctx_p)
 {
     return gs_currentcacheupper(ifont_dir);
 }
-private int
+static int
 set_MaxFontItem(i_ctx_t *i_ctx_p, long val)
 {
     return gs_setcacheupper(ifont_dir, val);
 }
-private long
+static long
 current_MinFontCompress(i_ctx_t *i_ctx_p)
 {
     return gs_currentcachelower(ifont_dir);
 }
-private int
+static int
 set_MinFontCompress(i_ctx_t *i_ctx_p, long val)
 {
     return gs_setcachelower(ifont_dir, val);
 }
-private long
+static long
 current_MaxOpStack(i_ctx_t *i_ctx_p)
 {
     return ref_stack_max_count(&o_stack);
 }
-private int
+static int
 set_MaxOpStack(i_ctx_t *i_ctx_p, long val)
 {
     return ref_stack_set_max_count(&o_stack, val);
 }
-private long
+static long
 current_MaxDictStack(i_ctx_t *i_ctx_p)
 {
     return ref_stack_max_count(&d_stack);
 }
-private int
+static int
 set_MaxDictStack(i_ctx_t *i_ctx_p, long val)
 {
     return ref_stack_set_max_count(&d_stack, val);
 }
-private long
+static long
 current_MaxExecStack(i_ctx_t *i_ctx_p)
 {
     return ref_stack_max_count(&e_stack);
 }
-private int
+static int
 set_MaxExecStack(i_ctx_t *i_ctx_p, long val)
 {
     return ref_stack_set_max_count(&e_stack, val);
 }
-private long
+static long
 current_MaxLocalVM(i_ctx_t *i_ctx_p)
 {
     gs_memory_gc_status_t stat;
@@ -364,7 +370,7 @@ current_MaxLocalVM(i_ctx_t *i_ctx_p)
     gs_memory_gc_status(iimemory_local, &stat);
     return stat.max_vm;
 }
-private int
+static int
 set_MaxLocalVM(i_ctx_t *i_ctx_p, long val)
 {
     gs_memory_gc_status_t stat;
@@ -374,7 +380,7 @@ set_MaxLocalVM(i_ctx_t *i_ctx_p, long val)
     gs_memory_set_gc_status(iimemory_local, &stat);
     return 0;
 }
-private long
+static long
 current_VMReclaim(i_ctx_t *i_ctx_p)
 {
     gs_memory_gc_status_t gstat, lstat;
@@ -383,7 +389,7 @@ current_VMReclaim(i_ctx_t *i_ctx_p)
     gs_memory_gc_status(iimemory_local, &lstat);
     return (!gstat.enabled ? -2 : !lstat.enabled ? -1 : 0);
 }
-private long
+static long
 current_VMThreshold(i_ctx_t *i_ctx_p)
 {
     gs_memory_gc_status_t stat;
@@ -391,34 +397,56 @@ current_VMThreshold(i_ctx_t *i_ctx_p)
     gs_memory_gc_status(iimemory_local, &stat);
     return stat.vm_threshold;
 }
-private long
+static long
 current_WaitTimeout(i_ctx_t *i_ctx_p)
 {
     return 0;
 }
-private int
+static int
 set_WaitTimeout(i_ctx_t *i_ctx_p, long val)
 {
     return 0;
 }
-private long
+static long
 current_MinScreenLevels(i_ctx_t *i_ctx_p)
 {
     return gs_currentminscreenlevels();
 }
-private int
+static int
 set_MinScreenLevels(i_ctx_t *i_ctx_p, long val)
 {
     gs_setminscreenlevels((uint) val);
     return 0;
 }
-private const long_param_def_t user_long_params[] =
+static long
+current_AlignToPixels(i_ctx_t *i_ctx_p)
+{
+    return gs_currentaligntopixels(ifont_dir);
+}
+static int
+set_AlignToPixels(i_ctx_t *i_ctx_p, long val)
+{
+    gs_setaligntopixels(ifont_dir, (uint)val);
+    return 0;
+}
+static long
+current_GridFitTT(i_ctx_t *i_ctx_p)
+{
+    return gs_currentgridfittt(ifont_dir);
+}
+static int
+set_GridFitTT(i_ctx_t *i_ctx_p, long val)
+{
+    gs_setgridfittt(ifont_dir, (uint)val);
+    return 0;
+}
+static const long_param_def_t user_long_params[] =
 {
     {"JobTimeout", 0, MAX_UINT_PARAM,
      current_JobTimeout, set_JobTimeout},
-    {"MaxFontItem", 0, MAX_UINT_PARAM,
+    {"MaxFontItem", MIN_INT_PARAM, MAX_UINT_PARAM,
      current_MaxFontItem, set_MaxFontItem},
-    {"MinFontCompress", 0, MAX_UINT_PARAM,
+    {"MinFontCompress", MIN_INT_PARAM, MAX_UINT_PARAM,
      current_MinFontCompress, set_MinFontCompress},
     {"MaxOpStack", 0, MAX_UINT_PARAM,
      current_MaxOpStack, set_MaxOpStack},
@@ -436,27 +464,43 @@ private const long_param_def_t user_long_params[] =
      current_WaitTimeout, set_WaitTimeout},
     /* Extensions */
     {"MinScreenLevels", 0, MAX_UINT_PARAM,
-     current_MinScreenLevels, set_MinScreenLevels}
+     current_MinScreenLevels, set_MinScreenLevels},
+    {"AlignToPixels", 0, 1,
+     current_AlignToPixels, set_AlignToPixels},
+    {"GridFitTT", 0, 3, 
+     current_GridFitTT, set_GridFitTT}
 };
 
 /* Boolean values */
-private bool
+static bool
 current_AccurateScreens(i_ctx_t *i_ctx_p)
 {
     return gs_currentaccuratescreens();
 }
-private int
+static int
 set_AccurateScreens(i_ctx_t *i_ctx_p, bool val)
 {
     gs_setaccuratescreens(val);
     return 0;
 }
-private bool
+/* Boolean values */
+static bool
+current_UseWTS(i_ctx_t *i_ctx_p)
+{
+    return gs_currentusewts();
+}
+static int
+set_UseWTS(i_ctx_t *i_ctx_p, bool val)
+{
+    gs_setusewts(val);
+    return 0;
+}
+static bool
 current_LockFilePermissions(i_ctx_t *i_ctx_p)
 {
     return i_ctx_p->LockFilePermissions;
 }
-private int
+static int
 set_LockFilePermissions(i_ctx_t *i_ctx_p, bool val)
 {
     /* allow locking even if already locked */
@@ -465,14 +509,27 @@ set_LockFilePermissions(i_ctx_t *i_ctx_p, bool val)
     i_ctx_p->LockFilePermissions = val;
     return 0;
 }
-private const bool_param_def_t user_bool_params[] =
+static bool
+current_RenderTTNotdef(i_ctx_t *i_ctx_p)
+{
+    return i_ctx_p->RenderTTNotdef;
+}
+static int
+set_RenderTTNotdef(i_ctx_t *i_ctx_p, bool val)
+{
+    i_ctx_p->RenderTTNotdef = val;
+    return 0;
+}
+static const bool_param_def_t user_bool_params[] =
 {
     {"AccurateScreens", current_AccurateScreens, set_AccurateScreens},
-    {"LockFilePermissions", current_LockFilePermissions, set_LockFilePermissions}
+    {"UseWTS", current_UseWTS, set_UseWTS},
+    {"LockFilePermissions", current_LockFilePermissions, set_LockFilePermissions},
+    {"RenderTTNotdef", current_RenderTTNotdef, set_RenderTTNotdef}
 };
 
 /* The user parameter set */
-private const param_set user_param_set =
+static const param_set user_param_set =
 {
     user_long_params, countof(user_long_params),
     user_bool_params, countof(user_bool_params),
@@ -495,7 +552,7 @@ set_user_params(i_ctx_t *i_ctx_p, const ref *paramdict)
     iparam_list_release(&list);
     return code;
 }
-private int
+static int
 zsetuserparams(i_ctx_t *i_ctx_p)
 {
     os_ptr op = osp;
@@ -511,14 +568,14 @@ zsetuserparams(i_ctx_t *i_ctx_p)
 }
 
 /* - .currentuserparams <name1> <value1> ... */
-private int
+static int
 zcurrentuserparams(i_ctx_t *i_ctx_p)
 {
     return currentparams(i_ctx_p, &user_param_set);
 }
 
 /* <name> .getuserparam <value> */
-private int
+static int
 zgetuserparam(i_ctx_t *i_ctx_p)
 {
     return currentparam1(i_ctx_p, &user_param_set);
@@ -546,7 +603,7 @@ const op_def zusparam_op_defs[] =
 
 /* Set the values of a parameter set from a parameter list. */
 /* We don't attempt to back out if anything fails. */
-private int
+static int
 setparams(i_ctx_t *i_ctx_p, gs_param_list * plist, const param_set * pset)
 {
     int i, code;
@@ -588,7 +645,7 @@ setparams(i_ctx_t *i_ctx_p, gs_param_list * plist, const param_set * pset)
 }
 
 /* Get the current values of a parameter set to the stack. */
-private bool
+static bool
 pname_matches(const char *pname, const ref * psref)
 {
     return
@@ -596,13 +653,13 @@ pname_matches(const char *pname, const ref * psref)
 	 !bytes_compare((const byte *)pname, strlen(pname),
 			psref->value.const_bytes, r_size(psref)));
 }
-private int
+static int
 current_param_list(i_ctx_t *i_ctx_p, const param_set * pset,
 		   const ref * psref /*t_string */ )
 {
     stack_param_list list;
     gs_param_list *const plist = (gs_param_list *)&list;
-    int i;
+    int i, code = 0;
 
     stack_param_list_write(&list, &o_stack, NULL, iimemory);
     for (i = 0; i < pset->long_count; i++) {
@@ -610,8 +667,8 @@ current_param_list(i_ctx_t *i_ctx_p, const param_set * pset,
 
 	if (pname_matches(pname, psref)) {
 	    long val = (*pset->long_defs[i].current)(i_ctx_p);
-	    int code = param_write_long(plist, pname, &val);
 
+	    code = param_write_long(plist, pname, &val);
 	    if (code < 0)
 		return code;
 	}
@@ -621,8 +678,8 @@ current_param_list(i_ctx_t *i_ctx_p, const param_set * pset,
 
 	if (pname_matches(pname, psref)) {
 	    bool val = (*pset->bool_defs[i].current)(i_ctx_p);
-	    int code = param_write_bool(plist, pname, &val);
 
+	    code = param_write_bool(plist, pname, &val);
 	    if (code < 0)
 		return code;
 	}
@@ -632,7 +689,6 @@ current_param_list(i_ctx_t *i_ctx_p, const param_set * pset,
 
 	if (pname_matches(pname, psref)) {
 	    gs_param_string val;
-	    int code;
 
 	    (*pset->string_defs[i].current)(i_ctx_p, &val);
 	    code = param_write_string(plist, pname, &val);
@@ -640,18 +696,49 @@ current_param_list(i_ctx_t *i_ctx_p, const param_set * pset,
 		return code;
 	}
     }
-    return 0;
+    if (psref) {
+	/*
+	 * Scanner options can be read, but only individually by .getuserparam.
+	 * This avoids putting them into userparams, and being affected by save/restore.
+	 */
+	const char *pname;
+	bool val;
+	int code;
+
+	switch (ztoken_get_scanner_option(psref, i_ctx_p->scanner_options, &pname)) {
+	    case 0:
+		code = param_write_null(plist, pname);
+		break;
+	    case 1:
+		val = true;
+		code = param_write_bool(plist, pname, &val);
+		break;
+	    default:
+		code = 0;
+		break;
+	}
+	if (code < 0)
+	    return code;
+    }
+#if ENABLE_CUSTOM_COLOR_CALLBACK
+    if (pset == &system_param_set) {
+        /* The custom_color callback pointer */
+	if (pname_matches(CustomColorCallbackParamName, psref))
+	    code = custom_color_callback_get_params(i_ctx_p->pgs, plist);
+    }
+#endif
+    return code;
 }
 
 /* Get the current values of a parameter set to the stack. */
-private int
+static int
 currentparams(i_ctx_t *i_ctx_p, const param_set * pset)
 {
     return current_param_list(i_ctx_p, pset, NULL);
 }
 
 /* Get the value of a single parameter to the stack, or signal an error. */
-private int
+static int
 currentparam1(i_ctx_t *i_ctx_p, const param_set * pset)
 {
     os_ptr op = osp;
@@ -660,7 +747,7 @@ currentparam1(i_ctx_t *i_ctx_p, const param_set * pset)
 
     check_type(*op, t_name);
     check_ostack(2);
-    name_string_ref((const ref *)op, &sref);
+    name_string_ref(imemory, (const ref *)op, &sref);
     code = current_param_list(i_ctx_p, pset, &sref);
     if (code < 0)
 	return code;
